/**
 * Copyright (c) 2000-2012 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.qti.slayer.service.persistence;

import com.liferay.portal.service.persistence.BasePersistence;

import com.qti.slayer.model.Assignment;

/**
 * The persistence interface for the assignment service.
 *
 * <p>
 * Caching information and settings can be found in <code>portal.properties</code>
 * </p>
 *
 * @author Hector
 * @see AssignmentPersistenceImpl
 * @see AssignmentUtil
 * @generated
 */
public interface AssignmentPersistence extends BasePersistence<Assignment> {
	/*
	 * NOTE FOR DEVELOPERS:
	 *
	 * Never modify or reference this interface directly. Always use {@link AssignmentUtil} to access the assignment persistence. Modify <code>service.xml</code> and rerun ServiceBuilder to regenerate this interface.
	 */

	/**
	* Caches the assignment in the entity cache if it is enabled.
	*
	* @param assignment the assignment
	*/
	public void cacheResult(com.qti.slayer.model.Assignment assignment);

	/**
	* Caches the assignments in the entity cache if it is enabled.
	*
	* @param assignments the assignments
	*/
	public void cacheResult(
		java.util.List<com.qti.slayer.model.Assignment> assignments);

	/**
	* Creates a new assignment with the primary key. Does not add the assignment to the database.
	*
	* @param assignmentPK the primary key for the new assignment
	* @return the new assignment
	*/
	public com.qti.slayer.model.Assignment create(
		com.qti.slayer.service.persistence.AssignmentPK assignmentPK);

	/**
	* Removes the assignment with the primary key from the database. Also notifies the appropriate model listeners.
	*
	* @param assignmentPK the primary key of the assignment
	* @return the assignment that was removed
	* @throws com.qti.slayer.NoSuchAssignmentException if a assignment with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment remove(
		com.qti.slayer.service.persistence.AssignmentPK assignmentPK)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchAssignmentException;

	public com.qti.slayer.model.Assignment updateImpl(
		com.qti.slayer.model.Assignment assignment, boolean merge)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the assignment with the primary key or throws a {@link com.qti.slayer.NoSuchAssignmentException} if it could not be found.
	*
	* @param assignmentPK the primary key of the assignment
	* @return the assignment
	* @throws com.qti.slayer.NoSuchAssignmentException if a assignment with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment findByPrimaryKey(
		com.qti.slayer.service.persistence.AssignmentPK assignmentPK)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchAssignmentException;

	/**
	* Returns the assignment with the primary key or returns <code>null</code> if it could not be found.
	*
	* @param assignmentPK the primary key of the assignment
	* @return the assignment, or <code>null</code> if a assignment with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment fetchByPrimaryKey(
		com.qti.slayer.service.persistence.AssignmentPK assignmentPK)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns all the assignments where userId = &#63;.
	*
	* @param userId the user ID
	* @return the matching assignments
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.Assignment> findByUserId(
		long userId) throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns a range of all the assignments where userId = &#63;.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param userId the user ID
	* @param start the lower bound of the range of assignments
	* @param end the upper bound of the range of assignments (not inclusive)
	* @return the range of matching assignments
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.Assignment> findByUserId(
		long userId, int start, int end)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns an ordered range of all the assignments where userId = &#63;.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param userId the user ID
	* @param start the lower bound of the range of assignments
	* @param end the upper bound of the range of assignments (not inclusive)
	* @param orderByComparator the comparator to order the results by (optionally <code>null</code>)
	* @return the ordered range of matching assignments
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.Assignment> findByUserId(
		long userId, int start, int end,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the first assignment in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the first matching assignment
	* @throws com.qti.slayer.NoSuchAssignmentException if a matching assignment could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment findByUserId_First(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchAssignmentException;

	/**
	* Returns the first assignment in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the first matching assignment, or <code>null</code> if a matching assignment could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment fetchByUserId_First(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the last assignment in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the last matching assignment
	* @throws com.qti.slayer.NoSuchAssignmentException if a matching assignment could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment findByUserId_Last(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchAssignmentException;

	/**
	* Returns the last assignment in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the last matching assignment, or <code>null</code> if a matching assignment could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment fetchByUserId_Last(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the assignments before and after the current assignment in the ordered set where userId = &#63;.
	*
	* @param assignmentPK the primary key of the current assignment
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the previous, current, and next assignment
	* @throws com.qti.slayer.NoSuchAssignmentException if a assignment with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.Assignment[] findByUserId_PrevAndNext(
		com.qti.slayer.service.persistence.AssignmentPK assignmentPK,
		long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchAssignmentException;

	/**
	* Returns all the assignments.
	*
	* @return the assignments
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.Assignment> findAll()
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns a range of all the assignments.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param start the lower bound of the range of assignments
	* @param end the upper bound of the range of assignments (not inclusive)
	* @return the range of assignments
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.Assignment> findAll(int start,
		int end) throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns an ordered range of all the assignments.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param start the lower bound of the range of assignments
	* @param end the upper bound of the range of assignments (not inclusive)
	* @param orderByComparator the comparator to order the results by (optionally <code>null</code>)
	* @return the ordered range of assignments
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.Assignment> findAll(int start,
		int end,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Removes all the assignments where userId = &#63; from the database.
	*
	* @param userId the user ID
	* @throws SystemException if a system exception occurred
	*/
	public void removeByUserId(long userId)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Removes all the assignments from the database.
	*
	* @throws SystemException if a system exception occurred
	*/
	public void removeAll()
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the number of assignments where userId = &#63;.
	*
	* @param userId the user ID
	* @return the number of matching assignments
	* @throws SystemException if a system exception occurred
	*/
	public int countByUserId(long userId)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the number of assignments.
	*
	* @return the number of assignments
	* @throws SystemException if a system exception occurred
	*/
	public int countAll()
		throws com.liferay.portal.kernel.exception.SystemException;
}