/**
 * Copyright (c) 2000-2012 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.qti.slayer.service.persistence;

import com.liferay.portal.service.persistence.BasePersistence;

import com.qti.slayer.model.TestResult;

/**
 * The persistence interface for the test result service.
 *
 * <p>
 * Caching information and settings can be found in <code>portal.properties</code>
 * </p>
 *
 * @author Hector
 * @see TestResultPersistenceImpl
 * @see TestResultUtil
 * @generated
 */
public interface TestResultPersistence extends BasePersistence<TestResult> {
	/*
	 * NOTE FOR DEVELOPERS:
	 *
	 * Never modify or reference this interface directly. Always use {@link TestResultUtil} to access the test result persistence. Modify <code>service.xml</code> and rerun ServiceBuilder to regenerate this interface.
	 */

	/**
	* Caches the test result in the entity cache if it is enabled.
	*
	* @param testResult the test result
	*/
	public void cacheResult(com.qti.slayer.model.TestResult testResult);

	/**
	* Caches the test results in the entity cache if it is enabled.
	*
	* @param testResults the test results
	*/
	public void cacheResult(
		java.util.List<com.qti.slayer.model.TestResult> testResults);

	/**
	* Creates a new test result with the primary key. Does not add the test result to the database.
	*
	* @param testResultPK the primary key for the new test result
	* @return the new test result
	*/
	public com.qti.slayer.model.TestResult create(
		com.qti.slayer.service.persistence.TestResultPK testResultPK);

	/**
	* Removes the test result with the primary key from the database. Also notifies the appropriate model listeners.
	*
	* @param testResultPK the primary key of the test result
	* @return the test result that was removed
	* @throws com.qti.slayer.NoSuchTestResultException if a test result with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult remove(
		com.qti.slayer.service.persistence.TestResultPK testResultPK)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	public com.qti.slayer.model.TestResult updateImpl(
		com.qti.slayer.model.TestResult testResult, boolean merge)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the test result with the primary key or throws a {@link com.qti.slayer.NoSuchTestResultException} if it could not be found.
	*
	* @param testResultPK the primary key of the test result
	* @return the test result
	* @throws com.qti.slayer.NoSuchTestResultException if a test result with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult findByPrimaryKey(
		com.qti.slayer.service.persistence.TestResultPK testResultPK)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns the test result with the primary key or returns <code>null</code> if it could not be found.
	*
	* @param testResultPK the primary key of the test result
	* @return the test result, or <code>null</code> if a test result with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult fetchByPrimaryKey(
		com.qti.slayer.service.persistence.TestResultPK testResultPK)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns all the test results where userId = &#63;.
	*
	* @param userId the user ID
	* @return the matching test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findByUserId(
		long userId) throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns a range of all the test results where userId = &#63;.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param userId the user ID
	* @param start the lower bound of the range of test results
	* @param end the upper bound of the range of test results (not inclusive)
	* @return the range of matching test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findByUserId(
		long userId, int start, int end)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns an ordered range of all the test results where userId = &#63;.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param userId the user ID
	* @param start the lower bound of the range of test results
	* @param end the upper bound of the range of test results (not inclusive)
	* @param orderByComparator the comparator to order the results by (optionally <code>null</code>)
	* @return the ordered range of matching test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findByUserId(
		long userId, int start, int end,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the first test result in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the first matching test result
	* @throws com.qti.slayer.NoSuchTestResultException if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult findByUserId_First(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns the first test result in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the first matching test result, or <code>null</code> if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult fetchByUserId_First(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the last test result in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the last matching test result
	* @throws com.qti.slayer.NoSuchTestResultException if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult findByUserId_Last(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns the last test result in the ordered set where userId = &#63;.
	*
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the last matching test result, or <code>null</code> if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult fetchByUserId_Last(long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the test results before and after the current test result in the ordered set where userId = &#63;.
	*
	* @param testResultPK the primary key of the current test result
	* @param userId the user ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the previous, current, and next test result
	* @throws com.qti.slayer.NoSuchTestResultException if a test result with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult[] findByUserId_PrevAndNext(
		com.qti.slayer.service.persistence.TestResultPK testResultPK,
		long userId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns all the test results where uolId = &#63;.
	*
	* @param uolId the uol ID
	* @return the matching test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findByUolId(
		long uolId) throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns a range of all the test results where uolId = &#63;.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param uolId the uol ID
	* @param start the lower bound of the range of test results
	* @param end the upper bound of the range of test results (not inclusive)
	* @return the range of matching test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findByUolId(
		long uolId, int start, int end)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns an ordered range of all the test results where uolId = &#63;.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param uolId the uol ID
	* @param start the lower bound of the range of test results
	* @param end the upper bound of the range of test results (not inclusive)
	* @param orderByComparator the comparator to order the results by (optionally <code>null</code>)
	* @return the ordered range of matching test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findByUolId(
		long uolId, int start, int end,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the first test result in the ordered set where uolId = &#63;.
	*
	* @param uolId the uol ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the first matching test result
	* @throws com.qti.slayer.NoSuchTestResultException if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult findByUolId_First(long uolId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns the first test result in the ordered set where uolId = &#63;.
	*
	* @param uolId the uol ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the first matching test result, or <code>null</code> if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult fetchByUolId_First(long uolId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the last test result in the ordered set where uolId = &#63;.
	*
	* @param uolId the uol ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the last matching test result
	* @throws com.qti.slayer.NoSuchTestResultException if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult findByUolId_Last(long uolId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns the last test result in the ordered set where uolId = &#63;.
	*
	* @param uolId the uol ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the last matching test result, or <code>null</code> if a matching test result could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult fetchByUolId_Last(long uolId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the test results before and after the current test result in the ordered set where uolId = &#63;.
	*
	* @param testResultPK the primary key of the current test result
	* @param uolId the uol ID
	* @param orderByComparator the comparator to order the set by (optionally <code>null</code>)
	* @return the previous, current, and next test result
	* @throws com.qti.slayer.NoSuchTestResultException if a test result with the primary key could not be found
	* @throws SystemException if a system exception occurred
	*/
	public com.qti.slayer.model.TestResult[] findByUolId_PrevAndNext(
		com.qti.slayer.service.persistence.TestResultPK testResultPK,
		long uolId,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException,
			com.qti.slayer.NoSuchTestResultException;

	/**
	* Returns all the test results.
	*
	* @return the test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findAll()
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns a range of all the test results.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param start the lower bound of the range of test results
	* @param end the upper bound of the range of test results (not inclusive)
	* @return the range of test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findAll(int start,
		int end) throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns an ordered range of all the test results.
	*
	* <p>
	* Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	* </p>
	*
	* @param start the lower bound of the range of test results
	* @param end the upper bound of the range of test results (not inclusive)
	* @param orderByComparator the comparator to order the results by (optionally <code>null</code>)
	* @return the ordered range of test results
	* @throws SystemException if a system exception occurred
	*/
	public java.util.List<com.qti.slayer.model.TestResult> findAll(int start,
		int end,
		com.liferay.portal.kernel.util.OrderByComparator orderByComparator)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Removes all the test results where userId = &#63; from the database.
	*
	* @param userId the user ID
	* @throws SystemException if a system exception occurred
	*/
	public void removeByUserId(long userId)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Removes all the test results where uolId = &#63; from the database.
	*
	* @param uolId the uol ID
	* @throws SystemException if a system exception occurred
	*/
	public void removeByUolId(long uolId)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Removes all the test results from the database.
	*
	* @throws SystemException if a system exception occurred
	*/
	public void removeAll()
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the number of test results where userId = &#63;.
	*
	* @param userId the user ID
	* @return the number of matching test results
	* @throws SystemException if a system exception occurred
	*/
	public int countByUserId(long userId)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the number of test results where uolId = &#63;.
	*
	* @param uolId the uol ID
	* @return the number of matching test results
	* @throws SystemException if a system exception occurred
	*/
	public int countByUolId(long uolId)
		throws com.liferay.portal.kernel.exception.SystemException;

	/**
	* Returns the number of test results.
	*
	* @return the number of test results
	* @throws SystemException if a system exception occurred
	*/
	public int countAll()
		throws com.liferay.portal.kernel.exception.SystemException;
}