package com.qti;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.DynamicQueryFactoryUtil;
import com.liferay.portal.kernel.dao.orm.PropertyFactoryUtil;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.service.CompanyLocalServiceUtil;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.util.bridges.mvc.MVCPortlet;
import com.qti.slayer.model.Assignment;
import com.qti.slayer.model.Uol;
import com.qti.slayer.model.impl.AssignmentImpl;
import com.qti.slayer.model.impl.UolImpl;
import com.qti.slayer.service.AssignmentLocalServiceUtil;
import com.qti.slayer.service.UolLocalServiceUtil;
import com.liferay.portal.model.User;
import com.liferay.portal.model.Company;

/**
 * Portlet implementation class AssignmentPortlet
 */
public class AssignmentPortlet extends MVCPortlet {
	
	/**
	* Updates Unit of Learning Assignment.
	*
	* @param actionRequest  ActionRequest object
	* @param actionResponse Action Response object
	* @throws IOException if system exception occurred
	* @throws PortletException if a portlet exception occurred
	*/
	public void updateUolAssignment(ActionRequest actionRequest,ActionResponse actionResponse)	throws IOException, PortletException 
	{
	String uolTitle = ParamUtil.getString(actionRequest, "uolTitle");
	String userName = ParamUtil.getString(actionRequest, "userName");
	System.out.println("Els inputs ==> " + uolTitle + ", " + userName);
	
	
	Assignment assAux = new AssignmentImpl();
	//set primary key
	long uolId =0L;
	long userId =0L;
	User userAux = null;
	Uol uolAux = new UolImpl();
	Company compAux = null;
	
	try 
	{
		

		try {
			//get current user Id (Company Administrator??)
			String userIdAdm = actionRequest.getRemoteUser();
			System.out.println("Usuario Remoto: " + userIdAdm);
			
			long compId=0;
			
			try
			{
				compId = CompanyLocalServiceUtil.getCompanyIdByUserId(Long.parseLong(userIdAdm));
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
				}
			
			compAux = CompanyLocalServiceUtil.getCompanyById(compId);
			
			userAux = UserLocalServiceUtil.getUserByScreenName(compAux.getCompanyId(), userName);
			
			userId = UserLocalServiceUtil.getUserIdByScreenName(compAux.getCompanyId(), userName);
			
			System.out.println("UserId: " + userId);
			
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		DynamicQuery dynamicQuery = DynamicQueryFactoryUtil.forClass(Uol.class);
		dynamicQuery.add(PropertyFactoryUtil.forName("title").eq(uolTitle));
		List<Uol> uols = UolLocalServiceUtil.dynamicQuery(dynamicQuery);
		if (!uols.isEmpty() && uols.size()==1)
		{
			uolId = uols.get(0).getI_Id();
			uolAux = uols.get(0);
		}
			
		
	} 
	
	catch (SystemException e)
	{
		e.printStackTrace();
		
	}
	
	assAux.setUserId(userId);
	assAux.setUolId(uolId);
	
	//Extra fields...
	
	assAux.setUsername(userAux.getScreenName());
	assAux.setTitle(uolAux.getTitle());
	
	//status=0 (non active), Type=0 (QTI)
	assAux.setStatus(0);
	assAux.setUoltype(0);
	
	
	//insert the assignment using persistence API
	
	try 
	{
		AssignmentLocalServiceUtil.addAssignment(assAux);
	}
	catch (SystemException e) 
	{
		e.printStackTrace();	
	}
	
	// gracefully redirecting to the default portlet view
	String redirectURL = ParamUtil.getString(actionRequest, "redirectURL");
	actionResponse.sendRedirect(redirectURL);
 
	}
	
	/**
	* Removes Unit of Learning Assignment.
	*
	* @param actionRequest  ActionRequest object
	* @param actionResponse Action Response object
	* @throws IOException if system exception occurred
	* @throws PortletException if a portlet exception occurred
	*/
	public void removeAssignmentAction(ActionRequest actionRequest,ActionResponse actionResponse)	throws IOException, PortletException 
	{
		String uolId = ParamUtil.getString(actionRequest, "uolId");
		String userId = ParamUtil.getString(actionRequest, "userId");
		System.out.println("Els inputs ==> " + uolId + ", " + userId);
		
		
		Assignment assAux = AssignmentLocalServiceUtil.getAssignment(uolId, userId);
		System.out.println("Deleting assignment:" + assAux.toString());
		
		//remove the assignment using persistence API
		
		try 
		{
			AssignmentLocalServiceUtil.deleteAssignment(assAux);
		}
		catch (SystemException e) 
		{
			e.printStackTrace();	
		}
		
	}
 

}
