/**
 * Copyright (c) 2000-2012 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.qti.slayer.model.impl;

import com.liferay.portal.kernel.bean.AutoEscapeBeanHandler;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.json.JSON;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.ProxyUtil;
import com.liferay.portal.kernel.util.StringBundler;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.CacheModel;
import com.liferay.portal.model.impl.BaseModelImpl;
import com.liferay.portal.util.PortalUtil;

import com.qti.slayer.model.Assignment;
import com.qti.slayer.model.AssignmentModel;
import com.qti.slayer.model.AssignmentSoap;
import com.qti.slayer.service.persistence.AssignmentPK;

import java.io.Serializable;

import java.sql.Types;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The base model implementation for the Assignment service. Represents a row in the &quot;QTI_Assignment&quot; database table, with each column mapped to a property of this class.
 *
 * <p>
 * This implementation and its corresponding interface {@link com.qti.slayer.model.AssignmentModel} exist only as a container for the default property accessors generated by ServiceBuilder. Helper methods and all application logic should be put in {@link AssignmentImpl}.
 * </p>
 *
 * @author Hector
 * @see AssignmentImpl
 * @see com.qti.slayer.model.Assignment
 * @see com.qti.slayer.model.AssignmentModel
 * @generated
 */
@JSON(strict = true)
public class AssignmentModelImpl extends BaseModelImpl<Assignment>
	implements AssignmentModel {
	/*
	 * NOTE FOR DEVELOPERS:
	 *
	 * Never modify or reference this class directly. All methods that expect a assignment model instance should use the {@link com.qti.slayer.model.Assignment} interface instead.
	 */
	public static final String TABLE_NAME = "QTI_Assignment";
	public static final Object[][] TABLE_COLUMNS = {
			{ "userId", Types.BIGINT },
			{ "uolId", Types.BIGINT },
			{ "status", Types.INTEGER },
			{ "uoltype", Types.INTEGER },
			{ "title", Types.VARCHAR },
			{ "username", Types.VARCHAR }
		};
	public static final String TABLE_SQL_CREATE = "create table QTI_Assignment (userId LONG not null,uolId LONG not null,status INTEGER,uoltype INTEGER,title VARCHAR(75) null,username VARCHAR(75) null,primary key (userId, uolId))";
	public static final String TABLE_SQL_DROP = "drop table QTI_Assignment";
	public static final String ORDER_BY_JPQL = " ORDER BY assignment.id.userId ASC";
	public static final String ORDER_BY_SQL = " ORDER BY QTI_Assignment.userId ASC";
	public static final String DATA_SOURCE = "liferayDataSource";
	public static final String SESSION_FACTORY = "liferaySessionFactory";
	public static final String TX_MANAGER = "liferayTransactionManager";
	public static final boolean ENTITY_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.entity.cache.enabled.com.qti.slayer.model.Assignment"),
			true);
	public static final boolean FINDER_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.finder.cache.enabled.com.qti.slayer.model.Assignment"),
			true);
	public static final boolean COLUMN_BITMASK_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.column.bitmask.enabled.com.qti.slayer.model.Assignment"),
			true);
	public static long USERID_COLUMN_BITMASK = 1L;

	/**
	 * Converts the soap model instance into a normal model instance.
	 *
	 * @param soapModel the soap model instance to convert
	 * @return the normal model instance
	 */
	public static Assignment toModel(AssignmentSoap soapModel) {
		if (soapModel == null) {
			return null;
		}

		Assignment model = new AssignmentImpl();

		model.setUserId(soapModel.getUserId());
		model.setUolId(soapModel.getUolId());
		model.setStatus(soapModel.getStatus());
		model.setUoltype(soapModel.getUoltype());
		model.setTitle(soapModel.getTitle());
		model.setUsername(soapModel.getUsername());

		return model;
	}

	/**
	 * Converts the soap model instances into normal model instances.
	 *
	 * @param soapModels the soap model instances to convert
	 * @return the normal model instances
	 */
	public static List<Assignment> toModels(AssignmentSoap[] soapModels) {
		if (soapModels == null) {
			return null;
		}

		List<Assignment> models = new ArrayList<Assignment>(soapModels.length);

		for (AssignmentSoap soapModel : soapModels) {
			models.add(toModel(soapModel));
		}

		return models;
	}

	public static final long LOCK_EXPIRATION_TIME = GetterUtil.getLong(com.liferay.util.service.ServiceProps.get(
				"lock.expiration.time.com.qti.slayer.model.Assignment"));

	public AssignmentModelImpl() {
	}

	public AssignmentPK getPrimaryKey() {
		return new AssignmentPK(_userId, _uolId);
	}

	public void setPrimaryKey(AssignmentPK primaryKey) {
		setUserId(primaryKey.userId);
		setUolId(primaryKey.uolId);
	}

	public Serializable getPrimaryKeyObj() {
		return new AssignmentPK(_userId, _uolId);
	}

	public void setPrimaryKeyObj(Serializable primaryKeyObj) {
		setPrimaryKey((AssignmentPK)primaryKeyObj);
	}

	public Class<?> getModelClass() {
		return Assignment.class;
	}

	public String getModelClassName() {
		return Assignment.class.getName();
	}

	@Override
	public Map<String, Object> getModelAttributes() {
		Map<String, Object> attributes = new HashMap<String, Object>();

		attributes.put("userId", getUserId());
		attributes.put("uolId", getUolId());
		attributes.put("status", getStatus());
		attributes.put("uoltype", getUoltype());
		attributes.put("title", getTitle());
		attributes.put("username", getUsername());

		return attributes;
	}

	@Override
	public void setModelAttributes(Map<String, Object> attributes) {
		Long userId = (Long)attributes.get("userId");

		if (userId != null) {
			setUserId(userId);
		}

		Long uolId = (Long)attributes.get("uolId");

		if (uolId != null) {
			setUolId(uolId);
		}

		Integer status = (Integer)attributes.get("status");

		if (status != null) {
			setStatus(status);
		}

		Integer uoltype = (Integer)attributes.get("uoltype");

		if (uoltype != null) {
			setUoltype(uoltype);
		}

		String title = (String)attributes.get("title");

		if (title != null) {
			setTitle(title);
		}

		String username = (String)attributes.get("username");

		if (username != null) {
			setUsername(username);
		}
	}

	@JSON
	public long getUserId() {
		return _userId;
	}

	public void setUserId(long userId) {
		_columnBitmask = -1L;

		if (!_setOriginalUserId) {
			_setOriginalUserId = true;

			_originalUserId = _userId;
		}

		_userId = userId;
	}

	public String getUserUuid() throws SystemException {
		return PortalUtil.getUserValue(getUserId(), "uuid", _userUuid);
	}

	public void setUserUuid(String userUuid) {
		_userUuid = userUuid;
	}

	public long getOriginalUserId() {
		return _originalUserId;
	}

	@JSON
	public long getUolId() {
		return _uolId;
	}

	public void setUolId(long uolId) {
		_uolId = uolId;
	}

	@JSON
	public int getStatus() {
		return _status;
	}

	public void setStatus(int status) {
		_status = status;
	}

	@JSON
	public int getUoltype() {
		return _uoltype;
	}

	public void setUoltype(int uoltype) {
		_uoltype = uoltype;
	}

	@JSON
	public String getTitle() {
		if (_title == null) {
			return StringPool.BLANK;
		}
		else {
			return _title;
		}
	}

	public void setTitle(String title) {
		_title = title;
	}

	@JSON
	public String getUsername() {
		if (_username == null) {
			return StringPool.BLANK;
		}
		else {
			return _username;
		}
	}

	public void setUsername(String username) {
		_username = username;
	}

	public long getColumnBitmask() {
		return _columnBitmask;
	}

	@Override
	public Assignment toEscapedModel() {
		if (_escapedModelProxy == null) {
			_escapedModelProxy = (Assignment)ProxyUtil.newProxyInstance(_classLoader,
					_escapedModelProxyInterfaces,
					new AutoEscapeBeanHandler(this));
		}

		return _escapedModelProxy;
	}

	@Override
	public Object clone() {
		AssignmentImpl assignmentImpl = new AssignmentImpl();

		assignmentImpl.setUserId(getUserId());
		assignmentImpl.setUolId(getUolId());
		assignmentImpl.setStatus(getStatus());
		assignmentImpl.setUoltype(getUoltype());
		assignmentImpl.setTitle(getTitle());
		assignmentImpl.setUsername(getUsername());

		assignmentImpl.resetOriginalValues();

		return assignmentImpl;
	}

	public int compareTo(Assignment assignment) {
		int value = 0;

		if (getUserId() < assignment.getUserId()) {
			value = -1;
		}
		else if (getUserId() > assignment.getUserId()) {
			value = 1;
		}
		else {
			value = 0;
		}

		if (value != 0) {
			return value;
		}

		return 0;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		Assignment assignment = null;

		try {
			assignment = (Assignment)obj;
		}
		catch (ClassCastException cce) {
			return false;
		}

		AssignmentPK primaryKey = assignment.getPrimaryKey();

		if (getPrimaryKey().equals(primaryKey)) {
			return true;
		}
		else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return getPrimaryKey().hashCode();
	}

	@Override
	public void resetOriginalValues() {
		AssignmentModelImpl assignmentModelImpl = this;

		assignmentModelImpl._originalUserId = assignmentModelImpl._userId;

		assignmentModelImpl._setOriginalUserId = false;

		assignmentModelImpl._columnBitmask = 0;
	}

	@Override
	public CacheModel<Assignment> toCacheModel() {
		AssignmentCacheModel assignmentCacheModel = new AssignmentCacheModel();

		assignmentCacheModel.userId = getUserId();

		assignmentCacheModel.uolId = getUolId();

		assignmentCacheModel.status = getStatus();

		assignmentCacheModel.uoltype = getUoltype();

		assignmentCacheModel.title = getTitle();

		String title = assignmentCacheModel.title;

		if ((title != null) && (title.length() == 0)) {
			assignmentCacheModel.title = null;
		}

		assignmentCacheModel.username = getUsername();

		String username = assignmentCacheModel.username;

		if ((username != null) && (username.length() == 0)) {
			assignmentCacheModel.username = null;
		}

		return assignmentCacheModel;
	}

	@Override
	public String toString() {
		StringBundler sb = new StringBundler(13);

		sb.append("{userId=");
		sb.append(getUserId());
		sb.append(", uolId=");
		sb.append(getUolId());
		sb.append(", status=");
		sb.append(getStatus());
		sb.append(", uoltype=");
		sb.append(getUoltype());
		sb.append(", title=");
		sb.append(getTitle());
		sb.append(", username=");
		sb.append(getUsername());
		sb.append("}");

		return sb.toString();
	}

	public String toXmlString() {
		StringBundler sb = new StringBundler(22);

		sb.append("<model><model-name>");
		sb.append("com.qti.slayer.model.Assignment");
		sb.append("</model-name>");

		sb.append(
			"<column><column-name>userId</column-name><column-value><![CDATA[");
		sb.append(getUserId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>uolId</column-name><column-value><![CDATA[");
		sb.append(getUolId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>status</column-name><column-value><![CDATA[");
		sb.append(getStatus());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>uoltype</column-name><column-value><![CDATA[");
		sb.append(getUoltype());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>title</column-name><column-value><![CDATA[");
		sb.append(getTitle());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>username</column-name><column-value><![CDATA[");
		sb.append(getUsername());
		sb.append("]]></column-value></column>");

		sb.append("</model>");

		return sb.toString();
	}

	private static ClassLoader _classLoader = Assignment.class.getClassLoader();
	private static Class<?>[] _escapedModelProxyInterfaces = new Class[] {
			Assignment.class
		};
	private long _userId;
	private String _userUuid;
	private long _originalUserId;
	private boolean _setOriginalUserId;
	private long _uolId;
	private int _status;
	private int _uoltype;
	private String _title;
	private String _username;
	private long _columnBitmask;
	private Assignment _escapedModelProxy;
}