/**
 * Copyright (c) 2000-2012 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.qti.slayer.model.impl;

import com.liferay.portal.kernel.bean.AutoEscapeBeanHandler;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.json.JSON;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.ProxyUtil;
import com.liferay.portal.kernel.util.StringBundler;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.CacheModel;
import com.liferay.portal.model.impl.BaseModelImpl;
import com.liferay.portal.util.PortalUtil;

import com.qti.slayer.model.TestResult;
import com.qti.slayer.model.TestResultModel;
import com.qti.slayer.model.TestResultSoap;
import com.qti.slayer.service.persistence.TestResultPK;

import java.io.Serializable;

import java.sql.Types;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The base model implementation for the TestResult service. Represents a row in the &quot;QTI_TestResult&quot; database table, with each column mapped to a property of this class.
 *
 * <p>
 * This implementation and its corresponding interface {@link com.qti.slayer.model.TestResultModel} exist only as a container for the default property accessors generated by ServiceBuilder. Helper methods and all application logic should be put in {@link TestResultImpl}.
 * </p>
 *
 * @author Hector
 * @see TestResultImpl
 * @see com.qti.slayer.model.TestResult
 * @see com.qti.slayer.model.TestResultModel
 * @generated
 */
@JSON(strict = true)
public class TestResultModelImpl extends BaseModelImpl<TestResult>
	implements TestResultModel {
	/*
	 * NOTE FOR DEVELOPERS:
	 *
	 * Never modify or reference this class directly. All methods that expect a test result model instance should use the {@link com.qti.slayer.model.TestResult} interface instead.
	 */
	public static final String TABLE_NAME = "QTI_TestResult";
	public static final Object[][] TABLE_COLUMNS = {
			{ "userId", Types.BIGINT },
			{ "uolId", Types.BIGINT },
			{ "result", Types.INTEGER },
			{ "autoComment", Types.VARCHAR },
			{ "teacherComment", Types.VARCHAR },
			{ "title", Types.VARCHAR },
			{ "username", Types.VARCHAR }
		};
	public static final String TABLE_SQL_CREATE = "create table QTI_TestResult (userId LONG not null,uolId LONG not null,result INTEGER,autoComment VARCHAR(75) null,teacherComment VARCHAR(75) null,title VARCHAR(75) null,username VARCHAR(75) null,primary key (userId, uolId))";
	public static final String TABLE_SQL_DROP = "drop table QTI_TestResult";
	public static final String ORDER_BY_JPQL = " ORDER BY testResult.id.userId ASC";
	public static final String ORDER_BY_SQL = " ORDER BY QTI_TestResult.userId ASC";
	public static final String DATA_SOURCE = "liferayDataSource";
	public static final String SESSION_FACTORY = "liferaySessionFactory";
	public static final String TX_MANAGER = "liferayTransactionManager";
	public static final boolean ENTITY_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.entity.cache.enabled.com.qti.slayer.model.TestResult"),
			true);
	public static final boolean FINDER_CACHE_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.finder.cache.enabled.com.qti.slayer.model.TestResult"),
			true);
	public static final boolean COLUMN_BITMASK_ENABLED = GetterUtil.getBoolean(com.liferay.util.service.ServiceProps.get(
				"value.object.column.bitmask.enabled.com.qti.slayer.model.TestResult"),
			true);
	public static long UOLID_COLUMN_BITMASK = 1L;
	public static long USERID_COLUMN_BITMASK = 2L;

	/**
	 * Converts the soap model instance into a normal model instance.
	 *
	 * @param soapModel the soap model instance to convert
	 * @return the normal model instance
	 */
	public static TestResult toModel(TestResultSoap soapModel) {
		if (soapModel == null) {
			return null;
		}

		TestResult model = new TestResultImpl();

		model.setUserId(soapModel.getUserId());
		model.setUolId(soapModel.getUolId());
		model.setResult(soapModel.getResult());
		model.setAutoComment(soapModel.getAutoComment());
		model.setTeacherComment(soapModel.getTeacherComment());
		model.setTitle(soapModel.getTitle());
		model.setUsername(soapModel.getUsername());

		return model;
	}

	/**
	 * Converts the soap model instances into normal model instances.
	 *
	 * @param soapModels the soap model instances to convert
	 * @return the normal model instances
	 */
	public static List<TestResult> toModels(TestResultSoap[] soapModels) {
		if (soapModels == null) {
			return null;
		}

		List<TestResult> models = new ArrayList<TestResult>(soapModels.length);

		for (TestResultSoap soapModel : soapModels) {
			models.add(toModel(soapModel));
		}

		return models;
	}

	public static final long LOCK_EXPIRATION_TIME = GetterUtil.getLong(com.liferay.util.service.ServiceProps.get(
				"lock.expiration.time.com.qti.slayer.model.TestResult"));

	public TestResultModelImpl() {
	}

	public TestResultPK getPrimaryKey() {
		return new TestResultPK(_userId, _uolId);
	}

	public void setPrimaryKey(TestResultPK primaryKey) {
		setUserId(primaryKey.userId);
		setUolId(primaryKey.uolId);
	}

	public Serializable getPrimaryKeyObj() {
		return new TestResultPK(_userId, _uolId);
	}

	public void setPrimaryKeyObj(Serializable primaryKeyObj) {
		setPrimaryKey((TestResultPK)primaryKeyObj);
	}

	public Class<?> getModelClass() {
		return TestResult.class;
	}

	public String getModelClassName() {
		return TestResult.class.getName();
	}

	@Override
	public Map<String, Object> getModelAttributes() {
		Map<String, Object> attributes = new HashMap<String, Object>();

		attributes.put("userId", getUserId());
		attributes.put("uolId", getUolId());
		attributes.put("result", getResult());
		attributes.put("autoComment", getAutoComment());
		attributes.put("teacherComment", getTeacherComment());
		attributes.put("title", getTitle());
		attributes.put("username", getUsername());

		return attributes;
	}

	@Override
	public void setModelAttributes(Map<String, Object> attributes) {
		Long userId = (Long)attributes.get("userId");

		if (userId != null) {
			setUserId(userId);
		}

		Long uolId = (Long)attributes.get("uolId");

		if (uolId != null) {
			setUolId(uolId);
		}

		Integer result = (Integer)attributes.get("result");

		if (result != null) {
			setResult(result);
		}

		String autoComment = (String)attributes.get("autoComment");

		if (autoComment != null) {
			setAutoComment(autoComment);
		}

		String teacherComment = (String)attributes.get("teacherComment");

		if (teacherComment != null) {
			setTeacherComment(teacherComment);
		}

		String title = (String)attributes.get("title");

		if (title != null) {
			setTitle(title);
		}

		String username = (String)attributes.get("username");

		if (username != null) {
			setUsername(username);
		}
	}

	@JSON
	public long getUserId() {
		return _userId;
	}

	public void setUserId(long userId) {
		_columnBitmask = -1L;

		if (!_setOriginalUserId) {
			_setOriginalUserId = true;

			_originalUserId = _userId;
		}

		_userId = userId;
	}

	public String getUserUuid() throws SystemException {
		return PortalUtil.getUserValue(getUserId(), "uuid", _userUuid);
	}

	public void setUserUuid(String userUuid) {
		_userUuid = userUuid;
	}

	public long getOriginalUserId() {
		return _originalUserId;
	}

	@JSON
	public long getUolId() {
		return _uolId;
	}

	public void setUolId(long uolId) {
		_columnBitmask |= UOLID_COLUMN_BITMASK;

		if (!_setOriginalUolId) {
			_setOriginalUolId = true;

			_originalUolId = _uolId;
		}

		_uolId = uolId;
	}

	public long getOriginalUolId() {
		return _originalUolId;
	}

	@JSON
	public int getResult() {
		return _result;
	}

	public void setResult(int result) {
		_result = result;
	}

	@JSON
	public String getAutoComment() {
		if (_autoComment == null) {
			return StringPool.BLANK;
		}
		else {
			return _autoComment;
		}
	}

	public void setAutoComment(String autoComment) {
		_autoComment = autoComment;
	}

	@JSON
	public String getTeacherComment() {
		if (_teacherComment == null) {
			return StringPool.BLANK;
		}
		else {
			return _teacherComment;
		}
	}

	public void setTeacherComment(String teacherComment) {
		_teacherComment = teacherComment;
	}

	@JSON
	public String getTitle() {
		if (_title == null) {
			return StringPool.BLANK;
		}
		else {
			return _title;
		}
	}

	public void setTitle(String title) {
		_title = title;
	}

	@JSON
	public String getUsername() {
		if (_username == null) {
			return StringPool.BLANK;
		}
		else {
			return _username;
		}
	}

	public void setUsername(String username) {
		_username = username;
	}

	public long getColumnBitmask() {
		return _columnBitmask;
	}

	@Override
	public TestResult toEscapedModel() {
		if (_escapedModelProxy == null) {
			_escapedModelProxy = (TestResult)ProxyUtil.newProxyInstance(_classLoader,
					_escapedModelProxyInterfaces,
					new AutoEscapeBeanHandler(this));
		}

		return _escapedModelProxy;
	}

	@Override
	public Object clone() {
		TestResultImpl testResultImpl = new TestResultImpl();

		testResultImpl.setUserId(getUserId());
		testResultImpl.setUolId(getUolId());
		testResultImpl.setResult(getResult());
		testResultImpl.setAutoComment(getAutoComment());
		testResultImpl.setTeacherComment(getTeacherComment());
		testResultImpl.setTitle(getTitle());
		testResultImpl.setUsername(getUsername());

		testResultImpl.resetOriginalValues();

		return testResultImpl;
	}

	public int compareTo(TestResult testResult) {
		int value = 0;

		if (getUserId() < testResult.getUserId()) {
			value = -1;
		}
		else if (getUserId() > testResult.getUserId()) {
			value = 1;
		}
		else {
			value = 0;
		}

		if (value != 0) {
			return value;
		}

		return 0;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}

		TestResult testResult = null;

		try {
			testResult = (TestResult)obj;
		}
		catch (ClassCastException cce) {
			return false;
		}

		TestResultPK primaryKey = testResult.getPrimaryKey();

		if (getPrimaryKey().equals(primaryKey)) {
			return true;
		}
		else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return getPrimaryKey().hashCode();
	}

	@Override
	public void resetOriginalValues() {
		TestResultModelImpl testResultModelImpl = this;

		testResultModelImpl._originalUserId = testResultModelImpl._userId;

		testResultModelImpl._setOriginalUserId = false;

		testResultModelImpl._originalUolId = testResultModelImpl._uolId;

		testResultModelImpl._setOriginalUolId = false;

		testResultModelImpl._columnBitmask = 0;
	}

	@Override
	public CacheModel<TestResult> toCacheModel() {
		TestResultCacheModel testResultCacheModel = new TestResultCacheModel();

		testResultCacheModel.userId = getUserId();

		testResultCacheModel.uolId = getUolId();

		testResultCacheModel.result = getResult();

		testResultCacheModel.autoComment = getAutoComment();

		String autoComment = testResultCacheModel.autoComment;

		if ((autoComment != null) && (autoComment.length() == 0)) {
			testResultCacheModel.autoComment = null;
		}

		testResultCacheModel.teacherComment = getTeacherComment();

		String teacherComment = testResultCacheModel.teacherComment;

		if ((teacherComment != null) && (teacherComment.length() == 0)) {
			testResultCacheModel.teacherComment = null;
		}

		testResultCacheModel.title = getTitle();

		String title = testResultCacheModel.title;

		if ((title != null) && (title.length() == 0)) {
			testResultCacheModel.title = null;
		}

		testResultCacheModel.username = getUsername();

		String username = testResultCacheModel.username;

		if ((username != null) && (username.length() == 0)) {
			testResultCacheModel.username = null;
		}

		return testResultCacheModel;
	}

	@Override
	public String toString() {
		StringBundler sb = new StringBundler(15);

		sb.append("{userId=");
		sb.append(getUserId());
		sb.append(", uolId=");
		sb.append(getUolId());
		sb.append(", result=");
		sb.append(getResult());
		sb.append(", autoComment=");
		sb.append(getAutoComment());
		sb.append(", teacherComment=");
		sb.append(getTeacherComment());
		sb.append(", title=");
		sb.append(getTitle());
		sb.append(", username=");
		sb.append(getUsername());
		sb.append("}");

		return sb.toString();
	}

	public String toXmlString() {
		StringBundler sb = new StringBundler(25);

		sb.append("<model><model-name>");
		sb.append("com.qti.slayer.model.TestResult");
		sb.append("</model-name>");

		sb.append(
			"<column><column-name>userId</column-name><column-value><![CDATA[");
		sb.append(getUserId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>uolId</column-name><column-value><![CDATA[");
		sb.append(getUolId());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>result</column-name><column-value><![CDATA[");
		sb.append(getResult());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>autoComment</column-name><column-value><![CDATA[");
		sb.append(getAutoComment());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>teacherComment</column-name><column-value><![CDATA[");
		sb.append(getTeacherComment());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>title</column-name><column-value><![CDATA[");
		sb.append(getTitle());
		sb.append("]]></column-value></column>");
		sb.append(
			"<column><column-name>username</column-name><column-value><![CDATA[");
		sb.append(getUsername());
		sb.append("]]></column-value></column>");

		sb.append("</model>");

		return sb.toString();
	}

	private static ClassLoader _classLoader = TestResult.class.getClassLoader();
	private static Class<?>[] _escapedModelProxyInterfaces = new Class[] {
			TestResult.class
		};
	private long _userId;
	private String _userUuid;
	private long _originalUserId;
	private boolean _setOriginalUserId;
	private long _uolId;
	private long _originalUolId;
	private boolean _setOriginalUolId;
	private int _result;
	private String _autoComment;
	private String _teacherComment;
	private String _title;
	private String _username;
	private long _columnBitmask;
	private TestResult _escapedModelProxy;
}