/**
 * Copyright (c) 2000-2012 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.qti.slayer.service.base;

import com.liferay.counter.service.CounterLocalService;

import com.liferay.portal.kernel.bean.BeanReference;
import com.liferay.portal.kernel.bean.IdentifiableBean;
import com.liferay.portal.kernel.dao.jdbc.SqlUpdate;
import com.liferay.portal.kernel.dao.jdbc.SqlUpdateFactoryUtil;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.DynamicQueryFactoryUtil;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.search.Indexable;
import com.liferay.portal.kernel.search.IndexableType;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.model.PersistedModel;
import com.liferay.portal.service.BaseLocalServiceImpl;
import com.liferay.portal.service.PersistedModelLocalServiceRegistryUtil;
import com.liferay.portal.service.ResourceLocalService;
import com.liferay.portal.service.ResourceService;
import com.liferay.portal.service.UserLocalService;
import com.liferay.portal.service.UserService;
import com.liferay.portal.service.persistence.ResourcePersistence;
import com.liferay.portal.service.persistence.UserPersistence;

import com.qti.slayer.model.TestResult;
import com.qti.slayer.service.AssignmentLocalService;
import com.qti.slayer.service.AssignmentService;
import com.qti.slayer.service.TestResultLocalService;
import com.qti.slayer.service.TestResultService;
import com.qti.slayer.service.UolLocalService;
import com.qti.slayer.service.UolService;
import com.qti.slayer.service.persistence.AssignmentPersistence;
import com.qti.slayer.service.persistence.TestResultPK;
import com.qti.slayer.service.persistence.TestResultPersistence;
import com.qti.slayer.service.persistence.UolPersistence;

import java.io.Serializable;

import java.util.List;

import javax.sql.DataSource;

/**
 * The base implementation of the test result local service.
 *
 * <p>
 * This implementation exists only as a container for the default service methods generated by ServiceBuilder. All custom service methods should be put in {@link com.qti.slayer.service.impl.TestResultLocalServiceImpl}.
 * </p>
 *
 * @author Hector
 * @see com.qti.slayer.service.impl.TestResultLocalServiceImpl
 * @see com.qti.slayer.service.TestResultLocalServiceUtil
 * @generated
 */
public abstract class TestResultLocalServiceBaseImpl
	extends BaseLocalServiceImpl implements TestResultLocalService,
		IdentifiableBean {
	/*
	 * NOTE FOR DEVELOPERS:
	 *
	 * Never modify or reference this class directly. Always use {@link com.qti.slayer.service.TestResultLocalServiceUtil} to access the test result local service.
	 */

	/**
	 * Adds the test result to the database. Also notifies the appropriate model listeners.
	 *
	 * @param testResult the test result
	 * @return the test result that was added
	 * @throws SystemException if a system exception occurred
	 */
	@Indexable(type = IndexableType.REINDEX)
	public TestResult addTestResult(TestResult testResult)
		throws SystemException {
		testResult.setNew(true);

		return testResultPersistence.update(testResult, false);
	}

	/**
	 * Creates a new test result with the primary key. Does not add the test result to the database.
	 *
	 * @param testResultPK the primary key for the new test result
	 * @return the new test result
	 */
	public TestResult createTestResult(TestResultPK testResultPK) {
		return testResultPersistence.create(testResultPK);
	}

	/**
	 * Deletes the test result with the primary key from the database. Also notifies the appropriate model listeners.
	 *
	 * @param testResultPK the primary key of the test result
	 * @return the test result that was removed
	 * @throws PortalException if a test result with the primary key could not be found
	 * @throws SystemException if a system exception occurred
	 */
	@Indexable(type = IndexableType.DELETE)
	public TestResult deleteTestResult(TestResultPK testResultPK)
		throws PortalException, SystemException {
		return testResultPersistence.remove(testResultPK);
	}

	/**
	 * Deletes the test result from the database. Also notifies the appropriate model listeners.
	 *
	 * @param testResult the test result
	 * @return the test result that was removed
	 * @throws SystemException if a system exception occurred
	 */
	@Indexable(type = IndexableType.DELETE)
	public TestResult deleteTestResult(TestResult testResult)
		throws SystemException {
		return testResultPersistence.remove(testResult);
	}

	public DynamicQuery dynamicQuery() {
		Class<?> clazz = getClass();

		return DynamicQueryFactoryUtil.forClass(TestResult.class,
			clazz.getClassLoader());
	}

	/**
	 * Performs a dynamic query on the database and returns the matching rows.
	 *
	 * @param dynamicQuery the dynamic query
	 * @return the matching rows
	 * @throws SystemException if a system exception occurred
	 */
	@SuppressWarnings("rawtypes")
	public List dynamicQuery(DynamicQuery dynamicQuery)
		throws SystemException {
		return testResultPersistence.findWithDynamicQuery(dynamicQuery);
	}

	/**
	 * Performs a dynamic query on the database and returns a range of the matching rows.
	 *
	 * <p>
	 * Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	 * </p>
	 *
	 * @param dynamicQuery the dynamic query
	 * @param start the lower bound of the range of model instances
	 * @param end the upper bound of the range of model instances (not inclusive)
	 * @return the range of matching rows
	 * @throws SystemException if a system exception occurred
	 */
	@SuppressWarnings("rawtypes")
	public List dynamicQuery(DynamicQuery dynamicQuery, int start, int end)
		throws SystemException {
		return testResultPersistence.findWithDynamicQuery(dynamicQuery, start,
			end);
	}

	/**
	 * Performs a dynamic query on the database and returns an ordered range of the matching rows.
	 *
	 * <p>
	 * Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	 * </p>
	 *
	 * @param dynamicQuery the dynamic query
	 * @param start the lower bound of the range of model instances
	 * @param end the upper bound of the range of model instances (not inclusive)
	 * @param orderByComparator the comparator to order the results by (optionally <code>null</code>)
	 * @return the ordered range of matching rows
	 * @throws SystemException if a system exception occurred
	 */
	@SuppressWarnings("rawtypes")
	public List dynamicQuery(DynamicQuery dynamicQuery, int start, int end,
		OrderByComparator orderByComparator) throws SystemException {
		return testResultPersistence.findWithDynamicQuery(dynamicQuery, start,
			end, orderByComparator);
	}

	/**
	 * Returns the number of rows that match the dynamic query.
	 *
	 * @param dynamicQuery the dynamic query
	 * @return the number of rows that match the dynamic query
	 * @throws SystemException if a system exception occurred
	 */
	public long dynamicQueryCount(DynamicQuery dynamicQuery)
		throws SystemException {
		return testResultPersistence.countWithDynamicQuery(dynamicQuery);
	}

	public TestResult fetchTestResult(TestResultPK testResultPK)
		throws SystemException {
		return testResultPersistence.fetchByPrimaryKey(testResultPK);
	}

	/**
	 * Returns the test result with the primary key.
	 *
	 * @param testResultPK the primary key of the test result
	 * @return the test result
	 * @throws PortalException if a test result with the primary key could not be found
	 * @throws SystemException if a system exception occurred
	 */
	public TestResult getTestResult(TestResultPK testResultPK)
		throws PortalException, SystemException {
		return testResultPersistence.findByPrimaryKey(testResultPK);
	}

	public PersistedModel getPersistedModel(Serializable primaryKeyObj)
		throws PortalException, SystemException {
		return testResultPersistence.findByPrimaryKey(primaryKeyObj);
	}

	/**
	 * Returns a range of all the test results.
	 *
	 * <p>
	 * Useful when paginating results. Returns a maximum of <code>end - start</code> instances. <code>start</code> and <code>end</code> are not primary keys, they are indexes in the result set. Thus, <code>0</code> refers to the first result in the set. Setting both <code>start</code> and <code>end</code> to {@link com.liferay.portal.kernel.dao.orm.QueryUtil#ALL_POS} will return the full result set.
	 * </p>
	 *
	 * @param start the lower bound of the range of test results
	 * @param end the upper bound of the range of test results (not inclusive)
	 * @return the range of test results
	 * @throws SystemException if a system exception occurred
	 */
	public List<TestResult> getTestResults(int start, int end)
		throws SystemException {
		return testResultPersistence.findAll(start, end);
	}

	/**
	 * Returns the number of test results.
	 *
	 * @return the number of test results
	 * @throws SystemException if a system exception occurred
	 */
	public int getTestResultsCount() throws SystemException {
		return testResultPersistence.countAll();
	}

	/**
	 * Updates the test result in the database or adds it if it does not yet exist. Also notifies the appropriate model listeners.
	 *
	 * @param testResult the test result
	 * @return the test result that was updated
	 * @throws SystemException if a system exception occurred
	 */
	@Indexable(type = IndexableType.REINDEX)
	public TestResult updateTestResult(TestResult testResult)
		throws SystemException {
		return updateTestResult(testResult, true);
	}

	/**
	 * Updates the test result in the database or adds it if it does not yet exist. Also notifies the appropriate model listeners.
	 *
	 * @param testResult the test result
	 * @param merge whether to merge the test result with the current session. See {@link com.liferay.portal.service.persistence.BatchSession#update(com.liferay.portal.kernel.dao.orm.Session, com.liferay.portal.model.BaseModel, boolean)} for an explanation.
	 * @return the test result that was updated
	 * @throws SystemException if a system exception occurred
	 */
	@Indexable(type = IndexableType.REINDEX)
	public TestResult updateTestResult(TestResult testResult, boolean merge)
		throws SystemException {
		testResult.setNew(false);

		return testResultPersistence.update(testResult, merge);
	}

	/**
	 * Returns the assignment local service.
	 *
	 * @return the assignment local service
	 */
	public AssignmentLocalService getAssignmentLocalService() {
		return assignmentLocalService;
	}

	/**
	 * Sets the assignment local service.
	 *
	 * @param assignmentLocalService the assignment local service
	 */
	public void setAssignmentLocalService(
		AssignmentLocalService assignmentLocalService) {
		this.assignmentLocalService = assignmentLocalService;
	}

	/**
	 * Returns the assignment remote service.
	 *
	 * @return the assignment remote service
	 */
	public AssignmentService getAssignmentService() {
		return assignmentService;
	}

	/**
	 * Sets the assignment remote service.
	 *
	 * @param assignmentService the assignment remote service
	 */
	public void setAssignmentService(AssignmentService assignmentService) {
		this.assignmentService = assignmentService;
	}

	/**
	 * Returns the assignment persistence.
	 *
	 * @return the assignment persistence
	 */
	public AssignmentPersistence getAssignmentPersistence() {
		return assignmentPersistence;
	}

	/**
	 * Sets the assignment persistence.
	 *
	 * @param assignmentPersistence the assignment persistence
	 */
	public void setAssignmentPersistence(
		AssignmentPersistence assignmentPersistence) {
		this.assignmentPersistence = assignmentPersistence;
	}

	/**
	 * Returns the test result local service.
	 *
	 * @return the test result local service
	 */
	public TestResultLocalService getTestResultLocalService() {
		return testResultLocalService;
	}

	/**
	 * Sets the test result local service.
	 *
	 * @param testResultLocalService the test result local service
	 */
	public void setTestResultLocalService(
		TestResultLocalService testResultLocalService) {
		this.testResultLocalService = testResultLocalService;
	}

	/**
	 * Returns the test result remote service.
	 *
	 * @return the test result remote service
	 */
	public TestResultService getTestResultService() {
		return testResultService;
	}

	/**
	 * Sets the test result remote service.
	 *
	 * @param testResultService the test result remote service
	 */
	public void setTestResultService(TestResultService testResultService) {
		this.testResultService = testResultService;
	}

	/**
	 * Returns the test result persistence.
	 *
	 * @return the test result persistence
	 */
	public TestResultPersistence getTestResultPersistence() {
		return testResultPersistence;
	}

	/**
	 * Sets the test result persistence.
	 *
	 * @param testResultPersistence the test result persistence
	 */
	public void setTestResultPersistence(
		TestResultPersistence testResultPersistence) {
		this.testResultPersistence = testResultPersistence;
	}

	/**
	 * Returns the uol local service.
	 *
	 * @return the uol local service
	 */
	public UolLocalService getUolLocalService() {
		return uolLocalService;
	}

	/**
	 * Sets the uol local service.
	 *
	 * @param uolLocalService the uol local service
	 */
	public void setUolLocalService(UolLocalService uolLocalService) {
		this.uolLocalService = uolLocalService;
	}

	/**
	 * Returns the uol remote service.
	 *
	 * @return the uol remote service
	 */
	public UolService getUolService() {
		return uolService;
	}

	/**
	 * Sets the uol remote service.
	 *
	 * @param uolService the uol remote service
	 */
	public void setUolService(UolService uolService) {
		this.uolService = uolService;
	}

	/**
	 * Returns the uol persistence.
	 *
	 * @return the uol persistence
	 */
	public UolPersistence getUolPersistence() {
		return uolPersistence;
	}

	/**
	 * Sets the uol persistence.
	 *
	 * @param uolPersistence the uol persistence
	 */
	public void setUolPersistence(UolPersistence uolPersistence) {
		this.uolPersistence = uolPersistence;
	}

	/**
	 * Returns the counter local service.
	 *
	 * @return the counter local service
	 */
	public CounterLocalService getCounterLocalService() {
		return counterLocalService;
	}

	/**
	 * Sets the counter local service.
	 *
	 * @param counterLocalService the counter local service
	 */
	public void setCounterLocalService(CounterLocalService counterLocalService) {
		this.counterLocalService = counterLocalService;
	}

	/**
	 * Returns the resource local service.
	 *
	 * @return the resource local service
	 */
	public ResourceLocalService getResourceLocalService() {
		return resourceLocalService;
	}

	/**
	 * Sets the resource local service.
	 *
	 * @param resourceLocalService the resource local service
	 */
	public void setResourceLocalService(
		ResourceLocalService resourceLocalService) {
		this.resourceLocalService = resourceLocalService;
	}

	/**
	 * Returns the resource remote service.
	 *
	 * @return the resource remote service
	 */
	public ResourceService getResourceService() {
		return resourceService;
	}

	/**
	 * Sets the resource remote service.
	 *
	 * @param resourceService the resource remote service
	 */
	public void setResourceService(ResourceService resourceService) {
		this.resourceService = resourceService;
	}

	/**
	 * Returns the resource persistence.
	 *
	 * @return the resource persistence
	 */
	public ResourcePersistence getResourcePersistence() {
		return resourcePersistence;
	}

	/**
	 * Sets the resource persistence.
	 *
	 * @param resourcePersistence the resource persistence
	 */
	public void setResourcePersistence(ResourcePersistence resourcePersistence) {
		this.resourcePersistence = resourcePersistence;
	}

	/**
	 * Returns the user local service.
	 *
	 * @return the user local service
	 */
	public UserLocalService getUserLocalService() {
		return userLocalService;
	}

	/**
	 * Sets the user local service.
	 *
	 * @param userLocalService the user local service
	 */
	public void setUserLocalService(UserLocalService userLocalService) {
		this.userLocalService = userLocalService;
	}

	/**
	 * Returns the user remote service.
	 *
	 * @return the user remote service
	 */
	public UserService getUserService() {
		return userService;
	}

	/**
	 * Sets the user remote service.
	 *
	 * @param userService the user remote service
	 */
	public void setUserService(UserService userService) {
		this.userService = userService;
	}

	/**
	 * Returns the user persistence.
	 *
	 * @return the user persistence
	 */
	public UserPersistence getUserPersistence() {
		return userPersistence;
	}

	/**
	 * Sets the user persistence.
	 *
	 * @param userPersistence the user persistence
	 */
	public void setUserPersistence(UserPersistence userPersistence) {
		this.userPersistence = userPersistence;
	}

	public void afterPropertiesSet() {
		PersistedModelLocalServiceRegistryUtil.register("com.qti.slayer.model.TestResult",
			testResultLocalService);
	}

	public void destroy() {
		PersistedModelLocalServiceRegistryUtil.unregister(
			"com.qti.slayer.model.TestResult");
	}

	/**
	 * Returns the Spring bean ID for this bean.
	 *
	 * @return the Spring bean ID for this bean
	 */
	public String getBeanIdentifier() {
		return _beanIdentifier;
	}

	/**
	 * Sets the Spring bean ID for this bean.
	 *
	 * @param beanIdentifier the Spring bean ID for this bean
	 */
	public void setBeanIdentifier(String beanIdentifier) {
		_beanIdentifier = beanIdentifier;
	}

	public Object invokeMethod(String name, String[] parameterTypes,
		Object[] arguments) throws Throwable {
		return _clpInvoker.invokeMethod(name, parameterTypes, arguments);
	}

	protected Class<?> getModelClass() {
		return TestResult.class;
	}

	protected String getModelClassName() {
		return TestResult.class.getName();
	}

	/**
	 * Performs an SQL query.
	 *
	 * @param sql the sql query
	 */
	protected void runSQL(String sql) throws SystemException {
		try {
			DataSource dataSource = testResultPersistence.getDataSource();

			SqlUpdate sqlUpdate = SqlUpdateFactoryUtil.getSqlUpdate(dataSource,
					sql, new int[0]);

			sqlUpdate.update();
		}
		catch (Exception e) {
			throw new SystemException(e);
		}
	}

	@BeanReference(type = AssignmentLocalService.class)
	protected AssignmentLocalService assignmentLocalService;
	@BeanReference(type = AssignmentService.class)
	protected AssignmentService assignmentService;
	@BeanReference(type = AssignmentPersistence.class)
	protected AssignmentPersistence assignmentPersistence;
	@BeanReference(type = TestResultLocalService.class)
	protected TestResultLocalService testResultLocalService;
	@BeanReference(type = TestResultService.class)
	protected TestResultService testResultService;
	@BeanReference(type = TestResultPersistence.class)
	protected TestResultPersistence testResultPersistence;
	@BeanReference(type = UolLocalService.class)
	protected UolLocalService uolLocalService;
	@BeanReference(type = UolService.class)
	protected UolService uolService;
	@BeanReference(type = UolPersistence.class)
	protected UolPersistence uolPersistence;
	@BeanReference(type = CounterLocalService.class)
	protected CounterLocalService counterLocalService;
	@BeanReference(type = ResourceLocalService.class)
	protected ResourceLocalService resourceLocalService;
	@BeanReference(type = ResourceService.class)
	protected ResourceService resourceService;
	@BeanReference(type = ResourcePersistence.class)
	protected ResourcePersistence resourcePersistence;
	@BeanReference(type = UserLocalService.class)
	protected UserLocalService userLocalService;
	@BeanReference(type = UserService.class)
	protected UserService userService;
	@BeanReference(type = UserPersistence.class)
	protected UserPersistence userPersistence;
	private String _beanIdentifier;
	private TestResultLocalServiceClpInvoker _clpInvoker = new TestResultLocalServiceClpInvoker();
}