create database if not exists tenc_apis;
use tenc_apis;

create table if not exists language (
  language_id bigint not null auto_increment,
  language varchar(250),
  primary key (language_id)
  );

create table if not exists assessmenttest (
  test_key bigint auto_increment,
  test_id varchar(500),
  unique(test_id),
  primary key (test_key)
  );

create table if not exists assessmentitem (
  item_key bigint not null auto_increment,
  item_id varchar(500),
  type varchar(250),
  unique(item_id),
  primary key (item_key)
  );

create table if not exists closeanswer (
  closeanswer_key bigint not null auto_increment,
  closeanswer_id varchar(500),
  content varchar(800),
  primary key (closeanswer_key)
  );

create table if not exists login_test_case (
  test_case_id bigint not null auto_increment,
  user varchar(250),
  test_key bigint,
  run integer,
  repetition integer,
  done boolean,
  uolid varchar(600),
  page integer,
  language bigint,
  foreign key (test_key) references assessmenttest(test_key),
  foreign key (language) references language(language_id),
  primary key (test_case_id)
  );

create table if not exists item_case (
  item_case_id bigint not null auto_increment,
  test_case bigint,
  item_key bigint,
  answer_key bigint,
  result integer,
  foreign key (test_case) references login_test_case(test_case_id),
  foreign key (item_key) references assessmentitem(item_key),
  foreign key (answer_key) references closeanswer(closeanswer_key),
  primary key (item_case_id)
  );

create table if not exists answer_set (
  answer_set_key bigint not null auto_increment,
  set_num bigint,
  answer bigint,
  type varchar(250),
  foreign key (answer) references closeanswer(closeanswer_key),
  foreign key (set_num) references item_case(item_case_id),
  primary key (answer_set_key)
  );

create table if not exists test_feedback_text (
  test_feedback_text_key bigint not null auto_increment,
  variable varchar(250),
  text varchar(250),
  language bigint,
  foreign key (language) references language(language_id),
  primary key (test_feedback_text_key)
  );

insert into language (language) values('english');
insert into language (language) values('dutch');
insert into language (language) values('spanish');

-- english
insert into test_feedback_text (variable, text, language) values
  ('final', 'Final score:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('correct', 'Number of correct:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('incorrect', 'Number of incorrect:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('presented', 'Number of presented:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('responded', 'Number of responded:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('selected', 'Number of selected:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('percentage', 'Percentage of correct:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('submit', 'submit',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('previous', 'previous',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('next', 'next',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('title', 'RESULTS OF THE TEST:',
  (select language_id from language where language = 'english'));

insert into test_feedback_text (variable, text, language) values
  ('breakdown', 'The test is now complete. The  following table shows a breakdown of your scores:',
  (select language_id from language where language = 'english'));

-- dutch
insert into test_feedback_text (variable, text, language) values
  ('final', 'Eindscore:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('correct', 'Aantal juiste:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('incorrect', 'Aantal onjuist:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('presented', 'Aantal aangeboden:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('responded', 'Aantal gereageerd:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('selected', 'Aantal geselecteerde:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('percentage', 'Percentage juist:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('submit', 'indienen',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('previous', 'vorige',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('next', 'volgende',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('title', 'RESULTATEN VAN DE TEST:',
  (select language_id from language where language = 'dutch'));

insert into test_feedback_text (variable, text, language) values
  ('breakdown', 'De test is nu voltooid. De volgende tabel geeft een overzicht van uw scores:',
  (select language_id from language where language = 'dutch'));

-- spanish
insert into test_feedback_text (variable, text, language) values
  ('final', 'Puntuaci\u00f3n final:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('correct', 'Numero de correctas:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('incorrect', 'Numero de incorrectas:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('presented', 'Numero de presentadas:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('responded', 'Numero de respondidas:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('selected', 'Numero de seleccionadas:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('percentage', 'Porcentaje de correctas:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('submit', 'Enviar',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('previous', 'Anterior',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('next', 'Siguiente',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('title', 'RESULTADOS DEL TEST:',
  (select language_id from language where language = 'spanish'));

insert into test_feedback_text (variable, text, language) values
  ('breakdown', 'El test ha sido completado. La siguiente tabla muestra un resumen de tu puntuaci\u00f3n:',
  (select language_id from language where language = 'spanish'));