/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.apis.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Enumeration;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.tencompetence.business.facade.AssessmentItemFacade;
import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.apis.TENCQtiParser;
import org.xml.sax.SAXException;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.PropsUtil;

/**
 * Servlet to serve apisTests.
 * 
 * @author venrooijma
 * @author jal
 * @author habetsr
 * 
 * @version $Revision: 1.13 $, $Date: 2009/11/27 16:53:23 $
 * 
 */
public class ApisServlet extends HttpServlet {

	private static final long serialVersionUID = -3398813134626664710L;
	private static final String TEST_PREFIX = "/AT-";
	private static final Log LOG = LogFactoryUtil.getLog(ApisServlet.class);
	private static final String SUFFIX = ".xml";
	private static final String TEXT_HTML = "text/html";
	private static final String APPLICATION_XML = "application/xml";
	private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	private static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
	private static final String DOCTYPE = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\""
			+ " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";

	private static final String CSS_FILE = "tenc.apis.css.file";

	/**
	 * {@inheritDoc}
	 */
	protected void doGet(final HttpServletRequest request, final HttpServletResponse res) throws IOException {
		final String strUserId = request.getParameter("user_id");
		final String strCompanyId = request.getParameter("company_id");
		final String strGroupId = request.getParameter("group_id");
		final String strLanguage = request.getParameter("language") == null ? "english" : request
				.getParameter("language");
		// final String strMode = request.getParameter("mode");
		// final Boolean testMode = strMode != null ? strMode.equals("test") : false;

		Long userId = null;
		Long companyId = null;
		Long groupId = null;

		if (strUserId == null) {
			userId = (Long) request.getSession().getAttribute("user_id");
		} else {
			userId = Long.valueOf(strUserId);
			request.getSession().setAttribute("user_id", userId);
		}

		if (strCompanyId == null) {
			companyId = (Long) request.getSession().getAttribute("company_id");
		} else {
			companyId = Long.valueOf(strCompanyId);
			request.getSession().setAttribute("company_id", companyId);
		}

		if (strGroupId == null) {
			groupId = (Long) request.getSession().getAttribute("group_id");
		} else {
			groupId = Long.valueOf(strGroupId);
			request.getSession().setAttribute("group_id", groupId);
		}

		PrintWriter out = res.getWriter();

		String path = request.getPathInfo();

		if (path.equals("/")) {
			setHeaders(res, TEXT_HTML);

			sendTestData(request.getParameter("test_id"), out, userId, request, companyId, groupId, strLanguage);
		} else if (path.startsWith(TEST_PREFIX)) {
			String fileName = request.getParameter("file");
			setHeaders(res, APPLICATION_XML);

			returnTestOrItemXML(out, fileName);
		}
	}

	private void sendTestData(final String testId, final PrintWriter prOut, final Long user,
			final HttpServletRequest request, final Long companyId, final Long groupId, final String language) {

		printHeader(prOut);

		try {
			/*
			 * URL realUrl = new URL(url); if ("file".equalsIgnoreCase(realUrl.getProtocol())) { throw new
			 * MalformedURLException("file protocol not allowed"); }
			 */
			StringWriter result = new StringWriter();

			SAXParser parser = createSaxParser();

			String paramString = fillParamString(request);

			String fullUrl = createFullUrl(testId, request);

			TENCQtiParser a = new TENCQtiParser(Long.toString(user), 0, result, fullUrl, paramString, language);

			a.setCompanyId(companyId);
			a.setGroupId(groupId);
			parser.parse(fullUrl, a);

			prOut.println(result.toString());
			printFooter(prOut);

		} catch (Exception e) {
			logException(e);
		}

	}

	/**
	 * printHeader get this value from portal-ext.properties, so it should contain a line like this:
	 * 
	 * tenc.apis.css.file=/org.tencompetence.default-theme/css/custom.css
	 * 
	 * .
	 * 
	 * @param prOut
	 */
	private void printHeader(final PrintWriter prOut) {
		String cssFile = "";

		prOut.println(DOCTYPE);
		prOut.println("<html><head><title>APIS testing</title>\n");

		prOut.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
		prOut.println("<meta http-equiv='Pragma' content='no-cache'>\n");
		prOut.println("<meta http-equiv='Expires' content='-1'>\n");

		try {
			cssFile = PropsUtil.get(CSS_FILE);
		} catch (Exception e) {
			logException(e);
		}

		if (cssFile != null && cssFile.length() > 0) {
			prOut.println("<link rel=stylesheet href=\"");
			prOut.println(cssFile);
			prOut.println("\" type=\"text/css\" media=screen>");
		}
		prOut.println("</head><body>");
	}

	private SAXParser createSaxParser() throws ParserConfigurationException, SAXException {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(false);
		factory.setNamespaceAware(true);
		SAXParser parser = factory.newSAXParser();
		parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
		return parser;
	}

	@SuppressWarnings("unchecked")
	private String fillParamString(final HttpServletRequest request) {
		String paramString = "";
		Enumeration myMap = request.getParameterNames();
		while (myMap.hasMoreElements()) {
			String paramName = (String) myMap.nextElement();
			paramString += paramName + "=" + request.getParameter(paramName) + "&";
		}
		paramString = paramString.substring(0, paramString.length() - 1);
		return paramString;
	}

	private String createFullUrl(final String testId, final HttpServletRequest request) {
		String fullUrl = "";

		if (testId == null) {
			fullUrl = request.getParameter("file");
		} else {
			fullUrl = request.getScheme() + "://";
			fullUrl += request.getServerName() + ":" + request.getLocalPort() + request.getContextPath() + "/apis/"
					+ testId + "?file=" + testId;
		}
		return fullUrl;
	}

	private void printFooter(final PrintWriter prOut) {
		prOut.println("</body></html>");
	}

	private void setHeaders(final HttpServletResponse res, final String contentType) {
		res.setContentType(contentType);
		res.setCharacterEncoding("UTF-8");
	}

	private void returnTestOrItemXML(final PrintWriter out, final String fileName) {
		if (isTestRequest(fileName)) {
			returnTestXML(out, fileName);
		} else {
			returnItemXML(out, fileName);
		}
	}

	private boolean isTestRequest(final String fileName) {
		return fileName.lastIndexOf('/') == -1;
	}

	private void returnTestXML(final PrintWriter out, final String fileName) {
		String testId = fileName;
		try {
			AssessmentTest assessmentTest = AssessmentTestFacade.getAssessmentTestByIdentifier(testId, null);
			out.print(assessmentTest.getXmlData());
		} catch (SystemException e) {
			logException(e);
		} catch (PortalException e) {
			logException(e);
		}
	}

	private void returnItemXML(final PrintWriter out, final String fileName) {
		String tmpFileName = fileName.substring(fileName.lastIndexOf("/"), fileName.length() - SUFFIX.length());
		String itemId = tmpFileName.substring(1);
		try {
			AssessmentItem assessmentItem = AssessmentItemFacade.getAssessmentItemByIdentifier(itemId, null);
			out.print(assessmentItem.getXmlData());
		} catch (SystemException e) {
			logException(e);
		} catch (PortalException e) {
			logException(e);
		}
	}

	private void logException(final Exception e) {
		if (LOG.isDebugEnabled()) {
			LOG.debug(e);
		}
		throw new RuntimeException(e);
	}
}
