/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.assessment.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.portlet.PortletPreferences;
import javax.portlet.ReadOnlyException;

import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.BasePreferences;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * Preferences.
 * 
 * @author venrooijma
 * @author Anton Finders
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.15 $, $Date: 2009/11/24 14:10:06 $
 * 
 */
public class AssessmentPreferences extends BasePreferences {

	private static final Log LOG = LogFactoryUtil.getLog(AssessmentPreferences.class);

	/** The Constant SELECTED_GOAL. */
	public static final String SELECTED_GOAL = "selectedGoal";

	private Goal selectedGoal = null;

	/**
	 * Instantiates a new assessment preferences.
	 */
	public AssessmentPreferences() {
		super();

		final long goalId = Long.parseLong(getPrefs().getValue(SELECTED_GOAL, "0"));
		if (goalId != 0) {
			try {
				selectedGoal = GoalFacade.getGoal(goalId, PortletUtils.getServiceContext());
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		} else {
			selectedGoal = null;
		}
	}

	/**
	 * Get Goals for the actual user.
	 * 
	 * @return the administrator goals
	 */
	public List<SelectItem> getAdministratorGoals() {
		final List<SelectItem> selectItems = new ArrayList<SelectItem>();

		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			final List<Goal> goals = GoalFacade.getGoalsForUser(ctx.getUserId(), ctx);

			for (final Goal goal : goals) {
				selectItems.add(new SelectItem(Long.toString(goal.getPrimaryKey()), goal.getTitle()));
			}

			return selectItems;
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * Gets the selected goal.
	 * 
	 * @return the selected goal
	 */
	public String getSelectedGoal() {
		if (selectedGoal == null) {
			return "0";
		} else {
			return Long.toString(selectedGoal.getPrimaryKey());
		}
	}

	/**
	 * Sets the selected goal.
	 * 
	 * @param selectedGoal
	 *            the new selected goal
	 */
	public void setSelectedGoal(final String selectedGoal) {
		final long goalId = Long.parseLong(selectedGoal);
		if (goalId != 0) {
			try {
				this.selectedGoal = GoalFacade.getGoal(goalId, PortletUtils.getServiceContext());
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		} else {
			this.selectedGoal = null;
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void updatePreferences(final PortletPreferences prefs) throws ReadOnlyException {
		prefs.setValue(SELECTED_GOAL, getSelectedGoal());
	}
}