/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common;

import java.util.ArrayList;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.portlet.RenderRequest;
import javax.portlet.WindowState;

import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

/**
 * This class implements a JSF PhaseListener that ensures that the correct JSF view is shown when a portlet is maximized
 * and when the portlet return to normal state.
 * <p />
 * When the portlet is maximized and TC_TARGET_VIEW or TC_REDIRECT parameter is passed, the value of this parameter is
 * used as the action outcome to navigate to the correct view.
 * <p />
 * When the portlet returns to normal state, the value of the TC_STAMP_VIEW portlet init configuration param is used as
 * the action outcome to navigate to the correct view.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * 
 * @version $Revision: 1.21 $, $Date: 2009/11/24 14:10:06 $
 */
public class TCPhaseListener implements PhaseListener {

	private List<String> pagesToBeHandled = new ArrayList<String>();

	/**
	 * Instantiates a new TC phase listener.
	 */
	public TCPhaseListener() {
		pagesToBeHandled.add("/views/modeleditor/smallView.jspx");
		pagesToBeHandled.add("/views/goaleditor/goalSmallView.jspx");
		pagesToBeHandled.add("/views/lpe/learningPathOverview.jspx");
		pagesToBeHandled.add("/views/evidence/evidenceOverview.jspx");
		pagesToBeHandled.add("/views/assessment/assessmentSmallView.jspx");
		pagesToBeHandled.add("/views/portfolio/portfolioSmallView.jspx");
		pagesToBeHandled.add("/views/pdp/pdpSmallView.jspx");
	}

	private static final long serialVersionUID = 5701769763251382557L;

	/** {@inheritDoc} */
	public void afterPhase(final PhaseEvent event) {
		// Nothing to do
	}

	/** {@inheritDoc} */
	public void beforePhase(final PhaseEvent event) {
		final RenderRequest request = PortletUtils.getRenderRequest();
		final String viewId = FacesContext.getCurrentInstance().getViewRoot().getViewId();

		if (pagesToBeHandled.contains(viewId)) {

			/*
			if (WindowState.NORMAL.equals(request.getWindowState())) {
				final String returnTo = request.getParameter(UrlUtils.TC_RETURN_TO);
				request.getParameterMap().remove(UrlUtils.TC_RETURN_TO);
				
				if (returnTo != null) {
					PortletUtils.handleNavigation(returnTo);
				}
			} else {
				request.getParameterMap().put(
						UrlUtils.TC_RETURN_TO,
						new String[] { (String) PortletUtils.getPortletConfig()
								.getInitParameter(UrlUtils.TC_STAMP_VIEW) });

				final String outcome = request.getParameter(UrlUtils.TC_TARGET_VIEW);
				request.getParameterMap().remove(UrlUtils.TC_TARGET_VIEW);

				if (outcome != null) {
					// Apparently switched to maximize mode for the first time
					PortletUtils.handleNavigation(outcome);
				}

				final String redirect = request.getParameter(UrlUtils.TC_REDIRECT);
				request.getParameterMap().remove(UrlUtils.TC_REDIRECT);

				if (redirect != null) {
					PortletUtils.handleNavigation(redirect);
				}
			}
			*/

			// Returning to the small view follows same path as navigating to the default VIEW mode of the
			// portlet, which is defined in the portlet.xml
			if (!WindowState.NORMAL.equals(request.getWindowState())) {
				final String outcome = request.getParameter(UrlUtils.TC_TARGET_VIEW);
				request.getParameterMap().remove(UrlUtils.TC_TARGET_VIEW);
				if (outcome != null) {
					// Apparently switched to maximize mode for the first time
					PortletUtils.handleNavigation(outcome);
				}
			}
		}
	}

	/** {@inheritDoc} */
	//@Override
	public PhaseId getPhaseId() {
		// react as soon as possible as we will intervene the default rendering
		// when appropriate
		return PhaseId.RENDER_RESPONSE;
	}
}