/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class URLHandler.
 * 
 * @version $Revision: 1.8 $, $Date: 2009/11/24 14:10:06 $
 */
public final class URLHandler {

	private URLHandler() {
		super();
	}

	/**
	 * Translate url. An url defined like : http://localhost:8080/CPL.500 will be translated. The part after the last /
	 * is used to get the assigned object. E.g. CPL.500 will return the Competence Profile Level with id 500. throws an
	 * URLHandlerException if the translation of the localURL fails
	 * 
	 * @param localURL
	 *            the local url
	 * 
	 * @return the object
	 * 
	 */
	public static Object translateURL(final String localURL) {
		String sURL = localURL;
		int position = sURL.lastIndexOf("/");

		if (position == -1) {
			throw new URLHandlerException("invalid URL: " + localURL);
		}

		String objectId = sURL.substring(position);
		String[] stringPair = objectId.split("\\.");

		if (stringPair.length != 2) {
			throw new URLHandlerException("invalid URL: " + localURL);
		}

		Object result = null;
		try {
			TencObjIdentifier classString = TencObjIdentifier.valueOf(stringPair[0]);
			ServiceContext ctx = PortletUtils.getServiceContext();
			long id = 0;
			try {
				id = Long.parseLong(stringPair[1]);
			} catch (NumberFormatException e) {
				throw new URLHandlerException(stringPair[1] + " could not be translated to a long value");
			}

			switch (classString) {
			case C:
				result = CompetenceFacade.getCompetence(id, ctx);
				break;
			case CP:
				result = CompetenceProfileFacade.getCompetenceProfile(id, ctx);
				break;
			case CL:
				result = CompetenceLevelFacade.getCompetenceLevel(id, ctx);
				break;
			case CPL:
				result = CompetenceProfileLevelFacade.getCompetenceProfileLevel(id, ctx);
				break;
			default:
				throw new URLHandlerException("Could not be here");
			}
		} catch (IllegalArgumentException e) {
			throw new URLHandlerException(stringPair[0] + " could not be translated in a TencObjIdentifier");
		} catch (NullPointerException e) {
			throw new URLHandlerException(stringPair[0] + " could not be translated in a TencObjIdentifier");
		} catch (PortalException e) {
			throw new URLHandlerException(sURL + " could not be translated in an entity");
		} catch (SystemException e) {
			throw new URLHandlerException(sURL + " could not be translated in an entity");
		}

		return result;

	}

	/**
	 * The Enum TencObjIdentifier.
	 */
	public enum TencObjIdentifier {
		/** The Competence. */
		C,
		/** The Competence Profile. */
		CP,
		/** The Competence Level. */
		CL,
		/** The Competence Profile Level. */
		CPL;
	}

}
