/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import org.tencompetence.portlet.common.utils.TENCActionKey;

import com.liferay.portal.model.BaseModel;

/**
 * Generic Data Type.
 * 
 * @param <T>
 * 
 * @author sheyenrath
 *            
 * @version $Revision: 1.10 $, $Date: 2009/11/24 14:10:06 $
 */
public class ActionsMenuRowItem<T> extends PermissionsBean {

	private final T data;
	private final Long id;
	private boolean selected;

	/**
	 * Instantiates a new row item.
	 * 
	 * @param data
	 *            the data associated with this row
	 * @param id
	 *            the id
	 */
	public ActionsMenuRowItem(final T data, final Long id) {
		super();
		this.data = data;
		this.id = id;
	}

	/**
	 * Gets the id of this row item.
	 * 
	 * @return id.
	 */
	public long getId() {
		return id;
	}

	/**
	 * Gets the data associated with this RowItem.
	 * 
	 * @return the item
	 */
	public T getData() {
		return data;
	}

	/**
	 * Is this row selected ?
	 * 
	 * @return true/false
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * Set this row selected.
	 * 
	 * @param selected
	 *            true/false
	 */
	public void setSelected(final boolean selected) {
		this.selected = selected;
	}

	/**
	 * Show the actions menu ?
	 * 
	 * @return showActionsMenu
	 */
	public boolean isActionAllowed() {
		return isAllowView() || isAllowEdit() || isAllowDelete() || isAllowPermissions();
	}

	/**
	 * Show the actions menu ?
	 * 
	 * @return showActionsMenu
	 */
	public boolean isShowActionsMenu() {
		return isActionAllowed();
	}

	/**
	 * Is the user allowed to view this item ?
	 * 
	 * @return allowView
	 */
	public boolean isAllowView() {
		return false; // By default we disable view because it's normally handled by the link.
	}

	/**
	 * Is the user allowed to edit this item ?
	 * 
	 * @return allowEdit
	 */
	public boolean isAllowEdit() {
		return checkAllow(TENCActionKey.UPDATE);
	}

	/**
	 * Is the user allowed to delete this item ?
	 * 
	 * @return allowDelete
	 */
	public boolean isAllowDelete() {
		return false; // By default we disable delete.
	}

	/**
	 * @return customTitle
	 */
	public String getCustomTitle() {
		return "Override";
	}

	/**
	 * Is the user allowed to edit permissions for this item ?
	 * 
	 * @return allowPermissions
	 */
	public boolean isAllowPermissions() {
		return checkAllow(TENCActionKey.PERMISSIONS);
	}

	/**
	 * Check allow action.
	 * 
	 * @param action
	 *            the action
	 * 
	 * @return true, if successful
	 */
	protected boolean checkAllow(final TENCActionKey action) {
		if (data instanceof BaseModel<?>) {
			return checkModelPermission((BaseModel<?>) data, action);
		} else {
			return false;
		}
	}
}
