/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.Locale;

import javax.faces.context.FacesContext;

import org.tencompetence.business.utils.UserLocaleResolver;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;
import com.liferay.util.bridges.jsf.common.JSFPortletUtil;

/**
 * The Class RatingPopupController.
 * 
 * @version $Revision: 1.8 $, $Date: 2009/11/24 14:10:06 $
 */
public abstract class AssessmentPopupController {

	/** The Constant LOG. */
	private static final Log LOG = LogFactoryUtil.getLog(AssessmentPopupController.class);

	private static final String CONTEXT = "/apis/";

	private static final String PARAM_USER_ID = "user_id";
	private static final String PARAM_TEST_ID = "test_id";
	private static final String PARAM_COMPANY_ID = "company_id";
	private static final String PARAM_GROUP_ID = "group_id";
	private static final String PARAM_LANGUAGE = "language";

	private static final String SERVER_NAME = JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance())
			.getServerName();
	private static final int SERVER_PORT = JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance())
			.getServerPort();
	private static final String SERVER_SCHEME = JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance())
			.getScheme();

	private String url;

	private boolean visible;

	private String assessmentId;

	/**
	 * Instantiates a new rating popup controller.
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public AssessmentPopupController() throws SystemException, PortalException {
		super();
		url = SERVER_SCHEME + "://" + SERVER_NAME + ":" + SERVER_PORT
				+ JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance()).getContextPath();

	}

	/**
	 * Checks if is visible.
	 * 
	 * @return true, if is visible
	 */
	public boolean isVisible() {
		return visible;
	}

	/**
	 * Sets the visibility of this dialog.
	 * 
	 * @param visible
	 *            the new visibility value
	 */
	public void setVisible(final boolean visible) {
		if (this.visible != visible) {
			this.visible = visible;
			if (visible) {
				this.assessmentId = getAssessmentId();
			}
		}
	}

	/**
	 * Do cancel.
	 * 
	 * @return the string
	 */
	public String doCancel() {
		setVisible(false);
		return null;
	}

	/**
	 * This method is called when the OK button is pressed. This method should not be overwritten. Instead implementing
	 * classes should put their logic in the handleOK method.
	 * 
	 * @return null
	 */
	public String doOk() {
		setVisible(false);
		handleOk();
		return null;
	}

	/**
	 * Implementations of this method may perform specific logic here when the OK button is pressed.
	 */
	public abstract void handleOk();

	/**
	 * Implementations of this method must return the identifier (URI) of the assessment test or item associated with
	 * this controller.
	 * 
	 * @return abstract String
	 */
	public abstract String getAssessmentId();

	/**
	 * Returns the correct, fully qualified URL for the QTI rendering servlet.
	 * 
	 * @return the correct, fully qualified URL for the QTI rendering servlet
	 */
	public String getIframeURL() {
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return url + CONTEXT + "?" + PARAM_USER_ID + "=" + ctx.getUserId() + "&" + PARAM_TEST_ID + "=" + assessmentId
				+ "&" + PARAM_GROUP_ID + "=" + ctx.getScopeGroupId() + "&" + PARAM_COMPANY_ID + "="
				+ ctx.getCompanyId() + "&" + PARAM_LANGUAGE + "=" + getLanguage();
	}

	private String getLanguage() {
		try {
			Locale loc = UserLocaleResolver.getUserLocale();
			String localeLanguage = loc.getLanguage();
			if (localeLanguage == "nl") {
				return "dutch";
			} else if (localeLanguage == "es") {
				return "spanish";
			} else {
				// return the default
				return "english";
			}
		} catch (Exception e) {
			return "english";
		}
	}
}
