/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import javax.el.ELException;
import javax.el.ValueExpression;
import javax.faces.component.UIComponentBase;

/**
 * The Class BaseCustomComponent.
 * 
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:06 $
 */
public abstract class BaseCustomComponent extends UIComponentBase {

	/** The Constant WIDTH. */
	protected static final String WIDTH = "width";

	/** The Constant STYLE. */
	protected static final String STYLE = "style";

	/** The Constant CLASS. */
	protected static final String CLASS = "class";

	/** The Constant CELLSPACING. */
	protected static final String CELLSPACING = "cellspacing";

	/** The Constant CELLPADDING. */
	protected static final String CELLPADDING = "cellpadding";

	/**
	 * Gets an attribute.
	 * 
	 * @param <T>
	 *            Type
	 * @param name
	 *            the name
	 * @param defaultValue
	 *            the default value
	 * 
	 * @return the value
	 */
	@SuppressWarnings("unchecked")
	protected <T> T getAttribute(final String name, final T defaultValue) {
		final Object value = getValue(name);

		if (value == null) {
			return defaultValue;
		}

		return (T) value;
	}

	/**
	 * Gets a required string attribute.
	 * 
	 * @param name
	 *            the name
	 * 
	 * @return the required integer
	 */
	protected String getRequiredString(final String name) {
		final Object value = getValue(name);

		if (!(value instanceof String)) {
			throw new ELException("Error: Attribute '" + name + "' must be a String.");
		}

		return (String) value;
	}

	private Object getValue(final String name) {
		final ValueExpression ve = getValueExpression(name);
		if (ve == null) {
			throw new ELException("Error: Attribute '" + name + "' is required and must be a EL.");
		}

		return ve.getValue(getFacesContext().getELContext());
	}
}