/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import javax.faces.event.ActionEvent;

import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.icesoft.faces.component.ContextActionEvent;
import com.liferay.portal.model.BaseModel;

/**
 * Base delete class.
 * @author sheyenrath
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:33:59 $
 */
public abstract class BaseDeletePortletBean extends PermissionsBean {
	
	/**
	 * Initialize the fields of this bean. This method is called whenever the input has been changed or set.
	 */
	protected abstract void initFields();

	/**
	 * Use this method in the jspx to correctly initialize this page / bean.
	 * 
	 * @return null
	 */
	public String getPageInit() {
		final boolean initFields = PortletUtils.getObjectFromSession(Defines.INIT_FIELDS, false);
		if (initFields) {
			initFields();
			PortletUtils.setObjectToSession(Defines.INIT_FIELDS, false);
		}
		return null;
	}
	
	/**
	 * Delete an Entity.
	 * 
	 * @param entity
	 *            BaseModel<?> entity
	 */
	protected abstract void deleteListEntity(final BaseModel<?> entity);

	/**
	 * Listen to to delete-action event from teh actions-menu.
	 * 
	 * @param event
	 *            ActionEvent
	 */
	public void deleteActionListener(final ActionEvent event) {
		if (event instanceof ContextActionEvent) {
			final ContextActionEvent contextEvent = (ContextActionEvent) event;
			final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) contextEvent.getContextValue();
			final BaseModel<?> entity = (BaseModel<?>) rowItem.getData();

			deleteListEntity(entity);
		}
	}
}
