/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.common.components;

import javax.faces.event.ActionEvent;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.ReadOnlyException;

import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * BasePreferences.
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.4 $, $Date: 2009/11/25 12:42:20 $
 */
public abstract class BasePreferences {
	private static final Log LOG = LogFactoryUtil.getLog(BasePreferences.class);

	private static final String COMMUNITYMODE = "COMMUNITYMODE";

	private final String smallView;
	private final PortletPreferences prefs;
	private CommunityMode communityMode;

	/**
	 * The Enum PortletCommunityMode.
	 */
	public enum CommunityMode {

		/** Current Community. */
		CURRENT,

		/** ALL Communities. */
		ALL
	}

	/**
	 * Instantiates a new BasePreferences.
	 */
	public BasePreferences() {
		smallView = UrlUtils.getWindowStateNormalUrl(PortletMode.VIEW);
		prefs = PortletUtils.getPortletRequest().getPreferences();
		communityMode = CommunityMode.valueOf(prefs.getValue(COMMUNITYMODE, CommunityMode.CURRENT.name()));
	}

	/**
	 * Save the preferences.
	 * 
	 * @param prefs
	 *            the prefs
	 * 
	 * @throws ReadOnlyException
	 *             the read only exception
	 */
	protected abstract void updatePreferences(final PortletPreferences prefs) throws ReadOnlyException;

	/**
	 * Do save and navigate to smallview.
	 * 
	 * @param ae
	 *            the action event
	 */
	public void doSave(final ActionEvent ae) {
		try {
			prefs.setValue(COMMUNITYMODE, communityMode.name());
			updatePreferences(prefs);
			prefs.store();
			PortletUtils.redirect(smallView);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Navigate to smallview.
	 * 
	 * @param ae
	 *            the action event
	 * 
	 */
	public void doCancel(final ActionEvent ae) {
		PortletUtils.redirect(smallView);
	}

	/**
	 * Gets the prefs.
	 * 
	 * @return the prefs
	 */
	public PortletPreferences getPrefs() {
		return prefs;
	}

	/**
	 * Gets the portlet community mode (CURRENT or ALL).
	 * 
	 * @return the portlet community mode
	 */
	public String getSelectedPortletCommunityMode() {
		return communityMode.name();
	}

	/**
	 * Sets the portlet community mode.
	 * 
	 * @param mode
	 *            the new portlet community mode
	 */
	public void setSelectedPortletCommunityMode(final String mode) {
		communityMode = CommunityMode.valueOf(mode);
	}

	/**
	 * Gets the portlet community mode (CURRENT or ALL).
	 * 
	 * @return the portlet community mode
	 */
	public CommunityMode getPortletCommunityMode() {
		return communityMode;
	}
}
