/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.List;

/**
 * CrumbController.
 * 
 * Abstract class that should be extended by the classes that want to use the new crumblepath.
 * 
 * @author jal
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:06 $
 */
public abstract class CrumbController {
	private static final String DOTS = "...";
	private static final int MAX_TITLE_LEN = 30;

	private ICrumbEntry selectedCrumb;

	/**
	 * handleCrumb.
	 * 
	 * @param selectedCrumb
	 *            the selectedCrumb
	 * 
	 * @return the string
	 */
	public abstract String handleCrumbEvent(final ICrumbEntry selectedCrumb);

	/**
	 * Gets the crumbs.
	 * 
	 * @return the crumbs
	 */
	public abstract List<ICrumbEntry> getCrumbs();

	/**
	 * Handle click.
	 * 
	 * @return the navigation string
	 */
	public String crumbClicked() {
		return handleCrumbEvent(selectedCrumb);
	}

	/**
	 * Sets the selected crumb.
	 * 
	 * @param selectedCrumb
	 *            the new selected crumb
	 */
	public void setSelectedCrumb(final ICrumbEntry selectedCrumb) {
		this.selectedCrumb = selectedCrumb;
	}
	
	/**
	 * Format the title from the BreadCrumb.
	 * 
	 * @param title the title
	 * 
	 * @return the string
	 */
	protected String formatTitle(final String title) {
		final StringBuilder sb = new StringBuilder();
		if (title.length() >  MAX_TITLE_LEN) {
			sb.append(title.substring(0, MAX_TITLE_LEN - DOTS.length()));
			sb.append(DOTS);
		} else {
			sb.append(title);
		}

		return sb.toString();
	}
}
