/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.goal.beans.GoalUtil;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.service.ServiceContext;

/**
 * This abstract class implements the controller for the goalTitle facelet.
 * 
 * @author Harrie Martens
 * @version $Revision: 1.2 $, $Date: 2009/12/01 09:37:45 $
 */

public class GoalTitleController {

	private String title;
	private String levelTitle;
	private boolean multiLevel;

	/**
	 * Instantiates a new goal title controller.
	 * 
	 * @param goal
	 *            the goal
	 * @param ctx
	 *            the Liferay service context
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public GoalTitleController(final Goal goal, final ServiceContext ctx) throws SystemException, PortalException {
		super();
		setInput(goal, ctx);
	}

	private void setInput(final Goal goal, final ServiceContext ctx) throws SystemException, PortalException {
		title = "";
		levelTitle = "";
		multiLevel = false;
		if (goal != null && GoalUtil.checkValidTargetType(goal)) {

			// get the competenceProfileLevel from the goal
			CompetenceProfileLevel level = CompetenceProfileLevelFacade
					.getCompetenceProfileLevel(goal.getTarget(), ctx);

			if (level != null) {
				levelTitle = level.getTitle();
				CompetenceProfile profile = CompetenceProfileFacade.getCompetenceProfile(
						level.getCompetenceProfileId(), ctx);
				if (profile != null) {
					title = profile.getTitle();
					multiLevel = CompetenceProfileLevelFacade.getCompetenceProfileLevelsByCompetenceProfileId(
							level.getCompetenceProfileId(), PortletUtils.getServiceContext()).size() > 1;
				}
			}
		}
	}

	/**
	 * Checks if the competence profile has multiple profile levels.
	 * 
	 * @return true, if the competence profile has multiple profile levels
	 */
	public boolean isMultiLevel() {
		return multiLevel;
	}

	/**
	 * Gets the title of the targeted competence profile level.
	 * 
	 * @return the title of the targeted competence profile level
	 */
	public String getLevelTitle() {
		return levelTitle;
	}

	/**
	 * Gets the title of the goal i.e. the title of the targeted competence profile.
	 * 
	 * @return the the title of the goal i.e. the title of the targeted competence profile
	 */
	public String getGoalTitle() {
		return title;
	}

}