/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.List;

import com.liferay.portal.model.BaseModel;

/**
 * The Interface IActionsMenuListController2.
 * 
 * @param <T>
 *            the type
 * @version $Revision: 1.2 $, $Date: 2009/11/24 14:10:06 $
 */
public interface IActionsMenuListController2<T extends BaseModel<T>> {

	/**
	 * Gets the rows.
	 * 
	 * @return the rows
	 */
	List<? extends IActionsMenuRow<T>> getRows();

	/**
	 * Checks if is show select column.
	 * 
	 * @return true, if is show select column
	 */
	boolean isShowSelectColumn();

	/**
	 * Gets the title header.
	 * 
	 * @return the title header
	 */
	String getTitleHeader();

	/**
	 * Checks if is show actions button.
	 * 
	 * @return true, if is show actions button
	 */
	boolean isShowActionsButton();

	/**
	 * Checks if is show extra column.
	 * 
	 * @return true, if is show extra column
	 */
	boolean isShowExtraColumn();

	/**
	 * Gets the extra column header.
	 * 
	 * @return the extra column header
	 */
	String getExtraColumnHeader();

	/**
	 * Gets the max url.
	 * 
	 * @return the max url
	 */
	String getMaxUrl();

	/**
	 * Sets the max url.
	 * 
	 * @param url
	 *            the new max url
	 */
	void setMaxUrl(String url);

	/**
	 * Gets the normal url.
	 * 
	 * @return the normal url
	 */
	String getNormalUrl();

	/**
	 * Sets the normal url.
	 * 
	 * @param url
	 *            the new normal url
	 */
	void setNormalUrl(String url);

	/**
	 * Adds the row.
	 * 
	 * @param entity
	 *            the entity
	 * @param clickLinkListener
	 *            the click link listener
	 * @param items
	 *            the items
	 */
	void addRow(T entity, final IActionEventListener<T> clickLinkListener, List<ActionsMenuItem<T>> items);

	/**
	 * Gets the selected items.
	 * 
	 * @return the selected items
	 */
	List<? extends IActionsMenuRow<T>> getSelectedItems();

}
