/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import javax.faces.event.ActionEvent;

import com.liferay.portal.model.BaseModel;

/**
 * The Interface IActionsMenuRow.
 * 
 * @param <T>
 *            the type
 * @version $Revision: 1.2 $, $Date: 2009/11/24 14:10:06 $
 */
public interface IActionsMenuRow<T extends BaseModel<T>> {

	/**
	 * Gets the entity.
	 * 
	 * @return the entity
	 */
	T getEntity();

	/**
	 * Sets the selected.
	 * 
	 * @param selected
	 *            the new selected
	 */
	void setSelected(final boolean selected);

	/**
	 * Checks if is selected.
	 * 
	 * @return true, if is selected
	 */
	boolean isSelected();

	/**
	 * Checks if is selectable enabled.
	 * 
	 * @return true, if is selectable enabled
	 */
	boolean isSelectableEnabled();

	/**
	 * Checks if is show title as link.
	 * 
	 * @return true, if is show title as link
	 */
	boolean isShowTitleAsLink();

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	String getTitle();

	/**
	 * Click link.
	 * 
	 * @param ae
	 *            the ae
	 */
	void clickLink(final ActionEvent ae);

	/**
	 * Gets the actions button controller.
	 * 
	 * @return the actions button controller
	 */
	ActionButtonController<T> getActionsButtonController();

	/**
	 * Gets the extra column value.
	 * 
	 * @return the extra column value
	 */
	String getExtraColumnValue();

}
