/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import com.liferay.portal.model.BaseModel;

/**
 * Interface for a confirmation dialog controller.
 * <p>
 * The confirmationDialog facelet expects a controller implementing this interface to control its behavior. The
 * controller defines the visibility of the dialog via the {@code isPopupVisible} method. And when clicking on either
 * the OK or Cancel button, the respective action handlers are called.
 * 
 * @author Harrie Martens
 * @version $Revision: 1.2 $, $Date: 2009/09/03 10:45:43 $, $Author: bat266 $
 */
public interface IConfirmationDialogController extends IOkCancelDialog {

	/**
	 * Intercepts navigation from this view to another by checking if the current editor is dirty. If so it asks the
	 * user for a confirmation to discard the changes made.
	 * 
	 * @param nextView
	 *            the id of the view to navigate to
	 * @param nextInput
	 *            the editor input for the view being navigated to
	 * 
	 * @return the string
	 */
	String canLeave(final String nextView, final BaseModel<?> nextInput);
}