/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.PortletUtils;

/**
 * The class InitEntityController.
 * 
 * @param <T>
 *            GenericType
 * @author sheyenrath
 * @version $Revision: 1.2 $, $Date: 2009/11/02 21:50:08 $
 */
public abstract class InitEntityController<T> {

	private T entity;
	private boolean needsInit = true;

	/**
	 * Initialize the fields of this bean. This method is called whenever the input has been changed or set.
	 */
	protected abstract void initFields();

	/**
	 * Use this method in the jspx to correctly initialize this page / bean.
	 * 
	 * @return null
	 */
	public String getPageInit() {
		final boolean initFields = PortletUtils.getObjectFromSession(Defines.INIT_FIELDS, false);
		if (initFields || needsInit) {
			initFields();
			PortletUtils.setObjectToSession(Defines.INIT_FIELDS, false);
			needsInit = false;
		}
		return null;
	}

	/**
	 * Fetches the entity.
	 * 
	 * @return the entity, or null if no entity was set
	 */
	@SuppressWarnings("unchecked")
	public T getEntity() {
		final Object result = PortletUtils.getObjectFromSession(Defines.EDITOR_INPUT_PARAM_NAME);
		if (result != null) {
			entity = (T) result;
			return entity;
		}

		return null;
	}

	/**
	 * Sets the entity.
	 * 
	 * @param entity
	 *            the new entity
	 */
	public void setEntity(final T entity) {
		PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, entity);
		this.entity = entity;
		needsInit = true;
	}
}
