/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import org.tencompetence.portlet.common.utils.Bundle;

/**
 * The Class PermissionsData.
 * 
 * @author sheyenrath
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:10:06 $
 */
public class PermissionsData {

	/** The action. */
	private String action;
	
	/** The guest permission. */
	private boolean guestPermission;
	
	/** The guest unsupported permission. */
	private boolean guestUnsupportedPermission;
	
	/** The community permission. */
	private boolean communityPermission;

	/**
	 * The Constructor.
	 * 
	 * @param action action
	 * @param community community
	 * @param guest guest
	 * @param guestUnsupported guestUnsupported
	 */
	public PermissionsData(final String action, final boolean community, final boolean guest,
			final boolean guestUnsupported) {
		this.action = action;
		this.guestPermission = guest;
		this.communityPermission = community;
		this.guestUnsupportedPermission = guestUnsupported;
	}

	/**
	 * Checks if is guest unsupported permission.
	 * 
	 * @return true, if is guest unsupported permission
	 */
	public boolean isGuestUnsupportedPermission() {
		return guestUnsupportedPermission;
	}

	/**
	 * Gets the action.
	 * 
	 * @return the action
	 */
	public String getAction() {
		return action;
	}

	/**
	 * Gets the action translated.
	 * 
	 * @return the action translated
	 */
	public String getActionTranslated() {
		return Bundle.getLabel("action." + action);
	}

	/**
	 * Checks if is guest permission.
	 * 
	 * @return true, if is guest permission
	 */
	public boolean isGuestPermission() {
		return guestPermission;
	}

	/**
	 * Sets the guest permission.
	 * 
	 * @param guestPermission the new guest permission
	 */
	public void setGuestPermission(final boolean guestPermission) {
		this.guestPermission = guestPermission;
	}

	/**
	 * Checks if is community permission.
	 * 
	 * @return true, if is community permission
	 */
	public boolean isCommunityPermission() {
		return communityPermission;
	}

	/**
	 * Sets the community permission.
	 * 
	 * @param communityPermission the new community permission
	 */
	public void setCommunityPermission(final boolean communityPermission) {
		this.communityPermission = communityPermission;
	}
}
