/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.ArrayList;
import java.util.List;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.PortalClassInvoker;

/**
 * The Class PermissionsList.
 * 
 * @param <T> generic type
 * 
 * @author sheyenrath
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:10:06 $
 */
public class PermissionsList<T> extends PermissionsBean {
	
	/** The Constant RESOURCEUTIL. */
	private static final String RESOURCEUTIL = "com.liferay.portal.security.permission.ResourceActionsUtil";
	
	/** The list. */
	private List<PermissionsData> list = null;

	/** The log. */
	private static final Log LOG = LogFactoryUtil.getLog(PermissionsList.class);

	/**
	 * Create PermissionsList.
	 * 
	 * @param plId PorletId
	 * @param entity Entity
	 */
	@SuppressWarnings("unchecked")
	public PermissionsList(final String plId, final T entity) {

		final String entityClassName = entity.getClass().getInterfaces()[0].getName();
		try {
			final List<String> resourceActions = (List<String>) PortalClassInvoker.invoke(RESOURCEUTIL,
					"getModelResourceActions", entityClassName, false);

			final List<String> resourceCommunityActions = (List<String>) PortalClassInvoker.invoke(RESOURCEUTIL,
					"getModelResourceCommunityDefaultActions", entityClassName, false);

			final List<String> resourceGuestActions = (List<String>) PortalClassInvoker.invoke(RESOURCEUTIL,
					"getModelResourceGuestDefaultActions", entityClassName, false);

			final List<String> resourceGuestUnsupportedActions = (List<String>) PortalClassInvoker.invoke(RESOURCEUTIL,
					"getModelResourceGuestUnsupportedActions", entityClassName, false);

			list = new ArrayList<PermissionsData>();

			for (final String action : resourceActions) {
				final boolean communityDefefault = resourceCommunityActions.contains(action);
				final boolean guestDefault = resourceGuestActions.contains(action);
				final boolean guestUnsupported = resourceGuestUnsupportedActions.contains(action);
				list.add(new PermissionsData(action, communityDefefault, guestDefault, guestUnsupported));
			}

		} catch (Exception e) {
			LOG.error(e);
		}
	}

	/**
	 * Get a list from the permissions.
	 * 
	 * @return List<PermissionsData>
	 */
	public List<PermissionsData> getAvailablePermissions() {
		return list;
	}

	/**
	 * Gets the community permissions.
	 * 
	 * @return the community permissions
	 */
	public String[] getCommunityPermissions() {
		final List<String> list = new ArrayList<String>();

		for (final PermissionsData data : getAvailablePermissions()) {
			if (data.isCommunityPermission()) {
				list.add(data.getAction());
			}
		}

		return list.toArray(new String[list.size()]);
	}

	/**
	 * Gets the guest permissions.
	 * 
	 * @return the guest permissions
	 */
	public String[] getGuestPermissions() {
		final List<String> list = new ArrayList<String>();

		for (final PermissionsData data : getAvailablePermissions()) {
			if (data.isGuestPermission()) {
				list.add(data.getAction());
			}
		}

		return list.toArray(new String[list.size()]);
	}
}
