/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Paint;

import org.krysalis.jcharts.axisChart.AxisChart;
import org.krysalis.jcharts.chartData.AxisChartDataSet;
import org.krysalis.jcharts.chartData.ChartDataException;
import org.krysalis.jcharts.chartData.DataSeries;
import org.krysalis.jcharts.properties.AxisProperties;
import org.krysalis.jcharts.properties.AxisTypeProperties;
import org.krysalis.jcharts.properties.ChartProperties;
import org.krysalis.jcharts.properties.DataAxisProperties;
import org.krysalis.jcharts.properties.LegendAreaProperties;
import org.krysalis.jcharts.properties.LegendProperties;
import org.krysalis.jcharts.properties.PropertyException;
import org.krysalis.jcharts.properties.StackedBarChartProperties;
import org.krysalis.jcharts.properties.util.ChartFont;
import org.krysalis.jcharts.properties.util.ChartStroke;
import org.krysalis.jcharts.types.ChartType;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.icesoft.faces.component.outputchart.OutputChart;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * The Class ProgressIndicatorController.
 * 
 * Controller for the progressindicator of the e.g. pdp. It exists of an stackedbar with 3 columns stacked. Default the
 * attained, planned and not planned properties.
 * @version $Revision: 1.8 $, $Date: 2009/11/24 14:10:06 $
 */
public class ProgressIndicatorController {
	private static final Log LOG = LogFactoryUtil.getLog(RatingComponentController.class);

	private static final int CHART_SCALE = 25;
	private static final int CHART_PADDING = 18;
	private static final int CHART_WIDTH = 360;
	private static final int CHART_HEIGHT = 108;
	private static final int FONT_SIZE = 10;

	/** The data. */
	private int[] data = new int[] { 0, 0, 0 };

	/** The labels. */
	private String[] labels = new String[] { Bundle.getLabel("progress_indicator.label.attained"),
			Bundle.getLabel("progress_indicator.label.planned"),
			Bundle.getLabel("progress_indicator.label.not_planned") };

	/** The title. */
	private String title = " ";

	/**
	 * Instantiates a new rating component controller.
	 */
	public ProgressIndicatorController() {

	}

	/**
	 * Gets the data.
	 * 
	 * @return the data
	 */
	public int[] getData() {
		return data;
	}

	/**
	 * Sets the data.
	 * 
	 * @param data
	 *            the new data
	 */
	public void setData(final int[] data) {
		this.data = data;
	}

	/**
	 * Gets the labels.
	 * 
	 * @return the labels
	 */
	public String[] getLabels() {
		String[] formattedLabels = labels.clone();
		// only format labels if data and labels are in sync
		if (data.length == formattedLabels.length) {
			for (int i = 0; i < data.length; i++) {
				formattedLabels[i] = data[i] + "% " + formattedLabels[i];
			}
		}
		return formattedLabels;
	}

	/**
	 * Sets the labels.
	 * 
	 * @param labels
	 *            the new labels
	 */
	public void setLabels(final String[] labels) {
		this.labels = labels;
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		System.out.println("get title" + title);
		return title;
	}

	/**
	 * Sets the title.
	 * 
	 * @param title
	 *            the new title
	 */
	public void setTitle(final String title) {
		this.title = title;
	}

	/**
	 * New chart.
	 * 
	 * @param component
	 *            the component
	 * 
	 * @return true, if successful
	 */
	public boolean newChart(final OutputChart component) {
		try {
			String[] xAxisLabels = { " " };
			String xAxisTitle = " ";
			String yAxisTitle = " ";
			DataSeries dataSeries = new DataSeries(xAxisLabels, xAxisTitle, yAxisTitle, title);

			int length = data.length;
			double[][] chartData = new double[length][1];

			for (int i = 0; i < length; i++) {
				chartData[i][0] = data[i];
			}

			// add 3 to fill gap in stacked bar probably bug
			for (int i = length - 1; i >= 0; i--) {
				if (data[i] > 0) {
					chartData[i][0] = data[i] + 2;
					break;
				}
			}

			String[] legendLabels = getLabels();
			ChartFont font = new ChartFont(new Font("Verdana", Font.PLAIN, FONT_SIZE), Color.BLACK);

			// Add colors to bars
			Paint[] paints = { Defines.PROGRESS_IND_ATTAINED_COLOR, Defines.PROGRESS_IND_PLANNED_COLOR,
					Defines.PROGRESS_IND_NOT_PLANNED_COLOR };

			StackedBarChartProperties stackedBarChartProperties = new StackedBarChartProperties();
			ChartStroke borderStroke = new ChartStroke(new BasicStroke(0.0f), Color.black);
			stackedBarChartProperties.setBarOutlineStroke(borderStroke);
			stackedBarChartProperties.setWidthPercentage(1);

			AxisChartDataSet axisChartDataSet = new AxisChartDataSet(chartData, legendLabels, paints,
					ChartType.BAR_STACKED, stackedBarChartProperties);
			dataSeries.addIAxisPlotDataSet(axisChartDataSet);

			ChartProperties chartProperties = new ChartProperties();
			chartProperties.setEdgePadding(0);
			chartProperties.setBackgroundPaint(Defines.TENCC_BACKGROUND_COLOR);

			AxisProperties axisProperties = new AxisProperties(true);
			DataAxisProperties dataAxisProperties = (DataAxisProperties) axisProperties.getXAxisProperties();
			dataAxisProperties.setUserDefinedScale(0, CHART_SCALE);
			dataAxisProperties.setBackgroundPaint(Defines.TENCC_BACKGROUND_COLOR);
			axisProperties.getXAxisProperties().setShowTicks(AxisTypeProperties.TICKS_NONE);
			axisProperties.getXAxisProperties().setScaleChartFont(font);
			axisProperties.getYAxisProperties().setShowTicks(AxisTypeProperties.TICKS_NONE);
			axisProperties.getYAxisProperties().setScaleChartFont(font);
			axisProperties.getYAxisProperties().setPaddingBetweenAxisTitleAndLabels(0);
			axisProperties.setBackgroundPaint(Defines.TENCC_BACKGROUND_COLOR);

			LegendProperties legendProperties = new LegendProperties();
			legendProperties.setPlacement(LegendAreaProperties.RIGHT);
			legendProperties.setNumColumns(1);
			legendProperties.setChartPadding(CHART_PADDING);
			legendProperties.setChartFont(font);

			AxisChart axisChart = new AxisChart(dataSeries, chartProperties, axisProperties, legendProperties,
					CHART_WIDTH, CHART_HEIGHT);
			component.setChart(axisChart);
			return true;
		} catch (ChartDataException e) {
			PortletUtils.handleError(LOG, e);
			return false;
		} catch (PropertyException e) {
			PortletUtils.handleError(LOG, e);
			return false;
		}

	}

}
