/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.common.utils;

import java.util.Calendar;
import java.util.Date;



/**
 * The Enum DateRangeEnum.
 * @version $Revision: 1.6 $, $Date: 2009/11/24 14:10:06 $
 */
public enum DateRangeEnum {

	// Calendar range selectors defaults
	/** The last two weeks. */
	LAST_TWO_WEEKS(-14),
	/** The YESTERDAY. */
	YESTERDAY(-2),
	/** The TODAY. */
	TODAY(-1),
	/** The last week. */
	LAST_WEEK(-7),
	/** The last month. */
	LAST_MONTH(-1),
	/** The last three months. */
	LAST_THREE_MONTHS(-3),
	/** The last half year. */
	LAST_HALF_YEAR(-6),
	/** The last year. */
	LAST_YEAR(-1),
	/** ALL. */
	ALL(0);

	private int value;

	private DateRangeEnum(final int value) {
		this.value = value;
	}

	/**
	 * Gets the label of the DateRangeType.
	 * 
	 * @return the label
	 */
	public String getLabel() {
		return Bundle.getLabel("label." + name());
	}

	/**
	 * Gets the date.
	 * 
	 * @param date
	 *            the date
	 * @param today
	 *            the today Is the startdate for this function
	 * 
	 * @return the date
	 */
	public static Date getDate(final DateRangeEnum date, final Date today) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(today);

		switch (date) {
		case ALL:
			return null;
		case TODAY:
			cal.add(Calendar.DATE, LAST_WEEK.value);
			break;
		case YESTERDAY:
			cal.add(Calendar.DATE, YESTERDAY.value);
			break;
		case LAST_WEEK:
			cal.add(Calendar.DATE, LAST_WEEK.value);
			break;
		case LAST_TWO_WEEKS:
			cal.add(Calendar.DATE, LAST_TWO_WEEKS.value);
			break;
		case LAST_MONTH:
			cal.add(Calendar.MONTH, LAST_MONTH.value);
			break;
		case LAST_THREE_MONTHS:
			cal.add(Calendar.MONTH, LAST_THREE_MONTHS.value);
			break;
		case LAST_HALF_YEAR:
			cal.add(Calendar.MONTH, LAST_HALF_YEAR.value);
			break;
		case LAST_YEAR:
			cal.add(Calendar.YEAR, LAST_YEAR.value);
			break;
		default:
			break;
		}

		return cal.getTime();
	}

	/**
	 * Gets the type by name.
	 * 
	 * @param dateRangeTypeName
	 *            the date range type name
	 * 
	 * @return the type by name
	 */
	public static DateRangeEnum getTypeByName(final String dateRangeTypeName) {
		for (DateRangeEnum type : DateRangeEnum.values()) {
			if (type.name().equals(dateRangeTypeName)) {
				return type;
			}
		}

		throw new IllegalArgumentException("No dateRangeTypeEnum defined with name " + dateRangeTypeName);

	}

}
