/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.utils;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Map;

import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.business.utils.StringUtils;


import com.liferay.portal.model.BaseModel;
import com.liferay.portal.model.Layout;
import com.liferay.portal.theme.ThemeDisplay;

/**
 * Permission utility class.
 * 
 * @author Anton Finders
 * 
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:06 $
 */
public final class EntityPermissionsUtils {

	private static final String EDITPERMISSIONSPORTLETID = "86";
	
	private EntityPermissionsUtils() {
		// No public constructor
	}

	/**
	 * Get the permissionsURL for an BaseModel<T> entity.
	 * 
	 * @param <T>
	 *            Generic Type
	 * @param entity
	 *            BaseModel entity
	 * @return URL
	 */
	public static <T> String getEntityPermissionsURL(final BaseModel<T> entity) {

		if (entity == null) {
			return null;
		}

		// PortletRequest request = (PortletRequest) PortletUtils.getPortletRequest();
		final ThemeDisplay themeDisplay = PortletUtils.getThemeDisplay();
		final Layout layout = themeDisplay.getLayout();

		final String entityClassName = entity.getClass().getInterfaces()[0].getName();
		final String entityId = entity.getPrimaryKeyObj().toString();
		final String resourceDescription = ClassUtils.getProperty(entity, "title", entityClassName + ":" + entityId);
		final String baseURL = themeDisplay.getURLHome() + layout.getFriendlyURL();

		String currentURL;
		try {
			final URL url = new URL(themeDisplay.getURLPortal() + themeDisplay.getURLCurrent());
			final Map<String, String> queryMap = StringUtils.getQueryMap(url.getQuery());
			//final String key = PortletUtils.getPortletId() + UrlUtils.TC_REDIRECT;
			final String key = PortletUtils.getPortletId() + UrlUtils.TC_TARGET_VIEW;
			queryMap.put(key, PortletUtils.getViewId());
			currentURL = URLEncoder.encode(baseURL + "?" + StringUtils.createQuery(queryMap), "UTF-8");
		} catch (UnsupportedEncodingException e) {
			return null;
		} catch (MalformedURLException e) {
			return null;
		}

		final StringBuilder permissionsPortletURL = new StringBuilder();
		permissionsPortletURL.append(baseURL);
		permissionsPortletURL.append("?");

		permissionsPortletURL.append("p_p_id=");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);

		permissionsPortletURL.append("&p_p_lifecycle=0&p_p_state=maximized");

		permissionsPortletURL.append("&p_p_col_id=column-2&p_p_col_pos=1&p_p_col_count=2");

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_struts_action=%2Fportlet_configuration%2Fedit_permissions");

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_redirect=");
		permissionsPortletURL.append(currentURL);

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_returnToFullPageURL=");
		permissionsPortletURL.append(currentURL);

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_portletResource=");
		permissionsPortletURL.append(PortletUtils.getLiferayPortletId());

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_modelResource=");
		permissionsPortletURL.append(entityClassName);

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_modelResourceDescription=");
		permissionsPortletURL.append(resourceDescription);

		permissionsPortletURL.append("&_");
		permissionsPortletURL.append(EDITPERMISSIONSPORTLETID);
		permissionsPortletURL.append("_resourcePrimKey=");
		permissionsPortletURL.append(entityId);

		return permissionsPortletURL.toString();
	}
}
