/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.utils;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;
import java.util.Map;

import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.ValueExpression;
import javax.faces.application.Application;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletConfig;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.CompetenceMap;

import com.icesoft.jasper.Constants;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
//import com.liferay.portlet.expando.model.impl.ExpandoBridgeImpl;
import com.liferay.portlet.expando.util.ExpandoBridgeFactoryUtil;
//import com.liferay.portlet.expando.model.ExpandoBridge;

import com.liferay.util.bridges.jsf.common.JSFPortletUtil;

/**
 * The Class PortletUtils.
 * 
 * Class with all kind of generic portlet functions.
 * 
 * @author leon schaeps
 * 
 * @version $Revision: 1.13 $, $Date: 2009/12/01 09:37:45 $
 */
public final class PortletUtils {

	private PortletUtils() {
		// Empty constructor
	}

	private static final Log LOG = LogFactoryUtil.getLog(PortletUtils.class);

	/**
	 * Gets the group id of the Liferay ThemeDisplay.
	 * 
	 * @return the group id
	 */
	public static long getGroupId() {
		return getThemeDisplay().getScopeGroupId();
	}

	/**
	 * Gets the context of the Liferay FacesContext.
	 * 
	 * @return the context
	 */
	public static FacesContext getContext() {
		return FacesContext.getCurrentInstance();
	}

	/**
	 * Gets the context of the Liferay FacesContext.
	 * 
	 * @return the context
	 */
	public static Application getApplication() {
		return getContext().getApplication();
	}

	/**
	 * navigate to other view.
	 * 
	 * @param outcome
	 *            the url to navigate to
	 */
	public static void handleNavigation(final String outcome) {
		getApplication().getNavigationHandler().handleNavigation(getContext(), null, outcome);
	}

	/**
	 * Gets the Liferay theme display.
	 * 
	 * @return the theme display
	 */
	public static ThemeDisplay getThemeDisplay() {
		final PortletRequest portletRequest = (PortletRequest) getContext().getExternalContext().getRequest();
		return (ThemeDisplay) portletRequest.getAttribute(WebKeys.THEME_DISPLAY);
	}

	/**
	 * Gets the request map of Liferay.
	 * 
	 * @return the request map
	 */
	public static Map<String, Object> getSessionMap() {
		return getContext().getExternalContext().getSessionMap();
	}

	/**
	 * Gets the object from session.
	 * 
	 * @param <T>
	 *            Generic
	 * 
	 * @param key
	 *            the key
	 * @param defaultValue
	 *            the default value
	 * 
	 * @return the object from map
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getObjectFromSession(final String key, final T defaultValue) {
		final Object object = PortletUtils.getSessionMap().get(key);

		T result = defaultValue;
		if (object != null) {
			result = (T) object;
		}

		return result;
	}

	/**
	 * Gets the object from session.
	 * 
	 * @param key
	 *            the key
	 * 
	 * @return the object from map
	 */
	public static Object getObjectFromSession(final String key) {
		return PortletUtils.getSessionMap().get(key);
	}

	/**
	 * Sets the object to session.
	 * 
	 * @param key
	 *            the key
	 * @param value
	 *            the value
	 */
	public static void setObjectToSession(final String key, final Object value) {
		PortletUtils.getSessionMap().put(key, value);
	}

	/**
	 * Sets the object to session.
	 * 
	 * @param key
	 *            the key
	 */
	public static void resetObjectOnSession(final String key) {
		PortletUtils.getSessionMap().put(key, null);
	}

	/**
	 * Gets the portlet config.
	 * 
	 * @return the portlet config
	 */
	public static PortletConfig getPortletConfig() {
		return (PortletConfig) getContext().getExternalContext().getRequestMap().get("javax.portlet.config");
	}

	/**
	 * Gets the portlet request.
	 * 
	 * @return the portlet request
	 */
	public static PortletRequest getPortletRequest() {
		return JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance());
	}

	/**
	 * Gets the portlet id.
	 * 
	 * @return the portlet id
	 */
	public static String getPortletId() {
		return (String) getPortletRequest().getAttribute(Constants.NAMESPACE_KEY);
	}

	/**
	 * Gets the liferay portlet id. If the portlet is instanceble, this is the instance-if.
	 * 
	 * @return the liferay portlet id
	 */
	public static String getLiferayPortletId() {
		return (String) getPortletRequest().getAttribute("PORTLET_ID");
	}

	/**
	 * Gets the company id.
	 * 
	 * @return the company id
	 */
	public static long getCompanyId() {
		return getThemeDisplay().getCompanyId();
	}

	/**
	 * Adds the error message to context.
	 * 
	 * @param id
	 *            the id of the errormessage, the portletId and form etc should be included. e.g. private String
	 *            CompetenceLevelsWarningId = PortletUtils.getPortletId()+
	 *            ":form:levelValuesTab:0:competencelevelEditButton";
	 * @param errorId
	 *            id of the errormessage defined in the Language.properties
	 * @param params
	 *            params used in the error message defined as {0}. {1} etc
	 * 
	 */
	public static void addErrorMessageToContext(final String id, final String errorId, final Object... params) {
		String sId = null;

		if (id != null) {
			sId = getPortletId() + id;
		}

		final FacesMessage message = new FacesMessage();
		message.setSeverity(FacesMessage.SEVERITY_ERROR);
		message.setSummary(Bundle.getLabel(errorId, params));

		final FacesContext context = FacesContext.getCurrentInstance();
		context.addMessage(sId, message);
	}

	/**
	 * Adds the error message to context.
	 * 
	 * @param errorId
	 *            id of the errormessage defined in the Language.properties
	 * @param params
	 *            params used in the error message defined as {0}. {1} etc
	 * 
	 */
	public static void addErrorMessageToContext(final String errorId, final Object... params) {
		addErrorMessageToContext(null, errorId, params);
	}

	/**
	 * Handle an error. Create an log entry with the Liferay log mechanisme and also throw an RuntimeException
	 * 
	 * @param log
	 *            the liferay log that should be used to handle the error
	 * @param e
	 *            the error e which should be handled
	 * 
	 */
	public static void handleError(final Log log, final Exception e) {
		if (log.isErrorEnabled()) {
			log.error(e);
		}
		throw new RuntimeException(e);
	}

	/**
	 * Returns a managed bean of the specified type and the specified name.
	 * 
	 * @param managedBeanName
	 *            the name of the managed bean to look up
	 * @param clazz
	 *            expected type of the managed bean
	 * @param <T>
	 *            generic param
	 * 
	 * @return the requested managed bean coerced to the expected type
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getManagedBean(final String managedBeanName, final Class<T> clazz) {
		final FacesContext facesContext = FacesContext.getCurrentInstance();

		// get application from faces context
		final ExpressionFactory exprFactory = facesContext.getApplication().getExpressionFactory();
		// getting the ELContext from faces context
		final ELContext elContext = facesContext.getELContext();
		// creating value expression with the help of the expression factory and
		// the ELContext
		final ValueExpression valExpr = exprFactory.createValueExpression(elContext, managedBeanName, clazz);

		return (T) valExpr.getValue(elContext);
	}

	/**
	 * Converts an List &lt;Long&gt; to an array of long[].
	 * 
	 * @param listIn
	 *            the list in
	 * 
	 * @return the array of long's
	 */
	public static long[] toLongArray(final List<Long> listIn) {
		long[] result = new long[listIn.size()];
		for (int i = 0; i < result.length; i++) {
			result[i] = listIn.get(i);
		}

		return result;
	}

	/**
	 * Normalize line break.
	 * 
	 * @param input
	 *            the input
	 * 
	 * @return the string
	 */
	public static String normalizeLineBreak(final String input) {
		return input.replaceAll(StringPool.RETURN, StringPool.BLANK);
		
	}

	/**
	 * Get the RenderResponse from context.
	 * 
	 * @return The rendered response from ICEFace
	 */
	public static RenderResponse getRenderResponse() {
		return (RenderResponse) getContext().getExternalContext().getResponse();
	}

	/**
	 * Get the RenderRequest from context.
	 * 
	 * @return RenderRequest
	 */
	public static RenderRequest getRenderRequest() {
		return (RenderRequest) getContext().getExternalContext().getRequest();
	}

	/**
	 * Get a specific UIComponent.
	 * 
	 * @param <A>
	 *            generic return type
	 * @param rootComponent
	 *            the root component
	 * @param clazz
	 *            the class to find
	 * 
	 * @return the generic type <A>
	 */
	public static <A extends UIComponent> A findUIComponent(final UIComponent rootComponent, final Class<A> clazz) {
		return findUIComponent(rootComponent, clazz, null);
	}

	/**
	 * Get a specific UIComponent.
	 * 
	 * @param <A>
	 *            Generic return type
	 * @param rootComponent
	 *            the root component
	 * @param clazz
	 *            the class to find
	 * @param id
	 *            the id of the component
	 * 
	 * @return A extends UIComponent
	 */
	@SuppressWarnings("unchecked")
	public static <A extends UIComponent> A findUIComponent(final UIComponent rootComponent, final Class<A> clazz,
			final String id) {

		if (rootComponent != null && rootComponent.getClass().getName().equalsIgnoreCase(clazz.getName())) {
			if (id == null) {
				return (A) rootComponent;
			} else {
				if (rootComponent.getId().equals(id)) {
					return (A) rootComponent;
				}
			}
		}
		if (rootComponent != null) {
			final List<UIComponent> children = rootComponent.getChildren();
			for (final UIComponent childComponent : children) {
				final A found = findUIComponent(childComponent, clazz, id);
				if (found != null) {
					return found;
				}
			}
		}

		return null;
	}

	/**
	 * Gets the competence map by current community.
	 * 
	 * @return the competence map by current community
	 */
	public static CompetenceMap getCompetenceMapByCurrentCommunity() {
		CompetenceMap map = null;
		try {
			map = CompetenceMapFacade.getCompetenceMapByGroupId(PortletUtils.getGroupId(), getServiceContext());
			if (map == null) {
				map = CompetenceMapFacade.createCompetenceMap();
			}
		} catch (SystemException e) {
			handleError(LOG, e);
		} catch (PortalException e) {
			handleError(LOG, e);
		}

		return map;
	}

	/**
	 * Get the current viewId.
	 * 
	 * @return viewId
	 */
	public static String getViewId() {
		return getContext().getViewRoot().getViewId();
	}

	/**
	 * Let the page redirect to the url.
	 * 
	 * @param url
	 *            the url to redirect to
	 */
	public static void redirect(final String url) {
		try {
			final ExternalContext ec = getContext().getExternalContext();
			ec.redirect(ec.encodeResourceURL(url));
		} catch (IOException e) {
			handleError(LOG, e);
		}
	}

	/**
	 * SetOwnership.
	 * 
	 * @param entity
	 *            BaseModel<?> Entity
	 * @param ctx
	 *            ServiceContext
	 * @throws SystemException
	 *             SystemException
	 */
	public static void setOwnership(final BaseModel<?> entity, final ServiceContext ctx) throws SystemException {
		EntityUtils.setUserOnEntity(entity, ctx.getUserId());
		EntityUtils.setCompanyIdOnEntity(entity, ctx.getCompanyId());
		EntityUtils.setGroupIdOnEntity(entity, ctx.getScopeGroupId());
		return;
	}

	/**
	 * Get the ServiceContext from the portletRequest.
	 * 
	 * @return ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static ServiceContext getServiceContext() throws PortalException, SystemException {
		return getServiceContext(null);
	}

	/**
	 * Get the ServiceContext from the portletRequest for a specific entity.
	 * 
	 * @param <T>
	 *            Generic type
	 * @param entity
	 *            The entity
	 * 
	 * @return ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static <T> ServiceContext getServiceContext(final BaseModel<T> entity) throws PortalException,
			SystemException {

		final PortletRequest portletRequest = getPortletRequest();
		final ServiceContext serviceContext = new ServiceContext();
		final ThemeDisplay themeDisplay = getThemeDisplay();

		serviceContext.setCompanyId(themeDisplay.getCompanyId());
		serviceContext.setLanguageId(themeDisplay.getLanguageId());
		serviceContext.setLayoutURL(PortalUtil.getLayoutURL(themeDisplay));
		serviceContext.setPathMain(PortalUtil.getPathMain());
		serviceContext.setPlid(themeDisplay.getPlid());
		serviceContext.setPortalURL(PortalUtil.getPortalURL(portletRequest));
		serviceContext.setScopeGroupId(themeDisplay.getScopeGroupId());
		serviceContext.setUserDisplayURL(themeDisplay.getUser().getDisplayURL(themeDisplay));
		serviceContext.setUserId(themeDisplay.getUserId());

		// Only add expando if needed.
		if (entity != null) {
			final String className = entity.getClass().getName();
			final Map<String, Serializable> attributes = PortalUtil.getExpandoBridgeAttributes(ExpandoBridgeFactoryUtil.getExpandoBridge(serviceContext.getCompanyId(),className, 0) 
					/*new ExpandoBridgeImpl(className, 0)*/, portletRequest);
			serviceContext.setExpandoBridgeAttributes(attributes);
		}

		// Permissions
		//final boolean addCommunityPermissions = ParamUtil.getBoolean(portletRequest, "addCommunityPermissions");
		final boolean addGuestPermissions = ParamUtil.getBoolean(portletRequest, "addGuestPermissions");
		//final String[] communityPermissions = PortalUtil.getCommunityPermissions(portletRequest);
		final String[] guestPermissions = PortalUtil.getGuestPermissions(portletRequest);

		//serviceContext.setAddCommunityPermissions(addCommunityPermissions);
		serviceContext.setAddGuestPermissions(addGuestPermissions);
		//serviceContext.setCommunityPermissions(communityPermissions);
		serviceContext.setGuestPermissions(guestPermissions);

		// Tags
		//final String[] tagsCategories = PortalUtil.getTagsCategories(portletRequest);
		//final String[] tagsEntries = PortalUtil.getTagsEntries(portletRequest);

		//serviceContext.setTagsCategories(tagsCategories);
		//serviceContext.setTagsEntries(tagsEntries);

		return serviceContext;
	}

	/**
	 * Returns the userid of the user that is currently loggedIn.
	 * 
	 * @return the userId
	 */
	public static long getCurrentUserId() {
		return getThemeDisplay().getUserId();
	}
	
	/**
	 * Forward to a jspx page.
	 * 
	 * @param jspx the jspx
	 */
	public static void forward(final String jspx) {
		final FacesContext fc = getContext();
		final UIViewRoot viewRoot = getApplication().getViewHandler().createView(fc, jspx);
		fc.setViewRoot(viewRoot);
		fc.renderResponse();
	}
}
