/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.goal.beans;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.ActionsMenuItem;
import org.tencompetence.portlet.common.components.IActionEventListener;
import org.tencompetence.portlet.common.components.IActionsMenuListController2;
import org.tencompetence.portlet.common.components.IActionsMenuRow;
import org.tencompetence.portlet.common.utils.Bundle;

/**
 * The Class GoalActionsMenuListController.
 * @version $Revision: 1.3 $, $Date: 2009/11/24 14:10:06 $
 */
public class GoalActionsMenuListController implements IActionsMenuListController2<Goal> {
	private String maxUrl;
	private String normalUrl;
	private List<GoalActionsMenuRowItem> rows = new ArrayList<GoalActionsMenuRowItem>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#getExtraColumnHeader()
	 */
	//@Override
	public String getExtraColumnHeader() {
		return " ";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#getMaxUrl()
	 */
	//@Override
	public String getMaxUrl() {
		return maxUrl;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#getNormalUrl()
	 */
	//@Override
	public String getNormalUrl() {
		return normalUrl;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#getRows()
	 */
	//@Override
	public List<? extends IActionsMenuRow<Goal>> getRows() {
		return rows;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#getTitleHeader()
	 */
	//@Override
	public String getTitleHeader() {
		return Bundle.getLabel("label.title");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#isShowActionsButton()
	 */
	//@Override
	public boolean isShowActionsButton() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#isShowExtraColumn()
	 */
	//@Override
	public boolean isShowExtraColumn() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#isShowSelectColumn()
	 */
	//@Override
	public boolean isShowSelectColumn() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#setMaxUrl(java.lang.String)
	 */
	//@Override
	public void setMaxUrl(final String url) {
		this.maxUrl = url;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#setNormalUrl(java.lang.String)
	 */
	//@Override
	public void setNormalUrl(final String url) {
		this.normalUrl = url;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.tencompetence.portlet.common.components.IActionsMenuListController2#addRow(com.liferay.portal.model.BaseModel
	 * , org.tencompetence.portlet.common.components.IActionEventListener, java.util.List)
	 */
	//@Override
	public void addRow(final Goal goal, final IActionEventListener<Goal> clickLinkListener,
			final List<ActionsMenuItem<Goal>> actionMenuItems) {
		rows.add(new GoalActionsMenuRowItem(goal, clickLinkListener, actionMenuItems));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuListController2#getSelectedItems()
	 */
	//@Override
	public List<GoalActionsMenuRowItem> getSelectedItems() {
		List<GoalActionsMenuRowItem> result = new ArrayList<GoalActionsMenuRowItem>();
		if (!isShowSelectColumn()) {
			// there is no selection possible so return emptyList
			return result;
		}
		for (GoalActionsMenuRowItem row : rows) {
			if (row.isSelected()) {
				result.add(row);
			}
		}

		return result;
	}

	/**
	 * Sets the selected goal.
	 * 
	 * @param goal
	 *            the new selected goal
	 */
	public void setSelectedGoal(final Goal goal) {
		for (GoalActionsMenuRowItem row : rows) {
			if (row.getEntity().equals(goal)) {
				row.setSelected(true);
			}
		}

	}

}
