/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.goal.beans;

import java.util.List;

import javax.faces.event.ActionEvent;

import org.tencompetence.business.services.SubscribeService;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.ActionButtonController;
import org.tencompetence.portlet.common.components.ActionsMenuItem;
import org.tencompetence.portlet.common.components.IActionEventListener;
import org.tencompetence.portlet.common.components.IActionsMenuRow;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;

/**
 * The Class GoalActionsMenuRowItem.
 * 
 * An extension to the ActionsMenuRowItem so we can display an icon in the frontEnd
 * @version $Revision: 1.9 $, $Date: 2009/11/24 14:10:06 $
 */
public class GoalActionsMenuRowItem implements IActionsMenuRow<Goal> {
	private static final Log LOG = LogFactoryUtil.getLog(GoalActionsMenuRowItem.class);

	private Goal entity;
	private ActionButtonController<Goal> buttonController = new ActionButtonController<Goal>();
	private boolean selected;

	private IActionEventListener<Goal> clickListener;

	/**
	 * Instantiates a new goal actions menu row item.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @param clickLinkListener
	 *            the click link listener
	 * @param actionMenuItems
	 *            the action menu items
	 */
	public GoalActionsMenuRowItem(final Goal entity, final IActionEventListener<Goal> clickLinkListener,
			final List<ActionsMenuItem<Goal>> actionMenuItems) {
		this.entity = entity;

		buttonController.addMenuItems(actionMenuItems);
		this.clickListener = clickLinkListener;
	}

	/**
	 * Checks if this user is allowed to subscribe to this goal.
	 * 
	 * @return true, if is allow subscribe
	 */
	public boolean isAllowSubscribe() {
		return !isSubscribedToGoal();
	}

	/**
	 * Checks if this user is allowed to unsubscribe to this goal.
	 * 
	 * @return true, if is allow unsubscribe
	 */
	public boolean isAllowUnsubscribe() {
		return isSubscribedToGoal();
	}

	/**
	 * Checks if this user is subscribed to the goal.
	 * 
	 * @return true, if is subscribed to goal
	 */
	private boolean isSubscribedToGoal() {
		boolean subscribed = false;
		try {
			return SubscribeService.isSubscribedToEntity(PortletUtils.getCurrentUserId(), getEntity(), PortletUtils
					.getServiceContext());
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}

		return subscribed;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#clickLink(javax.faces.event.ActionEvent)
	 */
	//@Override
	public void clickLink(final ActionEvent ae) {
		clickListener.doAction(ae, entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#getActionsButtonController()
	 */
	//@Override
	public ActionButtonController<Goal> getActionsButtonController() {
		return buttonController;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#getExtraColumnValue()
	 */
	//@Override
	public String getExtraColumnValue() {
		if (isGoalWithTarget()) {
			return StringPool.BLANK;
		} else {
			return "<img src='/org.tencompetence.all-portlet/images/goal/exclamation.png' alt='"
					+ Bundle.getLabel("goal.notarget") + "' title='" + Bundle.getLabel("goal.notarget") + "'  />";
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#getTitle()
	 */
	//@Override
	public String getTitle() {
		return entity.getTitle();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#isSelectableEnabled()
	 */
	//@Override
	public boolean isSelectableEnabled() {
		return isGoalWithTarget();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#isSelected()
	 */
	//@Override
	public boolean isSelected() {
		return selected;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#isShowTitleAsLink()
	 */
	//@Override
	public boolean isShowTitleAsLink() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#setSelected(boolean)
	 */
	//@Override
	public void setSelected(final boolean selected) {
		this.selected = selected;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.common.components.IActionsMenuRow#getEntity()
	 */
	//@Override
	public Goal getEntity() {
		return entity;
	}

	private boolean isGoalWithTarget() {
		return entity.getTarget() != 0 && entity.getTargetType() != null;
	}
}
