/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.goal.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class GoalEdit.
 * 
 * @author venrooijma
 * @author lsc
 * @version $Revision: 1.16 $, $Date: 2009/11/24 14:10:06 $
 */
public class GoalEdit extends BaseEditorBean<Goal> {

	/** The targets. */
	private List<CompetenceProfileLevel> targets = new ArrayList<CompetenceProfileLevel>();

	/** The parent targets. */
	private List<CompetenceProfile> parentTargets = new ArrayList<CompetenceProfile>();

	/** The Constant LOG. */
	private static final Log LOG = LogFactoryUtil.getLog(GoalEdit.class);

	/** The selected target. */
	private CompetenceProfileLevel selectedTarget;

	/** The selected target parent. */
	private CompetenceProfile selectedTargetParent;

	/** The ctx. */
	private ServiceContext ctx;

	/**
	 * Gets the parents of the possible targets. This makes the selection easier to use.
	 * 
	 * @return the target parents
	 */
	public List<SelectItem> getTargetParents() {

		final List<SelectItem> result = new ArrayList<SelectItem>();
		for (CompetenceProfile profilelevel : parentTargets) {
			result.add(new SelectItem(Long.toString(profilelevel.getPrimaryKey()), profilelevel.getTitle()));
		}
		return result;
	}

	/**
	 * Gets the targets.
	 * 
	 * @return the targets
	 */
	public List<SelectItem> getTargets() {
		final List<SelectItem> result = new ArrayList<SelectItem>();
		for (CompetenceProfileLevel level : targets) {
			result.add(new SelectItem(Long.toString(level.getPrimaryKey()), level.getTitle()));
		}
		return result;
	}

	/**
	 * Checks if is target selection disabled.
	 * 
	 * @return true, if is target selection disabled
	 */
	public boolean isTargetSelectionDisabled() {		
		return this.getEditorInput() != null && this.getEditorInput().getTarget() != 0;
	}

	/**
	 * Gets the selected target.
	 * 
	 * @return the selected target
	 */
	public String getSelectedTarget() {
		if (selectedTarget == null) {
			return "0";
		}
		return Long.toString(selectedTarget.getPrimaryKey());
	}

	/**
	 * Sets the selected target.
	 * 
	 * @param targetId
	 *            the new selected target
	 */
	public void setSelectedTarget(final String targetId) {
		final long id = Long.valueOf(targetId);
		if (id == 0) {
			selectedTarget = null;
		} else {
			try {
				ctx = PortletUtils.getServiceContext();
				selectedTarget = CompetenceProfileLevelFacade.getCompetenceProfileLevel(id, ctx);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void doSave(final Goal editorInput) {
		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setMotivation(getDescriptionForInternalUse());
		editorInput.setCompetenceMapId(PortletUtils.getCompetenceMapByCurrentCommunity().getCompetenceMapId());

		if (selectedTarget != null) {
			editorInput.setTarget(selectedTarget.getPrimaryKey());
			editorInput.setTargetType(selectedTarget.getClass().getInterfaces()[0].getName());
		}
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (editorInput.isNew()) {
			Goal goal = editorInput;
			try {
				PortletUtils.setOwnership(goal, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				GoalFacade.addGoal(goal, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			try {
				GoalFacade.updateGoal(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final Goal editorInput) {
		try {
			ctx = PortletUtils.getServiceContext();
			parentTargets = CompetenceProfileFacade.getCompetenceProfilesByCompetenceMapId(PortletUtils
					.getCompetenceMapByCurrentCommunity().getPrimaryKey(), ctx);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		if (editorInput.getTarget() != 0) {
			try {
				selectedTarget = CompetenceProfileLevelFacade.getCompetenceProfileLevel(editorInput.getTarget(),
						ctx);
				if (selectedTarget.getCompetenceProfileId() != 0) {
					selectedTargetParent = CompetenceProfileFacade.getCompetenceProfile(selectedTarget
							.getCompetenceProfileId(), ctx);
					//Fill the dropdownbox with the correct competenceprofilelevels
					setSelectedTargetParent(StringPool.BLANK + selectedTarget.getCompetenceProfileId());
				}				
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}

		this.setDescriptionForInternalUse(getEditorInput().getMotivation());
		this.setTitle(getEditorInput().getTitle());
	}

	/**
	 * Sets the selected target parent.
	 * 
	 * @param targetParentId
	 *            the new selected target parent
	 */
	public void setSelectedTargetParent(final String targetParentId) {
		final long id = Long.valueOf(targetParentId);
		targets.clear();
		if (id == 0) {
			selectedTargetParent = null;
		} else {
			try {
				selectedTargetParent = CompetenceProfileFacade.getCompetenceProfile(id, ctx);
				targets.addAll(CompetenceProfileLevelFacade.getCompetenceProfileLevelsByCompetenceProfileId(
						selectedTargetParent.getPrimaryKey(), ctx));
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	/**
	 * Gets the selected target parent.
	 * 
	 * @return the selected target parent
	 */
	public String getSelectedTargetParent() {
		if (selectedTargetParent == null) {
			return "0";
		}
		return Long.toString(selectedTargetParent.getPrimaryKey());
	}
}
