/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.goal.beans;

import java.util.HashSet;
import java.util.Set;

import org.tencompetence.business.services.GoalService;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.icesoft.faces.async.render.OnDemandRenderer;
import com.icesoft.faces.async.render.RenderManager;
import com.icesoft.faces.async.render.Renderable;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * The Class to manage several listeners to the selectedGoal for use on a PDP page.
 * 
 * @author venrooijma
 * @author Leon Schaeps
 * 
 * @version $Revision: 1.8 $, $Date: 2009/11/24 14:10:06 $
 */
public class GoalManager {
	private OnDemandRenderer goalRenderer;
	private Set<Renderable> listeners = new HashSet<Renderable>();
	private static final Log LOG = LogFactoryUtil.getLog(GoalManager.class);

	/**
	 * Sets the render manager.
	 * 
	 * @param renderManager
	 *            the new render manager
	 */
	public void setRenderManager(final RenderManager renderManager) {
		this.goalRenderer = renderManager.getOnDemandRenderer("goalManager");
	}

	/**
	 * Select goal. Triggers refresh on all registered listeners.
	 * 
	 * @param userId
	 *            the userid
	 * @param selectedGoal
	 *            the selected goal
	 * @param groupId
	 *            The liferay groupid
	 */
	public void selectGoal(final Goal selectedGoal, final long userId, final long groupId) {

		try {
			GoalService.selectGoalForUserCommunity(userId, groupId, selectedGoal, PortletUtils.getServiceContext());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		goalRenderer.requestRender();
	}

	/**
	 * Gets the selected goal.
	 * 
	 * @param userId
	 *            the userid
	 * @param groupId
	 *            The liferay groupid
	 * @return the selected goal
	 */
	public Goal getSelectedGoal(final long userId, final long groupId) {
		try {
			return GoalService.getSelectedGoalForUserCommunity(userId, groupId, PortletUtils.getServiceContext());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * Register a new Listener.
	 * 
	 * @param renderable
	 *            the renderable
	 */
	public void registerListener(final Renderable renderable) {
		if (!hasListener(renderable)) {
			listeners.add(renderable);
			goalRenderer.add(renderable);
		}
	}

	/**
	 * Checks if a listener is registered.
	 * 
	 * @param renderable
	 *            the renderable
	 * 
	 * @return true, if successful
	 */
	public boolean hasListener(final Renderable renderable) {
		return listeners.contains(renderable);
	}

	/**
	 * Removes a listener.
	 * 
	 * @param renderable
	 *            the renderable
	 */
	public void removeListener(final Renderable renderable) {
		goalRenderer.remove(renderable);
		listeners.remove(renderable);
	}
}