/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.goal.beans;

import javax.portlet.PortletPreferences;
import javax.portlet.ReadOnlyException;

import org.tencompetence.portlet.common.components.BasePreferences;

/**
 * The Class GoalPreferences.
 * @version $Revision: 1.6 $, $Date: 2009/11/24 14:10:06 $
 */
public class GoalPreferences extends BasePreferences {

	/** The Constant SHOW_ONLY_COMMUNITY_GOALS. */
	public static final String SHOW_ONLY_COMMUNITY_GOALS = "show_only_community_goals";
	// private static final Log LOG = LogFactoryUtil.getLog(GoalPreferences.class);
	private boolean onlyForCommunity;
	private boolean needsInit = true;

	/**
	 * Inits the fields.
	 */
	public void initFields() {
		if (needsInit) {
			needsInit = false;
			onlyForCommunity = Boolean.valueOf(getPrefs().getValue(SHOW_ONLY_COMMUNITY_GOALS, "false"));
		}
	}

	/**
	 * Gets the only for community.
	 * 
	 * @return the only for community
	 */
	public boolean getOnlyForCommunity() {
		initFields();
		return onlyForCommunity;
	}

	/**
	 * Sets the only for community.
	 * 
	 * @param onlyForCommunity
	 *            the new only for community
	 */
	public void setOnlyForCommunity(final boolean onlyForCommunity) {
		needsInit = true;
		this.onlyForCommunity = onlyForCommunity;
	}

	@Override
	protected void updatePreferences(final PortletPreferences prefs) throws ReadOnlyException {
		prefs.setValue(SHOW_ONLY_COMMUNITY_GOALS, Boolean.toString(onlyForCommunity));
	}

}
