/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.goal.beans;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;
import javax.portlet.PortletPreferences;

import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.NavigationDataHolder;
import org.tencompetence.portlet.common.components.PermissionsBean;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;
import org.tencompetence.portlet.common.utils.UrlUtils;
import org.tencompetence.portlet.goal.controllers.ProfileEntry;
import org.tencompetence.portlet.goal.controllers.ProfileEntryController;

import com.icesoft.faces.async.render.Renderable;
import com.icesoft.faces.component.ext.RowSelectorEvent;
import com.icesoft.faces.webapp.xmlhttp.PersistentFacesState;
import com.icesoft.faces.webapp.xmlhttp.RenderingException;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class GoalSmallView.
 * 
 * @author venrooijma
 * @version $Revision: 1.38 $, $Date: 2009/11/24 14:10:06 $
 */
public class GoalSmallView extends PermissionsBean implements Renderable {
	private static final Log LOG = LogFactoryUtil.getLog(GoalSmallView.class);
	private String maximizedUrl = UrlUtils.getWindowStateMaximizedUrl();
	private String normalUrl = UrlUtils.getWindowStateNormalUrl(Outcomes.GOAL_SMALL_VIEW);

	private List<ProfileEntry> profileList = new ArrayList<ProfileEntry>();
	private ProfileEntryController profileEntryController;
	private ProfileEntry selectedEntry;
	private ProfileEntry activeEntry;

	private GoalManager goalManager;
	private PersistentFacesState state;
	private boolean needsInit = true;
	private boolean isCollapsed;

	/**
	 * Instantiates a new goal small view.
	 */
	public GoalSmallView() {
		state = PersistentFacesState.getInstance();
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		initFields();
		return activeEntry != null ? activeEntry.getTitle() : "";
	}

	/**
	 * Create an icon.
	 * 
	 * @return the url
	 */
	public String getIconUrl() {
		initFields();
		return "<img src='/org.tencompetence.all-portlet/images/goal/exclamation.png' alt='"
				+ Bundle.getLabel("goal.notarget") + "' title='" + Bundle.getLabel("goal.notarget") + "'  />";
	}

	/**
	 * Gets the profile list.
	 * 
	 * @return the profile list
	 */
	public List<ProfileEntry> getProfileList() {
		initFields();
		return profileList;
	}

	
	/**
	 * Checks if user is signed in.
	 * 
	 * @return true, if user is signed in
	 */
	public boolean isSignedIn() {
		return PortletUtils.getThemeDisplay().isSignedIn();
	}
	
	/**
	 * Gets the ctrl.
	 * 
	 * @return the ctrl
	 */
	public ProfileEntryController getCtrl() {
		initFields();
		return profileEntryController;
	}

	/** {@inheritDoc} */
	//@Override
	public PersistentFacesState getState() {
		return state;
	}

	/**
	 * Gets the profile levels for the active profile.
	 * 
	 * @return the levels
	 */
	public List<SelectItem> getLevels() {
		initFields();
		List<SelectItem> result = new ArrayList<SelectItem>();
		if (activeEntry != null) {
			for (CompetenceProfileLevel level : activeEntry.getLevels()) {
				SelectItem item = new SelectItem(String.valueOf(level.getPrimaryKey()), level.getTitle());
				result.add(item);
			}
		}
		return result;
	}

	/**
	 * Gets the selected level.
	 * 
	 * @return the selected level
	 */
	public String getSelectedLevel() {
		initFields();
		return activeEntry != null ? String.valueOf(activeEntry.getSelectedLevel()) : "";
	}

	/**
	 * Gets the motivation.
	 * 
	 * @return the motivation
	 */
	public String getMotivation() {
		initFields();
		return activeEntry != null ? activeEntry.getMotivation() : "";
	}

	/**
	 * Gets the internationalized collapse text.
	 * 
	 * @return the internationalized collapse text
	 */
	public String getCollapseText() {
		initFields();
		return Bundle.getLabel("goal.collapse." + (isCollapsed ? "open" : "close"));
	}

	/**
	 * Sets the goal manager.
	 * 
	 * @param goalManager
	 *            the new goal manager
	 */
	public void setGoalManager(final GoalManager goalManager) {
		this.goalManager = goalManager;
		goalManager.registerListener(this);
	}

	/**
	 * Sets the selected profile level as the target of the goal.
	 * 
	 * @param level
	 *            the new selected level
	 */
	public void setSelectedLevel(final String level) {
		if (activeEntry != null) {
			activeEntry.setSelectedLevel(Long.valueOf(level));
		}
	}

	/**
	 * Sets the motivation.
	 * 
	 * @param value
	 *            the new motivation
	 */
	public void setMotivation(final String value) {
		if (activeEntry != null) {
			activeEntry.setMotivation(value);
		}
	}

	/**
	 * Check if user is allowed to add a Goal.
	 * 
	 * @return true / false
	 */
	public boolean isAddGoalAllowed() {
		initFields();
		return checkPortletPermission(TENCActionKey.ADD_GOAL);
	}

	/**
	 * Check if user is allowed to select a Goal.
	 * 
	 * @return true / false
	 */
	public boolean isSelectGoalAllowed() {
		initFields();
		return checkPortletPermission(TENCActionKey.SELECT_GOAL);
	}

	/**
	 * Checks if there are any profiles available.
	 * 
	 * @return true, if there are any profiles available
	 */
	public boolean isProfilesAvailable() {
		initFields();
		return profileList != null && !profileList.isEmpty();
	}

	/**
	 * Checks if a profile is selected in the list.
	 * 
	 * @return true, if a profile is selected
	 */
	public boolean isElementSelected() {
		initFields();
		return selectedEntry != null;
	}

	/**
	 * Checks if is active goal is selected in the list.
	 * 
	 * @return true, if is active goal is selected in the list
	 */
	public boolean isActiveSelected() {
		initFields();
		return activeEntry != null ? activeEntry.getSelected() : false;
	}

	/**
	 * Checks if the active profile has multiple (>1) profile levels.
	 * 
	 * @return true, if there are multiple levels available
	 */
	public boolean isMultiLevel() {
		initFields();
		return activeEntry != null ? activeEntry.hasMultipleLevels() : false;
	}

	/**
	 * Checks if there is an active profile selected.
	 * 
	 * @return true, if is active profile is available
	 */
	public boolean isActiveAvailable() {
		initFields();
		return activeEntry != null;
	}

	/**
	 * Checks if the goal selection box is collapsed.
	 * 
	 * @return true, if it is collapsed
	 */
	public boolean isCollapsed() {
		initFields();
		return isCollapsed;
	}

	/**
	 * Handle the select goal button click.
	 * 
	 * @return null because navigation is not needed
	 */
	public String doSelectGoal() {
		if (selectedEntry != null && !selectedEntry.equals(activeEntry)) {
			try {
				if (activeEntry != null) {
					activeEntry.deactivate();
				}
				activeEntry = selectedEntry;
				ServiceContext ctx = PortletUtils.getServiceContext();
				Goal goal = activeEntry.activate(ctx);
				goalManager.selectGoal(goal, PortletUtils.getServiceContext().getUserId(), PortletUtils.getGroupId());
				Collections.sort(profileList);
				isCollapsed = true;
			} catch (Exception e1) {
				PortletUtils.handleError(LOG, e1);
			}
		}

		return null;
	}

	/**
	 * Handle the collapse hyperlink by toggling between the collapse modes.
	 * 
	 * @return null because navigation is not needed
	 */
	public String doCollapse() {
		isCollapsed = !isCollapsed;

		return null;
	}

	/**
	 * Handle the selection event from the profile level list.
	 * <p />
	 * Sets the target of the goal to the selected level and saves the goal.
	 * 
	 * @param event
	 *            the selection event
	 */
	public void doLevelSelected(final ValueChangeEvent event) {
		activeEntry.setSelectedLevel(Long.valueOf((String) event.getNewValue()));
		try {
			activeEntry.save(PortletUtils.getServiceContext());
			goalManager.selectGoal(activeEntry.getGoal(), PortletUtils.getServiceContext().getUserId(), PortletUtils
					.getGroupId());

		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Saves the motivation of the goal.
	 * 
	 * @return null because navigation is not needed
	 */
	public String doSave() {
		if (activeEntry != null) {
			try {
				activeEntry.save(PortletUtils.getServiceContext());
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}
		return null;
	}

	/**
	 * Selection listener called when the selection in the profile level list changes.
	 * 
	 * @param event
	 *            the event describing the selection
	 */
	public void selectionListener(final RowSelectorEvent event) {
		initFields();
		handleSelection(profileList.get(event.getRow()));
	}

	/** {@inheritDoc} */
	//@Override
	public void renderingException(final RenderingException e) {
		// An FatalRenderingException will occur if a user has closed his browser
		// the update functionality tries to render a view that doesn't exist anymore so only for debugging there is
		// some logging.
		if (LOG.isDebugEnabled()) {
			LOG.debug(e);
		}
	}

	/**
	 * Logout.
	 * 
	 * @param event
	 *            the event
	 */
	public void removeListener(final ActionEvent event) {
		goalManager.removeListener(this);
		selectedEntry = null;
	}

	/**
	 * Dispose callback called due to a view closing or session invalidation/timeout.
	 * 
	 * clean closed views.
	 * 
	 * @throws Exception
	 *             the exception
	 */
	public void dispose() throws Exception {
		if (LOG.isDebugEnabled()) {
			LOG.debug("Dispose Participant - cleaning up");
		}
		performCleanup();
	}

	private void initFields() {

		if (needsInit) {

			createNavDataHolder();

			profileEntryController = new ProfileEntryController(null);

			try {
				fillList();
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}

			isCollapsed = activeEntry != null;

			needsInit = false;
		}
	}

	private boolean performCleanup() {
		try {
			if (goalManager.hasListener(this)) {
				removeListener(null);
			}
			return true;
		} catch (Exception failedCleanup) {
			if (LOG.isErrorEnabled()) {
				LOG.error("Failed to cleanup a Participant", failedCleanup);
			}
		}
		return false;
	}

	private void createNavDataHolder() {
		final NavigationDataHolder nav = NavigationDataHolder.getInstance();
		if (nav != null) {
			nav.setNormalWindowStateURL(normalUrl);
			nav.setMaximizedWindowStateURL(maximizedUrl);
			nav.clear();
		}
	}

	private void fillList() throws SystemException, PortalException {
		Goal selectedGoal = goalManager.getSelectedGoal(PortletUtils.getCurrentUserId(), PortletUtils.getGroupId());
		profileList = new ArrayList<ProfileEntry>();

		final ServiceContext ctx = PortletUtils.getServiceContext();
		List<Goal> goals = getGoals();
		List<CompetenceProfile> profiles = CompetenceProfileFacade.getCompetenceProfilesByCompetenceMapId(PortletUtils
				.getCompetenceMapByCurrentCommunity().getPrimaryKey(), ctx);

		for (CompetenceProfile profile : profiles) {
			ProfileEntry entry = new ProfileEntry(profile, ctx);
			if (entry.hasLevels()) {
				if (entry.associateGoal(goals, selectedGoal)) {
					activeEntry = entry;
				}
				profileList.add(entry);
			}
		}

		Collections.sort(profileList);

		// select the first path, when available
		if (!profileList.isEmpty()) {
			if (activeEntry != null) {
				handleSelection(activeEntry);
			} else {
				handleSelection(profileList.get(0));

			}

		}
	}

	private void handleSelection(final ProfileEntry selectedEntry) {
		this.selectedEntry = selectedEntry;
		// reset selected state of list items to resemble normal selection list
		for (ProfileEntry entry : profileList) {
			entry.setSelected(entry.equals(selectedEntry));
		}

		// inform the element viewer
		profileEntryController.setInput(selectedEntry);
	}

	private List<Goal> getGoals() {
		List<Goal> goals = null;
		PortletPreferences prefs = PortletUtils.getPortletRequest().getPreferences();
		try {
			ServiceContext ctx = PortletUtils.getServiceContext();
			if (Boolean.valueOf(prefs.getValue(GoalPreferences.SHOW_ONLY_COMMUNITY_GOALS, "false"))) {
				goals = GoalFacade.getGoalsForUserAndCurrentMap(ctx.getUserId(), PortletUtils
						.getCompetenceMapByCurrentCommunity().getPrimaryKey(), ctx);
			} else {
				goals = GoalFacade.getGoalsForUser(ctx.getUserId(), ctx);
			}
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		return goals;
	}
}
