/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.goal.beans;

import org.tencompetence.business.services.SubscribeService;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * Generic Goal utility class.
 * 
 * @author Anton Finders
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:10:06 $
 */
public final class GoalUtil {

	private static final Log LOG = LogFactoryUtil.getLog(GoalUtil.class);

	private GoalUtil() {
		// No public constructor
	}

	/**
	 * Check valid target type.
	 * 
	 * @param selectedGoal
	 *            the selected goal
	 * 
	 * @return true, if successful
	 */
	public static boolean checkValidTargetType(final Goal selectedGoal) {
		if (selectedGoal == null) {
			return false;
		}
		if (selectedGoal.getTargetType().equalsIgnoreCase(CompetenceProfileLevel.class.getName())) {
			return true;
		}
		if (selectedGoal.getTargetType().equalsIgnoreCase(CompetenceLevel.class.getName())) {
			return true;
		}
		return false;
	}

	/**
	 * isSubscribedToSelectedGoal.
	 * 
	 * @param selectedGoal
	 *            the selected goal
	 * 
	 * @return boolean isSubscribedToSelectedGoal
	 */
	public static boolean isSubscribedToSelectedGoal(final Goal selectedGoal) {
		if (selectedGoal == null) {
			return false;
		}
		try {
			return SubscribeService.isSubscribedToEntity(PortletUtils.getCurrentUserId(), selectedGoal, PortletUtils
					.getServiceContext());
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return false;
	}

	/**
	 * doSubscribeToGoal.
	 * 
	 * @param selectedGoal
	 *            the selected goal
	 * 
	 * @return String null
	 */
	public static String doSubscribeToGoal(final Goal selectedGoal) {
		try {
			SubscribeService.subscribeToEntity(PortletUtils.getCurrentUserId(), selectedGoal, null, PortletUtils
					.getServiceContext());
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}
}