/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.goal.controllers;

import com.liferay.portal.kernel.util.StringPool;

/**
 * This abstract class implements the controller for the lpElement facelet.
 * <p />
 * It provides the facelet with the data from the lpElement passed to the controller. It optionally shows a action
 * button. Default this button is shown when:
 * <ul>
 * <li>true is passed for the hasButton parameter in the class constructor</li>
 * <li>the element shows is either a {@code ILearningAction} or a {@code ILearningPath}</li>
 * <li>the element is not completed</li>
 * </ul>
 * 
 * When the user clicks the button the abstract handleClickEvent method is called. Concrete implementations of this
 * controller need to subclass and implement this method to handle the event.
 * <p />
 * 
 * @author Harrie Martens
 * @version $Revision: 1.2 $, $Date: 2009/11/11 16:09:28 $
 */

public class ProfileEntryController {
	private ProfileEntry element;
	private String description;



	/**
	 * Instantiates a new element controller.
	 * 
	 * @param entry
	 *            the ProfileEntry for this control.

	 */
	public ProfileEntryController(final ProfileEntry entry) {
		setInput(entry);
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return this.element != null ? this.element.getTitle() : StringPool.BLANK;
	}

	/**
	 * Gets the description.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		return this.description;
	}

	/**
	 * Gets the element associated with this controller.
	 * 
	 * @return the element
	 */
	public ProfileEntry getElement() {
		return this.element;
	}

	/**
	 * Sets the ILPElement input for this controller.
	 * <p>
	 * This input should not be null, otherwise an IllegalArgumentException is thrown
	 * 
	 * @param input
	 *            the new input for the controller
	 */
	public void setInput(final ProfileEntry input) {
		this.element = input;
		this.description = element != null ? (element.getDescription() == null ? StringPool.BLANK : element
				.getDescription()) : StringPool.BLANK;
	}
}