/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.PermissionsList;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;
import org.tencompetence.portlet.model.CompetenceTypeEnum;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceEdit.
 * 
 * @author Anton Finders
 * @version $Revision: 1.35 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceEdit extends BaseEditorBean<Competence> {

	/** The type. */
	private CompetenceTypeEnum type;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceEdit.class);
	private PermissionsList<Competence> pList = null;
	private ActionsMenuListController<ActionsMenuRowItem<CompetenceLevel>> controller;
	private long selectedTestId;

	/**
	 * Gets the controller.
	 * 
	 * @return the controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<CompetenceLevel>> getController() {
		checkEditorInput();
		return controller;
	}

	private List<ActionsMenuRowItem<CompetenceLevel>> loadLevelList() {
		final List<CompetenceLevel> levels;
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			levels = CompetenceLevelFacade.getRequirableCompetenceLevelsByCompetenceId(getEditorInput().getCompetenceId(),
					ctx);
			final List<ActionsMenuRowItem<CompetenceLevel>> result = new ArrayList<ActionsMenuRowItem<CompetenceLevel>>(
					levels.size());
			for (CompetenceLevel level : levels) {
				result.add(new ActionsMenuRowItem<CompetenceLevel>(level, level.getPrimaryKey()));
			}
			return result;
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * Gets the type.
	 * 
	 * @return the type
	 */
	public CompetenceTypeEnum getType() {
		checkEditorInput();
		if (type == null) {
			type = CompetenceTypeEnum.SELECT_ONE;
		}
		return type;
	}

	/**
	 * Sets the type.
	 * 
	 * @param type
	 *            the new type
	 */
	public void setType(final CompetenceTypeEnum type) {
		if (!isDirty()) {
			setDirty(!this.type.toString().equals(type.toString()));
		}
		this.type = type;
	}

	/**
	 * Gets the types.
	 * 
	 * @return the types
	 */
	public List<SelectItem> getTypes() {
		final List<SelectItem> items = new ArrayList<SelectItem>();
		for (final CompetenceTypeEnum value : CompetenceTypeEnum.values()) {
			items.add(new SelectItem(value, value.toString()));
		}
		return items;
	}

	/**
	 * Validate type.
	 * 
	 * @param context
	 *            the context
	 * @param component
	 *            the component
	 * @param value
	 *            the value
	 */
	public void validateType(final FacesContext context, final UIComponent component, final Object value) {
		final UIInput input = ((UIInput) component);
		if (input.isRequired() && CompetenceTypeEnum.SELECT_ONE.equals(value)) {
			PortletUtils.addErrorMessageToContext(":form:typeListbox", "competenceedit.error.notypeset");
			input.setValid(false);
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws PortalException
	 */
	@Override
	protected void doSave(final Competence editorInput){
		getEditorInput().setTitle(getTitleForInternalUse());
		getEditorInput().setDescription(getDescriptionForInternalUse());
		getEditorInput().setType(type.name());
		editorInput.setTestId(selectedTestId);
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (getEditorInput().isNew()) {
			ctx.setCommunityPermissions(pList.getCommunityPermissions());
			ctx.setGuestPermissions(pList.getGuestPermissions());
			Competence competence = getEditorInput();
			try {
				PortletUtils.setOwnership(competence, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				CompetenceFacade.addCompetence(competence, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			getEditorInput().setNew(false);

			// refresh the newly created levels
			controller.setInput(loadLevelList());
		} else {
			// update
			try {
				CompetenceFacade.updateCompetence(getEditorInput(),  ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final Competence editorInput) {
		controller = new ActionsMenuListController<ActionsMenuRowItem<CompetenceLevel>>();
		controller.setConfirmationDialogController(getConfirmationController());

		if (getEditorInput() != null) {
			setTitle(getEditorInput().getTitle());
			setDescriptionForInternalUse(getEditorInput().getDescription());
			controller.setInput(loadLevelList());
			if (getEditorInput().getType() == null || getEditorInput().getType().isEmpty()) {
				type = CompetenceTypeEnum.SELECT_ONE;
			} else {
				type = CompetenceTypeEnum.valueOf(getEditorInput().getType());
			}
			selectedTestId = editorInput.getTestId();
		}
		
		pList = new PermissionsList<Competence>(null, getEditorInput());
	}

	/**
	 * Gets a list of permissions for the actual Competanceprofile instance.
	 * 
	 * @return pList
	 */
	public PermissionsList<Competence> getPermissionsList() {
		return pList;
	}

	/**
	 * Check if user is allowed to add a Competence.
	 * 
	 * @return true / false
	 */
	public boolean isAddCompetenceAllowed() {
		return checkPortletPermission(TENCActionKey.ADD_COMPETENCE);
	}
	
	
	/**
	 * Gets the all tests.
	 * 
	 * @return the all tests
	 */
	public List<SelectItem> getAllTests() {
		List<SelectItem> result = new ArrayList<SelectItem>();
		try {
			final List<AssessmentTest> tests = AssessmentTestFacade
					.getAssessmentTests(PortletUtils.getServiceContext());
			for (AssessmentTest test : tests) {
				result.add(new SelectItem(Long.toString(test.getAssessmentTestId()), test.getTitle()));
			}
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return result;
	}
	
	
	/**
	 * Gets the selected test.
	 * 
	 * @return the selected test
	 */
	public String getSelectedTest() {
		return Long.toString(selectedTestId);
	}
	
	/**
	 * Sets the selected test.
	 * 
	 * @param id
	 *            the new selected test
	 */
	public void setSelectedTest(final String id) {
		selectedTestId = Long.valueOf(id);
	}
	

}