/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.modeleditor.common.CompetenceRow;
import org.tencompetence.portlet.modeleditor.common.ListController.SelectionType;
import org.tencompetence.portlet.modeleditor.controller.ChildLevelController;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceLevelEdit.
 * 
 * @author Anton Finders
 * 
 * @version $Revision: 1.47 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceLevelEdit extends BaseEditorBean<CompetenceLevel> {
	
	/** The Constant LOG. */
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceLevelEdit.class);
	
	/** The selected level value id. */
	private String selectedLevelValueId;
	
	/** The child levels controller. */
	private ChildLevelController childLevelsController;
	
	/** The meta levels controller. */
	private ChildLevelController metaLevelsController;
	
	/** The popup visible. */
	private boolean popupVisible = false;
	
	/** The level values. */
	private final List<SelectItem> levelValues = new ArrayList<SelectItem>();
	
	/** The new version tag. */
	private String newVersionTag;

	/**
	 * gets the newVersionTag to set in the version popup.
	 * 
	 * @return the newVersionTag
	 */
	public String getNewVersionTag() {
		return newVersionTag;
	}

	/**
	 * sets the newVersionTag from the version popup.
	 * 
	 * @param newVersionTag the newVersionTag
	 */
	public void setNewVersionTag(final String newVersionTag) {
		this.newVersionTag = newVersionTag;
	}

	/**
	 * Gets the child levels controller.
	 * 
	 * @return the child levels controller
	 */
	public ChildLevelController getChildLevelsController() {
		checkEditorInput();
		return this.childLevelsController;
	}

	/**
	 * Gets the meta levels controller.
	 * 
	 * @return the meta levels controller
	 */
	public ChildLevelController getMetaLevelsController() {
		checkEditorInput();
		return this.metaLevelsController;
	}

	/**
	 * Gets the level values.
	 * 
	 * @return the level values
	 */
	public List<SelectItem> getLevelValues() {
		return levelValues;
	}

	/**
	 * Gets the selected value.
	 * 
	 * @return the selected value
	 */
	public String getSelectedValue() {
		return selectedLevelValueId;
	}

	/**
	 * Sets the selected value.
	 * 
	 * @param selectedValue the new selected value
	 */
	public void setSelectedValue(final String selectedValue) {
		if (this.selectedLevelValueId == null || !this.selectedLevelValueId.equals(selectedValue)) {
			this.setDirty(true);
			selectedLevelValueId = selectedValue;
		}
	}

	/**
	 * Gets the versionTag.
	 * 
	 * @return the versionTag
	 */
	public String getVersionTag() {
		checkEditorInput();
		return getEditorInput().getVersionTag();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws PortalException
	 */
	@Override
	protected void doSave(final CompetenceLevel editorInput) {
		// fill the currentCompetenceLevel with the values entered in the form
		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setDescription(getDescriptionForInternalUse());
		editorInput.setLevelValueId(Long.valueOf(selectedLevelValueId));
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// level cannot be created manually so only update is possible
		try {
			setEditorInput(CompetenceLevelFacade.updateCompetenceLevel(editorInput, ctx));
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Go to the next page.
	 * 
	 * @return outcome
	 */
	public String changeMetaLevels() {
		final String result = this.getConfirmationController().canLeave(Outcomes.ADD, getEditorInput());
		if (result != null) {
			PortletUtils.getSessionMap().put(CompetenceSubCompetenceLevel.CHILD_OR_META_MODE,
					CompetenceSubCompetenceLevel.SubCompetenceMode.META);
		}
		return result;
	}

	/**
	 * Go to the next page.
	 * 
	 * @return outcome
	 */
	public String changeChildLevels() {
		final String result = this.getConfirmationController().canLeave(Outcomes.ADD, getEditorInput());
		if (result != null) {
			PortletUtils.getSessionMap().put(CompetenceSubCompetenceLevel.CHILD_OR_META_MODE,
					CompetenceSubCompetenceLevel.SubCompetenceMode.CHILD);
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final CompetenceLevel editorInput) {
		childLevelsController = new ChildLevelController("competenceleveleditor.childlevellist",
				SelectionType.MULTIPLE_ENHANCED);
		metaLevelsController = new ChildLevelController("competenceleveleditor.metalevellist",
				SelectionType.MULTIPLE_ENHANCED);
		// fill the form
		setDescriptionForInternalUse(editorInput.getDescription());
		setTitle(editorInput.getTitle());

		// prefill other entities for later use
		loadModelEntities();
	}

	/**
	 * Load model entities.
	 */
	private void loadModelEntities() {
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);

		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		try {
			if (levelValues.isEmpty()) {

				CompetenceMap map;
				map = CompetenceMapFacade.getCompetenceMapByGroupId(PortletUtils.getGroupId(), ctx);
				// get all level values that are possible to assign to a profile
				// level
				final List<LevelValue> list = LevelValueFacade.getCompetenceProfileLevelValues(
						map.getCompetenceMapId(), ctx);

				// selectitems are needed for showing in frontend
				for (LevelValue value : list) {
					levelValues.add(new SelectItem(Long.toString(value.getLevelValueId()), value.getTitle()));
				}
			}

			// if id not == 0 set the selected item
			if (this.getEditorInput() == null || this.getEditorInput().getLevelValueId() == 0) {
				// set the default level
				selectedLevelValueId = (String) levelValues.get(0).getValue();
			} else {
				selectedLevelValueId = Long.toString(this.getEditorInput().getLevelValueId());
			}

			// load the competenceLevels attached to the profile
			loadCompetenceLevels();

		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Reload competence levels.
	 */
	private void loadCompetenceLevels() {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			final List<CompetenceLevel> childLevels = CompetenceLevelFacade.getCompetenceLevelsByIds(this
					.getEditorInput().getChildLevelIds(), ctx);
			List<CompetenceRow> model = wrapLevels(childLevels);
			this.childLevelsController.setInput(model);
			final List<CompetenceLevel> metaLevels = CompetenceLevelFacade.getCompetenceLevelsByIds(this
					.getEditorInput().getMetaLevelIds(), ctx);
			model = wrapLevels(metaLevels);
			this.metaLevelsController.setInput(model);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Wrap levels.
	 * 
	 * @param competenceLevels the competence levels
	 * 
	 * @return the list< competence row>
	 */
	private List<CompetenceRow> wrapLevels(final List<CompetenceLevel> competenceLevels) {
		// add all Competences to the dataModel
		final List<CompetenceRow> model = new ArrayList<CompetenceRow>();
		for (CompetenceLevel level : competenceLevels) {
			model.add(new CompetenceRow(level));
		}
		return model;
	}

	/**
	 * Action to create a new version from this competenceProfileLevel.
	 * 
	 * @return The outcome
	 */
	public String createNewVersion() {
		popupVisible = true;
		newVersionTag = null;
		return null;
	}

	/**
	 * Cancel version.
	 * 
	 * @return the string
	 */
	public String cancelVersion() {
		popupVisible = false;
		return null;
	}

	/**
	 * Checks if is popup visible.
	 * 
	 * @return true, if is popup visible
	 */
	public boolean isPopupVisible() {
		return popupVisible;
	}


	/**
	 * Ok version.
	 * 
	 * @return the string
	 */
	public String okVersion() {
		try {
			final CompetenceLevel currentLevel = getEditorInput();
			final ServiceContext ctx = PortletUtils.getServiceContext();
			// save the tag
			currentLevel.setVersionTag(newVersionTag);
			currentLevel.setTitle(getTitleForInternalUse());
			currentLevel.setDescription(getDescriptionForInternalUse());

			final CompetenceLevel newLevel = CompetenceLevelFacade.addCompetenceLevelNewVersion(currentLevel, ctx);
			setEditorInput(newLevel);
			initFields(newLevel);
			popupVisible = false;
		} catch (PortalException pe) {
			PortletUtils.handleError(LOG, pe);
		} catch (SystemException se) {
			PortletUtils.handleError(LOG, se);
		}
		return null;
	}
}