/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This class is the managed bean for the competence.jspx. Which shows a list of Competences. It also gives the ability
 * to add a new Competence or edit a selected Competence object.
 * @version $Revision: 1.22 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceList extends BaseEditorBean<CompetenceMap> {
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceList.class);
	private ActionsMenuListController<ActionsMenuRowItem<Competence>> controller;

	/**
	 * Gets the controller.
	 * 
	 * @return the controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<Competence>> getController() {
		checkEditorInput();
		return controller;
	}

	/**
	 * Action handler for Adds the competence.
	 * 
	 * @return the string
	 */
	public String addCompetence() {

		try {
			final Competence competence = CompetenceFacade.createCompetence();
			competence.setCompetenceMapId(getEditorInput().getCompetenceMapId());

			return this.getConfirmationController().canLeave(Outcomes.ADD, competence);

		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return null;
	}

	/**
	 * Check if user is allowed to add a Competence.
	 * 
	 * @return true / false
	 */
	public boolean isAddCompetenceAllowed() {
		return checkPortletPermission(TENCActionKey.ADD_COMPETENCE);
	}

	/**
	 * Action handler for Delete competence.
	 * 
	 * @return the string
	 */
	public String deleteCompetence() {
		PortletUtils.addErrorMessageToContext("general.msg.deletenotimplemented", new Object[] {});
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void doSave(final CompetenceMap editorInput) {
		// Nothing to do
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final CompetenceMap editorInput) {
		controller = new ActionsMenuListController<ActionsMenuRowItem<Competence>>();
		controller.setInput(refreshList(editorInput));
		controller.setConfirmationDialogController(getConfirmationController());
	}

	/**
	 * Refresh competence list.
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	private List<ActionsMenuRowItem<Competence>> refreshList(final CompetenceMap map) {
		try {
			ServiceContext ctx = PortletUtils.getServiceContext();
			final List<Competence> competenceList = CompetenceFacade.getCompetencesByCompetenceMapId(map
					.getCompetenceMapId(), ctx);
			final List<ActionsMenuRowItem<Competence>> result = new ArrayList<ActionsMenuRowItem<Competence>>(
					competenceList.size());
			for (Competence c : competenceList) {
				result.add(new ActionsMenuRowItem<Competence>(c, c.getPrimaryKey()));
			}
			return result;
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

}
