/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.beans;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;

import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.NavigationDataHolder;
import org.tencompetence.portlet.common.components.PermissionsList;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This class is the managed bean for the mapview.jspx. It implements the possibility to add/edit a competencemap. Shows
 * also the list of the related level values.
 * 
 * @author lsc
 * @version $Revision: 1.34 $, $Date: 2009/11/24 14:10:06 $
 */

public class CompetenceMapEdit extends BaseEditorBean<CompetenceMap> {
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceMapEdit.class);
	/** The number profile levels. */
	private long numberProfileLevels;

	/** The number competence levels. */
	private long numberCompetenceLevels;

	private long orgNumberProfileLevels;
	private long orgNumberCompetenceLevels;

	private ActionsMenuListController<ActionsMenuRowItem<LevelValue>> competenceController;
	private ActionsMenuListController<ActionsMenuRowItem<LevelValue>> profileController;

	/**
	 * The MAX_NUMBER_OF_LEVELS a user can create as related to the competence map.
	 */
	private static final long MAX_NUMBER_OF_LEVELS = 20;

	/**
	 * The MAX_NUMBER_OF_LEVELS a user can create as related to the competence map.
	 */
	private static final long MIN_NUMBER_OF_LEVELS = 1;

	/**
	 * List of permissions.
	 */
	private PermissionsList<CompetenceMap> pList = null;
	private ServiceContext ctx = null;

	/**
	 * Instantiates a new competence map view.
	 */
	public CompetenceMapEdit() {
		super();
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Validate range.
	 * 
	 * @param context
	 *            the context
	 * @param validate
	 *            the validate
	 * @param value
	 *            the value
	 */
	public void validateRange(final FacesContext context, final UIComponent validate, final Object value) {
		final long valueLong = (Long) value;
		final UIInput input = (UIInput) validate;

		if (input.isRequired() && (valueLong < MIN_NUMBER_OF_LEVELS || valueLong > MAX_NUMBER_OF_LEVELS)) {
			final FacesMessage message = new FacesMessage();
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
			message.setSummary(MessageFormat.format(Bundle.getLabel("error.notinrange"), MIN_NUMBER_OF_LEVELS,
					MAX_NUMBER_OF_LEVELS));

			context.addMessage(validate.getClientId(context), message);

			input.setValid(false);
		}
	}

	/**
	 * Gets the org number profile levels.
	 * 
	 * @return the org number profile levels
	 */
	public long getOrgNumberProfileLevels() {
		checkEditorInput();
		return orgNumberProfileLevels;
	}

	/**
	 * Sets the org number profile levels.
	 * 
	 * @param orgNumberProfileLevels
	 *            the new org number profile levels
	 */
	public void setOrgNumberProfileLevels(final long orgNumberProfileLevels) {
		this.orgNumberProfileLevels = orgNumberProfileLevels;
	}

	/**
	 * Gets the org number competence levels.
	 * 
	 * @return the org number competence levels
	 */
	public long getOrgNumberCompetenceLevels() {
		checkEditorInput();
		return orgNumberCompetenceLevels;
	}

	/**
	 * Sets the org number competence levels.
	 * 
	 * @param orgNumberCompetenceLevels
	 *            the new org number competence levels
	 */
	public void setOrgNumberCompetenceLevels(final long orgNumberCompetenceLevels) {
		this.orgNumberCompetenceLevels = orgNumberCompetenceLevels;
	}

	/**
	 * Gets the number profile levels.
	 * 
	 * @return the number profile levels
	 */
	public long getNumberProfileLevels() {
		checkEditorInput();
		return numberProfileLevels;
	}

	/**
	 * Sets the number profile levels.
	 * 
	 * @param numberProfileLevels
	 *            the new number profile levels
	 */
	public void setNumberProfileLevels(final long numberProfileLevels) {
		this.numberProfileLevels = numberProfileLevels;
	}

	/**
	 * Gets the number competence levels.
	 * 
	 * @return the number competence levels
	 */
	public long getNumberCompetenceLevels() {
		checkEditorInput();
		return numberCompetenceLevels;
	}

	/**
	 * Sets the number competence levels.
	 * 
	 * @param numberCompetenceLevels
	 *            the new number competence levels
	 */
	public void setNumberCompetenceLevels(final long numberCompetenceLevels) {
		this.numberCompetenceLevels = numberCompetenceLevels;
	}


	/**
	 * {@inheritDoc}
	 * 
	 * @throws PortalException
	 */
	@Override
	protected void doSave(final CompetenceMap editorInput) {
		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setDescription(getDescriptionForInternalUse());
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// New CompetenceMap
		if (orgNumberCompetenceLevels == 0 && orgNumberProfileLevels == 0) {
			ctx.setCommunityPermissions(pList.getCommunityPermissions());
			ctx.setGuestPermissions(pList.getGuestPermissions());

			CompetenceMap competenceMap = editorInput;
			try {
				PortletUtils.setOwnership(competenceMap, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				CompetenceMapFacade.addCompetenceMap(competenceMap, (int) getNumberCompetenceLevels(),
						(int) getNumberProfileLevels(), ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			editorInput.setNew(false);
		}

		orgNumberCompetenceLevels = getNumberCompetenceLevels();
		orgNumberProfileLevels = getNumberProfileLevels();

		try {
			CompetenceMapFacade.updateCompetenceMap(editorInput, ctx);
			CompetenceMapFacade.updateCompetenceLevelValues(editorInput, (int) getNumberCompetenceLevels(), ctx);
			CompetenceMapFacade.updateCompetenceProfileLevelValues(editorInput, (int) getNumberProfileLevels(), ctx);
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// reload the list
		refreshLevels();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final CompetenceMap editorInput) {
		try {
			final long mapId = editorInput.getPrimaryKey();
			orgNumberProfileLevels = LevelValueFacade.getCompetenceProfileLevelValuesCount(mapId, ctx);
			orgNumberCompetenceLevels = LevelValueFacade.getRequiredCompetenceLevelValuesCount(mapId, ctx);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		setNumberCompetenceLevels(orgNumberCompetenceLevels);
		setNumberProfileLevels(orgNumberProfileLevels);
		setDescriptionForInternalUse(editorInput.getDescription());
		setTitle(editorInput.getTitle());

		// Create list controller for the competence level values list
		competenceController = new ActionsMenuListController<ActionsMenuRowItem<LevelValue>>();

		// Create list controller for the profile level values list
		profileController = new ActionsMenuListController<ActionsMenuRowItem<LevelValue>>();

		// Set ConfirmationDialogController
		competenceController.setConfirmationDialogController(getConfirmationController());
		profileController.setConfirmationDialogController(getConfirmationController());

		refreshLevels();
		
		pList = new PermissionsList<CompetenceMap>(null, getEditorInput());
	}

	private void refreshLevels() {
		// fill list controller with level values
		profileController.setInput(getProfileValues());
		// fill list controller with level values
		competenceController.setInput(getCompetenceValues());
	}

	private List<ActionsMenuRowItem<LevelValue>> getCompetenceValues() {
		if (getEditorInput() != null) {
			try {
				return createValueItems(LevelValueFacade.getAttainableCompetenceLevelValues(getEditorInput()
						.getCompetenceMapId(), ctx));
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			}
		}

		return new ArrayList<ActionsMenuRowItem<LevelValue>>(0);
	}

	private List<ActionsMenuRowItem<LevelValue>> getProfileValues() {
		if (getEditorInput() != null) {
			try {
				return createValueItems(LevelValueFacade.getCompetenceProfileLevelValues(getEditorInput()
						.getCompetenceMapId(), ctx));
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			}
		}

		return new ArrayList<ActionsMenuRowItem<LevelValue>>(0);
	}

	private List<ActionsMenuRowItem<LevelValue>> createValueItems(final List<LevelValue> levels) {
		final List<ActionsMenuRowItem<LevelValue>> result = new ArrayList<ActionsMenuRowItem<LevelValue>>(levels.size());
		for (LevelValue value : levels) {
			result.add(new ActionsMenuRowItem<LevelValue>(value, value.getPrimaryKey()));
		}

		return result;
	}

	/**
	 * Gets the competence controller.
	 * 
	 * @return the competence controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<LevelValue>> getCompetenceController() {
		checkEditorInput();
		return this.competenceController;
	}

	/**
	 * Gets the profile controller.
	 * 
	 * @return the profile controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<LevelValue>> getProfileController() {
		checkEditorInput();
		return this.profileController;
	}

	/**
	 * Gets the URL which tells Liferay to show this portlet in normal view.
	 * 
	 * @return The URL
	 */
	public String getWindowStateNormalUrl() {
		return NavigationDataHolder.getInstance().getNormalWindowStateURL();
	}

	/**
	 * Gets a list of permissions for the actual Map instance.
	 * 
	 * @return pList
	 */
	public PermissionsList<CompetenceMap> getPermissionsList() {
		return pList;
	}
}