/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.PermissionsList;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class EditCompetenceProfile.
 * 
 * @author Anton Finders
 * @version $Revision: 1.32 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceProfileEdit extends BaseEditorBean<CompetenceProfile> {
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceProfileEdit.class);
	private ActionsMenuListController<ActionsMenuRowItem<CompetenceProfileLevel>> controller;
	private PermissionsList<CompetenceProfile> pList = null;

	/**
	 * Constructor.
	 */
	public CompetenceProfileEdit() {
		super();
	}
	
	/**
	 * Gets the controller for the Competence Profile level list.
	 * 
	 * @return the controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<CompetenceProfileLevel>> getController() {
		checkEditorInput();
		return controller;
	}

	/** {@inheritDoc} */
	@Override
	protected void doSave(final CompetenceProfile editorInput) {
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (editorInput.isNew()) {
			editorInput.setTitle(getTitleForInternalUse());
			editorInput.setDescription(getDescriptionForInternalUse());
			CompetenceMap map = null;
			try {
				map = CompetenceMapFacade
						.getCompetenceMapByGroupId(PortletUtils.getGroupId(), ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			editorInput.setCompetenceMapId(map.getCompetenceMapId());
			editorInput.setNew(false);
			
			ctx.setCommunityPermissions(pList.getCommunityPermissions());
			ctx.setGuestPermissions(pList.getGuestPermissions());

			CompetenceProfile competenceProfile = editorInput;
			try {
				PortletUtils.setOwnership(competenceProfile, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			try {
				CompetenceProfileFacade.addCompetenceProfile(competenceProfile, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			editorInput.setTitle(getTitleForInternalUse());
			editorInput.setDescription(getDescriptionForInternalUse());
			try {
				CompetenceProfileFacade.updateCompetenceProfile(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final CompetenceProfile editorInput) {
		setTitle(editorInput.getTitle());
		setDescriptionForInternalUse(editorInput.getDescription());
		controller = new ActionsMenuListController<ActionsMenuRowItem<CompetenceProfileLevel>>();
		controller.setInput(refreshList(editorInput));
		controller.setConfirmationDialogController(getConfirmationController());
		
		pList = new PermissionsList<CompetenceProfile>(null, getEditorInput());
	}

	private List<ActionsMenuRowItem<CompetenceProfileLevel>> refreshList(final CompetenceProfile profile) {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			final List<CompetenceProfileLevel> levels = CompetenceProfileLevelFacade
					.getCompetenceProfileLevelsByCompetenceProfileId(profile.getCompetenceProfileId(), ctx);
			final List<ActionsMenuRowItem<CompetenceProfileLevel>> result =
				new ArrayList<ActionsMenuRowItem<CompetenceProfileLevel>>(levels.size());
			for (CompetenceProfileLevel level : levels) {
				result.add(new ActionsMenuRowItem<CompetenceProfileLevel>(level, level.getPrimaryKey()));
			}
			return result;
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * method called when add button below competence level is pressed.
	 * 
	 * A new CompetenceProfileLevel is created and passed to the next page
	 * 
	 * @return the string
	 */
	public String addProfileLevel() {
		// create empty one
		CompetenceProfileLevel level = null;
		try {
			level = CompetenceProfileLevelFacade.createCompetenceProfileLevel();
			level.setNew(true);
			level.setCompetenceProfileId(getEditorInput().getCompetenceProfileId());
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return this.getConfirmationController().canLeave(Outcomes.ADD, level);
	}
	
	/**
	 * Gets a list of permissions for the actual CompetenceProfile instance.
	 * @return pList
	 */
	public PermissionsList<CompetenceProfile> getPermissionsList() {
		return pList;
	}
	
	/**
	 * Check if user is allowed to add a Competenceprofilelevel.
	 * 
	 * @return true / false
	 */
	public boolean isAddCompetenceProfileLevelAllowed() {
		return checkPortletPermission(TENCActionKey.ADD_COMPETENCEPROFILE_LEVELS);
	}	
}
