/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.FacadeUtils;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.PermissionsList;
import org.tencompetence.portlet.common.components.RatingComponentController;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.modeleditor.common.CompetenceRow;
import org.tencompetence.portlet.modeleditor.common.ModelEditorRatingComponentController;
import org.tencompetence.portlet.modeleditor.common.ListController.SelectionType;
import org.tencompetence.portlet.modeleditor.controller.ChildLevelController;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceProfileLevelEdit.
 * @version $Revision: 1.49 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceProfileLevelEdit extends BaseEditorBean<CompetenceProfileLevel> {
	private ChildLevelController levelController;
	private ModelEditorRatingComponentController ratingController;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceProfileLevelEdit.class);
	/** The selected level value id. */
	private LevelValue selectedLevel;
	private boolean popupVisible = false;
	private PermissionsList<CompetenceProfileLevel> pList = null;
	private String newVersionTag = null;
	private String versionTag = null;

	/**
	 * gets the newVersionTag to set in the version popup.
	 * 
	 * @return the newVersionTag
	 */
	public String getNewVersionTag() {
		return newVersionTag;
	}

	/**
	 * sets the newVersionTag from the version popup.
	 * 
	 * @param newVersionTag
	 *            the newVersionTag
	 */
	public void setNewVersionTag(final String newVersionTag) {
		this.newVersionTag = newVersionTag;
	}

	/** All level values of the map. */
	private List<LevelValue> levelValues = new ArrayList<LevelValue>();

	/**
	 * Gets the level controller.
	 * 
	 * @return the level controller
	 */
	public ChildLevelController getLevelController() {
		checkEditorInput();
		return this.levelController;
	}

	/**
	 * Save competence levels by profile.
	 * 
	 * @param newCompetenceLevels
	 *            the new competence levels
	 */
	private void saveCompetenceLevelsByProfile(final List<Long> newCompetenceLevels) {
		try {
			if (this.getEditorInput() instanceof CompetenceProfileLevel) {
				CompetenceProfileLevel input = getEditorInput();
				input.setCompetenceLevelIds(FacadeUtils.encodeEntries(newCompetenceLevels));
			CompetenceProfileLevelFacade.updateCompetenceProfileLevel(input, PortletUtils.getServiceContext());
			}
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Gets the version.
	 * 
	 * @return the version
	 */
	public String getVersionTag() {
		checkEditorInput();
		return versionTag;
	}

	/**
	 * {@inheritDoc}
	 * 
	 */
	@Override
	protected void doSave(final CompetenceProfileLevel editorInput)  {
		// fill the currentProfileLevel with the values entered in the form
		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setDescription(getDescriptionForInternalUse());
		editorInput.setLevelValueId(Long.valueOf(ratingController.getAssignedLevel().getLevelValueId()));

		if (editorInput.isNew()) {
			// add a new profileLevel if isNew

			ServiceContext ctx = null;
			try {
				ctx = PortletUtils.getServiceContext();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			ctx.setCommunityPermissions(pList.getCommunityPermissions());
			ctx.setGuestPermissions(pList.getGuestPermissions());

			CompetenceProfileLevel competenceProfileLevel = editorInput;
			try {
				PortletUtils.setOwnership(competenceProfileLevel, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			// if is new the required versiontag needs to be saved with the initially filled oldVersionTag
			editorInput.setVersionTag(versionTag);
			try {
				CompetenceProfileLevelFacade.addCompetenceProfileLevel(competenceProfileLevel, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			editorInput.setNew(false);
		} else {
			// update the profileLevel

			// if update it is possible the CompetenceLevels in the list are
			// changed so save the current selections
			final List<Long> idsToSave = this.levelController.getIds();
			saveCompetenceLevelsByProfile(idsToSave);
			ServiceContext ctx = null;
			try {
				ctx = PortletUtils.getServiceContext();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				CompetenceProfileLevelFacade.updateCompetenceProfileLevel(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * Go to the next page.
	 * 
	 * @return outcome
	 */
	public String changeLevels() {
		return this.getConfirmationController().canLeave(Outcomes.ADD, getEditorInput());
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final CompetenceProfileLevel editorInput) {
		this.levelController = new ChildLevelController("profileleveleditor.levellist", SelectionType.MULTIPLE_ENHANCED);

		// fill the form
		setDescriptionForInternalUse(editorInput.getDescription());
		setTitle(editorInput.getTitle());

		// prefill other entities for later use
		loadModelEntities();

		if (editorInput.isNew()) {
			versionTag = Bundle.getLabel("CompetenceProfileLevelEdit.firstversion");
		} else {
			versionTag = editorInput.getVersionTag();
		}

		this.ratingController = new ModelEditorRatingComponentController(levelValues, selectedLevel, -1, true, true);
		
		pList = new PermissionsList<CompetenceProfileLevel>(null, getEditorInput());
	}

	/**
	 * Load model entities.
	 */
	private void loadModelEntities() {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			if (levelValues.isEmpty()) {
				final CompetenceMap map = CompetenceMapFacade.getCompetenceMapByGroupId(PortletUtils.getGroupId(), ctx);
				// get all level values that are possible to assign to a profile
				// level
				levelValues = LevelValueFacade.getCompetenceProfileLevelValues(map.getCompetenceMapId(), ctx);
			}

			// if id not == 0 set the selected item
			if (this.getEditorInput() == null || this.getEditorInput().getLevelValueId() == 0) {
				// set the default level
				selectedLevel = levelValues.get(0);
			} else {
				try {
					selectedLevel = LevelValueFacade.getLevelValue(this.getEditorInput().getLevelValueId(), ctx);
				} catch (PortalException e) {
					PortletUtils.handleError(LOG, e);
				} catch (SystemException e) {
					PortletUtils.handleError(LOG, e);
				}

			}
			// load the competenceLevels attached to the profile
			loadCompetenceLevels();

		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Reload competence levels.
	 */
	private void loadCompetenceLevels() {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			
			final List<CompetenceLevel> allAddedCompetenceLevels = CompetenceLevelFacade.getCompetenceLevelsByIds(this
					.getEditorInput().getCompetenceLevelIds(), ctx);

			// add all Competences to the dataModel
			final List<CompetenceRow> model = new ArrayList<CompetenceRow>();
			for (CompetenceLevel level : allAddedCompetenceLevels) {
				model.add(new CompetenceRow(level));
			}
			levelController.setInput(model);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Action to create a new version from this competenceProfileLevel.
	 * 
	 * @return The outcome
	 */
	public String createNewVersion() {
		popupVisible = true;
		newVersionTag = null;
		return null;
	}

	/**
	 * Controller for handling the ratingcomponent at the frontend.
	 * 
	 * @return The ratingcontroller
	 */
	public RatingComponentController getRatingController() {
		return ratingController;
	}

	/**
	 * Gets a list of permissions for the actual CompetenceProfilelevel instance.
	 * 
	 * @return pList
	 */
	public PermissionsList<CompetenceProfileLevel> getPermissionsList() {
		return pList;
	}

	
	/**
	 * Cancel version.
	 * 
	 * @return the string
	 */
	public String cancelVersion() {
		popupVisible = false;
		return null;
	}

	
	/**
	 * Checks if is popup visible.
	 * 
	 * @return true, if is popup visible
	 */
	public boolean isPopupVisible() {
		return popupVisible;
	}

	/**
	 * Ok version.
	 * 
	 * @return the string
	 */
	public String okVersion() {
		try {
			final CompetenceProfileLevel currentLevel = getEditorInput();
			currentLevel.setVersionTag(newVersionTag);
			currentLevel.setTitle(getTitleForInternalUse());
			currentLevel.setDescription(getDescriptionForInternalUse());
			currentLevel.setLevelValueId(Long.valueOf(ratingController.getAssignedLevel().getLevelValueId()));

			final CompetenceProfileLevel newLevel = CompetenceProfileLevelFacade.addCompetenceProfileLevelNewVersion(
					currentLevel, PortletUtils.getServiceContext());
			setEditorInput(newLevel);
			initFields(newLevel);
			popupVisible = false;
			versionTag = newVersionTag;
			newLevel.setNew(false);
		} catch (PortalException pe) {
			PortletUtils.handleError(LOG, pe);
		} catch (SystemException se) {
			PortletUtils.handleError(LOG, se);
		}
		return null;
	}

}
