/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceProfileView.
 * 
 * @author mvv
 * @version $Revision: 1.26 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceProfileList extends BaseEditorBean<CompetenceMap> {

	private ActionsMenuListController<ActionsMenuRowItem<CompetenceProfile>> controller;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceProfileList.class);

	/**
	 * Gets the controller.
	 * 
	 * @return the controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<CompetenceProfile>> getController() {
		checkEditorInput();
		return controller;
	}

	/**
	 * method called if add button is pressed.
	 * 
	 * No Profile may be selected otherwise a error message is shown
	 * 
	 * A new CompetenceProfile is passed to the next page
	 * 
	 * @return the string
	 */
	public String addCompetenceProfile() {
		try {
			final CompetenceProfile profile = CompetenceProfileFacade
					.createCompetenceProfile();
			return this.getConfirmationController().canLeave(Outcomes.ADD, profile);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return null;
	}

	/**
	 * Check if user is allowed to add a Competence profile.
	 * 
	 * @return true / false
	 */
	public boolean isAddCompetenceProfileAllowed() {
		return checkPortletPermission(TENCActionKey.ADD_COMPETENCEPROFILE);
	}

	/**
	 * Delete competence profile.
	 * 
	 * @return the string
	 */
	public String deleteCompetenceProfile() {
		final FacesMessage message = new FacesMessage();
		message.setSeverity(FacesMessage.SEVERITY_INFO);
		message.setSummary(Bundle.getLabel("general.msg.deletenotimplemented"));
		FacesContext.getCurrentInstance().addMessage(null, message);
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void doSave(final CompetenceMap editorInput) {
		// Nothing to do
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields(final CompetenceMap editorInput) {
		controller = new ActionsMenuListController<ActionsMenuRowItem<CompetenceProfile>>();
		controller.setInput(refreshList(editorInput));
		controller.setConfirmationDialogController(getConfirmationController());
	}

	private List<ActionsMenuRowItem<CompetenceProfile>> refreshList(final CompetenceMap map) {
		try {
			ServiceContext ctx = PortletUtils.getServiceContext();
			final List<CompetenceProfile> competenceProfileList = CompetenceProfileFacade
					.getCompetenceProfilesByCompetenceMapId(map.getCompetenceMapId(), ctx);
			final List<ActionsMenuRowItem<CompetenceProfile>> result =
				new ArrayList<ActionsMenuRowItem<CompetenceProfile>>(competenceProfileList.size());
			for (CompetenceProfile cp : competenceProfileList) {
				result.add(new ActionsMenuRowItem<CompetenceProfile>(cp, cp.getPrimaryKey()));
			}
			return result;
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

}
