/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.modeleditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;

import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.FacadeUtils;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.NavigationData;
import org.tencompetence.portlet.common.components.NavigationDataHolder;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.modeleditor.common.CompetenceRow;
import org.tencompetence.portlet.modeleditor.common.ListController.SelectionType;
import org.tencompetence.portlet.modeleditor.controller.ChildLevelController;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

// Auto-generated Javadoc
/**
 * The Class CompetenceProfileSubCompetenceLevel.
 * @version $Revision: 1.20 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceProfileSubCompetenceLevel {
	private boolean disabled;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceProfileSubCompetenceLevel.class);
	private CompetenceRow selectedValue;
	private CompetenceProfileLevel editorInput;
	private ChildLevelController listController;
	private boolean needsInit;

	/**
	 * Instantiates a new competence sub level edit.
	 */
	public CompetenceProfileSubCompetenceLevel() {
		needsInit = true;
		initFields();
	}

	/**
	 * Wrap levels.
	 * 
	 * @param levels
	 *            the levels
	 * 
	 * @return the list< competence row>
	 */
	private List<CompetenceRow> wrapLevels(final List<CompetenceLevel> levels) {
		final List<CompetenceRow> items = new ArrayList<CompetenceRow>();
		for (CompetenceLevel level : levels) {
			items.add(new CompetenceRow(level));
		}
		return items;
	}

	/**
	 * Gets the list controller.
	 * 
	 * @return the list controller
	 */
	public ChildLevelController getListController() {
		initFields();
		return listController;
	}

	private String getViewId() {
		// Get previous page
		final NavigationData data = NavigationDataHolder.getInstance().getPreviousNavigationData();
		BaseEditorBean.passEditorInput(data.getEditorInput());
		return data.getViewId();
	}

	/**
	 * Save.
	 * 
	 * @return the navigation string
	 */
	public String save() {
		try {
			this.editorInput.setCompetenceLevelIds(FacadeUtils.encodeEntries(listController.getIds()));
			CompetenceProfileLevelFacade.updateCompetenceProfileLevel(this.editorInput, PortletUtils
					.getServiceContext());
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		needsInit = true;
		return getViewId();
	}

	/**
	 * Cancel.
	 * 
	 * @return the navigation string
	 */
	public String cancel() {
		needsInit = true;
		return getViewId();
	}

	/**
	 * Cancel save.
	 * 
	 * @return the navigation string
	 */
	public String cancelSave() {
		return Outcomes.BACK;
	}

	/**
	 * Adds the save.
	 * 
	 * @return the string
	 */
	public String addSave() {
		if (listController.isAddPressed()) {
			if (listController.getSelectedCompetenceLevel() == null) {
				final String id = PortletUtils.getPortletId() + ":form:level";
				final UIInput input = (UIInput) FacesContext.getCurrentInstance().getViewRoot().findComponent(id);
				PortletUtils.addErrorMessageToContext("error.toolessselected", new Object[] { Bundle
						.getLabel("competencelevels") });
				input.setValid(false);
				return null;
			} else {
				final CompetenceRow row = new CompetenceRow(listController.getSelectedCompetenceLevel());
				listController.getRowItems().add(row);
			}
		} else {
			getSelectedRow().setLevel(listController.getSelectedCompetenceLevel());
		}
		return Outcomes.BACK;
	}

	/**
	 * Checks if is disabled.
	 * 
	 * @return true, if is disabled
	 */
	public boolean isDisabled() {
		return disabled;
	}

	/**
	 * Gets the selected row.
	 * 
	 * @return the selected row
	 */
	public CompetenceRow getSelectedRow() {
		initFields();
		if (listController.getSelection().size() == 1) {
			selectedValue = listController.getSelection().get(0);
		} else {
			selectedValue = null;
		}
		return selectedValue;
	}

	private void initFields() {
		if (needsInit) {
			needsInit = false;
			listController = new ChildLevelController("CompetenceProfileSubCompetenceLevel_controller",
					SelectionType.MULTIPLE);
			editorInput = (CompetenceProfileLevel) PortletUtils.getSessionMap().get(Defines.EDITOR_INPUT_PARAM_NAME);
			try {
				ServiceContext ctx = PortletUtils.getServiceContext();
				final List<CompetenceLevel> levels = CompetenceLevelFacade.getCompetenceLevelsByIds(editorInput
						.getCompetenceLevelIds(), ctx);
				listController.setInput(wrapLevels(levels));
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

}
