/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.beans;

import javax.faces.context.FacesContext;
import javax.portlet.RenderResponse;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.NavigationDataHolder;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

import com.icesoft.faces.context.effects.JavascriptContext;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the managed bean for the view.jspx.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.39 $, $Date: 2009/11/24 13:11:39 $
 */
public class SmallView {

	/**
	 * Instantiates a new small view.
	 */
	public SmallView() {
		super();
		//Add needed js libraries.
		JavascriptContext.includeLib("/js/ratingComponent.js", FacesContext.getCurrentInstance());
	}

	private static final int INT_NO_MAP = -1;
	private static final Log LOG = LogFactoryUtil.getLog(SmallView.class);
	private CompetenceMap map;
	private Integer competenceProfileCount = null;
	private Integer competenceCount = null;

	/**
	 * Returns a URL that Liferay uses to render this portlet in maximized state. The URL contains information to follow
	 * ADD_OR_EDIT outcome once maximized.
	 * 
	 * @return the Liferay URL to the maximized portlet
	 * @see org.tencompetence.portlet.common.TCPhaseListener
	 */
	public String getEditorLink() {
		setPortletTitle();
		resetHistory();
		BaseEditorBean.passEditorInput(getCompetenceMap());
		return UrlUtils.getWindowStateMaximizedUrl(Outcomes.ADD_VIEW_EDIT);
	}

	/**
	 * Sets the portlet title depending on the language.
	 */
	private void setPortletTitle() {
		final RenderResponse resp = PortletUtils.getRenderResponse();
		resp.setTitle(Bundle.getLabel("label.portlet.title"));
		PortletUtils.getContext().getExternalContext().setResponse(resp);
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in maximized state. The URL contains information to follow
	 * LIST_COMPETENCES outcome once maximized.
	 * 
	 * @return the Liferay URL to the maximized portlet
	 * 
	 * @see org.tencompetence.portlet.common.TCPhaseListener
	 */
	public String getMaxCompetenceUrl() {
		resetHistory();
		setPortletTitle();
		BaseEditorBean.passEditorInput(getCompetenceMap());
		return UrlUtils.getWindowStateMaximizedUrl(Outcomes.LIST_COMPETENCES);
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in maximized state. The URL contains information to follow
	 * LIST_PROFILES outcome once maximized.
	 * 
	 * @return the Liferay URL to the maximized portlet or null
	 * 
	 * @see org.tencompetence.portlet.common.TCPhaseListener
	 */
	public String getMaxProfileUrl() {
		resetHistory();
		BaseEditorBean.passEditorInput(getCompetenceMap());
		return UrlUtils.getWindowStateMaximizedUrl(Outcomes.LIST_PROFILES);
	}
	

	/**
	 * Gets the window state maximized url.
	 * 
	 * @return the window state maximized url
	 */
	public String getWindowStateMaximizedUrl() {
		final String url = UrlUtils.getWindowStateMaximizedUrl(Outcomes.ID_SMALL_VIEW);
		NavigationDataHolder.getInstance().setNormalWindowStateURL(url);
		return url;
	}

	private void resetHistory() {
		final String url = UrlUtils.getWindowStateNormalUrl(Outcomes.ID_SMALL_VIEW); 
		NavigationDataHolder.getInstance().setNormalWindowStateURL(url);
		
		final NavigationDataHolder nav = NavigationDataHolder.getInstance();
		if (nav != null) {
			nav.clear();
		}
	}

	/**
	 * Return the name of the competence map associated with this portlet. If no map can be found a default message is
	 * returned.
	 * 
	 * @return the name of the competence map
	 */
	public String getMapName() {
		final CompetenceMap map = getCompetenceMap();
		if (isMapReady(map) && map.getTitle() != null) {
			return map.getTitle();
		} else {
			return Bundle.getLabel("smallview.title.nomap");
		}
	}

	/**
	 * Gets a <code>String</code> representation for the number of competences contained in the current competence map.
	 * 
	 * @return the number of competences as a <code>String</code>
	 */
	public String getNumberOfCompetences() {
		return getInfoText("smallview.report.competences", getCompetenceCount());
	}

	/**
	 * Gets a <code>String</code> representation for the number of competence profiles contained in the current
	 * competence map.
	 * 
	 * @return the number of competence profiles as a <code>String</code>
	 */
	public String getNumberOfCompetenceProfiles() {

		return getInfoText("smallview.report.competenceprofiles", getCompetenceProfileCount());
	}

	private String getInfoText(final String bundleKey, final Integer count) {
		Object[] args = new Object[0];
		String key = bundleKey;
		if (INT_NO_MAP == count || count.equals(0)) {
			key += ".zero";
		} else if (count.equals(1)) {
			key += ".one";
		} else {
			key += ".many";
			args = new Object[] { count };
		}
		return Bundle.getLabel(key, args);
	}

	private CompetenceMap getCompetenceMap() {
		if (map == null) {
			try {
				final ServiceContext ctx = PortletUtils.getServiceContext();
				map = CompetenceMapFacade.getCompetenceMapByGroupId(PortletUtils.getGroupId(), ctx);
				if (map == null) {
					map = CompetenceMapFacade.createCompetenceMap();
				}
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
		return map;
	}

	/**
	 * Checks if is competences available.
	 * 
	 * @return true, if is competences available
	 */
	public boolean isCompetencesAvailable() {
		return (getCompetenceCount() != INT_NO_MAP);
	}

	/**
	 * Checks if is profiles available.
	 * 
	 * @return true, if is profiles available
	 */
	public boolean isProfilesAvailable() {
		return (getCompetenceProfileCount() != INT_NO_MAP);
	}

	private Integer getCompetenceCount() {
		if (this.competenceCount == null) {
			try {
				final ServiceContext ctx = PortletUtils.getServiceContext();
				final CompetenceMap map = getCompetenceMap();
				if (isMapReady(map)) {
					competenceCount = CompetenceFacade.getCompetencesCount(map.getCompetenceMapId(), ctx);
				} else {
					competenceCount = INT_NO_MAP;
				}
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
		return competenceCount;
	}

	private Integer getCompetenceProfileCount() {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			final CompetenceMap map = getCompetenceMap();
			if (isMapReady(map)) {
				competenceProfileCount = CompetenceProfileFacade.getCompetenceProfilesCount(map
						.getCompetenceMapId(), ctx);
			} else {
				competenceProfileCount = INT_NO_MAP;
			}
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}

		return competenceProfileCount;
	}

	private boolean isMapReady(final CompetenceMap map) {
		return map != null && !map.isNew();
	}
}
