/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.common;

import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceRow.
 * 
 * @author venrooijma
 * @version $Revision: 1.18 $, $Date: 2009/11/24 14:10:05 $
 */
public class CompetenceRow implements IRowItem {
	private final long competenceId;
	private CompetenceLevel selectedLevel;
	private List<CompetenceLevel> allCompetenceLevelsOfCompetence;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceRow.class);
	private boolean selected;
	private ServiceContext ctx = null;

	/**
	 * Gets the current level.
	 * 
	 * @return the current level
	 */
	public String getCurrentLevel() {
		return Long.toString(selectedLevel.getPrimaryKey());
	}

	/**
	 * Sets the current level.
	 * 
	 * @param currentLevel
	 *            the new current level
	 */
	public void setCurrentLevel(final String currentLevel) {
		// get the LevelObject by id from local all levels
		if (!"".equals(currentLevel)) {
			for (CompetenceLevel level : allCompetenceLevelsOfCompetence) {
				if (level.getPrimaryKey() == Long.valueOf(currentLevel)) {
					selectedLevel = level;
				}
			}
		}
	}

	/** {@inheritDoc} */
	public boolean isSelected() {
		return selected;
	}

	/** {@inheritDoc} */
	public void setSelected(final boolean selected) {
		this.selected = selected;
	}

	/**
	 * Instantiates a new competence row.
	 * 
	 * @param level
	 *            the level
	 */
	public CompetenceRow(final CompetenceLevel level) {
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		try {
			allCompetenceLevelsOfCompetence = CompetenceLevelFacade.getRequirableCompetenceLevelsByCompetenceId(level
					.getCompetenceId(), ctx);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		this.competenceId = level.getCompetenceId();
		this.selectedLevel = level;
	}

	/**
	 * Gets the level.
	 * 
	 * @return the level
	 */
	public CompetenceLevel getLevel() {
		return selectedLevel;
	}

	/**
	 * Sets the level.
	 * 
	 * @param level
	 *            the new level
	 */
	public void setLevel(final CompetenceLevel level) {
		this.selectedLevel = level;
	}

	/**
	 * Gets the CompetenceLevelValue.
	 * 
	 * @return the LevelValue
	 */
	public String getLevelValue() {
		try {
			return String.valueOf(LevelValueFacade.getLevelValue(selectedLevel.getLevelValueId(),
					PortletUtils.getServiceContext()).getValue());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

		return "";
	}

	/**
	 * Gets the all competence levels of competence.
	 * 
	 * @return the all competence levels of competence
	 */
	public List<SelectItem> getAllCompetenceLevelsOfCompetence() {
		final List<SelectItem> result = new ArrayList<SelectItem>();
		for (CompetenceLevel level : allCompetenceLevelsOfCompetence) {
			result.add(new SelectItem(level.getPrimaryKey(), level.getTitle()));
		}
		return result;
	}

	/**
	 * Gets the competence id.
	 * 
	 * @return the competence id
	 */
	public long getCompetenceId() {
		return competenceId;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object object) {
		return object instanceof CompetenceRow && this.competenceId == ((CompetenceRow) object).competenceId;
	}

	/** {@inheritDoc} */
	public int hashCode() {
		return Long.valueOf(competenceId).hashCode();
	}

	/** {@inheritDoc} */
	//@Override
	public long getId() {
		return selectedLevel.getPrimaryKey();
	}

	/**
	 * Gets the data.
	 * 
	 * @return the data
	 */
	public CompetenceLevel getData() {
		return selectedLevel;
	}

	/**
	 * Return title of competence.
	 * 
	 * @return title of competence
	 */
	public String getCompetenceTitle() {
		try {
			return CompetenceFacade.getCompetence(this.competenceId, PortletUtils.getServiceContext()).getTitle();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}
}
