/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.common;

/**
 * RowItem implements the UI data model for a row in a list supporting ice:rowSelector.
 * <p>
 * It contains a data of type T and adds a selected state to the original entity.
 * 
 * @param <T>
 *            generic parameter <code>&lt;T&gt;</code>
 * @version $Revision: 1.7 $, $Date: 2009/11/24 14:10:05 $
 */
public class RowItem<T> implements IRowItem {

	private final T data;
	private boolean selected;
	private final Long id;

	/**
	 * Instantiates a new row item.
	 * 
	 * @param data
	 *            the data associated with this row
	 * @param id
	 *            the id
	 */
	public RowItem(final T data, final Long id) {
		super();
		this.data = data;
		this.id = id;
	}

	/**
	 * Gets the data associated with this RowItem.
	 * 
	 * @return the item
	 */
	public T getData() {
		return data;
	}

	/** {@inheritDoc} */
	//@Override
	public long getId() {
		return id;
	}

	/** {@inheritDoc} */
	//@Override
	public boolean isSelected() {
		return this.selected;
	}

	/** {@inheritDoc} */
	//@Override
	public void setSelected(final boolean selected) {
		this.selected = selected;
	}

}
