/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.controller;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.modeleditor.beans.CompetenceProfileSubCompetenceLevel;
import org.tencompetence.portlet.modeleditor.common.CompetenceRow;
import org.tencompetence.portlet.modeleditor.common.ListController;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class ChildLevelController.
 * 
 * @author venrooijma
 * @version $Revision: 1.24 $, $Date: 2009/11/24 14:10:06 $
 */
public class ChildLevelController extends ListController<CompetenceRow> {
	private Competence selectedCompetence;
	private CompetenceLevel selectedCompetenceLevel;
	private boolean disabled;
	private boolean dirty;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceProfileSubCompetenceLevel.class);
	private CompetenceMap map;
	private CompetenceRow selectedValue;
	private List<SelectItem> competences;
	private List<SelectItem> currentLevels;
	private boolean addPressed;
	private ServiceContext ctx = null;

	/**
	 * Checks if is adds the pressed.
	 * 
	 * @return true, if is adds the pressed
	 */
	public boolean isAddPressed() {
		return addPressed;
	}

	/**
	 * Checks if is disabled.
	 * 
	 * @return true, if is disabled
	 */
	public boolean isDisabled() {
		return disabled;
	}

	/**
	 * Sets the disabled.
	 * 
	 * @param disabled
	 *            the new disabled
	 */
	public void setDisabled(final boolean disabled) {
		this.disabled = disabled;
	}

	/**
	 * Instantiates a new child level controller.
	 * 
	 * @param idString
	 *            the id string
	 * @param type
	 *            the type
	 */
	public ChildLevelController(final String idString, final SelectionType type) {
		super(idString, type);
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		map = PortletUtils.getCompetenceMapByCurrentCommunity();
	}

	/**
	 * Checks if is dirty.
	 * 
	 * @return true, if is dirty
	 */
	public boolean isDirty() {
		return dirty;
	}

	/**
	 * Show competence lookup screen and reset the list.
	 * 
	 * @return String
	 */
	public String addCompetenceSubLevel() {
		addPressed = true;
		selectedCompetence = null;
		selectedCompetenceLevel = null;
		currentLevels = null;
		final List<Competence> comp = getNotAlreadyContainedCompetences();
		if (comp == null || comp.isEmpty()) {
			PortletUtils.addErrorMessageToContext("error.subcompetencesfilled");
			return null;
		}
		competences = new ArrayList<SelectItem>();
		for (Competence c : comp) {
			competences.add(new SelectItem(c.getPrimaryKey(), c.getTitle()));
		}

		return Outcomes.ADD;
	}

	/**
	 * Delete selected competence levels.
	 * 
	 * @param event
	 *            the event
	 */
	public void deleteSelectedCompetenceLevels(final ActionEvent event) {
		final List<CompetenceRow> selected = this.getSelection();
		if (selected.isEmpty()) {
			PortletUtils.addErrorMessageToContext("error.toolessselected", new Object[] { Bundle
					.getLabel("competencelevel") });
		} else {
			getRowItems().removeAll(selected);
		}
		dirty = true;
		this.resetSelection();
	}

	/**
	 * Move selected items up.
	 * 
	 * @param event
	 *            the event
	 */
	public void moveItemsUp(final ActionEvent event) {
		final List<CompetenceRow> list = this.getSelection();
		if (getSelection().size() == 0) {
			PortletUtils.addErrorMessageToContext("error.toolessselected", new Object[] { Bundle
					.getLabel("competencelevels") });
		} else {
			for (CompetenceRow item : list) {
				final int index = getRowItems().indexOf(item);
				// only move when not first item and previous item not select (to avoid strange behavior)
				if (index > 0 && !list.contains(getRowItems().get(index - 1))) {
					getRowItems().remove(index);
					getRowItems().add(index - 1, item);
				}
			}
			dirty = true;
		}
	}

	/**
	 * Move selected items down.
	 * 
	 * @param event
	 *            the eevent
	 */
	public void moveItemsDown(final ActionEvent event) {
		final List<CompetenceRow> list = this.getSelection();
		if (getSelection().size() == 1) {
			CompetenceRow item;
			final int size = getRowItems().size();
			for (int i = list.size() - 1; i >= 0; i--) {
				item = list.get(i);
				final int index = getRowItems().indexOf(item);
				// only move when not last item and next item not select (to avoid strange behavior)
				if (index < size - 1 && !list.contains(getRowItems().get(index + 1))) {
					getRowItems().remove(index);
					getRowItems().add(index + 1, item);
				}
			}
			dirty = true;
		} else {
			PortletUtils.addErrorMessageToContext("error.exactlyonemustbeselected");
		}
	}

	/**
	 * Change level of a selected competenceRow.
	 * 
	 * @return String
	 */
	public String changeLevel() {
		if (getSelection().size() == 1) {
			addPressed = false;
			final long comp = getSelectedRow().getData().getCompetenceId();
			try {
				selectedCompetence = CompetenceFacade.getCompetence(comp, ctx);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
			try {
				selectedCompetenceLevel = CompetenceLevelFacade.getCompetenceLevel(getSelectedRow().getData()
						.getCompetenceLevelId(), ctx);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
			loadLevelsDropDown(getSelectedRow().getData().getCompetenceId());
			competences = new ArrayList<SelectItem>();
			competences.add(new SelectItem(Long.toString(selectedCompetence.getPrimaryKey()), selectedCompetence
					.getTitle()));
			return Outcomes.ADD;
		} else {
			PortletUtils.addErrorMessageToContext("error.exactlyonemustbeselected");
		}
		return null;
	}

	/**
	 * Returns all competences of the map excluding the ones of which a competenceLevel is already added.
	 * 
	 * @return
	 */
	private List<Competence> getNotAlreadyContainedCompetences() {
		try {
			final List<Competence> competencesByMap = CompetenceFacade.getCompetencesByCompetenceMapId(map
					.getPrimaryKey(), ctx);

			return wrapNotAddedCompetences(competencesByMap);

		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * Gets the competences.
	 * 
	 * @return the competences
	 */
	public List<SelectItem> getCompetences() {
		return competences;
	}

	/**
	 * Gets the levels.
	 * 
	 * @return the levels
	 */
	public List<SelectItem> getLevels() {
		return currentLevels;
	}

	private List<Competence> wrapNotAddedCompetences(final List<Competence> competencesByMap) {
		final List<Competence> result = new ArrayList<Competence>();
		// wrap and remove the already selected competences
		for (Competence competence : competencesByMap) {
			if (!containsCompetence(competence.getCompetenceId())) {
				result.add(competence);
			}
		}
		return result;
	}

	private boolean containsCompetence(final long competenceId) {
		for (CompetenceRow row : this.getRowItems()) {
			if (competenceId == row.getData().getCompetenceId()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Gets the selected row.
	 * 
	 * @return the selected row
	 */
	public CompetenceRow getSelectedRow() {
		if (getSelection().size() == 1) {
			selectedValue = getSelection().get(0);
		} else {
			selectedValue = null;
		}
		return selectedValue;
	}

	private void loadLevelsDropDown(final long competenceId) {
		try {
			final List<CompetenceLevel> lvl = CompetenceLevelFacade.getRequirableCompetenceLevelsByCompetenceId(
					competenceId, ctx);
			final List<SelectItem> lst = new ArrayList<SelectItem>();
			for (CompetenceLevel l : lvl) {
				lst.add(new SelectItem(l.getPrimaryKey(), l.getTitle()));
			}
			currentLevels = lst;
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Sets the selected competence.
	 * 
	 * @param value
	 *            the new selected competence
	 */
	public void setSelectedCompetence(final String value) {
		if (value != null && !value.isEmpty()) {
			final long id = Long.valueOf(value);
			try {
				selectedCompetence = CompetenceFacade.getCompetence(id, ctx);
				if (selectedCompetence != null) {
					loadLevelsDropDown(selectedCompetence.getPrimaryKey());
				}
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	/**
	 * Sets the selected level.
	 * 
	 * @param value
	 *            the new selected level
	 */
	public void setSelectedLevel(final String value) {
		if (value != null && !value.isEmpty()) {
			final long id = Long.valueOf(value);
			try {
				selectedCompetenceLevel = CompetenceLevelFacade.getCompetenceLevel(id, ctx);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	/**
	 * Competence on change.
	 * 
	 * @param event
	 *            the event
	 */
	public void competenceOnChange(final ValueChangeEvent event) {
		final String value = (String) event.getNewValue();
		if (value != null && !value.isEmpty()) {
			final long id = Long.valueOf(value);
			try {
				selectedCompetence = CompetenceFacade.getCompetence(id, ctx);
				if (selectedCompetence != null) {
					loadLevelsDropDown(selectedCompetence.getPrimaryKey());
				}
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	/**
	 * Gets the selected competence.
	 * 
	 * @return the selected competence
	 */
	public String getSelectedCompetence() {
		if (selectedCompetence == null) {
			return null;
		} else {
			return Long.toString(selectedCompetence.getPrimaryKey());
		}
	}

	/**
	 * Gets the selected level.
	 * 
	 * @return the selected level
	 */
	public String getSelectedLevel() {
		if (selectedCompetenceLevel == null) {
			return null;
		} else {
			return Long.toString(selectedCompetenceLevel.getPrimaryKey());
		}
	}

	/**
	 * Gets the selected competence level.
	 * 
	 * @return the selected competence level
	 */
	public CompetenceLevel getSelectedCompetenceLevel() {
		return this.selectedCompetenceLevel;
	}

}
