/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.controller;

import java.beans.Beans;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.el.ELContext;
import javax.el.ELException;
import javax.el.ExpressionFactory;
import javax.el.MethodExpression;
import javax.faces.FacesException;
import javax.faces.component.NamingContainer;
import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import nl.jkva.jsf.components.nonvalidatingcommand.UINonValidatingCommandLink;

import org.tencompetence.portlet.common.utils.Bundle;
import org.w3c.dom.Element;

import com.icesoft.faces.component.AttributeConstants;
import com.icesoft.faces.context.DOMContext;
import com.icesoft.faces.renderkit.dom_html_basic.DomBasicRenderer;
import com.icesoft.faces.renderkit.dom_html_basic.FormRenderer;
import com.icesoft.faces.renderkit.dom_html_basic.HTML;
import com.icesoft.faces.renderkit.dom_html_basic.PassThruAttributeRenderer;
import com.icesoft.faces.util.DOMUtils;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * This class creates a Tab for each label action pair provided in the jspx file.
 * @version $Revision: 1.3 $, $Date: 2009/11/24 14:10:06 $
 * 
 */
public class SubTabComponent extends UINonValidatingCommandLink {
	private String labels;
	private String actions;
	private int currentPage;
	private static final String[] PASS_ATTRIBUTES = AttributeConstants.getAttributes(AttributeConstants.H_COMMANDLINK);
	private static final String HIDDEN_FIELD_NAME = "tabCmp";
	private static final Log LOG = LogFactoryUtil.getLog(SubTabComponent.class);
	private static final String STYLE_CLASS = "class";
	private String[] splitActions;
	private String[] splitLabels;
	private DOMContext domContext;
	private FacesContext facesContext;

	/**
	 * Gets the current page.
	 * 
	 * The currentPage is the index of the tab which should be rendered in a different way.
	 * 
	 * @return the current page
	 */
	public int getCurrentPage() {
		return currentPage;
	}

	/**
	 * Sets the current page.
	 * 
	 * The currentPage is the index of the tab which should be rendered in a different way.
	 * 
	 * @param currentPage
	 *            the new current page
	 */
	public void setCurrentPage(final int currentPage) {
		this.currentPage = currentPage;
	}

	/**
	 * Gets the actions.
	 * 
	 * Actions is a space separated list of el expressions without the surrounding #{ [expr] } around the expressions.
	 * The list of actions is used to couple an action to a click on a tab. The order of the list is used to determine
	 * which tab includes which action.
	 * 
	 * 
	 * @return the actions
	 */
	public String getActions() {
		return actions;
	}

	/**
	 * Sets the actions.
	 * 
	 * Actions is a space separated list of el expressions without the surrounding #{ [expr] } around the expressions.
	 * The list of actions is used to couple an action to a click on a tab. The order of the list is used to determine
	 * which tab includes which action.
	 * 
	 * @param actions
	 *            the new actions
	 */
	public void setActions(final String actions) {
		this.splitActions = actions.split(" ");
		this.actions = actions;
	}

	/**
	 * Gets the labels.
	 * 
	 * Labels is a space separated list of keys of bundle items to display a correct text on the tab. The order of the
	 * list is used to determine which label should go on which tab.
	 * 
	 * @return the labels
	 */
	public String getLabels() {
		return labels;
	}

	/**
	 * Sets the labels.
	 * 
	 * Labels is a space separated list of keys of bundle items to display a correct text on the tab. The order of the
	 * list is used to determine which label should go on which tab.
	 * 
	 * @param labels
	 *            the new labels
	 */
	public void setLabels(final String labels) {
		// setValue(labels);
		this.labels = labels;
		this.splitLabels = labels.split(" ");
	}

	/** {@inheritDoc} */
	@Override
	public void encodeBegin(final FacesContext facesContext) throws IOException {
		this.facesContext = facesContext;
		validateParameters();
		domContext = DOMContext.attachDOMContext(facesContext, this);
		boolean disabled = false;
		try {
			disabled = Boolean.valueOf(this.getAttributes().get("disabled").toString());
		} catch (Exception e) {
			if (LOG.isDebugEnabled()) {
				LOG.debug(e);
			}
		}

		if (!domContext.isInitialized()) {
			// create the document output like <ul> <li><a>someTab </a>
			// </li></ul>
			initDomContext();
		}

		// for each label create a tab
		for (int index = 0; index < splitLabels.length; index++) {
			createTab(index, disabled);
		}
	}

	private void initDomContext() {
		final Element root = domContext.createElement("ul");
		root.setAttribute(STYLE_CLASS, "tabs ui-tabs");

		for (int index = 0; index < splitLabels.length; index++) {
			final Element htmlLi = domContext.createElement("li");
			root.appendChild(htmlLi);
			final Element htmlA = domContext.createElement("a");
			htmlA.setAttribute("href", "javascript:;");
			htmlLi.appendChild(htmlA);

			if (index == currentPage) {
				if (index == 0) {
					htmlLi.setAttribute(STYLE_CLASS, "current first");
				} else if (index == splitLabels.length - 1) {
					htmlLi.setAttribute(STYLE_CLASS, "current last");
				} else {
					htmlLi.setAttribute(STYLE_CLASS, "current");
				}
			}
			addIdToElement(htmlA, index);
		}
		domContext.setRootNode(root);
	}

	private void createTab(final int indexCurrentLabel, final boolean disabled) {
		final Element root = (Element) domContext.getRootNode();
		final Element listItem = (Element) root.getChildNodes().item(indexCurrentLabel);
		final Element link = (Element) listItem.getFirstChild();

		DOMContext.removeChildren(link);
		link.setTextContent(DOMUtils.escapeAnsi(getBundleText(splitLabels[indexCurrentLabel])));
		PassThruAttributeRenderer.renderHtmlAttributes(facesContext, this, PASS_ATTRIBUTES);
		renderOnClick(link, indexCurrentLabel);

		if (disabled) {
			link.removeAttribute("disabled");
		}

		FormRenderer.addHiddenField(facesContext, deriveCommonHiddenFieldNameWithLabel(indexCurrentLabel));
		final Iterator<String> parameterKeys = getParameterMap().keySet().iterator();
		while (parameterKeys.hasNext()) {
			final String nextKey = (String) parameterKeys.next();
			FormRenderer.addHiddenField(facesContext, nextKey);
		}
		domContext.stepInto(this);
	}

	private String getBundleText(final String label) {
		String bundleText;
		bundleText = Bundle.getLabel(label);
		if (bundleText == null) {
			bundleText = label;
		}
		return bundleText;
	}

	/** {@inheritDoc} */
	public void decode(final FacesContext facesContext) {
		this.facesContext = facesContext;
		validateParameters();
		if (DomBasicRenderer.isStatic(this)) {
			return;
		}
		final String commandLinkId = this.getClientId(facesContext);
		final Map<String, String> requestParams = facesContext.getExternalContext().getRequestParameterMap();

		// check if the requestmap contains a hiddenfield with a commonName
		for (int index = 0; index < splitLabels.length; index++) {
			final String commonFieldName = deriveCommonHiddenFieldNameWithLabel(index);
			final String fieldInRequest = (String) requestParams.get(commonFieldName);
			final boolean isCurrentComponent = fieldInRequest != null && !fieldInRequest.equals("")
					&& commandLinkId.equals(fieldInRequest);
			if (isCurrentComponent) {
				// this command link caused the submit so queue an event
				// translate action to methodbinding
				final MethodExpression expr = createActionExpression("#{" + splitActions[index] + "}");
				this.setActionExpression(expr);
				this.queueEvent(new ActionEvent(this));
			}
		}
	}

	// create the properly defined expression
	private MethodExpression createActionExpression(final String elExpression) {
		// get application from faces context
		final ExpressionFactory exprFactory = facesContext.getApplication().getExpressionFactory();
		// getting the ELContext from faces context
		final ELContext elContext = facesContext.getELContext();
		try {
			return exprFactory.createMethodExpression(elContext, elExpression, String.class, new Class[0]);
		} catch (ELException el) {
			if (LOG.isErrorEnabled()) {
				LOG.error(el);
			}
			throw new RuntimeException("Error while parsing action: " + elExpression, el);
		}
	}

	private void validateParameters() {
		if (facesContext == null) {
			LOG.error("Invalid Parameter - FacesContext instance must not be null");
			throw new NullPointerException("Invalid Parameter - FacesContext instance must not be null");
		}

		if (splitActions.length != splitLabels.length) {
			LOG.error("actions and label list should be of the same size");
			throw new IllegalArgumentException("actions and label list should be of the same size");
		}
	}

	private void addIdToElement(final Element element, final int index) {
		if (this.getId() != null) {
			element.setAttribute("id", this.getClientId(facesContext) + index);
		}
	}

	private void renderOnClick(final Element root, final int index) {
		final UIComponent uiForm = DomBasicRenderer.findForm(this);
		if (uiForm == null) {
			throw new FacesException("CommandLink must be contained in a form");
		}
		final Object onClick = this.getAttributes().get(HTML.ONCLICK_ATTR);
		// if onClick attribute set by the UI programmer, pre append it.
		final String rendererOnClick = getJavaScriptOnClickString(index);
		root.setAttribute("onclick", DomBasicRenderer.combinedPassThru((String) onClick, rendererOnClick));
	}

	private String getJavaScriptOnClickString(final int index) {
		return getJavascriptHiddenFieldSetters(index) + "return iceSubmit(form,this,event);";
	}

	private String getJavascriptHiddenFieldSetters(final int index) {
		final Map<String, String> parameters = getParameterMap();
		final StringBuilder sb = new StringBuilder("var form=formOf(this);form['");
		sb.append(deriveCommonHiddenFieldNameWithLabel(index));
		sb.append("'].value='");
		sb.append(this.getClientId(facesContext));
		sb.append("';");
		final Iterator<Entry<String, String>> parameterKeys = parameters.entrySet().iterator();
		while (parameterKeys.hasNext()) {
			final Entry<String, String> entry = parameterKeys.next();
			sb.append("form['");
			sb.append(entry.getKey());
			sb.append("'].value='");
			sb.append(entry.getValue());
			sb.append("';");
		}
		return sb.toString();
	}

	private Map<String, String> getParameterMap() {
		final Map<String, String> parameterMap = new HashMap<String, String>();
		if (this.getChildCount() > 0) {
			final Iterator<UIComponent> children = this.getChildren().iterator();
			while (children.hasNext()) {
				final UIComponent nextChild = children.next();
				if (nextChild instanceof UIParameter) {
					final UIParameter uiParam = (UIParameter) nextChild;
					parameterMap.put(uiParam.getName(), (String) uiParam.getValue());
				}
			}
		}
		return parameterMap;
	}

	private String deriveCommonHiddenFieldNameWithLabel(final int index) {
		String hiddenFieldName;
		if (Beans.isDesignTime()) {
			hiddenFieldName = "";
		} else {
			final UIComponent parentContainer = DomBasicRenderer.findForm(this);
			final String parentClientId = parentContainer.getClientId(facesContext);
			hiddenFieldName = parentClientId + NamingContainer.SEPARATOR_CHAR + UIViewRoot.UNIQUE_ID_PREFIX
					+ HIDDEN_FIELD_NAME + index;
		}
		return hiddenFieldName;
	}

}
