/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.beans;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.portlet.common.components.ElementController;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

/**
 * The class PdpViewAction.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.4 $, $Date: 2009/11/26 13:37:20 $
 */
public class PdpViewAction {
	private final String normalUrl = UrlUtils.getWindowStateNormalUrl();
	private ElementController elementController;
	private boolean needsInit;

	/**
	 * Instantiates a new pdp build plan.
	 */
	public PdpViewAction() {
		this.needsInit = true;
	}

	/**
	 * Gets the element controller.
	 * 
	 * @return the element controller
	 */
	public ElementController getElementController() {
		initFields();
		return this.elementController;
	}

	/**
	 * Return to the small view.
	 * 
	 * @return null as navigation is handled directly
	 */
	public String doOK() {
		PortletUtils.redirect(normalUrl);
		return null;
	}

	private void initFields() {
		if (needsInit) {
			needsInit = false;

			elementController = new ElementController(getTarget(), false, true) {

				@Override
				public String handleClickEvent(final ILPElement element) {
					return null;
				}
			};

		}
	}

	private ILearningAction getTarget() {
		ILearningAction result = null;
		final Object obj = PortletUtils.getSessionMap().get(Defines.EDITOR_INPUT_PARAM_NAME);

		if (!(obj instanceof ILearningAction)) {
			throw new IllegalArgumentException("Input should be of type ILearningAction");
		}

		result = (ILearningAction) obj;
		return result;
	}
}