/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import org.tencompetence.generated.model.CompetenceLevel;

/**
 * The Gap class implements a competence level gap controller.
 * 
 * @author Harrie Martens
 * @version $Revision: 1.1 $, $Date: 2009/10/25 11:09:54 $
 */

public class Gap {

	private String title;
	private long id;

	private int covered;
	private boolean selected;
	private final CompetenceLevel level;

	/**
	 * Instantiates a new competence data.
	 * 
	 * @param title
	 *            the title
	 * @param id
	 *            the id
	 */
	public Gap(final String title, final long id) {
		this.title = title;
		this.covered = 0;
		this.id = id;
		this.level = null;
	}

	/**
	 * Instantiates a new gap.
	 * 
	 * @param level
	 *            the level
	 */
	public Gap(final CompetenceLevel level) {
		this.level = level;
		this.covered = 0;
	}

	/**
	 * Sets the selected.
	 * 
	 * @param value
	 *            the new selected
	 */
	public void setSelected(final boolean value) {
		this.selected = value;
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		if (level != null) {
			return level.getTitle();
		}

		return this.title;
	}

	/**
	 * Checks if the gap is selected.
	 * 
	 * @return true, if is selected
	 */
	public boolean isSelected() {
		return this.selected;
	}

	/**
	 * Checks if the gap is covered by at least one path.
	 * 
	 * @return true, if is covered
	 */
	public boolean isCovered() {
		return this.covered > 0;
	}

	/**
	 * Gets the id.
	 * 
	 * @return the id
	 */
	protected long getId() {
		return this.id;
	}

	/**
	 * Increment.
	 * 
	 * @param isPartOfPlan
	 *            the is part of plan
	 */
	public void usePlan(final boolean isPartOfPlan) {
		if (isPartOfPlan) {
			this.covered++;
		} else if (this.covered > 0) {
			this.covered--;
		}
	}

	/**
	 * Gets the level.
	 * 
	 * @return the level
	 */
	public CompetenceLevel getLevel() {
		return this.level;
	}
}