/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import java.io.IOException;

import javax.faces.context.FacesContext;

import org.tencompetence.portlet.common.components.BaseCustomComponent;
import org.w3c.dom.Element;

import com.icesoft.faces.context.DOMContext;

/**
 * The Class ProgressBar.
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:10:06 $
 */
public abstract class PdpProgress extends BaseCustomComponent {

	/** The Constant NOT_PLANNED. */
	protected static final String NOT_PLANNED = "notPlanned";

	/** The Constant PLANNED. */
	protected static final String PLANNED = "planned";

	/** The Constant ATTAINED. */
	protected static final String ATTAINED = "attained";

	private DOMContext domContext;
	private String width;

	/**
	 * Inits the DOMContext.
	 * 
	 * @param domContext
	 *            the DOMContext
	 */
	protected abstract void initDomContext(final DOMContext domContext);

	/** {@inheritDoc} */
	@Override
	public void encodeBegin(final FacesContext context) throws IOException {
		super.encodeBegin(context);

		domContext = DOMContext.attachDOMContext(context, this);

		if (!domContext.isInitialized()) {
			initDomContext(domContext);
		}

		domContext.stepInto(this);
	}

	/**
	 * Creates the table.
	 * 
	 * @param styleClass
	 *            the style class
	 * @param width
	 *            the width            
	 * 
	 * @return the element
	 */
	protected Element createTable(final String styleClass, final String width) {
		final Element table = domContext.createElement("table");
		table.setAttribute(WIDTH, width);
		table.setAttribute(CELLPADDING, "0");
		table.setAttribute(CELLSPACING, "0");
		table.setAttribute(CLASS, styleClass);

		return table;
	}

	/**
	 * Creates the table cell.
	 * 
	 * @param styleClass
	 *            the style class
	 * 
	 * @return the element
	 */
	protected Element createTableCell(final String styleClass) {
		return createTableCell(styleClass, null);
	}
	
	/**
	 * Creates the table cell.
	 * 
	 * @param styleClass
	 *            the style class
	 * @param value
	 *            the value
	 * 
	 * @return the element
	 */
	protected Element createTableCell(final String styleClass, final String value) {
		final Element cell = domContext.createElement("td");
		cell.setAttribute(CLASS, styleClass);

		if (value != null) {
			cell.setTextContent(value);
		}

		return cell;
	}

	/**
	 * Creates the table row.
	 * 
	 * @return the element
	 */
	protected Element createTableRow() {
		return domContext.createElement("tr");
	}

	/**
	 * Sets the width.
	 * 
	 * @param width
	 *            the new width
	 */
	public void setWidth(final String width) {
		this.width = width;
	}

	/**
	 * Gets the width.
	 * 
	 * @return the width
	 */
	protected String getWidth() {
		return width;
	}
}