/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import org.w3c.dom.Element;

import com.icesoft.faces.context.DOMContext;

/**
 * The Class ProgressBar.
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.7 $, $Date: 2009/11/24 14:10:06 $
 */
public class PdpProgressBar extends PdpProgress {

	private static final int PCT = 100;

	/** {@inheritDoc} */
	@Override
	public String getFamily() {
		return "PdpProgressBar";
	}

	/** {@inheritDoc} */
	@Override
	protected void initDomContext(final DOMContext domContext) {
		final Element table = createTable("pdpProgressBarTable", getWidth());

		final Element tr = createTableRow();
		table.appendChild(tr);

		tr.appendChild(createTDElement("pdpProgressText pdpProgressBarAttained", getRequiredString(ATTAINED)));
		tr.appendChild(createTDElement("pdpProgressText pdpProgressBarPlanned", getRequiredString(PLANNED)));
		tr.appendChild(createTDElement("pdpProgressText pdpProgressBarNotPlanned", getRequiredString(NOT_PLANNED)));

		domContext.setRootNode(table);
	}

	private Element createTDElement(final String styleClass, final String pct) {
		final Element td = createTableCell(styleClass);
		final int pctValue = Integer.parseInt(pct);
		td.setAttribute(STYLE, String.format("width: %dpx;", calcWidth(pctValue)));

		// Only display pct value when > 0
		if (pctValue > 0) {
			td.setTextContent(pct + "%");
		}
		return td;
	}

	private int calcWidth(final int pctValue) {
		return (Integer.parseInt(getWidth()) * pctValue) / PCT;
	}
}