/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import org.tencompetence.portlet.common.utils.Bundle;
import org.w3c.dom.Element;

import com.icesoft.faces.context.DOMContext;
import com.liferay.portal.kernel.util.StringPool;

/**
 * The Class PdpProgressLegenda.
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.7 $, $Date: 2009/11/24 14:10:06 $
 */
public class PdpProgressLegenda extends PdpProgress {

	/** {@inheritDoc} */
	@Override
	public String getFamily() {
		return "PdpProgressLegenda";
	}

	/** {@inheritDoc} */
	@Override
	protected void initDomContext(final DOMContext domContext) {
		final Element table = createTable("pdpProgressLegendaTable", getWidth());

		final Element rowAttained = createTableRow();
		rowAttained.appendChild(createTableCell("pdpProgressLegendaImage pdpProgressLegendaAttained"));
		rowAttained.appendChild(createPctColumn("pdpProgressText pdpProgressLegendaPct", getRequiredString(ATTAINED)));
		rowAttained.appendChild(createTextColumn("pdpProgressText pdpProgressLegendaText", Bundle
				.getLabel("progress_indicator.label.attained")));

		final Element rowPlanned = createTableRow();
		rowPlanned.appendChild(createTableCell("pdpProgressLegendaImage pdpProgressLegendaPlanned"));
		rowPlanned.appendChild(createPctColumn("pdpProgressText pdpProgressLegendaPct", getRequiredString(PLANNED)));
		rowPlanned.appendChild(createTextColumn("pdpProgressText pdpProgressLegendaText", Bundle
				.getLabel("progress_indicator.label.planned")));

		final Element rowNotPlanned = createTableRow();
		rowNotPlanned.appendChild(createTableCell("pdpProgressLegendaImage pdpProgressLegendaNotPlanned"));
		rowNotPlanned.appendChild(createPctColumn("pdpProgressText pdpProgressLegendaPct",
				getRequiredString(NOT_PLANNED)));
		rowNotPlanned.appendChild(createTextColumn("pdpProgressText pdpProgressLegendaText", Bundle
				.getLabel("progress_indicator.label.not_planned")));

		table.appendChild(rowAttained);
		table.appendChild(rowPlanned);
		table.appendChild(rowNotPlanned);

		domContext.setRootNode(table);
	}

	private Element createTextColumn(final String styleClass, final String label) {
		return createTableCell(styleClass, StringPool.NBSP + label);
	}

	private Element createPctColumn(final String styleClass, final String pct) {
		return createTableCell(styleClass, pct + StringPool.PERCENT);
	}
}