/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.util;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.portlet.common.utils.Bundle;

/**
 * This class acts as a filter for LearningPaths.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.4 $, $Date: 2009/11/30 14:03:20 $
 */
public class Filter {
	private final List<FilterOption> filterOptions = new ArrayList<FilterOption>();

	/**
	 * Sets the filter option.
	 * 
	 * @param filterOption
	 *            the filter option
	 */
	public void setFilterOption(final FilterOption filterOption) {
		filterOptions.remove(filterOption);
		if (filterOption.getValue() != null && !"".equals(filterOption.getValue().trim())) {

			// filter option was set
			filterOptions.add(filterOption);
		}
	}

	/**
	 * Gets the filter options.
	 * 
	 * @return the filter options
	 */
	public List<FilterOption> getFilterOptions() {
		return filterOptions;
	}

	/**
	 * Return the
	 * <code>FilterOption</code> corresponding with the passed key. Return null if not <code>FilterOption</code> can be
	 * found.
	 * 
	 * @param key  the key representing the <code>FilterOption</code>
	 * @return the <code>FilterOption</code> corresponding with the passed key
	 */
	public FilterOption getFilterOption(final String key) {
		if (key == null) {
			return null;
		}
		for (FilterOption option : filterOptions) {
			if (key.equals(option.getKey())) {
				return option;
			}
		}

		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		StringBuffer result = new StringBuffer();
		boolean first = true;

		for (FilterOption option : filterOptions) {
			if (!first) {
				result.append(" " + Bundle.getLabel("pdp.filter.and") + " ");
			}

			first = false;
			result.append(option.toString());
		}

		return result.toString();
	}
}
