/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.util;

import org.tencompetence.portlet.common.utils.Bundle;

/**
 * A filter option represents an individual filter setting.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * 
 * @version $Revision: 1.6 $, $Date: 2009/12/01 09:37:45 $
 */
public class FilterOption {
	private final String key;
	private String value;

	/**
	 * Creates a new Filter option using the passed key.
	 * 
	 * @param key
	 *            the language independent key
	 */
	public FilterOption(final String key) {
		this.key = key;
	}

	/**
	 * Instantiates a new filter option using the passed key and value.
	 * 
	 * @param key
	 *            the language independent key
	 * @param value
	 *            the current value of the key
	 */
	public FilterOption(final String key, final String value) {
		this.key = key;
		this.value = value;
	}

	/**
	 * Returns the translated key value.
	 * 
	 * @return the translated key value
	 */
	public String getTranslatedKey() {
		return Bundle.getLabel("learningpath.metadata." + key);
	}

	/**
	 * Gets the language independent key for this filter option.
	 * 
	 * @return the language independent key
	 */
	public String getKey() {
		return key;
	}

	/**
	 * Gets the value of this filteroption.
	 * 
	 * @return the value
	 */
	public String getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * 
	 * @param value
	 *            the new value
	 */
	public void setValue(final String value) {
		this.value = value;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return getTranslatedKey() + " = " + getValue();
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((key == null) ? 0 : key.hashCode());
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof FilterOption)) {
			return false;
		}
		FilterOption other = (FilterOption) obj;
		if (key == null) {
			if (other.key != null) {
				return false;
			}
		} else if (!key.equals(other.key)) {
			return false;
		}
		return true;
	}

}
