/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import java.util.GregorianCalendar;

import org.tencompetence.business.facade.EvidenceFacade;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.PermissionsList;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.portfolio.common.PortfolioDefines;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class EvidenceEdit.
 * @version $Revision: 1.16 $, $Date: 2009/11/24 14:10:06 $
 */
public class EvidenceEdit extends BaseEditorBean<Evidence> {

	private static final Log LOG = LogFactoryUtil.getLog(EvidenceEdit.class);

	private PermissionsList<Evidence> pList = null;
	private Competence linkedCompetence;

	/**
	 * constructor.
	 */
	public EvidenceEdit() {
		super();
		checkEditorInput();
	}

	@Override
	protected void initFields(final Evidence evidence) {
		linkedCompetence = (Competence) PortletUtils.getSessionMap().get(PortfolioDefines.EVIDENCE_LINKED_COMPETENCE);

		setTitle(evidence.getTitle());
		setDescriptionForInternalUse(evidence.getDescription());

		if (evidence.getDate() == null) {
			evidence.setDate(new GregorianCalendar().getTime());
		}

		pList = new PermissionsList<Evidence>(null, getEditorInput());
	}

	@Override
	public String cancel() {
		return handleNavigation();
	}

	@Override
	public String save() {
		doSave(getEditorInput());

		return handleNavigation();
	}
	
	private String handleNavigation() {
		// Set the competence back on the session if needed.
		if (PortletUtils.getObjectFromSession("FROM_COMPETENCE_VIEW", false)) {
			PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, linkedCompetence);
			return Outcomes.VIEW_PORTFOLIO_COMPETENCE;
		} else {
			return Outcomes.VIEW_PORTFOLIO_EVIDENCE;
		}
	}

	/**
	 * Save the evidence.
	 * 
	 * @param evidence
	 *            Evidence
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public void doSave(final Evidence evidence) {
		evidence.setTitle(getTitleForInternalUse());
		evidence.setDescription(getDescriptionForInternalUse());

		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (evidence.isNew()) {
			evidence.setTargetType(EntityUtils.getInterfaceFromEntity(linkedCompetence));
			evidence.setTarget(linkedCompetence.getPrimaryKey());

			ctx.setCommunityPermissions(pList.getCommunityPermissions());
			ctx.setGuestPermissions(pList.getGuestPermissions());

			try {
				PortletUtils.setOwnership(evidence, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				EvidenceFacade.addEvidence(evidence, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			evidence.setNew(false);
		} else {
			try {
				EvidenceFacade.updateEvidence(evidence, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		// Refresh the calling page
		PortletUtils.getSessionMap().put(Defines.INIT_FIELDS, true);
	}

	/**
	 * Gets the evidence.
	 * 
	 * @return the evidence
	 */
	public Evidence getEvidence() {
		return getEditorInput();
	}

	/**
	 * Gets a list of permissions for the actual Evidence instance.
	 * 
	 * @return pList
	 */
	public PermissionsList<Evidence> getPermissionsList() {
		return pList;
	}

	/**
	 * Get the change effect.
	 * 
	 * @return effect
	 */

	/**
	 * Gets the linked competence.
	 * 
	 * @return the linked competence
	 */
	public Competence getLinkedCompetence() {
		return linkedCompetence;
	}
}
