/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.portlet.common.components.ElementController;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.portfolio.common.components.PortfolioFullView;

/**
 * This is the managed bean for the PortfolioActivityView.
 * 
 * @author Anton Finders
 * @version $Revision: 1.11 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioActivityView extends PortfolioFullView<ILearningAction> {

	private ElementController elementController;
	private final ILearningAction action;
	private final ILearningPath path;

	/**
	 * Instantiates a new PortfolioActivityView.
	 */
	public PortfolioActivityView() {
		super();

		action = getEntity();
		path = action.getLearningPath();
	}

	/** {@inheritDoc} */
	@Override
	protected String getPrimaryKey() {
		return getEntity().getUri();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
		super.initFields();
		elementController = new ElementController(getEntity(), false, false, true) {

			@Override
			public String handleClickEvent(final ILPElement element) {
				return null;
			}
		};		
	}

	/**
	 * Gets the element controller.
	 * 
	 * @return the element controller
	 */
	public ElementController getElementController() {
		return elementController;
	}

	/**
	 * Go to the LearningPath view.
	 * 
	 * @return Outcome
	 */
	public String selectLearningPath() {
		PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, path);
		PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
		return Outcomes.VIEW_PORTFOLIO_LEARNINGPATH;
	}

	/**
	 * Gets the learning path title.
	 * 
	 * @return the learning path title
	 */
	public String getLearningPathTitle() {
		return path.getTitle();
	}

	/**
	 * Gets the activity url.
	 * 
	 * @return the activity url
	 */
	public String getActivityURL() {
		return action.getUri();
	}

}
