/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.portfolio.beans;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.portlet.PortletPreferences;
import javax.portlet.ReadOnlyException;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.portfolio.common.PortfolioDefines;
import org.tencompetence.portlet.portfolio.common.components.PortfolioPreferencesComponent;

/**
 * Preferences. Implementation of the portfolio preference component case of admin preferences
 * 
 * @author Leon Schaeps
 * @author Anton Finders
 * @version $Revision: 1.18 $, $Date: 2009/11/27 16:22:30 $
 */
public class PortfolioAdminPreferencesController extends PortfolioPreferencesComponent {

	/** The Constant LOG. */
	// private static final Log LOG = LogFactoryUtil.getLog(PortfolioAdminPreferencesController.class);
	/**
	 * Instantiates a new portfolio admin preferences controller.
	 * 
	 */
	public PortfolioAdminPreferencesController() {
		super();
		// get the currently saved preference settings
		setSelectedEntities(getPrefs().getValues(PortfolioDefines.SELECTED_ENTITIES, new String[1]));
		setSelectedEvents(getPrefs().getValues(PortfolioDefines.SELECTED_EVENTS, new String[1]));
		setDateRangeType(getPrefs().getValue(PortfolioDefines.DATE_RANGE, "LAST_TWO_WEEKS"));
		if (getPrefs().getValue(PortfolioDefines.SHOW_EVENTS, "1").equals("1")) {
			setShowEvents(true);
		} else {
			setShowEvents(false);
		}
		setActionButtonLabel(Bundle.getLabel("button.save"));
		setShowEventCheckbox(true);
	}

	/** {@inheritDoc} */
	@Override
	public List<SelectItem> getEntities() {

		final List<String> list = Arrays.asList(Competence.class.getName(), Evidence.class.getName(), Goal.class
				.getName(), CompetenceProfileLevel.class.getName(), ILearningAction.class.getName(),
				ILearningPath.class.getName(), ILearningPlan.class.getName());
		final List<SelectItem> entitiesList = new ArrayList<SelectItem>();
		for (final String entityClass : list) {
			entitiesList.add(new SelectItem(entityClass, Bundle.getLabel(entityClass)));
		}

		return entitiesList;
	}

	/** {@inheritDoc} */
	@Override
	public List<SelectItem> getEventTypes() {
		final List<EventType> list = Arrays.asList(EventType.values());

		final List<SelectItem> eventItems = new ArrayList<SelectItem>();
		for (final EventType item : list) {
			// Exclude this eventtype
			if (!item.equals(EventType.FILTER_OPTIONS))   {
				eventItems.add(new SelectItem(item.name(), Bundle.getLabel(item.name())));
			}
		}

		return eventItems;
	}

	/** {@inheritDoc} */
	@Override
	public void doAction(final ActionEvent ae) {
		doSave(ae);
	}

	/** {@inheritDoc} */
	@Override
	protected void updatePreferences(final PortletPreferences prefs) throws ReadOnlyException {
		prefs.setValues(PortfolioDefines.SELECTED_ENTITIES, getSelectedEntities());
		prefs.setValues(PortfolioDefines.SELECTED_EVENTS, getSelectedEvents());
		prefs.setValue(PortfolioDefines.DATE_RANGE, getDateRangeType());
		if (isShowEvents()) {
			prefs.setValue(PortfolioDefines.SHOW_EVENTS, "1");
		} else {
			prefs.setValue(PortfolioDefines.SHOW_EVENTS, "0");
		}

	}

	/** {@inheritDoc} */
	@Override
	public boolean isSelectCommunityDropdownVisible() {
		return true;
	}
}