/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.portfolio.common.components.PortfolioFullView;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;

/**
 * This is the managed bean for the PortfolioCompetenceProfileLevelView.
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.9 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioCompetenceProfileLevelView extends PortfolioFullView<CompetenceProfileLevel> {
	private static final Log LOG = LogFactoryUtil.getLog(PortfolioCompetenceProfileLevelView.class);

	private final CompetenceProfileLevel profileLevel;
	private Goal goal;

	/**
	 * Instantiates a new PortfolioCompetenceProfileLevelView.
	 */
	public PortfolioCompetenceProfileLevelView() {
		super();

		profileLevel = getEntity();
	}

	/** {@inheritDoc} */
	@Override
	protected String getPrimaryKey() {
		return profileLevel.getPrimaryKeyObj().toString();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
		super.initFields();

		try {
			goal = GoalFacade.getGoalByCompetenceProfileLevelId(profileLevel.getPrimaryKey());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Checks if is target valid.
	 * 
	 * @return true, if is target valid
	 */
	public boolean isGoalValid() {
		return goal != null;
	}
	
	/**
	 * Gets the title of the goal.
	 * 
	 * @return the targets title
	 */
	public String getGoalTitle() {
		return goal != null ? goal.getTitle() : StringPool.BLANK;
	}

	/**
	 * Go to the Goal view.
	 * 
	 * @return Outcome
	 */
	public String selectGoal() {
		// Pass the Goal to the next page
		PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, goal);
		PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
		return Outcomes.VIEW_PORTFOLIO_GOAL;
	}
}
