/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;

import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.EvidenceFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.assessment.beans.controller.RatingController;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.IAssessementDialogInput;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;
import org.tencompetence.portlet.portfolio.common.PortfolioDefines;
import org.tencompetence.portlet.portfolio.common.components.PortfolioFullView;
import org.tencompetence.portlet.portfolio.common.components.PortfolioViewRowItem;

import com.icesoft.faces.context.effects.JavascriptContext;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the managed bean for the PortfolioCompetenceView.
 * 
 * @author sheyenrath
 * @version $Revision: 1.38 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioCompetenceView extends PortfolioFullView<Competence> implements IAssessementDialogInput {

	private static final Log LOG = LogFactoryUtil.getLog(PortfolioCompetenceView.class);
	private final Map<String, Map<String, String>> outcomeMapping = new HashMap<String, Map<String, String>>();

	private ActionsMenuListController<EvidenceRowItem> evidenceList;
	private final Competence competence;
	private LevelValue levelValue;
	private RatingController ratingController;

	/**
	 * Instantiates a new PortfolioCompetenceView.
	 */
	public PortfolioCompetenceView() {
		super();

		competence = (Competence) getEntity();

		// Add needed js libraries.
		JavascriptContext.includeLib("/js/ratingComponent.js", FacesContext.getCurrentInstance());
		
		PortletUtils.setObjectToSession(PortfolioDefines.EVIDENCE_LINKED_COMPETENCE, competence);
	}

	/**
	 * The Class EvidenceRowItem.
	 */
	public class EvidenceRowItem extends PortfolioViewRowItem<Evidence> {

		/**
		 * Instantiates a new EvidenceRowItem.
		 * 
		 * @param data
		 *            the data
		 * @param id
		 *            the id
		 */
		public EvidenceRowItem(final Evidence data, final Long id) {
			super(data, id);
		}

		@Override
		public boolean isAllowEdit() {
			return checkAllow(TENCActionKey.UPDATE);
		}

	}

	/** {@inheritDoc} */
	@Override
	protected String getPrimaryKey() {
		return competence.getPrimaryKeyObj().toString();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
		super.initFields();

		// Set outcome mappings
		final Map<String, String> viewMap = new HashMap<String, String>();
		viewMap.put(Evidence.class.getName(), Outcomes.VIEW_PORTFOLIO_EVIDENCE);
		outcomeMapping.put(Outcomes.VIEW, viewMap);

		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			initRatingController(ctx);
			initEvidences(ctx);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private void initRatingController(final ServiceContext ctx) throws SystemException, PortalException {
		// get the attained levels
		levelValue = PdpService.getAttainedLevelForUser(ctx.getUserId(), competence, ctx);
		ratingController = new RatingController(LevelValueFacade.getRequirableCompetenceLevelValues(competence
				.getCompetenceMapId(), ctx), levelValue, 0);
	}

	private void initEvidences(final ServiceContext ctx) throws SystemException {
		final List<EvidenceRowItem> result = new ArrayList<EvidenceRowItem>();

		for (final Evidence evidence : EvidenceFacade.getEvidencesByUserTargetTypeTargetId(ctx.getUserId(), EntityUtils
				.getInterfaceFromEntity(competence), competence.getPrimaryKey())) {
			result.add(new EvidenceRowItem(evidence, evidence.getPrimaryKey()));
		}

		evidenceList = new ActionsMenuListController<EvidenceRowItem>();
		evidenceList.setInput(result);
		evidenceList.setMaximizedUrl(getMaximizedUrl());
		evidenceList.setNormalUrl(getNormalUrl());
		evidenceList.setOutcomeMapping(outcomeMapping);
	}

	/**
	 * Gets the evidence list controller.
	 * 
	 * @return the evidence list
	 */
	public ActionsMenuListController<EvidenceRowItem> getEvidenceList() {
		return evidenceList;
	}

	/**
	 * Adds a new evidence.
	 * 
	 * @return the string
	 */
	public String addEvidence() {
		try {
			// create a new Evidence and pass to the next page
			PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, EvidenceFacade.createEvidence());
			PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
			PortletUtils.setObjectToSession("FROM_COMPETENCE_VIEW", true);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return Outcomes.ADD;
	}

	/**
	 * LevelValue.
	 * 
	 * @return levelValue
	 */
	public LevelValue getLevelValue() {
		return levelValue;
	}

	/**
	 * doSave.
	 * 
	 * @param levelValue
	 *            LevelValue
	 */
	//@Override
	public void doSave(final LevelValue levelValue) {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			PdpService.setAttainedLevelForUser(competence, levelValue, ctx.getUserId(), ctx);

			// The value is saved, now the underlying view should be refreshed. See AssessmentFullPage
			PortletUtils.getSessionMap().put(Defines.INIT_FIELDS, true);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	//@Override
	public CompetenceLevel getCompetenceLevel() {
		try {
			final List<CompetenceLevel> levelList = CompetenceLevelFacade.getCompetenceLevelsByCompetenceMapId(competence
					.getCompetenceMapId(), PortletUtils.getServiceContext());
			if (levelValue == null) {
				return levelList.get(0);

			} else {
				for (final CompetenceLevel cLevel : levelList) {
					if (cLevel.getLevelValueId() == levelValue.getPrimaryKey()) {
						return cLevel;
					}
				}
			}
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * getRatingController.
	 * 
	 * @return ratingController
	 */
	public RatingController getRatingController() {
		return ratingController;
	}
}
