/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.EvidenceFacade;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.portfolio.common.PortfolioDefines;
import org.tencompetence.portlet.portfolio.common.components.PortfolioFullView;
import org.tencompetence.portlet.portfolio.common.components.PortfolioViewRowItem;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the managed bean for the PortfolioEvidenceView.
 * 
 * @author sheyenrath
 * @version $Revision: 1.15 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioEvidenceView extends PortfolioFullView<Evidence> {

	private static final Log LOG = LogFactoryUtil.getLog(PortfolioEvidenceView.class);
	private final DateFormat dateFormatter = new SimpleDateFormat("dd-MM-yyyy");
	private final Map<String, Map<String, String>> outcomeMapping = new HashMap<String, Map<String, String>>();
	private ActionsMenuListController<PortfolioViewRowItem<Competence>> competenceList;

	private final Evidence evidence;

	/**
	 * Instantiates a new PortfolioCompetenceView.
	 */
	public PortfolioEvidenceView() {
		super();

		evidence = getEntity();
	}

	/** {@inheritDoc} */
	@Override
	protected String getPrimaryKey() {
		return evidence.getPrimaryKeyObj().toString();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
		super.initFields();

		// Set outcome mappings
		final Map<String, String> viewMap = new HashMap<String, String>();
		viewMap.put(Competence.class.getName(), Outcomes.VIEW_PORTFOLIO_COMPETENCE);
		outcomeMapping.put(Outcomes.VIEW, viewMap);

		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			initCompetences(ctx);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private void initCompetences(final ServiceContext ctx) throws PortalException, SystemException {
		final List<PortfolioViewRowItem<Competence>> result = new ArrayList<PortfolioViewRowItem<Competence>>();

		final List<Long> competenceIds = EvidenceFacade.getEntityIdsByEvidenceId(evidence.getPrimaryKey(),
				Competence.class.getName());
		for (final Competence entry : CompetenceFacade.getCompetenceByIds(competenceIds, ctx)) {
			result.add(new PortfolioViewRowItem<Competence>(entry, entry.getPrimaryKey()));
		}

		competenceList = new ActionsMenuListController<PortfolioViewRowItem<Competence>>();
		competenceList.setInput(result);
		competenceList.setMaximizedUrl(getMaximizedUrl());
		competenceList.setNormalUrl(getNormalUrl());
		competenceList.setOutcomeMapping(outcomeMapping);
	}

	/**
	 * Gets the Competence list controller.
	 * 
	 * @return the CompetenceList
	 */
	public ActionsMenuListController<PortfolioViewRowItem<Competence>> getCompetenceList() {
		return competenceList;
	}

	/**
	 * Gets the formatted date from the Evidence.
	 * 
	 * @return the date
	 */
	public String getDate() {
		return dateFormatter.format(evidence.getDate());
	}

	/**
	 * Edit the evidence.
	 * 
	 * @return the outcome
	 */
	public String editEvidence() {
		try {
			// Pass Evidence to the next page
			PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, evidence);

			// Get the linked competence and set it on the session.
			if (Competence.class.getName().equals(evidence.getTargetType())) {
				final Competence competence = CompetenceFacade.getCompetence(evidence.getTarget(), PortletUtils
						.getServiceContext());
				PortletUtils.setObjectToSession(PortfolioDefines.EVIDENCE_LINKED_COMPETENCE, competence);
			}

			PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException pe) {
			PortletUtils.handleError(LOG, pe);
		}

		return Outcomes.EDIT;
	}
}