/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.portfolio.common.components.PortfolioFullView;

import com.liferay.portal.kernel.util.StringPool;

/**
 * This is the managed bean for the PortfolioGoalView.
 * 
 * @author sheyenrath
 * @author venrooijma
 * @version $Revision: 1.9 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioGoalView extends PortfolioFullView<Goal> {
	private final Goal goal;
	private CompetenceProfileLevel profileLevel;

	/**
	 * Instantiates a new PortfolioGoalView.
	 */
	public PortfolioGoalView() {
		super();

		goal = (Goal) getEntity();
	}

	/** {@inheritDoc} */
	@Override
	protected String getPrimaryKey() {
		return getEntity().getPrimaryKeyObj().toString();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
		super.initFields();
		
		try {
			profileLevel = CompetenceProfileLevelFacade.getCompetenceProfileLevel(goal.getTarget(), PortletUtils
					.getServiceContext());
		} catch (Exception e) {
			profileLevel = null;
		}
	}

	/**
	 * Checks if is target valid.
	 * 
	 * @return true, if is target valid
	 */
	public boolean isTargetValid() {
		return profileLevel != null;
	}

	/**
	 * Gets the title of the target.
	 * 
	 * @return the targets title
	 */
	public String getTargetTitle() {
		return profileLevel != null ? profileLevel.getTitle() : StringPool.BLANK;
	}

	/**
	 * Go to the ProfileLevel view.
	 * 
	 * @return Outcome
	 */
	public String selectProfileLevel() {
		// Pass the ProfileLevel to the next page
		PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, profileLevel);
		PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
		return Outcomes.VIEW_PORTFOLIO_PROFILELEVEL;
	}
}