/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ElementController;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.portfolio.common.components.LPElementActionsMenuListController;
import org.tencompetence.portlet.portfolio.common.components.LPElementRowItem;
import org.tencompetence.portlet.portfolio.common.components.PortfolioFullView;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the managed bean for the PortfolioLearningPathView.
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.10 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioLearningPathView extends PortfolioFullView<ILearningPath> {
	private static final Log LOG = LogFactoryUtil.getLog(PortfolioLearningPathView.class);
	private final Map<String, Map<String, String>> outcomeMapping = new HashMap<String, Map<String, String>>();
	private LPElementActionsMenuListController<LPElementRowItem> childs;
	private ElementController elementController;

	private final ILearningPath path;

	/**
	 * Instantiates a new PortfolioLearningPathView.
	 */
	public PortfolioLearningPathView() {
		super();

		path = getEntity();

		// Set outcome mappings
		final Map<String, String> viewMap = new HashMap<String, String>();
		viewMap.put(ILearningPath.class.getName(), Outcomes.VIEW_PORTFOLIO_LEARNINGPATH);
		viewMap.put(ILearningAction.class.getName(), Outcomes.VIEW_PORTFOLIO_ACTIVITY);
		outcomeMapping.put(Outcomes.VIEW, viewMap);
	}

	/** {@inheritDoc} */
	@Override
	protected String getPrimaryKey() {
		return getEntity().getUri();
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
		super.initFields();

		elementController = new ElementController(getEntity(), false, false, true) {

			@Override
			public String handleClickEvent(final ILPElement element) {
				return null;
			}
		};

		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			initChilds(ctx);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private void initChilds(final ServiceContext ctx) throws PortalException, SystemException {
		final List<LPElementRowItem> result = new ArrayList<LPElementRowItem>();

		final List<ILearningAction> actions = path.getLearningActions();
		for (int idx = 0; idx < actions.size(); idx++) {
			final ILearningAction action = actions.get(idx);

			// Only show subscribed and completed.
			if (action.isCompleted() || action.isSubscribed()) {
				result.add(new LPElementRowItem(action, (long) idx));
			}
		}

		childs = new LPElementActionsMenuListController<LPElementRowItem>();
		childs.setInput(result);
		childs.setMaximizedUrl(getMaximizedUrl());
		childs.setNormalUrl(getNormalUrl());
		childs.setOutcomeMapping(outcomeMapping);
	}

	/**
	 * Gets the learning activities.
	 * 
	 * @return the learning activities
	 */
	public ActionsMenuListController<LPElementRowItem> getChilds() {
		return childs;
	}

	/**
	 * Gets the element controller.
	 * 
	 * @return the element controller
	 */
	public ElementController getElementController() {
		return elementController;
	}
}
