/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.beans;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;

import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.business.services.PortfolioService.PortfolioSearchResult;
import org.tencompetence.business.utils.DateUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.BasePortletBean;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.DateRangeEnum;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;
import org.tencompetence.portlet.portfolio.common.components.PortfolioActionsMenuListController;
import org.tencompetence.portlet.portfolio.common.components.PortfolioActionsMenuRowItem;
import org.tencompetence.portlet.portfolio.common.components.PortfolioCrumbController;

import com.icesoft.faces.context.effects.JavascriptContext;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the managed bean for the portfolioSmallView.jspx.
 * 
 * @author Anton Finders
 * @author Leon Schaeps
 * @author Stef Heyenrath
 * @version $Revision: 1.47 $, $Date: 2009/12/01 09:37:45 $
 */
public class PortfolioSmallView extends BasePortletBean {

	private static final Log LOG = LogFactoryUtil.getLog(PortfolioSmallView.class);
	private final String maximizedUrl = UrlUtils.getWindowStateMaximizedUrl();
	private final String normalUrl = UrlUtils.getWindowStateNormalUrl();

	private PortfolioActionsMenuListController<PortfolioActionsMenuRowItem> journalListController;
	private PortfolioActionsMenuListController<PortfolioActionsMenuRowItem> entityListController;
	private PortfolioUserPreferencesController prefsController;
	private boolean collapsed;
	private String searchParameter = "";
	private boolean buttonFromVisible;
	private boolean buttonToVisible;
	private boolean needsInit;

	private final Map<String, Map<String, String>> outcomeMapping = new HashMap<String, Map<String, String>>();

	/**
	 * Gets the journal list controller.
	 * 
	 * @return the journal list controller
	 */
	public ActionsMenuListController<PortfolioActionsMenuRowItem> getJournalListController() {
		initFields();
		return journalListController;
	}

	/**
	 * Gets the entity list controller.
	 * 
	 * @return the entity list controller
	 */
	public ActionsMenuListController<PortfolioActionsMenuRowItem> getEntityListController() {
		initFields();
		return entityListController;
	}

	/**
	 * Checks if is button from visible.
	 * 
	 * @return true, if is button from visible
	 */
	public boolean isButtonFromVisible() {
		return buttonFromVisible;
	}

	/**
	 * Checks if is button to visible.
	 * 
	 * @return true, if is button to visible
	 */
	public boolean isButtonToVisible() {
		return buttonToVisible;
	}

	/**
	 * Gets the search parameter.
	 * 
	 * @return the search parameter
	 */
	public String getSearchParameter() {
		return searchParameter;
	}

	/**
	 * Sets the search parameter.
	 * 
	 * @param searchParameter
	 *            the new search parameter
	 */
	public void setSearchParameter(final String searchParameter) {
		this.searchParameter = searchParameter;
	}

	/**
	 * Instantiates a new assessment small view.
	 */
	public PortfolioSmallView() {
		needsInit = true;

		// Add needed js libraries.
		JavascriptContext.includeLib("/js/ratingComponent.js", FacesContext.getCurrentInstance());
		prefsController = new PortfolioUserPreferencesController();

		// Set outcome mappings
		final Map<String, String> viewMap = new HashMap<String, String>();
		viewMap.put(Competence.class.getName(), Outcomes.VIEW_PORTFOLIO_COMPETENCE);
		viewMap.put(CompetenceProfileLevel.class.getName(), Outcomes.VIEW_PORTFOLIO_PROFILELEVEL);
		viewMap.put(Goal.class.getName(), Outcomes.VIEW_PORTFOLIO_GOAL);
		viewMap.put(Evidence.class.getName(), Outcomes.VIEW_PORTFOLIO_EVIDENCE);
		viewMap.put(ILearningAction.class.getName(), Outcomes.VIEW_PORTFOLIO_ACTIVITY);
		viewMap.put(ILearningPath.class.getName(), Outcomes.VIEW_PORTFOLIO_LEARNINGPATH);
		viewMap.put(ILearningPlan.class.getName(), Outcomes.VIEW_PORTFOLIO_LEARNINGPLAN);
		outcomeMapping.put(Outcomes.VIEW, viewMap);
	}

	/**
	 * Gets the controller.
	 * 
	 * @return the controller
	 */
	public PortfolioUserPreferencesController getPrefsController() {
		return prefsController;
	}

	/**
	 * Checks if is description expanded.
	 * 
	 * @return true, if is description expanded
	 */
	public boolean isCollapsed() {
		return this.collapsed;
	}

	/**
	 * Toggles between expanded state of advanced search properties on the page.
	 * 
	 * @return null as we remain on the same page
	 */
	public String toggle() {
		this.collapsed = !this.collapsed;
		return null;
	}

	/**
	 * Gets the toggle text for the hide/show additional information links.
	 * 
	 * @return the toggle text
	 */
	public String getToggleText() {
		return this.collapsed ? Bundle.getLabel("label.portfolio.advancedsearchhide") : Bundle
				.getLabel("label.portfolio.advancedsearch");
	}

	/**
	 * Search with filter on title.
	 * 
	 * @return the string
	 */
	public String searchWithFilter() {
		collapsed = false;
		Date maxDate = DateUtils.getMaximizedCurrentDate();

		final Date minDate = DateRangeEnum.getDate(DateRangeEnum.valueOf(prefsController.getDateRangeType()), maxDate);
		if (minDate == null) {
			maxDate = null;
		}

		search(prefsController.getSelectedEntities(), prefsController.getSelectedEvents(), getSearchParameter(),
				minDate, maxDate);
		return null;
	}

	/**
	 * Do search.
	 * 
	 * 
	 * @param entities
	 *            the entities
	 * @param events
	 *            the events
	 * @param filter
	 *            the filter on title
	 * @param maxDate
	 *            the date (most recent date of both dates, minDate < maxDate)
	 * 
	 * @param minDate
	 *            the date (less recent date of both dates, minDate < maxDate) the end date (minDate <= maxDate)
	 */
	public void search(final String[] entities, final String[] events, final String filter, final Date minDate,
			final Date maxDate) {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();

			// If no entities selected show no entities at all
			List<String> listEntities = new ArrayList<String>();
			listEntities.add(StringPool.BLANK);
			if (entities != null && entities.length > 0) {
				listEntities = Arrays.asList(entities);
			}
			// If no events selected show no events at all
			List<String> listEvents = new ArrayList<String>();
			listEvents.add(StringPool.BLANK);
			if (events != null && events.length > 0) {
				listEvents = Arrays.asList(events);
			}

			if (prefsController.isDisplayEventsBox()) {
				final List<PortfolioSearchResult> resultEntries = PortfolioService
						.getFilteredEntriesByUCGClassesEventTypesDateRange(ctx.getUserId(), ctx.getCompanyId(),
								getGroupId(ctx), listEntities, listEvents, minDate, maxDate, filter, ctx);

				final List<PortfolioActionsMenuRowItem> result = new ArrayList<PortfolioActionsMenuRowItem>(
						resultEntries.size());

				for (final PortfolioSearchResult entry : resultEntries) {
					final PortfolioActionsMenuRowItem row = new PortfolioActionsMenuRowItem(entry, entry.getEntry()
							.getPrimaryKey());
					result.add(row);
				}

				journalListController = createController(result);
			} else {

				final List<PortfolioSearchResult> resultEntries = PortfolioService
						.getFilteredEntitiesByUCGClassesEventTypesDateRange(ctx.getUserId(), ctx.getCompanyId(),
								getGroupId(ctx), listEntities, null, null, null, filter, ctx);

				final List<PortfolioActionsMenuRowItem> result = new ArrayList<PortfolioActionsMenuRowItem>(
						resultEntries.size());

				for (final PortfolioSearchResult entry : resultEntries) {
					final PortfolioActionsMenuRowItem row = new PortfolioActionsMenuRowItem(entry, entry.getEntry()
							.getPrimaryKey(), true);
					result.add(row);
				}

				entityListController = createController(result);
			}
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private long getGroupId(final ServiceContext ctx) {
		final long groupId;
		switch (getPrefsController().getPortletCommunityMode()) {
		case ALL:
			groupId = 0L; // Show entries from all communities.
			break;

		case CURRENT:
			groupId = ctx.getScopeGroupId(); // Show entries from current community.
			break;

		default:
			groupId = ctx.getScopeGroupId(); // Show entries from current community.
			break;
		}

		return groupId;
	}

	private PortfolioActionsMenuListController<PortfolioActionsMenuRowItem> createController(
			final List<PortfolioActionsMenuRowItem> result) {
		final PortfolioActionsMenuListController<PortfolioActionsMenuRowItem> controller = 
			new PortfolioActionsMenuListController<PortfolioActionsMenuRowItem>();
		controller.setInput(result);
		controller.setMaximizedUrl(maximizedUrl);
		controller.setNormalUrl(normalUrl);
		controller.setOutcomeMapping(outcomeMapping);
		return controller;
	}

	@Override
	protected void initFields() {
		if (needsInit) {
			search(prefsController.getSelectedEntities(), prefsController.getSelectedEvents(), StringPool.BLANK, null,
					null);
			needsInit = false;
		}

		// Reset the breadcrumb controller
		PortfolioCrumbController.reset();
	}
}
