/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.portfolio.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.portlet.PortletPreferences;
import javax.portlet.ReadOnlyException;

import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.portfolio.common.PortfolioDefines;
import org.tencompetence.portlet.portfolio.common.components.PortfolioPreferencesComponent;

/**
 * Preferences. Implementation of the portfolio preference component case of admin preferences
 * 
 * @author Leon Schaeps
 * @author Anton Finders
 * @version $Revision: 1.12 $, $Date: 2009/11/25 12:42:20 $
 */
public class PortfolioUserPreferencesController extends PortfolioPreferencesComponent {

	private List<SelectItem> entitiesList = new ArrayList<SelectItem>();
	private List<SelectItem> eventsList = new ArrayList<SelectItem>();

	/**
	 * Instantiates a new portfolio user preferences controller.
	 * 
	 */
	public PortfolioUserPreferencesController() {
		super();
		// get the currently saved preference settings
		setSelectedEntities(getPrefs().getValues(PortfolioDefines.SELECTED_ENTITIES, null));
		setSelectedEvents(getPrefs().getValues(PortfolioDefines.SELECTED_EVENTS, null));
		setDateRangeType(getPrefs().getValue(PortfolioDefines.DATE_RANGE, "LAST_TWO_WEEKS"));
		if (getPrefs().getValue(PortfolioDefines.SHOW_EVENTS, "1").equals("1")) {
			setDisplayEventsBox(true);
		} else {
			setDisplayEventsBox(false);
		}

		String[] entities = getSelectedEntities();
		if (entities != null) {
			for (String item : entities) {
				if (!item.equals("0")) {
					SelectItem selectItem = new SelectItem(item, Bundle.getLabel(item));
					entitiesList.add(selectItem);
				}
			}
		}

		String[] envents = getSelectedEvents();
		if (envents != null) {
			for (String item : envents) {
				if (!item.equals("0")) {
					SelectItem selectItem = new SelectItem(item, Bundle.getLabel(item));
					eventsList.add(selectItem);
				}
			}
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.tencompetence.portlet.portfolio.common.components.PortfolioPreferencesComponent#getEntities()
	 */
	@Override
	public List<SelectItem> getEntities() {
		return entitiesList;
	}

	/** {@inheritDoc} */	@Override
	public List<SelectItem> getEventTypes() {
		return eventsList;
	}

	/** {@inheritDoc} */
	@Override
	protected void updatePreferences(final PortletPreferences prefs) throws ReadOnlyException {		
		// User is not allowed to update the Preferences. 
	}

	/** {@inheritDoc} */
	@Override
	public void doAction(final ActionEvent ae) {
		
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSelectCommunityDropdownVisible() {
		return false;
	}
}
