/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.portfolio.common.components;

import javax.portlet.RenderRequest;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.PortletUtils;

/**
 * This class implements a controller for ICEFace components supporting ice:dataTable with an actionsMenu column.
 * 
 * @param <R>
 *            The generic row item
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.2 $, $Date: 2009/11/03 08:27:40 $
 */
public class LPElementActionsMenuListController<R extends ActionsMenuRowItem<? extends ILPElement>> extends
		ActionsMenuListController<R> {

	/**
	 * Foward to next page.
	 * 
	 * @param rowItem
	 *            rowItem
	 * @param outcome
	 *            outcome
	 */
	@Override
	protected void forwardToNextPage(final ActionsMenuRowItem<?> rowItem, final String outcome) {

		if (outcome != null && rowItem.getData() instanceof ILPElement) {
			final RenderRequest request = PortletUtils.getRenderRequest();
			final ILPElement result = (ILPElement) rowItem.getData();
			final String className = result.getClass().getInterfaces()[0].getName();

			PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, result);
			PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);

			final String newOutcome = getTargetOutcome(className, outcome);
			navigateToNextOutcome(request, newOutcome);
		}
	}
}
