/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.common.components;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.FacadeUtils;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.services.PortfolioService.PortfolioSearchResult;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.RatingComponentController;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class PortfolioActionsMenuRowItem.
 * 
 * An extension to the ActionsMenuRowItem so we can display an icon in the frontEnd
 * @version $Revision: 1.6 $, $Date: 2009/11/24 14:10:06 $
 */
public class PortfolioActionsMenuRowItem extends ActionsMenuRowItem<PortfolioSearchResult> {
	private static final Log LOG = LogFactoryUtil.getLog(PortfolioActionsMenuRowItem.class);
	private final DateFormat formatter = new SimpleDateFormat("dd-MM-yyyy hh:mm:ss");

	private ServiceContext ctx;
	private PortfolioRatingController ratingController;

	private EventType eventType;
	private boolean showValue = false;
	private ValueType valueType = ValueType.TEXT;
	private String iconURL = StringPool.BLANK;

	/**
	 * The Enum valueType.
	 */
	private enum ValueType {
		TEXT, IMAGE, ATTAINED_LEVEL
	}

	/**
	 * The Class PortfolioRatingController.
	 */
	private class PortfolioRatingController extends RatingComponentController {

		/**
		 * Instantiates a new Portfolio rating controller.
		 * 
		 * @param levels
		 *            the levels
		 * @param assignedLevel
		 *            the assigned level
		 * @param requiredLevelId
		 *            the required level id
		 */
		public PortfolioRatingController(final List<LevelValue> levels, final LevelValue assignedLevel,
				final long requiredLevelId) {
			super(levels, assignedLevel, requiredLevelId, false, false, false);
		}

		/** {@inheritDoc} */
		@Override
		public void ratingComponentClicked() {
		}
	}

	/**
	 * Instantiates a new PortfolioActions menu row item.
	 * 
	 * @param data
	 *            the data
	 * @param id
	 *            the id
	 */
	public PortfolioActionsMenuRowItem(final PortfolioSearchResult data, final Long id) {
		this(data, id, false);
	}

	/**
	 * Instantiates a new PortfolioActions actions menu row item.
	 * 
	 * @param data
	 *            the data
	 * @param id
	 *            the id
	 * @param showValue
	 *            showValue
	 */
	public PortfolioActionsMenuRowItem(final PortfolioSearchResult data, final Long id, final boolean showValue) {
		super(data, id);
		this.showValue = showValue;

		eventType = EventType.valueOf(data.getEntry().getEventType());

		initRow();
	}

	private void initRow() {
		if (getData().getObj() instanceof Competence) {
			valueType = ValueType.ATTAINED_LEVEL;
			initRatingController();
		}

		if (getData().getObj() instanceof ILearningAction) {
			final ILearningAction action = (ILearningAction) getData().getObj();

			// Set the value column to the title from the learningpath
			getData().getEntry().setValue(action.getLearningPath().getTitle());
		}

		if (getData().getObj() instanceof Goal && eventType == EventType.GOAL_SELECTED) {
			// Set the value column to the title from the map
			final long groupId = Long.parseLong(getData().getEntry().getValue());
			String value;
			try {
				value = CompetenceMapFacade.getCompetenceMapByGroupId(groupId, ctx).getTitle();
			} catch (Exception e) {
				value = StringPool.BLANK;
			}
			getData().getEntry().setValue(value);
		}

		if (getData().getObj() instanceof ILearningPlan) {

			// Make the value empty for the LEARNING_PLAN_CURSOR
			if (eventType == EventType.LEARNING_PLAN_CURSOR) {
				getData().getEntry().setValue("");
			}

			// Set the value column to the title from the Goal
			if (eventType == EventType.GOAL_LINKED) {
				final long goalId = Long.parseLong(getData().getEntry().getValue());
				String value;
				try {
					value = GoalFacade.getGoal(goalId, ctx).getTitle();
				} catch (Exception e) {
					value = StringPool.BLANK;
				}
				getData().getEntry().setValue(value);
			}
		}
	}

	private void initRatingController() {
		// Only create rating if needed
		if (showValue) {
			try {
				final long mapId = FacadeUtils.getCompetenceMapId((BaseModel<?>) getData().getObj());
				final List<LevelValue> allLevels = LevelValueFacade.getRequirableCompetenceLevelValues(mapId, ctx);
				final LevelValue attained;
				
				if (Validator.isNotNull(getData().getEntry().getValue())) {
					final Long attainedLevelId = (getData().getEntry().getValue() != null) ? Long.parseLong(getData()
							.getEntry().getValue()) : 0L;
					attained = LevelValueFacade.getLevelValue(attainedLevelId, ctx);
				} else {
					attained = null;
				}

				ratingController = new PortfolioRatingController(allLevels, attained, 0);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	@Override
	public boolean isAllowPermissions() {
		return false;
	}

	@Override
	public boolean isAllowEdit() {
		return false;
	}

	@Override
	public boolean isAllowView() {
		return true;
	}

	@Override
	public String getCustomTitle() {
		return formatter.format(getData().getEntry().getJournalDate());
	}

	/**
	 * @return the label of the entity
	 */
	public String getEntityName() {
		return Bundle.getLabel(getData().getEntry().getEntityClass());
	}

	/**
	 * @return the title of the entity
	 */
	public String getEntityTitle() {
		return ClassUtils.getProperty(getData().getObj(), "title", StringPool.BLANK);
	}

	/**
	 * @return the title of the event
	 */
	public String getEventTitle() {
		return Bundle.getLabel(eventType.toString());
	}

	/**
	 * Gets the value type (needed for rendering the value in the datatable).
	 * 
	 * @return the value type
	 */
	public String getValueType() {
		return valueType.toString();
	}

	/**
	 * Gets the value.
	 * 
	 * @return the value
	 */
	public String getValue() {
		return showValue ? getData().getEntry().getValue() : StringPool.BLANK;
	}

	/**
	 * Gets the rating controller.
	 * 
	 * @return the rating controller
	 */
	public PortfolioRatingController getRatingController() {
		return ratingController;
	}

	/**
	 * Gets the icon.
	 * 
	 * @return the icon
	 */
	public String getIcon() {
		return iconURL;
	}
}
