/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.common.components;

import javax.faces.context.FacesContext;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.business.utils.DateUtils;
import org.tencompetence.portlet.common.components.CrumbController;
import org.tencompetence.portlet.common.components.InitEntityController;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.ImageUtils;

/**
 * The Class PortfolioDetailsController.
 * 
 * @param <T>
 *            GenericType
 * @author sheyenrath
 * @version $Revision: 1.9 $, $Date: 2009/12/01 09:37:45 $
 */
public class PortfolioHeaderController<T> extends InitEntityController<T> {

	private final String entityTitle;
	private final String entityName;

	private PortfolioFullView<T> parent;
	private PortfolioCrumbController crumbController;

	/**
	 * Instantiates a new PortfolioHeaderController.
	 * 
	 * @param entity
	 *            BaseModel<?> entity
	 * @param page
	 *            the parentpage
	 */
	public PortfolioHeaderController(final PortfolioFullView<T> page, final T entity) {
		parent = page;
		setEntity(entity);

		entityTitle = ClassUtils.getProperty(entity, "title", "");
		entityName = Bundle.getLabel(entity.getClass().getInterfaces()[0].getName());

		// create new controller for crumble control
		crumbController = PortfolioCrumbController.getController();

		// update the crumb controller
		crumbController.processEditorInput(entity, FacesContext.getCurrentInstance().getViewRoot().getViewId());
	}

	/**
	 * Gets the crumb controller.
	 * 
	 * @return the crumb controller
	 */
	public CrumbController getCrumbController() {
		return crumbController;
	}

	@Override
	protected void initFields() {
	}

	/**
	 * @return the name of the entity
	 */
	public String getEntityName() {
		return entityName;
	}

	/**
	 * @return the title of the entity
	 */
	public String getEntityTitle() {
		return entityTitle;
	}

	/**
	 * Gets the image url.
	 * 
	 * @return the image url
	 */
	public String getImageUrl() {
		return ImageUtils.getImageUrl(getEntity());
	}

	/**
	 * Gets the legenda.
	 * 
	 * @return the legenda
	 */
	public String getLegenda() {
		final String result;

		if (getEntity() instanceof ILPElement) {
			final ILPElement element = (ILPElement) getEntity();
			// Get completed or subscribed + Date
			if (element.isCompleted()) {
				result = Bundle.getLabel("elementcontroller.completed") + "<br />"
						+ DateUtils.dateToString(element.getCompletedDate());
			} else if (element.isSubscribed()) {
				result = Bundle.getLabel("elementcontroller.subscribed") + "<br />"
						+ DateUtils.dateToString(element.getSubscribeDate());
			} else {
				result = "";
			}
		} else {
			result = "";
		}

		return result;
	}

	/**
	 * Gets the parent page (PortfolioFullView).
	 * 
	 * @return the parent
	 */
	public PortfolioFullView<T> getParent() {
		return parent;
	}
}