/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.portfolio.common.components;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

import org.tencompetence.portlet.common.components.BasePreferences;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.DateRangeEnum;

/**
 * The Class PortfolioPreferencesComponent. Abstract controller class for generic preferences of the Portfolio portlet.
 * 
 * @author Anton Finders
 * @author Leon Schaeps
 * @version $Revision: 1.12 $, $Date: 2009/11/25 12:42:20 $
 */
public abstract class PortfolioPreferencesComponent extends BasePreferences {

	private String[] selectedEntities;
	private String[] selectedEvents;
	private boolean showEvents;
	private String actionButtonLabel = Bundle.getLabel("button.search");
	private boolean showEventCheckbox = false;
	private boolean displayEventsBox = true;
	private String dateRange;

	/**
	 * Gets the date range type.
	 * 
	 * @return the date range type
	 */
	public String getDateRangeType() {
		return dateRange;
	}

	/**
	 * Sets the date range type.
	 * 
	 * @param dateRange
	 *            the new date range type
	 */
	public void setDateRangeType(final String dateRange) {
		this.dateRange = dateRange;
	}

	/**
	 * Gets the types.
	 * 
	 * @return the types
	 */
	public List<SelectItem> getDateRangeTypes() {
		final List<SelectItem> items = new ArrayList<SelectItem>();
		for (final DateRangeEnum value : DateRangeEnum.values()) {
			items.add(new SelectItem(value.name(), value.getLabel()));
		}
		return items;
	}

	/**
	 * Gets the selected events.
	 * 
	 * @return the selected events
	 */
	public String[] getSelectedEvents() {
		return selectedEvents;
	}

	/**
	 * Sets the selected events.
	 * 
	 * @param selectedEvents
	 *            the new selected events
	 */
	public void setSelectedEvents(final String[] selectedEvents) {
		this.selectedEvents = selectedEvents;
	}

	/**
	 * Checks if is show events.
	 * 
	 * @return true, if is show events
	 */
	public boolean isShowEvents() {
		return showEvents;
	}

	/**
	 * Sets the show events.
	 * 
	 * @param showEvents
	 *            the new show events
	 */
	public void setShowEvents(final boolean showEvents) {
		this.showEvents = showEvents;
	}

	/**
	 * Sets the selected entities.
	 * 
	 * @param selectedEntities
	 *            the new selected entities
	 */
	public void setSelectedEntities(final String[] selectedEntities) {
		this.selectedEntities = selectedEntities;
	}

	/**
	 * Gets the selected entities.
	 * 
	 * @return the selected entities
	 */
	public String[] getSelectedEntities() {
		return selectedEntities;
	}

	/**
	 * Sets the action button label.
	 * 
	 * @param label
	 *            the new action button label
	 */
	public void setActionButtonLabel(final String label) {
		actionButtonLabel = label;
	}

	/**
	 * Gets the action button label.
	 * 
	 * @return the action button label
	 */
	public String getActionButtonLabel() {
		return actionButtonLabel;
	}

	/**
	 * Sets the show event checkbox.
	 * 
	 * @param showEventCheckbox
	 *            the new show event checkbox
	 */
	public void setShowEventCheckbox(final boolean showEventCheckbox) {
		this.showEventCheckbox = showEventCheckbox;
	}

	/**
	 * Checks if is show event checkbox.
	 * 
	 * @return true, if is show event checkbox
	 */
	public boolean isShowEventCheckbox() {
		return showEventCheckbox;
	}

	/**
	 * Checks if is display events box.
	 * 
	 * @return true, if is display events box
	 */
	public boolean isDisplayEventsBox() {
		return displayEventsBox;
	}

	/**
	 * selectAllEntities.
	 * 
	 * @param ae
	 *            ActionEvent
	 */
	public void selectAllEntities(final ActionEvent ae) {
		int iSize = getEntities().size();
		if (iSize > 0) {
			List<SelectItem> entities = getEntities();
			selectedEntities = new String[iSize];
			for (int i = 0; i < iSize; i++) {
				selectedEntities[i] = (String) entities.get(i).getValue();
			}
		}
	}

	/**
	 * selectNoneEntities.
	 * 
	 * @param ae
	 *            ActionEvent
	 */
	public void selectNoneEntities(final ActionEvent ae) {
		selectedEntities = new String[1];
	}

	/**
	 * selectAllEvents.
	 * 
	 * @param ae
	 *            ActionEvent
	 */
	public void selectAllEvents(final ActionEvent ae) {
		int iSize = getEventTypes().size();
		if (iSize > 0) {
			List<SelectItem> events = getEventTypes();
			selectedEvents = new String[iSize];
			for (int i = 0; i < iSize; i++) {
				selectedEvents[i] = (String) events.get(i).getValue();
			}
		}
	}

	/**
	 * selectNoneEvents.
	 * 
	 * @param ae
	 *            ActionEvent
	 */
	public void selectNoneEvents(final ActionEvent ae) {
		selectedEvents = new String[1];
	}

	/**
	 * Sets the display events box.
	 * 
	 * @param displayEventsBox
	 *            the new display events box
	 */
	public void setDisplayEventsBox(final boolean displayEventsBox) {
		this.displayEventsBox = displayEventsBox;
	}

	/**
	 * Do click. Implement the action when clicked on the action button
	 * 
	 * @param ae
	 *            Action Event
	 */
	public abstract void doAction(final ActionEvent ae);

	/**
	 * Gets the entities.
	 * 
	 * @return the entities
	 */
	public abstract List<SelectItem> getEntities();

	/**
	 * Gets the event types.
	 * 
	 * @return the event types
	 */
	public abstract List<SelectItem> getEventTypes();
	
	/**
	 * Checks if the SelectCommunity Dropdown is visible.
	 * 
	 * @return true, false
	 */
	public abstract boolean isSelectCommunityDropdownVisible();
}