/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.AssessmentItemFacade;
import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;
import org.tencompetence.portlet.qtieditor.common.AssessmentItemListController;
import org.tencompetence.portlet.qtieditor.common.FeedbackItem;
import org.tencompetence.portlet.qtieditor.common.FeedbackListController;
import org.tencompetence.portlet.qtieditor.common.RowItem;
import org.tencompetence.portlet.qtieditor.common.ValueMapping;
import org.tencompetence.services.xml.qti.elements.AssessmentElementFactory;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.AssessmentItemRef;
import org.tencompetence.services.xml.qti.elements.AssessmentSection;
import org.tencompetence.services.xml.qti.elements.AssessmentTestInstance;
import org.tencompetence.services.xml.qti.elements.BasicElement;
import org.tencompetence.services.xml.qti.elements.BasicElementList;
import org.tencompetence.services.xml.qti.elements.Block;
import org.tencompetence.services.xml.qti.elements.ChoiceInteraction;
import org.tencompetence.services.xml.qti.elements.ExtendedTextInteraction;
import org.tencompetence.services.xml.qti.elements.FillingInBlankBlock;
import org.tencompetence.services.xml.qti.elements.InlineChoiceBlock;
import org.tencompetence.services.xml.qti.elements.SectionPart;
import org.tencompetence.services.xml.qti.elements.TestFeedback;
import org.tencompetence.services.xml.qti.elements.TestResultReport;

import com.icesoft.faces.component.ContextActionEvent;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;

/**
 * AssessmentTestEdit backing bean.
 *
 * @author Teodor Stoev
  * @version $Revision: 1.28 $, $Date: 2009/11/26 15:25:04 $
 */
public class AssessmentTestEdit extends BaseEditorBean<AssessmentTest> {
	private static final String EDITOR_INPUT_PREV_PARAM_NAME = "editor.input.prev";
	private static final String ITEM_TO_DELETE = "item.to.delete";
	private static final int EQF_LEVEL_1 = 1;
	private static final int EQF_LEVEL_2 = 2;
	private static final int EQF_LEVEL_3 = 3;
	private static final int EQF_LEVEL_4 = 4;
	private static final int EQF_LEVEL_5 = 5;
	private static final int EQF_LEVEL_6 = 6;
	private static final int EQF_LEVEL_7 = 7;
	private static final int EQF_LEVEL_8 = 8;
	private static final Log LOG = LogFactoryUtil.getLog(AssessmentTestEdit.class);
	private List<SelectItem> eqfLevels = null;
	private boolean numberOfCorrectResponses;
	private boolean numberOfIncorrectResponses;
	private boolean numberOfPresentedQuestions;
	private boolean numberOfRespondedQuestions;
	private boolean numberOfSelectedQuestions;
	private boolean overallPercentage;
	private String selectedQuestionType;
	private FeedbackListController feedbackController;
	private AssessmentItemListController itemsController;

	private ServiceContext ctx = null;

	/**
	 * Gets the feedback controller.
	 *
	 * @return the controller
	 */
	public FeedbackListController getFeedbackController() {
		checkEditorInput();
		return feedbackController;
	}
	
	/**
	 * Gets the items controller.
	 *
	 * @return the controller
	 */
	public AssessmentItemListController getItemsController() {
		checkEditorInput();
		return itemsController;
	}

	/**
	 * Check if user is allowed to add an Assessment item.
	 *
	 * @return true / false
	 */
	public boolean isAddAssessmentItemAllowed() {
		return checkPortletPermission(TENCActionKey.ADD_ASSESSMENT_ITEM);
	}
	
	/**
	 * Check if result report has number of correct responses.
	 *
	 * @return true / false
	 */
	public boolean isNumberOfCorrectResponses() {
		return numberOfCorrectResponses;
	}

	/**
	 * Sets if result report has number of correct responses.
	 * 
	 * @param numberOfCorrectResponses
	 *            if result report has number of correct responses
	 */
	public void setNumberOfCorrectResponses(final boolean numberOfCorrectResponses) {
		this.numberOfCorrectResponses = numberOfCorrectResponses;
	}

	/**
	 * Check if result report has number of incorrect responses.
	 *
	 * @return true / false
	 */
	public boolean isNumberOfIncorrectResponses() {
		return numberOfIncorrectResponses;
	}

	/**
	 * Sets if result report has number of incorrect responses.
	 * 
	 * @param numberOfIncorrectResponses
	 *            if result report has number of incorrect responses
	 */
	public void setNumberOfIncorrectResponses(final boolean numberOfIncorrectResponses) {
		this.numberOfIncorrectResponses = numberOfIncorrectResponses;
	}

	/**
	 * Check if result report has number of presented questions.
	 *
	 * @return true / false
	 */
	public boolean isNumberOfPresentedQuestions() {
		return numberOfPresentedQuestions;
	}

	/**
	 * Sets if result report has number of presented questions.
	 * 
	 * @param numberOfPresentedQuestions
	 *            if result report has number of presented questions
	 */
	public void setNumberOfPresentedQuestions(final boolean numberOfPresentedQuestions) {
		this.numberOfPresentedQuestions = numberOfPresentedQuestions;
	}

	/**
	 * Check if result report has number of responded questions.
	 *
	 * @return true / false
	 */
	public boolean isNumberOfRespondedQuestions() {
		return numberOfRespondedQuestions;
	}

	/**
	 * Sets if result report has number of responded questions.
	 * 
	 * @param numberOfRespondedQuestions
	 *            if result report has number of responded questions
	 */
	public void setNumberOfRespondedQuestions(final boolean numberOfRespondedQuestions) {
		this.numberOfRespondedQuestions = numberOfRespondedQuestions;
	}

	/**
	 * Check if result report has number of selected questions.
	 *
	 * @return true / false
	 */
	public boolean isNumberOfSelectedQuestions() {
		return numberOfSelectedQuestions;
	}

	/**
	 * Sets if result report has number of selected questions.
	 * 
	 * @param numberOfSelectedQuestions
	 *            if result report has number of selected questions
	 */
	public void setNumberOfSelectedQuestions(final boolean numberOfSelectedQuestions) {
		this.numberOfSelectedQuestions = numberOfSelectedQuestions;
	}

	/**
	 * Check if result report has overall percentage.
	 *
	 * @return true / false
	 */
	public boolean isOverallPercentage() {
		return overallPercentage;
	}
	
	/**
	 * Sets if result report has overall percentage.
	 * 
	 * @param overallPercentage
	 *            if result report has overall percentage
	 */
	public void setOverallPercentage(final boolean overallPercentage) {
		this.overallPercentage = overallPercentage;
	}
	
	/**
	 * Adds a test feedback. Method called if add button is pressed.
	 *
	 * A new AssessmentItem is passed to the next page
	 * 
	 * @return the string
	 */
	public String addFeedback() {
		getEqfLevels();
		AssessmentTest editorInput = getEditorInput();
		AssessmentTestInstance assessmentTest = AssessmentTestInstance.fromXmlString(editorInput.getXmlData());
		assessmentTest.setTitle(getTitleForInternalUse());
		updateTestResultReport(assessmentTest);

		assessmentTest.setTestFeedbackList(getTestFeedbacks());
		TestFeedback testFeedback = new TestFeedback(assessmentTest);
		testFeedback.getEqfEntry().setTargetValue("" + EQF_LEVEL_1);
		testFeedback.setId("" + EQF_LEVEL_1);
		assessmentTest.addTestFeedback(testFeedback);

		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setXmlData(assessmentTest.toXmlString());

		return this.getConfirmationController().canLeave(Outcomes.ADD, editorInput);
	}

	/**
	 * Adds a question. Method called if add button is pressed.
	 *
	 * A new AssessmentItem is passed to the next page
	 * 
	 * @return the string
	 */
	public String addQuestion() {
		try {
			final AssessmentItem item = AssessmentItemFacade.createAssessmentItem();
			item.setNew(true);

			AssessmentItemInstance assessmentItem = new AssessmentItemInstance();
			assessmentItem.setTitle("Untitled question");
			assessmentItem.getItemBody().setQuestionClass(selectedQuestionType);

			Block block = ValueMapping.createBlockCorrespondingToQuestionType(selectedQuestionType, assessmentItem);
			assessmentItem.getItemBody().setFirstBlock(block);

			item.setTitle("Untitled question");
			item.setXmlData(assessmentItem.toXmlString());
			item.setIdentifier(assessmentItem.getId());

			AssessmentTest editorInput = getEditorInput();
			AssessmentTestInstance assessmentTest = AssessmentTestInstance.fromXmlString(editorInput.getXmlData());
			updateTestResultReport(assessmentTest);
			assessmentTest.setTestFeedbackList(getTestFeedbacks());
			
			editorInput.setTitle(getTitleForInternalUse());
			editorInput.setXmlData(assessmentTest.toXmlString());
			
			PortletUtils.getSessionMap().put(EDITOR_INPUT_PREV_PARAM_NAME, editorInput);

			if (block instanceof ChoiceInteraction) {
				return this.getConfirmationController().canLeave("CHOICE_INTERACTION", item);
			} else if (block instanceof ExtendedTextInteraction) {
				return this.getConfirmationController().canLeave("EXTENDED_TEXT_INTERACTION", item);
			} else if (block instanceof FillingInBlankBlock) {
				return this.getConfirmationController().canLeave("FILL_IN_THE_BLANK", item);
			} else if (block instanceof InlineChoiceBlock) {
				return this.getConfirmationController().canLeave("INLINE_CHOICE", item);
			}
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return null;
	}
	
	/**
	 * Gets a list of EQF levels.
	 *
	 * @return the list of EQF levels
	 */
	public List<SelectItem> getEqfLevels() {
		if (eqfLevels == null) {
			eqfLevels = new ArrayList<SelectItem>();
			
			eqfLevels.add(new SelectItem(EQF_LEVEL_1, "1"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_2, "2"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_3, "3"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_4, "4"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_5, "5"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_6, "6"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_7, "7"));
			eqfLevels.add(new SelectItem(EQF_LEVEL_8, "8"));
		}
		
		return eqfLevels;
	}
	
	private List<RowItem<FeedbackItem>> getFeedbackItemList(final AssessmentTest editorInput) {
		try {
			AssessmentTestInstance testInstance = AssessmentTestInstance.fromXmlString(editorInput.getXmlData());
			BasicElementList feedbackItemList = testInstance.getTestFeedbackList();
			List<RowItem<FeedbackItem>> result = new ArrayList<RowItem<FeedbackItem>>(feedbackItemList.size());

			for (BasicElement element : feedbackItemList) {
				TestFeedback testFeedback = (TestFeedback) element;
				result.add(new RowItem<FeedbackItem>(
						new FeedbackItem(testFeedback), (long) testFeedback.getId().hashCode()));
			}
			return result;
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	private List<ActionsMenuRowItem<AssessmentItem>> getAssessmentItems(final AssessmentTest editorInput) {
		try {
			List<AssessmentItem> assessmentItemList = AssessmentTestFacade
				.getAssessmentItemsByAssessmentTest(editorInput.getAssessmentTestId(), ctx);
			List<ActionsMenuRowItem<AssessmentItem>> result = new ArrayList<ActionsMenuRowItem<AssessmentItem>>(
					assessmentItemList.size());

			for (AssessmentItem item : assessmentItemList) {
				result.add(new ActionsMenuRowItem<AssessmentItem>(item, item.getPrimaryKey()));
			}
			return result;
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	/**
	 * Gets a list of question types.
	 *
	 * @return the list of question types
	 */
	public List<SelectItem> getQuestionTypes() {
		List<SelectItem> questionTypes = new ArrayList<SelectItem>();
		questionTypes.add(new SelectItem(AssessmentElementFactory.TEXT_BASED_MULTIPLE_CHOICE,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.TEXT_BASED_MULTIPLE_CHOICE)));
		questionTypes.add(new SelectItem(AssessmentElementFactory.TEXT_BASED_MULTIPLE_RESPONSE,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.TEXT_BASED_MULTIPLE_RESPONSE)));
		questionTypes.add(new SelectItem(AssessmentElementFactory.LIKERT,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.LIKERT)));
		questionTypes.add(new SelectItem(AssessmentElementFactory.YES_NO,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.YES_NO)));
		questionTypes.add(new SelectItem(AssessmentElementFactory.OPEN,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.OPEN)));
		questionTypes.add(new SelectItem(AssessmentElementFactory.FILL_IN_THE_BLANK,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.FILL_IN_THE_BLANK)));
		questionTypes.add(new SelectItem(AssessmentElementFactory.INLINE_CHOICE,
				ValueMapping.getUserFriendlyQuestionType(AssessmentElementFactory.INLINE_CHOICE)));

		return questionTypes;
	}

	/**
	 * Gets the selected question type.
	 * 
	 * @return the selected question type
	 */
	public String getSelectedQuestionType() {
		return selectedQuestionType;
	}

	/**
	 * Sets the selected question type.
	 * 
	 * @param questionType
	 *            the new questionType
	 */
	public void setSelectedQuestionType(final String questionType) {
		selectedQuestionType = questionType;
	}
	
	private void deleteListEntity(final BaseModel<?> entity) {
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		
		try {
			if (entity instanceof AssessmentItem) {
				AssessmentTest test = getEditorInput();
				AssessmentItem itemToDelete = (AssessmentItem) entity;
				String itemIdentifier = itemToDelete.getIdentifier();

				List<AssessmentItem> items = AssessmentTestFacade
					.getAssessmentItemsByAssessmentTest(test.getAssessmentTestId(), ctx);
				for (AssessmentItem item : items) {
					if (item.getAssessmentItemId() == itemToDelete.getAssessmentItemId()) {
						AssessmentItemFacade.deleteAssessmentItem(item, ctx);
						break;
					}
				}
				
				AssessmentTestInstance testInstance = AssessmentTestInstance.fromXmlString(test.getXmlData());
				AssessmentSection assessmentSection = testInstance.getFirstTestPart().getFirstAssessmentSection();
				
				// Remove the reference to the item as well
				SectionPart partToDelete = null;
				for (BasicElement element : assessmentSection.getSectionPartList()) {
					if (element instanceof AssessmentItemRef) {
						AssessmentItemRef itemRef = (AssessmentItemRef) element;
						if (itemRef.getHref().contains(itemIdentifier)) {
							partToDelete = itemRef;
						}
					}
				}
				if (partToDelete != null) {
					assessmentSection.removeSectionPart(partToDelete);
					test.setXmlData(testInstance.toXmlString());
				}
				
				PortletUtils.getSessionMap().put(Defines.INIT_FIELDS, true);
				PortletUtils.handleNavigation(Outcomes.DELETE);
			}
		} catch (SystemException e) {
			LOG.error(e);
		}
	}
	
	/**
	 * Listen to to delete-action event from the actions-menu.
	 * 
	 * @param event
	 *            ActionEvent
	 */
	public void deleteActionListener(final ActionEvent event) {
		if (event instanceof ContextActionEvent) {
			final ContextActionEvent contextEvent = (ContextActionEvent) event;
			final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) contextEvent.getContextValue();
			final BaseModel<?> entity = (BaseModel<?>) rowItem.getData();

			deleteListEntity(entity);
		}
	}
	
	private void setTestResultReportVariables(final AssessmentTest editorInput) {
		AssessmentTestInstance assessmentTest = AssessmentTestInstance.fromXmlString(editorInput.getXmlData());
		TestResultReport resultReport = assessmentTest.getTestResultReport();
		
		numberOfCorrectResponses = resultReport.getNumberCorrect();
		numberOfIncorrectResponses = resultReport.getNumberIncorrect();
		numberOfPresentedQuestions = resultReport.getNumberPresented();
		numberOfRespondedQuestions = resultReport.getNumberResponded();
		numberOfSelectedQuestions = resultReport.getNumberSelected();
		overallPercentage = resultReport.getPercentage();
	}
	
	private void updateTestResultReport(final AssessmentTestInstance assessmentTest) {
		TestResultReport resultReport = assessmentTest.getTestResultReport();
		
		resultReport.setNumberCorrect(numberOfCorrectResponses);
		resultReport.setNumberIncorrect(numberOfIncorrectResponses);
		resultReport.setNumberPresented(numberOfPresentedQuestions);
		resultReport.setNumberResponded(numberOfRespondedQuestions);
		resultReport.setNumberSelected(numberOfSelectedQuestions);
		resultReport.setPercentage(overallPercentage);
		System.out.println("Updating Test Result Report Score:" + resultReport.getScore() + "org.tencompetence.portlet.qtieditor.beans.AssessmentTestEdit");
	}
	
	private BasicElementList getTestFeedbacks() {
		BasicElementList result = new BasicElementList();
		
		for (RowItem<FeedbackItem> rowItem : feedbackController.getRowItems()) {
			if (rowItem.getData().getTestFeedback() != null) {
				result.add(rowItem.getData().getTestFeedback());
			}
		}
		return result;
	}
	
	private void removeFeedback(final String feedbackId) {
		AssessmentTest editorInput = getEditorInput();
		AssessmentTestInstance assessmentTest = AssessmentTestInstance.fromXmlString(editorInput.getXmlData());
		TestFeedback feedbackToRemove = null;
		for (BasicElement element : assessmentTest.getTestFeedbackList()) {
			TestFeedback feedback = (TestFeedback) element;
			if (feedback.getId().equals(feedbackId)) {
				feedbackToRemove = feedback;
				break;
			}
		}
		if (feedbackToRemove != null) {
			assessmentTest.removeTestFeedback(feedbackToRemove);

			editorInput.setXmlData(assessmentTest.toXmlString());
		}
	}

	@Override
	protected void doSave(final AssessmentTest editorInput) {
		AssessmentTestInstance assessmentTest = AssessmentTestInstance.fromXmlString(editorInput.getXmlData());
		
		assessmentTest.setTitle(getTitleForInternalUse());
		updateTestResultReport(assessmentTest);
		assessmentTest.setTestFeedbackList(getTestFeedbacks());
		
		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setXmlData(assessmentTest.toXmlString());

		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (editorInput.isNew()) {
			try {
				PortletUtils.setOwnership(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				AssessmentTestFacade.addAssessmentTest(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			editorInput.setNew(false);
		} else {
			try {
				AssessmentTestFacade.updateAssessmentTest(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	protected void initFields(final AssessmentTest editorInput) {
		if (PortletUtils.getSessionMap().containsKey(ITEM_TO_DELETE)) {
			String feedbackId = (String) PortletUtils.getSessionMap().get(ITEM_TO_DELETE);
			removeFeedback(feedbackId);
			PortletUtils.getSessionMap().remove(ITEM_TO_DELETE);
		}
		
		setTitle(editorInput.getTitle());
		setDescriptionForInternalUse(editorInput.getXmlData());
		
		setTestResultReportVariables(editorInput);

		feedbackController = new FeedbackListController();
		feedbackController.setInput(getFeedbackItemList(editorInput));
		
		itemsController = new AssessmentItemListController();
		itemsController.setInput(getAssessmentItems(editorInput));
		itemsController.setConfirmationDialogController(getConfirmationController());
	}

	@Override
	public String cancel() {
		AssessmentTest editorInput = getEditorInput();
		if (editorInput.isNew()) {
			//If the creation of the test is canceled, remove all assessment items associated to it
			try {
				List<AssessmentItem> items = AssessmentTestFacade
					.getAssessmentItemsByAssessmentTest(editorInput.getAssessmentTestId(), ctx);
				for (AssessmentItem item : items) {
					AssessmentItemFacade.deleteAssessmentItem(item, ctx);
				}
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}

		return super.cancel();
	}
}