/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.context.FacesContext;

import org.tencompetence.business.facade.AssessmentItemFacade;
import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.components.BaseDeletePortletBean;
import org.tencompetence.portlet.common.components.NavigationDataHolder;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;
import org.tencompetence.portlet.common.utils.UrlUtils;
import org.tencompetence.services.xml.qti.elements.AssessmentTestInstance;

import com.icesoft.faces.context.effects.JavascriptContext;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;

/**
 * AssessmentTestList backing bean.
 * 
 * @author Teodor Stoev
 * @version $Revision: 1.23 $, $Date: 2009/11/24 14:10:06 $
 */
public class AssessmentTestList extends BaseDeletePortletBean {
	private static final Log LOG = LogFactoryUtil.getLog(AssessmentTestList.class);
	private String testTitle;
	private ActionsMenuListController<ActionsMenuRowItem<AssessmentTest>> controller;
	private ServiceContext ctx = null;

	/** Constructor. */
	public AssessmentTestList() {
		PortletUtils.getSessionMap().put(Defines.INIT_FIELDS, true);

		// Add needed js libraries.
		JavascriptContext.includeLib("/js/xmlCodeViewer.js", FacesContext.getCurrentInstance());
		JavascriptContext.includeLib("/js/selection.js", FacesContext.getCurrentInstance());
		JavascriptContext.includeLib("/js/fillInTheBlankEdit.js", FacesContext.getCurrentInstance());
	}

	/**
	 * Gets the controller.
	 * 
	 * @return the controller
	 */
	public ActionsMenuListController<ActionsMenuRowItem<AssessmentTest>> getController() {
		if (controller == null || PortletUtils.getObjectFromSession(Defines.INIT_FIELDS, false)) {
			controller = new ActionsMenuListController<ActionsMenuRowItem<AssessmentTest>>();
			controller.setInput(getAssessmentTests());
		}

		return controller;
	}

	/**
	 * Gets the test title.
	 * 
	 * @return the test title
	 */
	public String getTestTitle() {
		return testTitle;
	}

	/**
	 * Sets the test title.
	 * 
	 * @param title
	 *            the new test title
	 */
	public void setTestTitle(final String title) {
		testTitle = title;
	}

	/**
	 * Check if user is allowed to add an Assessment test.
	 * 
	 * @return true / false
	 */
	public boolean isAddAssessmentTestAllowed() {
		return checkPortletPermission(TENCActionKey.ADD_ASSESSMENT_TEST);
	}

	/**
	 * Adds a test. Method called if add button is pressed.
	 * 
	 * A new AssessmentTest is passed to the next page
	 * 
	 * @return the string
	 */
	public String addTest() {
		try {
			final AssessmentTest test = AssessmentTestFacade.createAssessmentTest();
			test.setNew(true);

			AssessmentTestInstance assessmentTest = new AssessmentTestInstance();

			test.setTitle("");
			test.setXmlData(assessmentTest.toXmlString());
			test.setIdentifier(assessmentTest.getId());

			PortletUtils.getSessionMap().put(Defines.EDITOR_INPUT_PARAM_NAME, test);
			PortletUtils.getSessionMap().put(Defines.INIT_FIELDS, true);
			return Outcomes.ADD;
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}

		return null;
	}

	private List<ActionsMenuRowItem<AssessmentTest>> getAssessmentTests() {
		try {
			final List<AssessmentTest> assessmentTestList = AssessmentTestFacade.getAssessmentTestsByGroupId(ctx
					.getScopeGroupId(), ctx);
			final List<ActionsMenuRowItem<AssessmentTest>> result = new ArrayList<ActionsMenuRowItem<AssessmentTest>>(
					assessmentTestList.size());
			for (final AssessmentTest test : assessmentTestList) {
				result.add(new ActionsMenuRowItem<AssessmentTest>(test, test.getPrimaryKey()));
			}
			return result;
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		return null;
	}

	@Override
	protected void deleteListEntity(final BaseModel<?> entity) {
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

		try {
			if (entity instanceof AssessmentTest) {
				final AssessmentTest test = (AssessmentTest) entity;

				final List<AssessmentItem> items = AssessmentTestFacade.getAssessmentItemsByAssessmentTest(test
						.getAssessmentTestId(), ctx);
				for (final AssessmentItem item : items) {
					AssessmentItemFacade.deleteAssessmentItem(item, ctx);
				}

				AssessmentTestFacade.setAssessmentItemsForAssessmentTest(test.getPrimaryKey(), new long[] {}, ctx);
				AssessmentTestFacade.deleteAssessmentTest(test, ctx);

				PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
			}
		} catch (Exception e) {
			LOG.error(e);
		}
	}

	@Override
	protected void initFields() {
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

		getController();

		final NavigationDataHolder nav = NavigationDataHolder.getInstance();

		if (nav != null) {
			if (nav.getNormalWindowStateURL() == null) {
				nav.setNormalWindowStateURL(UrlUtils.getWindowStateNormalUrl(Outcomes.LIST_ASSESSMENT_TESTS));
			}
			if (nav.getMaximizedWindowStateURL() == null) {
				nav.setMaximizedWindowStateURL(UrlUtils.getWindowStateMaximizedUrl(Outcomes.LIST_ASSESSMENT_TESTS));
			}
			nav.clear();
		}
	}
}
