/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;

import org.tencompetence.business.facade.AssessmentItemFacade;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.qtieditor.common.AssessmentItemEditorBean;
import org.tencompetence.portlet.qtieditor.common.RowItem;
import org.tencompetence.portlet.qtieditor.common.SimpleChoiceItem;
import org.tencompetence.portlet.qtieditor.common.SimpleChoiceListController;
import org.tencompetence.services.xml.qti.elements.AssessmentElementFactory;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.BasicElement;
import org.tencompetence.services.xml.qti.elements.BasicElementList;
import org.tencompetence.services.xml.qti.elements.BlockInteraction;
import org.tencompetence.services.xml.qti.elements.ChoiceInteraction;
import org.tencompetence.services.xml.qti.elements.InterpretationValue;
import org.tencompetence.services.xml.qti.elements.Mapping;
import org.tencompetence.services.xml.qti.elements.OutcomeDeclaration;
import org.tencompetence.services.xml.qti.elements.ResponseDeclaration;
import org.tencompetence.services.xml.qti.elements.SimpleChoice;
import org.tencompetence.services.xml.qti.elements.Value;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * ChoiceInteractionEdit backing bean.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.25 $, $Date: 2009/11/27 14:09:34 $
 */
public class ChoiceInteractionEdit extends AssessmentItemEditorBean {
	private static final String ITEM_TO_DELETE = "item.to.delete";
	private static final int MAX_CHOICES = 10;
	private SimpleChoiceListController controller;
	private List<SelectItem> radioCorrectItems;
	private boolean shuffle;
	private String selectedCorrect;
	private boolean addAndDeleteChoiceAllowed;
	private boolean onlyOneCorrectAllowed;
	private boolean correctAndScoreAllowed;
	private boolean hasItemFeedbacks;

	/**
	 * Gets the controller.
	 *
	 * @return the controller
	 */
	public SimpleChoiceListController getController() {
		checkEditorInput();
		return controller;
	}

	/**
	 * Adds a simple choice to a choice-interaction. Method called if add button is pressed.
	 *
	 * The current AssessmentItem is modified and passed to the next page
	 * 
	 * @return the string
	 */
	public String addChoice() {
		AssessmentItem editorInput = getEditorInput();
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		assessmentItem.setTitle(getTitleForInternalUse());
		((BlockInteraction) assessmentItem.getItemBody().getFirstBlock()).getPrompt().setData(getText());
		
		ChoiceInteraction choiceInteraction = (ChoiceInteraction) assessmentItem.getItemBody().getFirstBlock();
		choiceInteraction.setSimpleChoiceList(getSimpleChoices());
		if (hasItemFeedbacks) {
			// When the responseDeclaration is constructed from xml and no modalFeedback is defined,
			// the response's feedback outcome is null. So we first create the feedback outcome.
			ResponseDeclaration responseDeclaration = choiceInteraction.getResponseDeclaration();
			if (responseDeclaration.getFeedbackOutcomeDeclaration() == null) {
				OutcomeDeclaration aOutcomeDeclaration = new OutcomeDeclaration(responseDeclaration.getId() + ".FEEDBACK", 
						AssessmentElementFactory.IDENTIFIER, 
						AssessmentElementFactory.SINGLE,
						assessmentItem);
				responseDeclaration.setFeedbackOutcomeDeclaration(aOutcomeDeclaration);
			}
		}
		if (correctAndScoreAllowed) {
			choiceInteraction.getResponseDeclaration().setCorrectResponse(getCorrectResponse());
			choiceInteraction.getResponseDeclaration().setMapping(getMapping());
		}
		
		choiceInteraction.addSimpleChoice(new SimpleChoice(assessmentItem, choiceInteraction));

		editorInput.setXmlData(assessmentItem.toXmlString());

		return this.getConfirmationController().canLeave(Outcomes.ADD, editorInput);
	}

	private void removeChoice(final String choiceId) {
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(getEditorInput().getXmlData());
		ChoiceInteraction choiceInteraction = (ChoiceInteraction) assessmentItem.getItemBody().getFirstBlock();
		if (choiceInteraction.getSimpleChoiceList().size() > 2) {
			SimpleChoice choiceToRemove = null;
			for (BasicElement element : choiceInteraction.getSimpleChoiceList()) {
				SimpleChoice choice = (SimpleChoice) element;
				if (choice.getId().equals(choiceId)) {
					choiceToRemove = choice;
					break;
				}
			}
			if (choiceToRemove != null) {
				choiceInteraction.removeSimpleChoice(choiceToRemove);
	
				AssessmentItem editorInput = getEditorInput();
				editorInput.setXmlData(assessmentItem.toXmlString());
			}
		}
	}
	
	/**
	 * Gets the shuffle value.
	 * 
	 * @return true / false
	 */
	public boolean getShuffle() {
		return shuffle;
	}
	
	/**
	 * Sets the shuffle value.
	 * 
	 * @param shuffle
	 *            the new shuffle value
	 */
	public void setShuffle(final boolean shuffle) {
		this.shuffle = shuffle;
	}

	/**
	 * Gets the selected correct answer.
	 * 
	 * @return the selected correct answer
	 */
	public String getSelectedCorrect() {
		return selectedCorrect;
	}

	/**
	 * Sets the selected correct answer.
	 * 
	 * @param correct
	 *            the new correct answer
	 */
	public void setSelectedCorrect(final String correct) {
		selectedCorrect = correct;
	}

	/**
	 * Check if question type allows adding and deleting simple choices.
	 * 
	 * @return true / false
	 */
	public boolean isAddAndDeleteChoiceAllowed() {
		return addAndDeleteChoiceAllowed;
	}

	/**
	 * Check if question type allows only one correct answer.
	 * 
	 * @return true / false
	 */
	public boolean isOnlyOneCorrectAllowed() {
		return onlyOneCorrectAllowed;
	}

	/**
	 * Check if question type allows setting a score for each answer.
	 * 
	 * @return true / false
	 */
	public boolean isCorrectAndScoreAllowed() {
		return correctAndScoreAllowed;
	}

	private List<RowItem<SimpleChoiceItem>> getSimpleChoiceItems(final AssessmentItemInstance assessmentItem) {
		BasicElementList simpleChoiceList =
			((ChoiceInteraction) assessmentItem.getItemBody().getFirstBlock()).getSimpleChoiceList();
		List<RowItem<SimpleChoiceItem>> result = new ArrayList<RowItem<SimpleChoiceItem>>(simpleChoiceList.size());

		for (int i = 0; i < simpleChoiceList.size(); i++) {
			SimpleChoice choice = (SimpleChoice) simpleChoiceList.getBasicElementAt(i);
			result.add(new RowItem<SimpleChoiceItem>(
					new SimpleChoiceItem(choice, i), (long) choice.getId().hashCode()));
		}

		return result;
	}

	/**
	 * Gets the list of correct items to populate the radio buttons control.
	 * 
	 * @return the list of correct items
	 */
	public List<SelectItem> getRadioCorrectItems() {
		return radioCorrectItems;
	}
	
	/**
	 * Validate choice.
	 * 
	 * @param context
	 *            the context
	 * @param component
	 *            the component
	 * @param value
	 *            the value
	 */
	public void validateChoice(final FacesContext context, final UIComponent component, final Object value) {
		final UIInput input = ((UIInput) component);
		if (input.isRequired() && value == null) {
			PortletUtils.addErrorMessageToContext(":form:choiceSelectOneRadio", "choiceInteractionEdit.error.nocorrectset");
			input.setValid(false);
		}
	}

	private BasicElementList getSimpleChoices() {
		BasicElementList result = new BasicElementList();
		for (RowItem<SimpleChoiceItem> rowItem : controller.getRowItems()) {
			if (rowItem.getData().getSimpleChoice() != null) {
				result.add(rowItem.getData().getSimpleChoice());
				if (rowItem.getData().getFeedback() != null) {
					this.hasItemFeedbacks = true;
				}
			}
		}
		return result;
	}

	private InterpretationValue getCorrectResponse() {
		InterpretationValue result = new InterpretationValue(AssessmentElementFactory.CORRECT_RESPONSE);
		if (onlyOneCorrectAllowed) {
			for (RowItem<SimpleChoiceItem> rowItem : controller.getRowItems()) {
				SimpleChoiceItem choiceItem = rowItem.getData();
				if (choiceItem.getSimpleChoice().getId().equals(selectedCorrect)) {
					Value value = new Value(AssessmentElementFactory.IDENTIFIER, choiceItem.getSimpleChoice().getId());
					result.addValue(value);
					break;
				}
			}
		} else {
			for (RowItem<SimpleChoiceItem> rowItem : controller.getRowItems()) {
				SimpleChoiceItem choiceItem = rowItem.getData();
				if (choiceItem.isCorrect()) {
					Value value = new Value(AssessmentElementFactory.IDENTIFIER, choiceItem.getSimpleChoice().getId());
					result.addValue(value);
				}
			}
		}
		return result;
	}

	private Mapping getMapping() {
		Mapping result = new Mapping();
		for (RowItem<SimpleChoiceItem> rowItem : controller.getRowItems()) {
			SimpleChoiceItem choiceItem = rowItem.getData();
			if (choiceItem.getScore().length() > 0) {
				result.updateOrAddMapEntry(choiceItem.getSimpleChoice().getId(), choiceItem.getScore());
			}
		}
		return result;
	}

	private void determineIfAddAndDeleteChoiceIsAllowed(final String questionType) {
		if (questionType.equals(AssessmentElementFactory.YES_NO)) {
			addAndDeleteChoiceAllowed = false;
		} else {
			addAndDeleteChoiceAllowed = true;
		}
	}

	private void determineIfOnlyOneCorrectIsAllowed(final String questionType) {
		if (questionType.equals(AssessmentElementFactory.TEXT_BASED_MULTIPLE_CHOICE)
				|| questionType.equals(AssessmentElementFactory.YES_NO)) {
			onlyOneCorrectAllowed = true;
		} else {
			onlyOneCorrectAllowed = false;
		}
	}

	private void determineIfCorrectAndScoreAreAllowed(final String questionType) {
		if (questionType.equals(AssessmentElementFactory.LIKERT)) {
			correctAndScoreAllowed = false;
		} else {
			correctAndScoreAllowed = true;
		}
	}

	private List<SelectItem> getListOfRadioCorrectItems() {
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(getEditorInput().getXmlData());
		BasicElementList simpleChoiceList =
			((ChoiceInteraction) assessmentItem.getItemBody().getFirstBlock()).getSimpleChoiceList();
		ArrayList<SelectItem> result = new ArrayList<SelectItem>();

		for (int i = 0; i < simpleChoiceList.size(); i++) {
			SimpleChoice choice = (SimpleChoice) simpleChoiceList.getBasicElementAt(i);
			SelectItem newItem = new SelectItem(choice.getId());
			newItem.setLabel("");
			result.add(newItem);
		}

		return result;
	}

	@Override
	protected void doSave(final AssessmentItem editorInput) {
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		ChoiceInteraction choiceInteraction = (ChoiceInteraction) assessmentItem.getItemBody().getFirstBlock();

		assessmentItem.setTitle(getTitleForInternalUse());
		choiceInteraction.getPrompt().setData(getText());
		if (!onlyOneCorrectAllowed) {
			choiceInteraction.setMaxChoices(MAX_CHOICES);
		} else {
			choiceInteraction.setMaxChoices(1);
		}
		choiceInteraction.setShuffle(shuffle);
		choiceInteraction.setSimpleChoiceList(getSimpleChoices());
		if (hasItemFeedbacks) {
			// When the responseDeclaration is constructed from xml and no modalFeedback is defined,
			// the response's feedback outcome is null. So we first create the feedback outcome.
			ResponseDeclaration responseDeclaration = choiceInteraction.getResponseDeclaration();
			if (responseDeclaration.getFeedbackOutcomeDeclaration() == null) {
				OutcomeDeclaration aOutcomeDeclaration = new OutcomeDeclaration(responseDeclaration.getId() + ".FEEDBACK", 
						AssessmentElementFactory.IDENTIFIER, 
						AssessmentElementFactory.SINGLE,
						assessmentItem);
				responseDeclaration.setFeedbackOutcomeDeclaration(aOutcomeDeclaration);
			}
		}
		if (correctAndScoreAllowed) {
			choiceInteraction.getResponseDeclaration().setCorrectResponse(getCorrectResponse());
			choiceInteraction.getResponseDeclaration().setMapping(getMapping());
		}

		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setXmlData(assessmentItem.toXmlString());

		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (editorInput.isNew()) {
			try {
				PortletUtils.setOwnership(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				AssessmentItemFacade.addAssessmentItem(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			editorInput.setNew(false);
			addToAssessmentTest(editorInput);
		} else {
			try {
				AssessmentItemFacade.updateAssessmentItem(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	protected void initFields(final AssessmentItem editorInput) {
		if (PortletUtils.getSessionMap().containsKey(ITEM_TO_DELETE)) {
			String choiceId = (String) PortletUtils.getSessionMap().get(ITEM_TO_DELETE);
			removeChoice(choiceId);
			PortletUtils.getSessionMap().remove(ITEM_TO_DELETE);
		}

		setTitle(editorInput.getTitle());
		setDescriptionForInternalUse(editorInput.getXmlData());

		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		ChoiceInteraction choiceInteraction = (ChoiceInteraction) assessmentItem.getItemBody().getFirstBlock();
		setText(choiceInteraction.getPrompt().getData());
		setShuffle(choiceInteraction.getShuffle());

		String questionType = assessmentItem.getItemBody().getQuestionClass();
		determineIfAddAndDeleteChoiceIsAllowed(questionType);
		determineIfOnlyOneCorrectIsAllowed(questionType);
		determineIfCorrectAndScoreAreAllowed(questionType);

		if (onlyOneCorrectAllowed) {
			radioCorrectItems = getListOfRadioCorrectItems();

			InterpretationValue correctResponse = choiceInteraction.getResponseDeclaration().getCorrectResponse();
			if (correctResponse.getValueList().size() > 0) {
				String correctChoiceId = ((Value) correctResponse.getValueAt(0)).getData();
				for (SelectItem item : radioCorrectItems) {
					String value = (String) item.getValue();
					if (value.equals(correctChoiceId)) {
						selectedCorrect = value;
					}
				}
			}
		}

		controller = new SimpleChoiceListController();
		controller.setInput(getSimpleChoiceItems(assessmentItem));
	}
}