/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.beans;

import org.tencompetence.business.facade.AssessmentItemFacade;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.qtieditor.common.AssessmentItemEditorBean;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.BlockInteraction;
import org.tencompetence.services.xml.qti.elements.ExtendedTextInteraction;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * ExtendedTextInteractionEdit backing bean.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.11 $, $Date: 2009/11/24 14:10:06 $
 */
public class ExtendedTextInteractionEdit extends AssessmentItemEditorBean {
	
	@Override
	protected void doSave(final AssessmentItem editorInput) {
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		ExtendedTextInteraction textInteraction = (ExtendedTextInteraction) assessmentItem.getItemBody().getFirstBlock();
		
		assessmentItem.setTitle(getTitleForInternalUse());
		textInteraction.getPrompt().setData(getText());
		
		editorInput.setTitle(getTitleForInternalUse());
		editorInput.setXmlData(assessmentItem.toXmlString());
		
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (editorInput.isNew()) {
			try {
				PortletUtils.setOwnership(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				AssessmentItemFacade.addAssessmentItem(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			editorInput.setNew(false);
			addToAssessmentTest(editorInput);
		} else {
			try {
				AssessmentItemFacade.updateAssessmentItem(editorInput, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	protected void initFields(final AssessmentItem editorInput) {
		setTitle(editorInput.getTitle());
		setDescriptionForInternalUse(editorInput.getXmlData());
		
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		setText(((BlockInteraction) assessmentItem.getItemBody().getFirstBlock()).getPrompt().getData());
	}
}
