/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.common.components.BaseEditorBean;
import org.tencompetence.portlet.common.components.NavigationData;
import org.tencompetence.portlet.common.components.NavigationDataHolder;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.AssessmentItemRef;
import org.tencompetence.services.xml.qti.elements.AssessmentSection;
import org.tencompetence.services.xml.qti.elements.AssessmentTestInstance;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * AssessmentItemEditorBean.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.12 $, $Date: 2009/11/24 14:10:06 $
 */
public class AssessmentItemEditorBean extends BaseEditorBean<AssessmentItem> {
	private static final String EDITOR_INPUT_PREV_PARAM_NAME = "editor.input.prev";
	private static final Log LOG = LogFactoryUtil.getLog(AssessmentItemEditorBean.class);
	
	/** the prompt text of the question. */
	private String text;
	
	/**
	 * Gets the question type name.
	 * 
	 * @return the question type name
	 */
	public String getTypeName() {
		AssessmentItemInstance item = AssessmentItemInstance.fromXmlString(getEditorInput().getXmlData());
		String questionType = item.getItemBody().getQuestionClass();
		return ValueMapping.getUserFriendlyQuestionType(questionType);
	}

	/**
	 * Adds the assessment item to its parent assessment test.
	 * 
	 * @param item
	 *            the assessment item
	 */
	protected void addToAssessmentTest(final AssessmentItem item) {
		AssessmentTest assessmentTest =
			(AssessmentTest) PortletUtils.getSessionMap().get(EDITOR_INPUT_PREV_PARAM_NAME);
		AssessmentTestInstance testInstance = AssessmentTestInstance.fromXmlString(assessmentTest.getXmlData());

		AssessmentSection assessmentSection = testInstance.getFirstTestPart().getFirstAssessmentSection();
		AssessmentItemRef assessmentItemRef = new AssessmentItemRef(testInstance);
		assessmentItemRef.setHref(testInstance.getTitle() + "/" + item.getIdentifier() + ".xml");
		assessmentSection.addSectionPart(assessmentItemRef);

		assessmentTest.setXmlData(testInstance.toXmlString());

		long assessmentTestId = assessmentTest.getAssessmentTestId();
		try {
			ServiceContext ctx = PortletUtils.getServiceContext();
			
			if (!assessmentTest.isNew()) {
				AssessmentTestFacade.updateAssessmentTest(assessmentTest, ctx);
			}
			AssessmentTestFacade
				.addAssessmentItemToAssessmentTest(assessmentTestId, item.getAssessmentItemId(), ctx);
			PortletUtils.getSessionMap().put(EDITOR_INPUT_PREV_PARAM_NAME, assessmentTest);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Gets the prompt text of the question.
	 * 
	 * @return the prompt text
	 */
	public String getText() {
		return text;
	}
	
	/**
	 * Sets the prompt text of the question.
	 * 
	 * @param text
	 *            the prompt text
	 */
	public void setText(final String text) {
		this.text = text;
	}

	@Override
	protected void doSave(final AssessmentItem editorInput) {
		//Nothing to do
	}

	@Override
	protected void initFields(final AssessmentItem editorInput) {
		//Nothing to do
	}

	@Override
	protected String getViewId() {
		// Get previous page
		final NavigationData data = NavigationDataHolder.getInstance().getPreviousNavigationData();

		if (data == null) {
			// go back to the first page
			PortletUtils.redirect(NavigationDataHolder.getInstance().getNormalWindowStateURL());
		} else {
			if (PortletUtils.getSessionMap().containsKey(EDITOR_INPUT_PREV_PARAM_NAME)) {
				AssessmentTest assessmentTest =
					(AssessmentTest) PortletUtils.getSessionMap().get(EDITOR_INPUT_PREV_PARAM_NAME);
				BaseEditorBean.passEditorInput(assessmentTest);
				PortletUtils.getSessionMap().remove(EDITOR_INPUT_PREV_PARAM_NAME);
			} else {
				BaseEditorBean.passEditorInput(data.getEditorInput());
			}
			return data.getViewId();
		}

		return null;
	}
}