/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import javax.faces.event.ActionEvent;

import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.ActionsMenuRowItem;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.Block;
import org.tencompetence.services.xml.qti.elements.ChoiceInteraction;
import org.tencompetence.services.xml.qti.elements.ExtendedTextInteraction;
import org.tencompetence.services.xml.qti.elements.FillingInBlankBlock;
import org.tencompetence.services.xml.qti.elements.InlineChoiceBlock;

import com.icesoft.faces.component.ContextActionEvent;
import com.icesoft.faces.component.ext.HtmlCommandLink;
import com.icesoft.faces.component.ext.HtmlDataTable;
import com.icesoft.faces.component.ext.UIColumn;
import com.icesoft.faces.component.menubar.MenuItem;

/**
 * AssessmentItemListController. Extends the functionality of the ActionsMenuListController.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.10 $, $Date: 2009/11/24 14:10:06 $
 */
public class AssessmentItemListController extends ActionsMenuListController<ActionsMenuRowItem<AssessmentItem>> {
	/** The constant ITEM_TO_DELETE. */
	protected static final String ITEM_TO_DELETE = "item.to.delete";
	
	/** The constant INIT_FIELDS. */
	protected static final String INIT_FIELDS = "init.fields";
	
	/**
	 * Handles the clickLink event.
	 *
	 * @param event
	 *            ActionEvent
	 */
	@Override
	public void clickLink(final ActionEvent event) {
		final HtmlCommandLink link = (HtmlCommandLink) event.getComponent();
		final UIColumn column = (UIColumn) link.getParent();
		final HtmlDataTable table = (HtmlDataTable) column.getParent();
		final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) table.getRowData();

		forwardToNextPage(rowItem, getNextView(Outcomes.VIEW, rowItem));
	}

	/**
	 * This listener catches all click events from the popup menu.
	 *
	 * The supported events are : view, edit and delete.
	 *
	 * When a confirmationController is set, this is used to check the input and display a dialog.
	 *
	 * @param event
	 *            ActionEvent
	 */
	@Override
	public void menuActionListener(final ActionEvent event) {

		if (event instanceof ContextActionEvent) {
			final ContextActionEvent contextEvent = (ContextActionEvent) event;
			final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) contextEvent.getContextValue();
			final String id = ((MenuItem) contextEvent.getSource()).getId();

			String outcome;

			if (id.equalsIgnoreCase("view")) {
				outcome = Outcomes.EDIT;
			} else if (id.equalsIgnoreCase("edit")) {
				outcome = Outcomes.EDIT;
			} else if (id.equalsIgnoreCase("delete")) {
				final AssessmentItem assessmentItem = (AssessmentItem) rowItem.getData();
				PortletUtils.getSessionMap().put(ITEM_TO_DELETE, assessmentItem.getAssessmentItemId());
				PortletUtils.getSessionMap().put(INIT_FIELDS, true);
				
				outcome = Outcomes.DELETE;
			} else {
				outcome = null;
			}

			forwardToNextPage(rowItem, getNextView(outcome, rowItem));
		}
	}

	private String getNextView(final String outcome, final ActionsMenuRowItem<?> rowItem) {
		String result = null;

		if (outcome.equals(Outcomes.VIEW)) {
			final AssessmentItem editorInput = (AssessmentItem) rowItem.getData();
			AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
			Block block = assessmentItem.getItemBody().getFirstBlock();
			if (block instanceof ChoiceInteraction) {
				result = "CHOICE_INTERACTION";
			} else if (block instanceof ExtendedTextInteraction) {
				result = "EXTENDED_TEXT_INTERACTION";
			} else if (block instanceof FillingInBlankBlock) {
				result = "FILL_IN_THE_BLANK";
			} else if (block instanceof InlineChoiceBlock) {
				result = "INLINE_CHOICE";
			}
		}

		return result;
	}
}