/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Copyright (c) 2009 McDowell
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 */

package org.tencompetence.portlet.qtieditor.common;

import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;

/**
 * Utility methods for working with clientIds. Requires JEE5/JSF1.2 or above.
 *
 * @author McDowell
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:10:05 $
 */
public final class ClientIdUtils {
	private ClientIdUtils() {
	}
	
	/**
	 * This method takes the id of a component and returns its clientId.
	 *
	 * If the component cannot be found, an IllegalStateException is thrown. This
	 * method may only be called during a JSF request.
	 *
	 * @param id
	 *          the id of a JSF component
	 *          
	 * @return the component's clientId
	 * @see javax.faces.component.UIComponent#getClientId(javax.faces.context.FacesContext)
	 * @see javax.faces.component.UIComponent#getId()
	 */
	public static String findClientId(final String id) {
		FacesContext context = FacesContext.getCurrentInstance();
		UIViewRoot view = context.getViewRoot();
	    if (view == null) {
	    	throw new IllegalStateException("view == null");
	    }
	    UIComponent component = findComponent(view, id);
	    if (component == null) {
	    	throw new IllegalStateException("no component has id='" + id + "'");
	    }
	    return component.getClientId(context);
	}

	private static UIComponent findComponent(final UIComponent component, final String id) {
		if (id.equals(component.getId())) {
			return component;
	}

	Iterator<UIComponent> kids = component.getFacetsAndChildren();
    while (kids.hasNext()) {
    	UIComponent kid = kids.next();
    	UIComponent found = findComponent(kid, id);
    	if (found != null) {
    		return found;
    	}
    }

    return null;
  }
}