/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import org.tencompetence.services.xml.qti.elements.TestFeedback;

/**
 * FeedbackItem.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.8 $, $Date: 2009/11/26 15:25:04 $
 */
public class FeedbackItem {
	private TestFeedback testFeedback;

	/** Constructor.
	 * 
	 * @param testFeedback
	 *            the test feedback
	 */
	public FeedbackItem(final TestFeedback testFeedback) {
		this.testFeedback = testFeedback;
	}
	
	/**
	 * Gets the test feedback.
	 *
	 * @return the test feedback
	 */
	public TestFeedback getTestFeedback() {
		return testFeedback;
	}
	
	/**
	 * Gets the feedback minimum score.
	 *
	 * @return the feedback minimum score
	 */
	public double getMinScore() {
		if (testFeedback.getEqfEntry() != null) {
			return testFeedback.getEqfEntry().getSourceValue();
		} else {
			return 0;
		}
	}

	/**
	 * Sets the feedback minimum score.
	 *
	 * @param minScore
	 *            the feedback minimum score
	 */
	public void setMinScore(final double minScore) {
		testFeedback.getEqfEntry().setSourceValue(minScore);
	}
	
	/**
	 * Checks if boundary is included in the feedback.
	 *
	 * @return true / false
	 */
	public boolean isIncludeBoundary() {
		if (testFeedback.getEqfEntry() != null) {
			return testFeedback.getEqfEntry().getIncludeBoundary();
		} else {
			return false;
		}
	}

	/**
	 * Sets if boundary is included in the feedback.
	 *
	 * @param includeBoundary
	 *            if boundary is included in the feedback
	 */
	public void setIncludeBoundary(final boolean includeBoundary) {
		testFeedback.getEqfEntry().setIncludeBoundary(includeBoundary);
	}
	
	/**
	 * Gets the choice text.
	 *
	 * @return the choice text
	 */
	public String getText() {
		return testFeedback.getData();
	}

	/**
	 * Sets the choice text.
	 *
	 * @param text
	 *            the new text
	 */
	public void setText(final String text) {
		testFeedback.setData(text);
	}

	/**
	 * Gets the selected EQF level.
	 *
	 * @return the selected EQF level
	 */
	public String getSelectedEqfLevel() {
		if (testFeedback.getEqfEntry() != null) {
			return testFeedback.getEqfEntry().getTargetValue();
		} else {
			return "1";
		}
	}
	
	/**
	 * Sets the selected EQF level.
	 *
	 * @param selectedEqfLevel
	 *            the new selected EQF level
	 */
	public void setSelectedEqfLevel(final String selectedEqfLevel) {
		if (testFeedback.getEqfEntry() != null) {
			testFeedback.setId(selectedEqfLevel);
			testFeedback.getEqfEntry().setTargetValue(selectedEqfLevel);
		}
	}
}
