/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import org.tencompetence.services.xml.qti.elements.InlineChoice;

/**
 * InlineChoiceItem.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.2 $, $Date: 2009/11/24 14:10:05 $
 */
public class InlineChoiceItem {
	private InlineChoice inlineChoice;
	private int index;
	private boolean correct;
	
	/** Constructor.
	 * 
	 * @param inlineChoice
	 *            the inline choice
	 * 
	 * @param index
	 *            the index
	 */
	public InlineChoiceItem(final InlineChoice inlineChoice, final int index) {
		this.inlineChoice = inlineChoice;
		this.index = index;
	}
	
	/**
	 * Gets the inline choice.
	 *
	 * @return the inline choice
	 */
	public InlineChoice getInlineChoice() {
		return inlineChoice;
	}
	
	/**
	 * Gets the choice text.
	 *
	 * @return the choice text
	 */
	public String getText() {
		return inlineChoice.getData();
	}

	/**
	 * Sets the choice text.
	 *
	 * @param text
	 *            the new text
	 */
	public void setText(final String text) {
		inlineChoice.setData(text);
	}
	
	/**
	 * Gets the index of the item.
	 *
	 * @return the index
	 */
	public int getIndex() {
		return index;
	}
	
	/**
	 * Checks if the choice is currently marked as correct.
	 *
	 * @return true / false
	 */
	public boolean isCorrect() {
		return correct;
	}
}
