/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import javax.faces.event.ActionEvent;

import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;

/**
 * InlineChoiceListController. Extends and modifies the functionality of the ListController.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:05 $
 */
public class InlineChoiceListController extends ListController<RowItem<InlineChoiceItem>> {
	private static final String CHOICE_TO_DELETE = "choice.to.delete";
	private static final String BLANK_WITH_CHOICE_TO_DELETE = "blank.with.choice.to.delete";
	private static final String INIT_FIELDS = "init.fields";
	
	/** Constructor. */
	public InlineChoiceListController() {
		super("InlineChoiceEdit.dataTable", SelectionType.SINGLE);
	}
	
	/**
	 * Handles the clickLink event.
	 *
	 * @param event
	 *            ActionEvent
	 */
	public void deleteChoice(final ActionEvent event) {
		String blankId = (String) event.getComponent().getAttributes().get("blankId");
		String choiceId = (String) event.getComponent().getAttributes().get("choiceId");

		PortletUtils.getSessionMap().put(BLANK_WITH_CHOICE_TO_DELETE, blankId);
		PortletUtils.getSessionMap().put(CHOICE_TO_DELETE, choiceId);
		forwardToNextPage(Outcomes.DELETE);
	}

	/**
	 * Forward to next page.
	 *
	 * @param outcome
	 *            outcome
	 */
	protected void forwardToNextPage(final String outcome) {
		if (outcome != null) {
			PortletUtils.getSessionMap().put(INIT_FIELDS, true);
			PortletUtils.handleNavigation(outcome);
		}
	}
}
