/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import org.tencompetence.services.xml.qti.elements.TextEntryInteraction;

/**
 * SimpleChoiceListController. Extends and modifies the functionality of the ListController.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:10:06 $
 */
public class TextEntryInteractionItem {
	private TextEntryInteraction textEntryInteraction;
	private String answer;

	/** Constructor.
	 * 
	 * @param textEntryInteraction
	 *            the text entry interaction
	 * 
	 * @param answer
	 *            the answer
	 */
	public TextEntryInteractionItem(final TextEntryInteraction textEntryInteraction, final String answer) {
		this.textEntryInteraction = textEntryInteraction;
		this.answer = answer;
	}

	/**
	 * Gets the text entry interaction.
	 *
	 * @return the text entry interaction
	 */
	public TextEntryInteraction getTextEntryInteraction() {
		return textEntryInteraction;
	}

	/**
	 * Gets the position of the interaction.
	 *
	 * @return the position
	 */
	public String getPosition() {
		int start = textEntryInteraction.getStart();
		int end = start + textEntryInteraction.getLength();
		
		return start + " - " + end;
	}

	/**
	 * Gets the answer.
	 *
	 * @return the answer
	 */
	public String getAnswer() {
		return answer;
	}
}