/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import org.tencompetence.services.xml.qti.elements.AssessmentElementFactory;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.Block;
import org.tencompetence.services.xml.qti.elements.ChoiceInteraction;
import org.tencompetence.services.xml.qti.elements.ExtendedTextInteraction;
import org.tencompetence.services.xml.qti.elements.FillingInBlankBlock;
import org.tencompetence.services.xml.qti.elements.InlineChoiceBlock;

/**
 * ValueMapping.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:05 $
 */
public final class ValueMapping {
	private ValueMapping() {
	}
	
	/**
	 * Gets the user-friendly question type.
	 *
	 * @param questionType
	 *            the question type
	 * 
	 * @return the user-friendly question type
	 */
	public static String getUserFriendlyQuestionType(final String questionType) {
		String result = "Unknown question";

		if (questionType.equals(AssessmentElementFactory.TEXT_BASED_MULTIPLE_CHOICE)) {
			result = "Multiple choice";
		} else if (questionType.equals(AssessmentElementFactory.TEXT_BASED_MULTIPLE_RESPONSE)) {
			result = "Multiple response";
		} else if (questionType.equals(AssessmentElementFactory.LIKERT)) {
			result = "Likert scale";
		} else if (questionType.equals(AssessmentElementFactory.YES_NO)) {
			result = "Yes/no question";
		} else if (questionType.equals(AssessmentElementFactory.OPEN)) {
			result = "Open question";
		} else if (questionType.equals(AssessmentElementFactory.FILL_IN_THE_BLANK)) {
			result = "Fill-in-the-blank";
		} else if (questionType.equals(AssessmentElementFactory.INLINE_CHOICE)) {
			result = "Inline choice";
		}

		return result;
	}

	/**
	 * Created and returns a block corresponding to the question type.
	 *
	 * @param questionType
	 *            the question type
	 * 
	 * @param item
	 *            the assessment item instance
	 * 
	 * @return the user-friendly question type
	 */
	public static Block createBlockCorrespondingToQuestionType(
			final String questionType, final AssessmentItemInstance item) {
		Block result = null;

		if (questionType.equals(AssessmentElementFactory.TEXT_BASED_MULTIPLE_CHOICE)) {
			result = new ChoiceInteraction(item, AssessmentItemInstance.TBMC, 2);
		} else if (questionType.equals(AssessmentElementFactory.TEXT_BASED_MULTIPLE_RESPONSE)) {
			result = new ChoiceInteraction(item, AssessmentItemInstance.TBMR, 2);
		} else if (questionType.equals(AssessmentElementFactory.LIKERT)) {
			result = new ChoiceInteraction(item, AssessmentItemInstance.LS, 2);
		} else if (questionType.equals(AssessmentElementFactory.YES_NO)) {
			result = new ChoiceInteraction(item, AssessmentItemInstance.YN, 2);
		} else if (questionType.equals(AssessmentElementFactory.OPEN)) {
			result = new ExtendedTextInteraction(item);
		} else if (questionType.equals(AssessmentElementFactory.FILL_IN_THE_BLANK)) {
			result = new FillingInBlankBlock(item);
		} else if (questionType.equals(AssessmentElementFactory.INLINE_CHOICE)) {
			result = new InlineChoiceBlock(item);
		}

		return result;
	}
}