/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.qtieditor.common;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.XMLConstants;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

/**
 * XmlValidator.
 * 
 * @author Teodor Stoev
 * @version $Revision: 1.3 $, $Date: 2009/11/24 14:10:05 $
 */
public final class XmlValidator {
	private static final String QTI_SCHEMA = "http://www.imsglobal.org/question/qtiv2p1pd2/xsd/imsqti_v2p1.xsd";
	private static Schema schema = null;

	private XmlValidator() {
	}

	/**
	 * Validates the XML data.
	 * 
	 * @param xmlData
	 *            the XML data
	 * 
	 * @return the validation output
	 */
	public static String validateXmlData(final String xmlData) {
		String result = "The XML Code is valid.";
		try {
			Validator v = getSchema().newValidator();

			StringReader reader = new StringReader(xmlData);
			v.validate(new StreamSource(reader));
		} catch (SAXNotRecognizedException e1) {
			result = "The XML Schema cannot be recognized:\n" + e1.getMessage();
		} catch (SAXNotSupportedException e2) {
			result = "The XML Schema is not supported.";
		} catch (SAXException e3) {
			result = "Error while validating the XML Code:\n" + e3.getMessage();
		} catch (IOException e4) {
			result = "Error while validating the XML Code:\n" + e4.getMessage();
		}

		return result;
	}

	private static Schema getSchema() throws SAXException {
		if (schema == null) {
			SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			StreamSource ss = new StreamSource(QTI_SCHEMA);
			schema = factory.newSchema(ss);
		}
		return schema;
	}
}
