/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.test.beans;

import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.portlet.PortletPreferences;
import javax.portlet.ReadOnlyException;

import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.common.components.BasePreferences;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * Preferences for ...
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.3 $, $Date: 2009/11/24 14:10:06 $
 */
public class TestPreferences extends BasePreferences {

	private static final Log LOG = LogFactoryUtil.getLog(TestPreferences.class);
	private long selectedTestId;

	/** The Constant SELECTED_TEST. */
	public static final String SELECTED_TEST = "selectedTest";

	/**
	 * Instantiates a new assessment preferences.
	 */
	public TestPreferences() {
		super();
		final String selectedTestIdAsString = getPrefs().getValue(SELECTED_TEST, "0");
		selectedTestId = Long.parseLong(selectedTestIdAsString);
	}

	/**
	 * Gets the all tests.
	 * 
	 * @return the all tests
	 */
	public List<SelectItem> getAllTests() {
		List<SelectItem> result = new ArrayList<SelectItem>();
		try {
			final List<AssessmentTest> tests = AssessmentTestFacade
					.getAssessmentTests(PortletUtils.getServiceContext());
			for (AssessmentTest test : tests) {
				result.add(new SelectItem(Long.toString(test.getAssessmentTestId()), test.getTitle()));
			}
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}
		return result;
	}

	/**
	 * Gets the selected test.
	 * 
	 * @return the selected test
	 */
	public String getSelectedTest() {
		return Long.toString(selectedTestId);
	}

	/**
	 * Sets the selected test.
	 * 
	 * @param id
	 *            the new selected test
	 */
	public void setSelectedTest(final String id) {
		selectedTestId = Long.valueOf(id);
	}

	@Override
	protected void updatePreferences(final PortletPreferences prefs) throws ReadOnlyException {
		prefs.setValue(SELECTED_TEST, getSelectedTest());
	}
}