/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.test.beans;

import java.util.Locale;

import javax.faces.context.FacesContext;
import javax.portlet.PortletPreferences;

import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.business.utils.UserLocaleResolver;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;
import com.liferay.util.bridges.jsf.common.JSFPortletUtil;

/**
 * Backing bean for the simple test portlet.
 * 
 * @author jal
 * @version $Revision: 1.8 $, $Date: 2009/11/27 16:53:23 $
 */
public class TestSmallView {
	private static final Log LOG = LogFactoryUtil.getLog(TestSmallView.class);

	private static final String CONTEXT = "/apis/";
	private static final String PARAM_USER_ID = "user_id";
	private static final String PARAM_TEST_ID = "test_id";
	private static final String PARAM_COMPANY_ID = "company_id";
	private static final String PARAM_GROUP_ID = "group_id";
	private static final String PARAM_LANGUAGE = "language";
	private static final String PARAM_MODE = "mode";
	private static final String MODE = "test";

	private static final String SERVER_NAME = JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance())
			.getServerName();
	private static final int SERVER_PORT = JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance())
			.getServerPort();
	private static final String SERVER_SCHEME = JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance())
			.getScheme();

	private String baseUrl;
	private String assessmentIdentifier = null;

	/**
	 * Instantiates a new test small view.
	 */
	public TestSmallView() {
		try {
			initFields();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

	}

	private void initFields() throws SystemException, PortalException {
		final PortletPreferences prefs = PortletUtils.getPortletRequest().getPreferences();
		final Long assessmentId = Long.valueOf(prefs.getValue(TestPreferences.SELECTED_TEST, "0"));

		if (assessmentId > 0) {
			final AssessmentTest assessmentTest = AssessmentTestFacade.getAssessmentTest(assessmentId, PortletUtils
					.getServiceContext());
			assessmentIdentifier = assessmentTest.getIdentifier();

			baseUrl = SERVER_SCHEME + "://" + SERVER_NAME + ":" + SERVER_PORT
					+ JSFPortletUtil.getPortletRequest(FacesContext.getCurrentInstance()).getContextPath();
		} else {
			assessmentIdentifier = null;
		}
	}

	private String getLanguage() {
		try {
			Locale loc = UserLocaleResolver.getUserLocale();
			String localeLanguage = loc.getLanguage();
			if (localeLanguage == "nl") {
				return "dutch";
			} else if (localeLanguage == "es") {
				return "spanish";
			} else {
				// return the default
				return "english";
			}
		} catch (Exception e) {
			return "english";
		}
	}

	/**
	 * Returns the correct, fully qualified URL for the QTI rendering servlet.
	 * 
	 * @return the correct, fully qualified URL for the QTI rendering servlet
	 */
	public String getIframeURL() {
		ServiceContext ctx = null;
		try {
			ctx = PortletUtils.getServiceContext();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

		return baseUrl + CONTEXT + "?" + PARAM_USER_ID + "=" + ctx.getUserId() + "&" + PARAM_TEST_ID + "="
				+ assessmentIdentifier + "&" + PARAM_GROUP_ID + "=" + ctx.getScopeGroupId() + "&" + PARAM_COMPANY_ID
				+ "=" + ctx.getCompanyId() + "&" + PARAM_LANGUAGE + "=" + getLanguage() + "&" + PARAM_MODE + "=" + MODE;
	}

	/**
	 * Checks if a valid test has been selected.
	 * 
	 * @return true / false
	 */
	public boolean isValidTest() {
		return assessmentIdentifier != null;
	}
}