/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.constants;

import java.util.List;

/**
 * Enumeration for ClusterType.
 * 
 * @author sheyenrath
 * @version $Revision: 1.2 $, $Date: 2009/11/24 14:18:19 $
 * @since Khoper
 */
public enum ClusterType {

	/** NONE constant. */
	NONE("none"),
	/** FREE_ORDER constant. */
	FREE_ORDER("free order"),
	/** SEQUENCE constant. */
	SEQUENCE("sequence"),
	/** PARALLEL constant. */
	PARALLEL("parallel");

	/** The type. */
	private final String type;

	/**
	 * Instantiates a new EventType.
	 * 
	 * @param value
	 *            the value
	 */
	ClusterType(final String value) {
		this.type = value;
	}

	/**
	 * Gets the type.
	 * 
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * Gets the types as a string-array.
	 * 
	 * @param clusters
	 *            the events
	 * 
	 * @return the types
	 */
	public static String[] getTypes(final List<ClusterType> clusters) {
		final String[] strClusterTypes = new String[clusters.size()];

		int idx = 0;

		for (final ClusterType clusterType : clusters) {
			strClusterTypes[idx] = clusterType.getType();
			idx++;
		}

		return strClusterTypes;
	}

	/**
	 * Get the BType by string.
	 * 
	 * @param clusterType
	 *            clusterType
	 * @return BType
	 */
	public static ClusterType valueOfType(final String clusterType) {

		for (final ClusterType type : ClusterType.values()) {
			if (type.getType().contains(clusterType)) {
				return type;
			}
		}

		return ClusterType.NONE;
	}
}