/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.ActivityWrapper;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.service.ActivityWrapperLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The Class ActivityWrapperFacade.
 * 
 * @author venrooijma
 * @version $Id: ActivityWrapperFacade.java,v 1.7 2009/11/05 12:23:02 leike Exp $
 */
public final class ActivityWrapperFacade {

	private ActivityWrapperFacade() {
	}

	/**
	 * Creates a activityWrapper.
	 * 
	 * @return the activityWrapper
	 * @throws SystemException
	 *             the system exception
	 */
	public static ActivityWrapper createActivityWrapper() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();

		final ActivityWrapper entity = ActivityWrapperLocalServiceUtil.createActivityWrapper(id);
		entity.setNew(true);

		return entity;
	}

	/**
	 * Add an ActivityWrapper.
	 * 
	 * @param activityWrapper
	 *            The ActivityWrapper
	 * @param ctx
	 *            The ServiceContext
	 * @return ActivityWrapper
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static ActivityWrapper addActivityWrapper(final ActivityWrapper activityWrapper, final ServiceContext ctx)
			throws SystemException, PortalException {

		final ActivityWrapper newActivityWrapper = ActivityWrapperLocalServiceUtil.addActivityWrapper(activityWrapper);

		PortfolioService.addJournalEntry(newActivityWrapper, EventType.CREATE, ctx);

		FacadeUtils.addPermissions(newActivityWrapper, ctx);

		return newActivityWrapper;
	}

	/**
	 * Gets the activity wrappers by id.
	 * 
	 * @param ids
	 *            the ids
	 * @return the activity wrappers by id
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<ActivityWrapper> getActivityWrappersById(final List<Long> ids) throws SystemException {
		return ActivityWrapperLocalServiceUtil.getActivityWrappersByIds(ids);
	}

	/**
	 * Delete an ActivityWrapper.
	 * 
	 * @param activityWrapper
	 *            The ActivityWrapper
	 * @param ctx
	 *            The ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static void deleteActivityWrapper(final ActivityWrapper activityWrapper, final ServiceContext ctx)
			throws SystemException, PortalException {

		PortfolioService.addJournalEntry(activityWrapper, EventType.DELETE, ctx);

		FacadeUtils.removePermissions(activityWrapper, ctx);

		ActivityWrapperLocalServiceUtil.deleteActivityWrapper(activityWrapper);
	}

	/**
	 * Gets the competence levels.
	 * 
	 * @param activityWrapper
	 *            the activity wrapper
	 * @param ctx
	 *            the ctx
	 * @return the competence levels
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<CompetenceLevel> getCompetenceLevels(final ActivityWrapper activityWrapper,
			final ServiceContext ctx) throws SystemException, PortalException {

		return ActivityWrapperLocalServiceUtil.getCompetenceLevels(activityWrapper);
	}

	/**
	 * Adds the competence levels.
	 * 
	 * @param activityWrapper
	 *            the activity wrapper
	 * @param competenceLevels
	 *            the competence levels
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void addCompetenceLevels(final ActivityWrapper activityWrapper,
			final List<CompetenceLevel> competenceLevels, final ServiceContext ctx) throws SystemException,
			PortalException {
		PortfolioService.addJournalEntry(activityWrapper, EventType.UPDATE, ctx);

		ActivityWrapperLocalServiceUtil.addCompetenceLevels(activityWrapper, competenceLevels);
	}

	/**
	 * Clear competence levels.
	 * 
	 * @param activityWrapper
	 *            the activity wrapper
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void clearCompetenceLevels(final ActivityWrapper activityWrapper, final ServiceContext ctx)
			throws SystemException, PortalException {
		PortfolioService.addJournalEntry(activityWrapper, EventType.UPDATE, ctx);

		ActivityWrapperLocalServiceUtil.clearCompetenceLevels(activityWrapper);
	}

	/**
	 * Removes the competence levels.
	 * 
	 * @param activityWrapper
	 *            the activity wrapper
	 * @param competenceLevels
	 *            the competence levels
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void removeCompetenceLevels(final ActivityWrapper activityWrapper,
			final List<CompetenceLevel> competenceLevels, final ServiceContext ctx) throws SystemException,
			PortalException {
		PortfolioService.addJournalEntry(activityWrapper, EventType.UPDATE, ctx);

		ActivityWrapperLocalServiceUtil.removeCompetenceLevels(activityWrapper, competenceLevels);
	}

}
