/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.AssessmentTestException;
import org.tencompetence.business.facade.validators.AssessmentTestValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.generated.service.AssessmentTestLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.service.ServiceContext;
import com.qti.slayer.model.Uol;
import com.qti.slayer.model.UolModel;
import com.qti.slayer.service.UolLocalService;
import com.qti.slayer.service.UolLocalServiceUtil;

/**
 * The Class AssesmentTestFacade.
 * 
 * @author venrooijma
 * @version $Revision: 1.11 $, $Date: 2009/11/24 14:18:19 $
 */
public final class AssessmentTestFacade {
	private AssessmentTestFacade() {
	}

	/**
	 * Creates the assessment test.
	 * 
	 * @return the assessment test
	 * @throws SystemException
	 *             the system exception
	 */
	public static AssessmentTest createAssessmentTest() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();

		final AssessmentTest result = AssessmentTestLocalServiceUtil.createAssessmentTest(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the assessment test and updates Uol Repository.
	 * 
	 * @param assessmentTest
	 *            the assessment test
	 * @param ctx
	 *            the ctx
	 * @return the assessment test
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentTest addAssessmentTest(final AssessmentTest assessmentTest, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<String> errors = new ArrayList<String>();

		if (!AssessmentTestValidator.validate(assessmentTest, errors)) {
			throw new AssessmentTestException("Invalid AssessmentTest : " + errors.toString());
		}

		final AssessmentTest result = AssessmentTestLocalServiceUtil.addAssessmentTest(assessmentTest);
		
		//Creant entrada a la tabla Uol
		
		//set primary key
		long UolId =0L;
		
		try 
		{
			UolId = CounterLocalServiceUtil.increment(Uol.class.getName());
		} 
		
		catch (SystemException e)
		{
			e.printStackTrace();
			
		}
		
		Uol uolAux = UolLocalServiceUtil.createUol(UolId);
		//set Additional Fields
		uolAux.setAssessmentTestId(result.getAssessmentTestId());
		uolAux.setTitle(result.getTitle());
		uolAux.setDescription(result.getTitle());
		uolAux.setCreateDate(new Date());

		//insert the Uol using persistence API
		
		try 
		{
			UolLocalServiceUtil.addUol(uolAux);
		}
		catch (SystemException e) 
		{
			e.printStackTrace();	
		}
		
		//Fi entrada taula UOL
		
		FacadeUtils.addPermissions(result, ctx);

		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		return result;
	}

	/**
	 * Removes the assessment test.
	 * 
	 * @param assessmentTest
	 *            The assessment test
	 * @param ctx
	 *            The ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static void deleteAssessmentTest(final AssessmentTest assessmentTest, final ServiceContext ctx)
			throws SystemException, PortalException {

		PortfolioService.addJournalEntry(assessmentTest, EventType.DELETE, ctx);

		FacadeUtils.removePermissions(assessmentTest, ctx);

		AssessmentTestLocalServiceUtil.deleteAssessmentTest(assessmentTest);
	}

	/**
	 * Update assessment test.
	 * 
	 * @param assessmentTest
	 *            the assessment test
	 * @param ctx
	 *            the ctx
	 * @return the assessment test
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentTest updateAssessmentTest(final AssessmentTest assessmentTest, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<String> errors = new ArrayList<String>();

		if (!AssessmentTestValidator.validate(assessmentTest, errors)) {
			throw new AssessmentTestException("Invalid AssessmentTest : " + errors.toString());
		}

		final AssessmentTest result = AssessmentTestLocalServiceUtil.updateAssessmentTest(assessmentTest);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Gets the assessment test by its id.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the assessment test
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentTest getAssessmentTest(final long assessmentTestId, final ServiceContext ctx)
			throws SystemException, PortalException {
		return AssessmentTestLocalServiceUtil.getAssessmentTest(assessmentTestId);
	}

	/**
	 * Gets the assessment test by identifier.
	 * 
	 * @param identifier
	 *            the identifier
	 * @param ctx
	 *            the ctx
	 * @return the assessment test by identifier
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentTest getAssessmentTestByIdentifier(final String identifier, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<AssessmentTest> list = AssessmentTestLocalServiceUtil.getAssessmentTestsByIdentifier(identifier);
		if (list != null) {
			if (list.isEmpty()) {
				return null;
			} else if (list.size() == 1) {
				return list.get(0);
			} else {
				throw new AssessmentTestException("More then 1 AssessmentTest found for identifier : " + identifier);
			}
		}
		return null;
	}

	/**
	 * Gets the assessment tests.
	 * 
	 * @param ctx
	 *            the ctx
	 * @return the assessment tests
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<AssessmentTest> getAssessmentTests(final ServiceContext ctx) throws SystemException {
		return AssessmentTestLocalServiceUtil.getAssessmentTests();
	}

	/**
	 * Gets the assessment tests by groupId.
	 * 
	 * @param groupId
	 *            the group id
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the assessment tests
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<AssessmentTest> getAssessmentTestsByGroupId(final long groupId, final ServiceContext ctx)
			throws SystemException {
		return AssessmentTestLocalServiceUtil.getAssessmentTestsByGroupId(groupId);
	}

	/**
	 * Gets the assessment items by assessment test.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param ctx
	 *            the ctx
	 * @return the assessment items by assessment test
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<AssessmentItem> getAssessmentItemsByAssessmentTest(final long assessmentTestId,
			final ServiceContext ctx) throws SystemException {
		return AssessmentTestLocalServiceUtil.getAssessmentItemsByAssessmentTest(assessmentTestId);
	}

	/**
	 * Sets the assessment items for assessment test.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param assessmentItemIds
	 *            the assessment item ids
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 */
	public static void setAssessmentItemsForAssessmentTest(final long assessmentTestId, final long[] assessmentItemIds,
			final ServiceContext ctx) throws SystemException {
		AssessmentTestLocalServiceUtil.setAssessmentItemsForAssessmentTest(assessmentTestId, assessmentItemIds);
	}

	/**
	 * Adds the assessment item for assessment test.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param assessmentItemId
	 *            the assessment item id
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 */
	public static void addAssessmentItemToAssessmentTest(final long assessmentTestId, final long assessmentItemId,
			final ServiceContext ctx) throws SystemException {
		AssessmentTestLocalServiceUtil.addAssessmentItemToAssessmentTest(assessmentTestId, assessmentItemId);
	}

	/**
	 * Adds the assessment items for assessment test.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param assessmentItemIds
	 *            the assessment item ids
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 */
	public static void addAssessmentItemsToAssessmentTest(final long assessmentTestId, final long[] assessmentItemIds,
			final ServiceContext ctx) throws SystemException {
		AssessmentTestLocalServiceUtil.addAssessmentItemsToAssessmentTest(assessmentTestId, assessmentItemIds);
	}

	/**
	 * Removes the assessment item from assessment test.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param assessmentItemId
	 *            the assessment item id
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 */
	public static void removeAssessmentItemFromAssessmentTest(final long assessmentTestId, final long assessmentItemId,
			final ServiceContext ctx) throws SystemException {
		AssessmentTestLocalServiceUtil.removeAssessmentItemFromAssessmentTest(assessmentTestId, assessmentItemId);
	}

	/**
	 * Removes the assessment items from assessment test.
	 * 
	 * @param assessmentTestId
	 *            the assessment test id
	 * @param assessmentItemIds
	 *            the assessment item ids
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 */
	public static void removeAssessmentItemsFromAssessmentTest(final long assessmentTestId,
			final long[] assessmentItemIds, final ServiceContext ctx) throws SystemException {
		AssessmentTestLocalServiceUtil.removeAssessmentItemsfromAssessmentTest(assessmentTestId, assessmentItemIds);
	}

	/**
	 * Gets the assessment tests count.
	 * 
	 * @param ctx
	 *            the ctx
	 * @return the assessment tests count
	 * @throws SystemException
	 *             the system exception
	 */
	public static int getAssessmentTestsCount(final ServiceContext ctx) throws SystemException {
		return AssessmentTestLocalServiceUtil.getAssessmentTestsCount();
	}

	/**
	 * Gets the assessment tests by ids.
	 * 
	 * @param ids
	 *            the ids
	 * @return the assessment tests by ids
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<AssessmentTest> getAssessmentTestsByIds(final List<Long> ids) throws SystemException {
		return AssessmentTestLocalServiceUtil.getAssessmentTestsByIds(ids);
	}

	/**
	 * Gets a assessmentTest by the primary key.
	 * 
	 * Beware the difference between this method and the @see(getAssessmentTestByIdentifier)
	 * 
	 * @param assessmentTestId
	 *            the primary key of the assessmentTest
	 * @param ctx
	 *            the service context
	 * @return the assessmentTest
	 * @throws PortalException
	 *             the PortalException
	 * @throws SystemException
	 *             the {@link SystemException}
	 */
	public static AssessmentTest getAssessmentTestByLongId(final long assessmentTestId, final ServiceContext ctx)
			throws PortalException, SystemException {
		return AssessmentTestLocalServiceUtil.getAssessmentTest(assessmentTestId);
	}

}
