/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade.validators;

import java.util.List;

import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.NoSuchCompetenceProfileException;
import org.tencompetence.generated.NoSuchLevelValueException;
import org.tencompetence.generated.model.CompetenceProfileLevel;

import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;

/**
 * This validator is used to validate a CompetenceProfileLevel.
 * 
 * @author sheyenrath
 * @author Harrie Martens
 * @version $Id: CompetenceMapFacade.java,v 1.1 2009/10/02 10:46:24 sheyenrath
 *          Exp $
 */
public class CompetenceProfileLevelValidator extends GenericValidator {

	/**
	 * Validate the CompetenceProfileLevel.
	 * 
	 * @param entity
	 *            The CompetenceProfileLevel
	 * @param errors
	 *            A String list from the errors
	 * @param ctx
	 *            the ctx
	 * @return validate check
	 *             Exception
	 */
	public static boolean validate(final CompetenceProfileLevel entity, final List<String> errors,
			final ServiceContext ctx) {
		if (Validator.isNull(entity)) {
			errors.add("CompetenceProfileLevel required");
			return false;
		}
		boolean valid = validateGeneralPropertiesEntity(entity, errors);

		if (Validator.isNull(entity.getVersion())) {
			errors.add("Version required");
			valid = false;
		}

		if (Validator.isNull(entity.getVersionTag())) {
			errors.add("VersionTag required");
			valid = false;
		}

		try {
			CompetenceProfileFacade.getCompetenceProfile(entity.getCompetenceProfileId(), ctx);
		} catch (NoSuchCompetenceProfileException e) {
			errors.add("CompetenceProfile does not exist");
			valid = false;
		} catch (Exception e) {
			errors.add("Exception occurred when checking CompetenceProfile: " + e.getMessage());
			valid = false;
		}

		try {
			LevelValueFacade.getLevelValue(entity.getLevelValueId(), ctx);
		} catch (NoSuchLevelValueException e) {
			errors.add("LevelValue does not exist");
			valid = false;
		} catch (Exception e) {
			errors.add("Exception occurred when checking LevelValue: " + e.getMessage());
			valid = false;
		}

		valid &= validateEntities(entity.getCompetenceLevelIds(), errors);

		return valid;
	}
}
