/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade.validators;

import java.util.List;

import org.tencompetence.generated.model.Goal;

import com.liferay.portal.kernel.util.Validator;

/**
 * Validates a Goal entity.
 * 
 * @author venrooijma
 * @author Harrie Martens
 * @version $Id: GoalValidator.java,v 1.4 2009/11/11 16:09:25 hubertvogten Exp $
 */
public final class GoalValidator extends GenericValidator {
	
	private GoalValidator() {
		// empty private constructor
	}

	/**
	 * Method to validate a Goal.
	 * 
	 * @param entity
	 *            the goal to validate
	 * @param errors
	 *            a list to contain all errors
	 * @return boolean indicating if the entity is valid
	 */
	public static boolean validate(final Goal entity, final List<String> errors) {
		boolean valid = true;
		
		if (Validator.isNull(entity)) {
			errors.add("Goal required");
			return false;
		}
		
		valid &= validateTitle(entity, errors);

		if (entity.getMotivation() == null) {
			errors.add("Motivation required");
			valid = false;
		}
		
		valid &= validateUCG(entity, errors);

		if ((Validator.isNotNull(entity.getTargetType()) && Validator.isNull(entity.getTarget()))
				|| (Validator.isNull(entity.getTargetType()) && Validator.isNotNull(entity.getTarget()))) {
			errors.add("Target fields are either both required or both left null");
			valid = false;
		}

		return valid;
	}
}