/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade.validators;

import java.util.List;

import org.tencompetence.business.constants.LearningPathType;
import org.tencompetence.generated.model.LearningPath;

import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;

/**
 * This validator is used to validate a LearningPath.
 * 
 * @author sheyenrath
 * @author Harrie Martens
 * @version $Id: LearningPathValidator.java,v 1.7 2009/11/04 14:36:52 sheyenrath Exp $
 */
public class LearningPathValidator extends GenericValidator {

	/**
	 * Validate the CompetenceMap.
	 * 
	 * @param entity
	 *            The CompetenceMap
	 * @param errors
	 *            A String list from the errors
	 * @param ctx
	 *            the ctx
	 * @return validate check
	 */
	public static boolean validate(final LearningPath entity, final List<String> errors, final ServiceContext ctx) {

		if (Validator.isNull(entity)) {
			errors.add("LearningPath required");
			return false;
		}

		boolean valid = true;

		if (Validator.isNull(entity.getType())) {
			errors.add("Type required");
			valid &= false;
		} else {
			try {
				LearningPathType.valueOf(entity.getType());
			} catch (IllegalArgumentException e) {
				errors.add("Type is not valid. Must be PATH or PLAN.");
				valid &= false;
			}
		}

		valid &= validateUCG(entity, errors);

		valid &= validateEntities(entity.getCoeditorIds(), errors);

		return valid;
	}
}
