/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath;

import java.util.List;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

/**
 * The Interface ICursor.
 * @version $Revision: 1.3 $, $Date: 2009/11/24 14:18:19 $
 */
public interface ICursor {

	/**
	 * Push.
	 * 
	 * @param element
	 *            the element
	 */
	void push(final ILPElement element);

	/**
	 * Pop.
	 * 
	 * @return the iLP element
	 */
	ILPElement pop();

	/**
	 * Gets the current element.
	 * 
	 * @return the current element
	 */
	ILPElement getCurrentElement();

	/**
	 * Gets all <code>LPElement</code> of the cursor.
	 * 
	 * @return all <code>LPElement</code> of the cursor
	 */
	List<ILPElement> getElements();

	/**
	 * Returns the <code>ILearningPlan</code> for this <code>ICursor</code>.
	 * 
	 * @return the <code>ILearningPlan</code>
	 */
	ILearningPlan getLearningPlan();
	
	/**
	 * Subscribes to all elements of the cursor that require a subscription but are not yet subscribed.
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	void subScribe() throws SystemException, PortalException;
}