/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath;

import java.util.Date;
import java.util.List;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

/**
 * The interface ILPElement.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.3 $, $Date: 2009/11/17 22:07:54 $
 */
public interface ILPElement {

	/**
	 * Gets the id of the element.
	 * 
	 * @return the id
	 */
	String getId();

	/**
	 * Gets the uri of the element.
	 * 
	 * @return the uri
	 */
	String getUri();

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	String getTitle();

	/**
	 * Gets the description.
	 * 
	 * @return the description
	 */
	String getDescription();

	/**
	 * Gets the meta data.
	 * 
	 * @return the meta data
	 */
	List<MetaDataEntry> getMetaData();

	/**
	 * Return the learning path in which the current <code>ILPElement</code> is defined. (Note that this method returns
	 * 'null' when executed on a ILearningPath.
	 * 
	 * @return getLearningPath
	 */
	ILearningPath getLearningPath();

	/**
	 * Checks if is completed.
	 * 
	 * @return true, if is completed
	 */
	boolean isCompleted();

	/**
	 * Sets this LPElement to completed. (This is directly persisted in database)
	 * 
	 * @param value
	 *            the new completed
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	void setCompleted(final boolean value) throws SystemException, PortalException;

	/**
	 * Checks if is subscribed.
	 * 
	 * @return true, if is subscribed
	 */
	boolean isSubscribed();

	/**
	 * Returns true if a user can subscribe to this element. False otherwise.
	 * 
	 * @return true if a user can subscribe to this element. False otherwise
	 */
	boolean canSubscribe();

	/**
	 * Sets this LPElement to subscribed. (This is directly persisted in database)
	 * 
	 * @param value
	 *            the new subscribed
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	void setSubscribed(final boolean value) throws SystemException, PortalException;

	/**
	 * Gets the SubscribeDate.
	 * 
	 * @return the SubscribeDate
	 */
	Date getSubscribeDate();

	/**
	 * Gets the CompletedDate.
	 * 
	 * @return the CompletedDate
	 */
	Date getCompletedDate();
}