/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath;

import java.util.List;

import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.services.jaxb.model.lp.ExternalLearningPath;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

/**
 * The interface ILearningPath.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.3 $, $Date: 2009/11/11 13:18:54 $
 */
public interface ILearningPath extends ILPElement, ICluster {
	/**
	 * Gets the uri of the element.
	 * 
	 * @return the uri
	 */
	String getUri();

	/**
	 * Gets the jAXB representation.
	 * 
	 * @return the jAXB representation
	 */
	ExternalLearningPath createJAXBRepresentation();

	/**
	 * Sets the xml data and refreshes the internal model.
	 * 
	 * @param xmlData
	 *            the xmlData
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	void setXmlData(final String xmlData) throws PortalException, SystemException;

	/**
	 * get a list of CompetenceLevels belonging to this learningPath.
	 * 
	 * @return List<CompetenceLevel>
	 */
	List<CompetenceLevel> getCompetenceLevels();

	/**
	 * Gets the LPElement by the uri.
	 * 
	 * @param uri
	 *            the uri
	 * @return the LPElement
	 */
	ILPElement getLPElementByUri(final String uri);
	
	/**
	 * Gets the LPElement by the id.
	 * 
	 * @param id
	 *            the id
	 * @return the LPElement
	 */
	ILPElement getLPElementById(final String id);

	/**
	 * Checks if this LearningPath provides a competenceLevel. 'Provides' means that the competenceLevel is equal or
	 * less then the competenceLevel in the LearningPath.
	 * 
	 * @param competenceLevel
	 *            CompetenceLevel
	 * 
	 * @return boolean boolean
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	boolean providesLevel(final CompetenceLevel competenceLevel) throws SystemException, PortalException;

	/**
	 * getFinishCompetenceProfileLevel. Note: this is the finish competence profile level. The start isn't implemented
	 * in this release of TENC
	 * 
	 * @return CompetenceProfileLevel CompetenceProfileLevel
	 */
	CompetenceProfileLevel getFinishCompetenceProfileLevel();
	
	/**
	 * Gets a list from all actions from this path. Note : this is done with recursion.)
	 * 
	 * @return the actions
	 */
	List<ILearningAction> getLearningActions();
}
