/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath;

import java.util.List;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

/**
 * The interface ILearningPlan.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.2 $, $Date: 2009/11/17 22:07:54 $
 */
public interface ILearningPlan extends ILearningPath, ISelection {

	/**
	 * Sets the selected LearningPaths.
	 * 
	 * @param selectedLearningPaths
	 *            the LearningPaths
	 */
	void setChildren(final List<ILearningPath> selectedLearningPaths);

	/**
	 * Set the current cursor. When the cursor is set, it will be persisted at once.
	 * 
	 * @param cursor
	 *            ICursor
	 * 
	 * @throws PortalException
	 *             PortalException
	 * @throws SystemException
	 *             SystemException
	 */
	void setCursor(final ICursor cursor) throws SystemException, PortalException;

	/**
	 * return the cursor.
	 * 
	 * @return the cursor
	 * 
	 * @throws PortalException
	 *             PortalException
	 * @throws SystemException
	 *             SystemException
	 */
	ICursor getCursor() throws SystemException, PortalException;

	/**
	 * This method returns a <code>ICursor</code> representation of the next learning action a user can perform in the
	 * learning plan, or null if there are no more learning actions to perform.
	 * 
	 * @return the <code>ICursor</code> representation of the next learning action a user must perform in he current
	 *         context, or null if there are not learning actions to perform
	 */
	ICursor getNextLearningAction();
}
