/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.model.learningpath.ICompetenceLevel;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The class CompetenceLevelImpl.
 * 
 * @author Stef Heyenrath
 * @author jal
 * 
 * @version $Revision: 1.1 $, $Date: 2009/10/28 12:59:26 $
 */
public class CompetenceLevelImpl implements ICompetenceLevel {
	private final Competence competence;
	private final CompetenceLevel competenceLevel;
	private final LevelValue competenceLevelValue;

	/**
	 * Create a new CompetenceLevelImpl.
	 * 
	 * @param competenceLevel
	 *            CompetenceLevel
	 * @param ctx
	 *            ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public CompetenceLevelImpl(final CompetenceLevel competenceLevel, final ServiceContext ctx) throws SystemException,
			PortalException {
		this.competenceLevel = competenceLevel;
		this.competence = CompetenceFacade.getCompetence(competenceLevel.getCompetenceId(), ctx);
		this.competenceLevelValue = LevelValueFacade.getLevelValue(competenceLevel.getLevelValueId(), ctx);
	}

	/** {@inheritDoc} */
	@Override
	public Competence getCompetence() {
		return competence;
	}

	/** {@inheritDoc} */
	@Override
	public CompetenceLevel getCompetenceLevel() {
		return competenceLevel;
	}

	/** {@inheritDoc} */
	@Override
	public LevelValue getCompetenceLevelValue() {
		return competenceLevelValue;
	}

	/** {@inheritDoc} */
	@Override
	public int getLevelValue() {
		return competenceLevelValue.getValue();
	}
}