/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.MetaDataEntry;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.business.services.SubscribeService;
import org.tencompetence.business.services.PortfolioService.SubscribeType;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.generated.model.PortfolioEntry;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The class LPElementImpl.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * @version $Revision: 1.5 $, $Date: 2009/11/17 22:07:54 $
 */
public abstract class LPElementImpl implements ILPElement {
	private final String interfaceName = getClass().getInterfaces()[0].getName();;
	private final List<String> ignoreProperties = Arrays.asList("id", "uri", "title", "description");

	private final ILearningPath learningPath;
	private boolean isSubscribed = false;
	private boolean isCompleted = false;
	private Date subscribeDate = null;
	private Date completeDate = null;
	private String parentId = null;
	private ServiceContext ctx;

	/**
	 * Instantiates a new LPElementImpl.
	 * 
	 * @param ctx
	 *            the ctx
	 */
	public LPElementImpl(final ServiceContext ctx) {
		this(null, ctx);
	}

	/**
	 * Instantiates a new LPElementImpl.
	 * 
	 * @param learningPath
	 *            the learning path
	 * @param ctx
	 *            the ctx
	 */
	public LPElementImpl(final ILearningPath learningPath, final ServiceContext ctx) {
		this.learningPath = learningPath;
		this.ctx = ctx;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isCompleted() {
		return isCompleted;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSubscribed() {
		return isSubscribed;
	}

	/** {@inheritDoc} */
	@Override
	public List<MetaDataEntry> getMetaData() {
		final List<MetaDataEntry> result = new ArrayList<MetaDataEntry>();
		final Object metaData = ClassUtils.getProperty(getJAXBElement(), "Metadata");
		if (metaData != null) {
			crawlMetadata(metaData, result);
		}
	
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public Date getCompletedDate() {
		return completeDate;
	}

	/** {@inheritDoc} */
	@Override
	public ILearningPath getLearningPath() {
		return learningPath;
	}

	/** {@inheritDoc} */
	@Override
	public Date getSubscribeDate() {
		return subscribeDate;
	}

	/** {@inheritDoc} */
	@Override
	public void setCompleted(final boolean value) {
		if (value) {
			try {
				PortfolioService.addJournalEntry(getContext().getUserId(), interfaceName, getUri(), EventType.COMPLETE,
						parentId, ctx);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			completeDate = new Date();
		}
	
		isCompleted = value;
	}

	/** {@inheritDoc} */
	@Override
	public void setSubscribed(final boolean subscribe) {
		isSubscribed = subscribe;
		try {
			subscribeToEntity();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((getId() == null) ? 0 : getId().hashCode());
		result = prime * result + ((learningPath == null) ? 0 : learningPath.hashCode());
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof LPElementImpl)) {
			return false;
		}
		LPElementImpl other = (LPElementImpl) obj;
		if (getId() == null) {
			if (other.getId() != null) {
				return false;
			}
		} else if (!getId().equals(other.getId())) {
			return false;
		}
		if (learningPath == null) {
			if (other.learningPath != null) {
				return false;
			}
		} else if (!learningPath.equals(other.learningPath)) {
			return false;
		}
		return true;
	}

	/**
	 * Adds the metadata entry.
	 * 
	 * @param list
	 *            the metadata
	 * @param key
	 *            the key
	 * @param value
	 *            the value
	 */
	protected void addMetadataEntry(final List<MetaDataEntry> list, final String key, final Object value) {
		if (key != null && value != null) {
			final String strval = getString(value);
			if (strval != null) {
				list.add(new MetaDataEntry(key, strval));
			}
		}
	}

	/**
	 * Gets the JAXBElement.
	 * 
	 * @return the JAXBElement
	 */
	protected abstract Object getJAXBElement();

	/**
	 * Gets the context.
	 * 
	 * @return the context
	 */
	protected ServiceContext getContext() {
		return ctx;
	}

	/**
	 * Returns the cursor pointing to the next learning action a user can perform, or null if no learning action can be
	 * found. The cursor is build upon the passed cursor.
	 * 
	 * @param cursor
	 *            the starting point for the returned cursor
	 * @return the cursor pointing to the next learning action a user can perform
	 */
	protected abstract ICursor getNextLearningAction(ICursor cursor);

	/**
	 * Inits this LPElement.
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	protected void init() throws SystemException, PortalException {
		parentId = learningPath != null ? learningPath.getUri() : null;

		if (ctx != null) {
			final PortfolioEntry subscribed = SubscribeService.getSubscribedFromEntity(ctx.getUserId(), interfaceName,
					getUri(), parentId, ctx);
			final PortfolioEntry completed = PortfolioService.getLatestEntryByUserEventType(ctx.getUserId(),
					interfaceName, getUri(), EventType.COMPLETE, parentId);

			if (subscribed != null) {
				isSubscribed = true;
				subscribeDate = subscribed.getJournalDate();
			}

			if (completed != null) {
				isCompleted = true;
				completeDate = completed.getJournalDate();
			}
		}
	}

	/**
	 * Subscribe this entity.
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	protected void subscribeToEntity() throws SystemException, PortalException {
		if (isSubscribed) {
			SubscribeService.subscribeToEntity(getContext().getUserId(), interfaceName, getUri(), SubscribeType.ONCE,
					parentId, ctx);
			subscribeDate = new Date();
		} else {
			SubscribeService.unsubscribeFromEntity(getContext().getUserId(), interfaceName, getUri(), parentId, ctx);
		}
	}

	@SuppressWarnings("unchecked")
	private void crawlMetadata(final Object object, final List<MetaDataEntry> result) {
		final List<String> properties = ClassUtils.getAllProperties(object);
		properties.removeAll(ignoreProperties);

		for (final String property : properties) {
			final Object value = ClassUtils.getProperty(object, property);
			if (value != null) {
				if (value instanceof String || value instanceof Boolean || value instanceof Integer
						|| value instanceof BigInteger) {
					addMetadataEntry(result, property, value);
				} else if (value instanceof List) {
					final String csv = crawlList(property, (List) value, result);
					if (csv.length() > 0) {
						addMetadataEntry(result, property + "list", csv);
					}
				}
			}
		}
	}

	@SuppressWarnings("unchecked")
	private String crawlList(final String property, final List list, final List<MetaDataEntry> result) {
		String csv = "";
		for (final Object listObject : list) {
			if (listObject instanceof String || listObject instanceof Boolean || listObject instanceof Integer
					|| listObject instanceof BigInteger) {
				csv += listObject.toString() + " ";
			}
		}

		return csv.trim();
	}

	private static String getString(final Object object) {
		if (object instanceof String) {
			return (String) object;
		} else if (object instanceof Boolean) {
			return ((Boolean) object).toString();
		} else if (object instanceof Integer) {
			return ((Integer) object).toString();
		} else if (object instanceof BigInteger) {
			return ((BigInteger) object).toString();
		} else if (object instanceof XMLGregorianCalendar) {
			return ((XMLGregorianCalendar) object).toString();
		} else {
			return null;
		}
	}
}