/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import java.util.List;

import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.MetaDataEntry;
import org.tencompetence.services.jaxb.model.lp.LearningAction;
import org.tencompetence.services.jaxb.model.lp.LearningAction.ActionMetadata.ActionRunInformation;
import org.tencompetence.services.jaxb.model.lp.LearningAction.ActionMetadata.Costs;
import org.tencompetence.services.jaxb.model.lp.LearningAction.ActionMetadata.ActionRunInformation.Location;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;



/**
 * Enter the class documentation here.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.5 $, $Date: 2009/11/17 22:07:54 $
 */
public class LearningActionImpl extends LPElementImpl implements ILearningAction {
	private final LearningAction action;

	/**
	 * Instantiates a new learning action impl.
	 * 
	 * @param action
	 *            the action
	 * @param learningPathImpl
	 *            the learning path impl
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public LearningActionImpl(final LearningAction action, final LearningPathImpl learningPathImpl,
			final ServiceContext ctx) throws SystemException, PortalException {
		super(learningPathImpl, ctx);
		if (action == null) {
			throw new IllegalArgumentException("Action should not be null");
		}
		this.action = action;
		init();
	}
	
	/** {@inheritDoc} */
	@Override
	public List<MetaDataEntry> getMetaData() {
		final List<MetaDataEntry> metadata = super.getMetaData();

		if (action.getMetadata() != null) {
			final Costs costs = action.getMetadata().getCosts();
			if (costs != null) {
				addMetadataEntry(metadata, "costs.currency", costs.getCurrency());
				addMetadataEntry(metadata, "costs.value", costs.getValue());
			}

			final List<ActionRunInformation> runs = action.getMetadata().getRunInformation();
			if (runs != null) {
				for (final ActionRunInformation ri : runs) {
					final List<Location> locations = ri.getLocation();
					if (locations != null) {
						for (final Location location : locations) {
							addMetadataEntry(metadata, "locations", location);
						}
					}

					addMetadataEntry(metadata, "startDate", ri.getStartDate());
					addMetadataEntry(metadata, "endDate", ri.getEndDate());
				}
			}
		}

		return metadata;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getJAXBElement() {
		return action;
	}

	/** {@inheritDoc} */
	@Override
	public String getDescription() {
		return action.getMetadata().getDescription();
	}

	/** {@inheritDoc} */
	@Override
	public String getTitle() {
		return action.getMetadata().getTitle();
	}

	/** {@inheritDoc} */
	@Override
	public String getId() {
		return action.getId();
	}
	
	/** {@inheritDoc} */
	@Override
	public String getUri() {
		return action.getMetadata().getUri();
	}
	
	/** {@inheritDoc} */
	public ICursor getNextLearningAction(final ICursor cursor) {
		cursor.push(this);
		return isCompleted() ? null : cursor;
	}

	@Override
	public boolean canSubscribe() {
		return true;
	}
}
