/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.PortfolioEntry;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The class LearningPlanImpl.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * @version $Revision: 1.6 $, $Date: 2009/11/17 22:07:54 $
 */
public class LearningPlanImpl extends LearningPathImpl implements ILearningPlan {

	private List<ILearningPath> children = new ArrayList<ILearningPath>();
	private ICursor cursor;
	private boolean cursorNeedsRefresh;

	/**
	 * The Constructor.
	 * 
	 * @param xml
	 *            the xml
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public LearningPlanImpl(final String xml, final ServiceContext ctx) throws PortalException, SystemException {
		super(xml, ctx);
		// Get the selected LearningPaths from the children from the root Cluster.
		for (ILPElement child : super.getChildren()) {
			if (child instanceof ILearningPath) {
				children.add((ILearningPath) child);
			}
		}
		// make sure the cursor is retrieved at the next call of getCursor
		cursorNeedsRefresh = true;
	}

	/** {@inheritDoc} */
	@Override
	public List<? extends ILPElement> getChildren() {
		return children;
	}

	/** {@inheritDoc} */
	@Override
	public void setChildren(final List<ILearningPath> selectedLPs) {
		this.children = selectedLPs;
	}

	/** {@inheritDoc} */
	public void setCursor(final ICursor cursor) {
		try {
			PortfolioService.addJournalEntry(getContext().getUserId(), ILearningPlan.class.getName(), getUri(),
					EventType.LEARNING_PLAN_CURSOR, cursor.toString(), getContext());
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		cursorNeedsRefresh = false;
		this.cursor = cursor;
	}

	/** {@inheritDoc} */
	public ICursor getCursor() {
		if (cursorNeedsRefresh) {
			// get the cursor from the portfolio
			PortfolioEntry entry = null;
			try {
				entry = PortfolioService.getLatestEntryByUserEventType(getContext().getUserId(),
						ILearningPlan.class.getName(), getUri(), EventType.LEARNING_PLAN_CURSOR, null);
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if (entry != null) {
				String value = entry.getValue();
				if (value != null) {
					try {
						cursor = new CursorImpl(this, value);
					} catch (PortalException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (SystemException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
			cursorNeedsRefresh = false;
		}
		return this.cursor;
	}

	/** {@inheritDoc} */
	@Override
	public int getNumberToSelect() {
		return children.size();
	}

	/** {@inheritDoc} */
	@Override
	public void setCompleted(final boolean value) {
		// Do nothing
	}

	@Override
	public ICursor getNextLearningAction() {
		ICursor cursor = new CursorImpl(this);
		for (ILPElement lpe : getChildren()) {
			if (!lpe.isCompleted()) {
				ICursor result = ((LPElementImpl) lpe).getNextLearningAction(cursor);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}
}