/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.Map;

import org.json.simple.JSONValue;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.tencompetence.business.constants.EventType;
import org.tencompetence.generated.model.PortfolioEntry;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * Filter options service.
 * 
 * @author Anton Finders
 * 
 * @version $Revision: 1.3 $, $Date: 2009/11/30 14:03:19 $
 */
public final class FilterOptionsService {

	private static final Log LOG = LogFactoryUtil.getLog(FilterOptionsService.class);

	private FilterOptionsService() {
		// private constructor
	}

	/**
	 * Write user filteroptions to portFolio.
	 * 
	 * @param filter
	 *            the filter
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void writeFilterOptions(final Map<String, String> filter, final ServiceContext ctx)
			throws SystemException, PortalException {
		PortfolioService.addJournalEntry(ctx.getUserId(), FilterOptionsService.class.getName(), "-1",
				EventType.FILTER_OPTIONS, convertToJsonObject(filter), ctx);
	}

	/**
	 * Read user filteroptions from portFolio.
	 * 
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the map
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static Map<String, String> readFilterOptions(final ServiceContext ctx) throws SystemException {
		final PortfolioEntry entry = PortfolioService.getLatestEntryByUserEventType(ctx.getUserId(),
				FilterOptionsService.class.getName(), "-1", EventType.FILTER_OPTIONS, null);
		if (entry == null) {
			return null;
		}

		return convertFromJsonObject(entry.getValue());
	}

	private static String convertToJsonObject(final Map<String, String> options) {
		return JSONValue.toJSONString(options);
	}

	@SuppressWarnings("unchecked")
	private static Map<String, String> convertFromJsonObject(final String filters) {
		if (filters.isEmpty()) {
			return null;
		}

		final JSONParser parser = new JSONParser();
		try {
			return (Map<String, String>) parser.parse(filters);
		} catch (ParseException e) {
			LOG.error(e.getMessage());
		}

		return null;
	}
}