/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.business.facade.JournalEntryFacade;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.PortfolioEntry;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * Goal service.
 * 
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.2 $, $Date: 2009/11/09 15:57:53 $
 */
public final class GoalService {

	private static final Log LOG = LogFactoryUtil.getLog(GoalService.class);

	private GoalService() {
		// private constructor
	}

	/**
	 * Select goal for the user in this community.
	 * 
	 * @param userId
	 *            the user id
	 * @param groupId
	 *            the group id
	 * @param goal
	 *            the goal
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void selectGoalForUserCommunity(final long userId, final long groupId, final Goal goal,
			final ServiceContext ctx) throws SystemException, PortalException {
		PortfolioService.addJournalEntry(goal, EventType.GOAL_SELECTED, Long.toString(groupId), ctx);
	}

	/**
	 * Gets the selected goal for the user by community.
	 * 
	 * @param userId
	 *            the user id
	 * @param groupId
	 *            the group id
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the selected goal for user community
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Goal getSelectedGoalForUserCommunity(final long userId, final long groupId, final ServiceContext ctx)
			throws SystemException, PortalException {
		List<PortfolioEntry> list = JournalEntryFacade.getEntriesByUserClassEventType(ctx.getUserId(), Goal.class
				.getName(), EventType.GOAL_SELECTED.name());
		PortfolioEntry latestSelection = null;
		for (PortfolioEntry entry : list) {
			if (Long.valueOf(entry.getValue()).equals(Long.valueOf(groupId))) {
				if (latestSelection == null || latestSelection.getJournalDate().before(entry.getJournalDate())) {
					latestSelection = entry;
				}
			}
		}

		if (latestSelection == null) {
			return null;
		}
		try {
			return GoalFacade.getGoal(Long.valueOf(latestSelection.getEntityId()), ctx);
		} catch (NumberFormatException nfe) {
			if (LOG.isDebugEnabled()) {
				LOG.debug(nfe);
			}
			return null;
		}

	}
}