/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.LearningPathFacade;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.impl.LearningPathImpl;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LearningPath;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The class LearningPathService.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.1 $, $Date: 2009/10/28 12:59:27 $
 */
public final class LearningPathService {

	private LearningPathService() {
		// private constructor
	}

	/**
	 * Gets the learning path.
	 * 
	 * @param uri
	 *            the uri
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the learning path
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static ILearningPath getLearningPath(final String uri, final ServiceContext ctx) throws PortalException,
			SystemException {
		final LearningPath path = LearningPathFacade.getLearningPathByUri(uri, ctx);
		return path != null ? new LearningPathImpl(path.getXmlData(), ctx) : null;
	}

	/**
	 * Search ILearningPaths based on a list from required levels.
	 * 
	 * @param requiredLevels
	 *            requiredLevels
	 * 
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the list<ILearningPath>
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<ILearningPath> searchLearningPaths(final List<CompetenceLevel> requiredLevels,
			final ServiceContext ctx) throws SystemException, PortalException {
		final List<ILearningPath> list = new ArrayList<ILearningPath>();

		for (final LearningPath modelPath : LearningPathFacade.getLearningPaths()) {
			final ILearningPath path = new LearningPathImpl(modelPath.getXmlData(), ctx);

			int providedLevels = 0;
			for (final CompetenceLevel level : requiredLevels) {
				if (path.providesLevel(level)) {
					providedLevels++;
				}
			}

			if (providedLevels > 0) {
				list.add(path);
			}
		}
		return list;
	}
}
