/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.FacadeUtils;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * Calculate levels for user class.
 * @author sheyenrath
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:41:01 $
 */
public class UserLevelCalculation {

	private Map<Long, LevelValue> levelsMap = null;
	private Map<Long, Competence> competencesMap = null;
	private Map<Long, CompetenceLevel> competenceLevels = null;
	private Map<Competence, LevelValue> attainedLevels = null;
	private Map<Long, List<CompetenceLevel>> childLevelMap = null;
	private Map<Long, List<CompetenceLevel>> metaLevelMap = null;
	private LevelValue zeroLevel;

	/**
	 * Instantiates a new UserLevelCalculation object.
	 */
	public UserLevelCalculation() {
		levelsMap = null;
		competencesMap = null;
		competenceLevels = null;
		attainedLevels = null;
		childLevelMap = null;
		metaLevelMap = null;
	}

	/**
	 * This method computes and checks all CompetenceLevels for a user.
	 * 
	 * @param userId
	 *            the user id
	 * @param mapId
	 *            the map id
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public void computeAllCompetenceLevels(final long userId, final long mapId, final ServiceContext ctx)
			throws SystemException, PortalException {

		long startTotal = System.currentTimeMillis();
		long start = System.currentTimeMillis();

		// BLOCK 1 : Get all Competences
		start = System.currentTimeMillis();
		competencesMap = new HashMap<Long, Competence>();

		for (final Competence competence : CompetenceFacade.getBareCompetencesByCompetenceMapId(mapId)) {
			competencesMap.put(competence.getPrimaryKey(), competence);
		}
		System.out.println("getBareCompetencesByCompetenceMapId (" + competencesMap.size() + ") : "
				+ (System.currentTimeMillis() - start));

		// BLOCK 2 : Get all CompetenceLevels
		start = System.currentTimeMillis();
		competenceLevels = new HashMap<Long, CompetenceLevel>();
		for (final CompetenceLevel competenceLevel : CompetenceLevelFacade
				.getBareCompetenceLevelsByCompetenceMapId(mapId)) {
			competenceLevels.put(competenceLevel.getPrimaryKey(), competenceLevel);
		}
		System.out.println("getBareCompetenceLevels (" + competenceLevels.size() + ") : "
				+ (System.currentTimeMillis() - start));

		// BLOCK 3 : Get all ChildCompetenceLevels
		start = System.currentTimeMillis();
		childLevelMap = new HashMap<Long, List<CompetenceLevel>>();
		for (final CompetenceLevel competenceLevel : competenceLevels.values()) {
			final List<CompetenceLevel> childList = new ArrayList<CompetenceLevel>();

			for (final Long childId : FacadeUtils.decodeEntries(competenceLevel.getChildLevelIds())) {
				childList.add(competenceLevels.get(childId));
			}

			if (childList.size() > 0) {
				childLevelMap.put(competenceLevel.getPrimaryKey(), childList);
			}
		}
		System.out.println("getChildLevels (" + childLevelMap.size() + ") : " + (System.currentTimeMillis() - start));

		// BLOCK 4 : Get all MetaCompetenceLevels
		start = System.currentTimeMillis();
		metaLevelMap = new HashMap<Long, List<CompetenceLevel>>();
		for (final CompetenceLevel competenceLevel : competenceLevels.values()) {
			final List<CompetenceLevel> metaList = new ArrayList<CompetenceLevel>();

			for (final Long childId : FacadeUtils.decodeEntries(competenceLevel.getMetaLevelIds())) {
				metaList.add(competenceLevels.get(childId));
			}

			if (metaList.size() > 0) {
				metaLevelMap.put(competenceLevel.getPrimaryKey(), metaList);
			}
		}
		System.out.println("getMetaLevels (" + metaLevelMap.size() + ") : " + (System.currentTimeMillis() - start));

		// BLOCK 5 : Get all Attained Competences + LevelValues from the user
		// removed because it caused a class cast exception
		start = System.currentTimeMillis();
		attainedLevels = PdpService.getAttainedLevelsForUser(userId, ctx);
		System.out.println("getAttainedLevelsForUser (" + attainedLevels.size() + ") : "
				+ (System.currentTimeMillis() - start));

		// BLOCK 6 : Get all LevelValues
		start = System.currentTimeMillis();
		levelsMap = new HashMap<Long, LevelValue>();
		for (final LevelValue level : LevelValueFacade.getAttainableCompetenceLevelValues(mapId, ctx)) {
			levelsMap.put(level.getPrimaryKey(), level);
			if (level.getValue() == 0) {
				zeroLevel = level;
			}
		}
		System.out.println("getCompetenceLevelValues (" + levelsMap.size() + ") : "
				+ (System.currentTimeMillis() - start));

		computeCompetenceLevels(userId, ctx);

		System.out.println("Total = " + (System.currentTimeMillis() - startTotal));
	}

	private void computeCompetenceLevels(final long userId, final ServiceContext ctx) throws PortalException,
			SystemException {

		final Map<Competence, LevelValue> dataMap = new HashMap<Competence, LevelValue>();

		long startLoop = System.currentTimeMillis();
		for (final CompetenceLevel level : competenceLevels.values()) {
			final Competence competence = competencesMap.get(level.getCompetenceId());
			final LevelValue value = levelsMap.get(level.getLevelValueId());

			checkCompetenceLevel(userId, dataMap, competence, level, value, ctx);
		}
		System.out.println("Loop CompetenceLevels (" + competenceLevels.size() + ") : "
				+ (System.currentTimeMillis() - startLoop));

		int levelsSet = 0;
		startLoop = System.currentTimeMillis();
		// Loop the dataMap and store this in the portfolio
		for (final Entry<Competence, LevelValue> entry : dataMap.entrySet()) {
			final LevelValue value = entry.getValue();
			final LevelValue userLevel = attainedLevels.get(entry.getKey());

			// If no portfolio entry exists or value differs, add it.
			if (userLevel == null || value.getValue() != userLevel.getValue()) {
				PdpService.setAttainedLevel(entry.getKey(), value, userId, ctx);
				levelsSet++;
			}
		}
		System.out.println("setAttainedLevelForUser (" + levelsSet + ") : " + (System.currentTimeMillis() - startLoop));
	}

	private boolean checkCompetenceLevel(final long userId, final Map<Competence, LevelValue> dataMap,
			final Competence competence, final CompetenceLevel competenceLevel, final LevelValue reqLevel,
			final ServiceContext ctx) throws SystemException, PortalException {

		final long competenceLevelId = competenceLevel.getPrimaryKey();

		List<CompetenceLevel> childOrMetaLevels = new ArrayList<CompetenceLevel>();
		if (childLevelMap.containsKey(competenceLevelId)) {
			childOrMetaLevels = childLevelMap.get(competenceLevelId);
		}
		if (metaLevelMap.containsKey(competenceLevelId)) {
			childOrMetaLevels.addAll(metaLevelMap.get(competenceLevelId));
		}

		boolean attained = true;

		// Leaf
		if (childOrMetaLevels.size() == 0) {
			final LevelValue userLevel = attainedLevels.get(competence);

			if (userLevel == null || userLevel.getValue() < reqLevel.getValue()) {
				attained = false;
			}
		} else {
			// Non-Leaf
			for (final CompetenceLevel subLevel : childOrMetaLevels) {
				final Competence subCompetence = competencesMap.get(subLevel.getCompetenceId());
				final LevelValue subValue = levelsMap.get(subLevel.getLevelValueId());

				attained = checkCompetenceLevel(userId, dataMap, subCompetence, subLevel, subValue, ctx);

				if (!attained) {
					// If a leaf does not meet the requirements, set the value to the ZeroLevel and quit here.
					if (!dataMap.containsKey(competence)) {
						dataMap.put(competence, zeroLevel);
					}
					break;
				}
			}

			if (attained) {
				// Check if this value is already stored or higher.
				if (!dataMap.containsKey(competence) || (reqLevel.getValue() > dataMap.get(competence).getValue())) {
					dataMap.put(competence, reqLevel);
				}
			}
		}

		return attained;
	}
}