/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.utils;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * The Class ClassUtils.
 * 
 * Class with all kind of Class related functions.
 * @version $Revision: 1.9 $, $Date: 2009/11/24 14:18:19 $
 */
public final class ClassUtils {
	private static final Log LOG = LogFactoryUtil.getLog(ClassUtils.class);
	private static final String GET = "get";
	private static final String IS = "is";
	private static final String SET = "set";

	private ClassUtils() {
		// Empty constructor
	}

	/**
	 * Gets the all properties.
	 * 
	 * @param object
	 *            the object
	 * 
	 * @return the all properties
	 */
	public static List<String> getAllProperties(final Object object) {
		final List<String> properties = new ArrayList<String>();
		try {
			for (final Method method : object.getClass().getMethods()) {
				final String methodName = method.getName();
				if (methodName.startsWith(GET)) {
					if (!methodName.equals("getClass")) {
						final String name = methodName.substring(GET.length(), methodName.length());
						properties.add(name.substring(0, 1).toLowerCase() + name.substring(1));
					}
				}
			}
		} catch (Exception e) {
			if (LOG.isDebugEnabled()) {
				LOG.debug(e);
				System.out.println(e);
			}
		}

		return properties;
	}

	/**
	 * Set a property from a class to a value.
	 * 
	 * @param object
	 *            object
	 * @param property
	 *            property
	 * @param value
	 *            value
	 */
	public static void setProperty(final Object object, final String property, final Object value) {
		callSetMethod(object, SET + StringUtils.capitalize(property), new Object[] { value });
	}

	/**
	 * Get the value from a property from a class.
	 * 
	 * @param <T>
	 *            Generic Type
	 * @param object
	 *            object
	 * @param property
	 *            property
	 * @param defaultvalue
	 *            defaultvalue
	 * @return value
	 */
	public static <T> T getProperty(final Object object, final String property, final T defaultvalue) {
		return callMethod(object, GET + StringUtils.capitalize(property), defaultvalue);
	}
	
	/**
	 * @param object
	 *            object
	 * @param property
	 *            property
	 * @return value
	 */
	public static Object getProperty(final Object object, final String property) {
		return callMethod(object, GET + StringUtils.capitalize(property));
	}

	/**
	 * @param <T>
	 *            Generic Type
	 * @param object
	 *            object
	 * @param property
	 *            property
	 * @param defaultvalue
	 *            defaultvalue
	 * @return value
	 */
	public static <T> T isProperty(final Object object, final String property, final T defaultvalue) {
		return callMethod(object, IS + StringUtils.capitalize(property), defaultvalue);
	}

	@SuppressWarnings("unchecked")
	private static <T> T callMethod(final Object object, final String methodName, final T defaultValue) {
		try {
			final Method method = object.getClass().getMethod(methodName, new Class[] {});
			if (method != null) {
				return (T) method.invoke(object, new Object[] {});
			}
		} catch (Exception e) {
			if (LOG.isDebugEnabled()) {
				LOG.debug(e);
				System.out.println(e);
			}
		}

		return defaultValue;
	}
	
	private static Object callMethod(final Object object, final String methodName) {
		try {
			final Method method = object.getClass().getMethod(methodName, new Class[] {});
			if (method != null) {
				return method.invoke(object, new Object[] {});
			}
		} catch (Exception e) {
			if (LOG.isDebugEnabled()) {
				LOG.debug(e);
				System.out.println(e);
			}
		}

		return null;
	}

	private static void callSetMethod(final Object object, final String methodName, final Object[] params) {
		try {
			final Method method = object.getClass().getMethod(methodName, new Class[] {});
			if (method != null) {
				method.invoke(object, params);
			}
		} catch (Exception e) {
			if (LOG.isDebugEnabled()) {
				LOG.debug(e);
				System.out.println(e);
			}
		}
	}
}
