/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.utils;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.liferay.portal.kernel.util.StringPool;

/**
 * The Class DateUtils.
 * 
 * Class with some date / time related functions.
 * 
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.1 $, $Date: 2009/11/23 11:28:34 $
 */
public final class DateUtils {

	private static final DateFormat SIMPLE = new SimpleDateFormat("MM-dd-yyyy");
	private static final int HOURS = 11;
	private static final int MINUTES = 59;
	private static final int SECONDS = 59;

	private DateUtils() {
		// Empty constructor
	}

	/**
	 * Date to string.
	 * 
	 * @param date
	 *            the date
	 * 
	 * @return the string
	 */
	public static String dateToString(final Date date) {
		return date != null ? SIMPLE.format(date) : StringPool.BLANK;
	}

	/**
	 * Returns the current date with time set to 23:59:59 .
	 * 
	 * @return the current date
	 */
	public static Date getMaximizedCurrentDate() {
		final GregorianCalendar gregDate = new GregorianCalendar();
		gregDate.set(Calendar.HOUR, HOURS);
		gregDate.set(Calendar.MINUTE, MINUTES);
		gregDate.set(Calendar.SECOND, SECONDS);

		return gregDate.getTime();
	}
}