/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.utils;

import java.lang.reflect.Method;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.EvidenceFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.business.facade.LearningPathFacade;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LearningPath;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;

/**
 * Some helper methods for BaseModel<?> Entities.
 * @version $Revision: 1.13 $, $Date: 2009/11/24 14:18:19 $
 */
public final class EntityUtils {
	private EntityUtils() {
		// Private constructor
	}

	/**
	 * Get the primaryKey from a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @return PrimaryKey
	 * @throws SystemException
	 *             SystemException
	 */
	public static long getPrimaryKeyFromEntity(final BaseModel<?> entity) throws SystemException {
		return ClassUtils.getProperty(entity, "primaryKey", 0L);
	}

	/**
	 * Get the CompanyId from a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @return CompanyId
	 * @throws SystemException
	 *             SystemException
	 */
	public static long getCompanyIdFromEntity(final BaseModel<?> entity) throws SystemException {
		return ClassUtils.getProperty(entity, "companyId", 0L);
	}

	/**
	 * Get the GroupId from a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @return GroupId
	 * @throws SystemException
	 *             SystemException
	 */
	public static long getGroupIdFromEntity(final BaseModel<?> entity) throws SystemException {
		return ClassUtils.getProperty(entity, "groupId", 0L);
	}

	/**
	 * Get the UserId from a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @return UserId
	 * @throws SystemException
	 *             SystemException
	 */
	public static long getUserIdFromEntity(final BaseModel<?> entity) throws SystemException {
		return ClassUtils.getProperty(entity, "userId", 0L);
	}

	/**
	 * Set the CompanyId to a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @param companyId
	 *            companyId
	 * @throws SystemException
	 *             SystemException
	 */
	public static void setCompanyIdOnEntity(final BaseModel<?> entity, final long companyId) throws SystemException {
		setIdOnEntity(entity, "setCompanyId", companyId);
	}

	/**
	 * Set the GroupId to a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @param groupId
	 *            groupId
	 * @throws SystemException
	 *             SystemException
	 */
	public static void setGroupIdOnEntity(final BaseModel<?> entity, final long groupId) throws SystemException {
		setIdOnEntity(entity, "setGroupId", groupId);
	}

	/**
	 * Set the UserId to a BaseModel<?> Entity.
	 * 
	 * @param entity
	 *            Entity
	 * @param userId
	 *            userId
	 * @throws SystemException
	 *             SystemException
	 */
	public static void setUserOnEntity(final BaseModel<?> entity, final long userId) throws SystemException {
		setIdOnEntity(entity, "setUserId", userId);
	}

	private static void setIdOnEntity(final BaseModel<?> entity, final String methodName, final long id)
			throws SystemException {
		try {
			Method method = entity.getClass().getMethod(methodName, new Class[] { long.class });
			if (method != null) {
				method.invoke(entity, new Object[] { id });
			}
		} catch (Exception e) {
			throw new SystemException(e);
		}
	}

	/**
	 * Get interface from BaseModel<T>.
	 * 
	 * @param <T>
	 *            Generic Type
	 * @param entity
	 *            BaseModel<T> Entity
	 * @return Interface
	 */
	public static <T> String getInterfaceFromEntity(final BaseModel<T> entity) {
		if (entity != null && entity.getClass().getInterfaces().length > 0) {
			return entity.getClass().getInterfaces()[0].getName();
		}

		return null;
	}

	/**
	 * Gets the BaseModel<?> entity from a className + Id.
	 * 
	 * @param className
	 *            The className
	 * @param entityId
	 *            The entityId
	 * @param ctx
	 *            the ctx
	 * @param logInJournal
	 *            Log this action in the journal
	 * @return BaseModel<?>
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static BaseModel<?> getEntityFromClassAndId(final String className, final long entityId,
			final ServiceContext ctx, final boolean logInJournal) throws SystemException, PortalException {

		BaseModel<?> entity = null;

		// TODO add more entities
		if (Goal.class.getName().equals(className)) {
			entity = GoalFacade.getGoal(entityId, ctx);
		} else if (CompetenceProfileLevel.class.getName().equals(className)) {
			entity = CompetenceProfileLevelFacade.getCompetenceProfileLevel(entityId, ctx);
		} else if (CompetenceLevel.class.getName().equals(className)) {
			entity = CompetenceLevelFacade.getCompetenceLevel(entityId, ctx);
		} else if (Competence.class.getName().equals(className)) {
			entity = CompetenceFacade.getCompetence(entityId, ctx);
		} else if (LearningPath.class.getName().equals(className)) {
			entity = LearningPathFacade.getLearningPath(entityId, ctx);
		} else if (Evidence.class.getName().equals(className)) {
			entity = EvidenceFacade.getEvidence(entityId, ctx);
		}

		return entity;
	}

	/**
	 * Encode an entity into a string.
	 * 
	 * @param <T>
	 *            BaseModel
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the string
	 */
	public static <T> String encode(final BaseModel<T> entity) {
		if (entity == null) {
			return null;
		}

		return getInterfaceFromEntity(entity) + StringPool.COLON + entity.getPrimaryKeyObj().toString();
	}

	/**
	 * Get the Long id from 'classname:id'.
	 * 
	 * @param data
	 *            the data
	 * 
	 * @return the long
	 */
	public static Long decode(final String data) {
		Long id = 0L;

		if (Validator.isNotNull(data)) {
			final String[] values = data.split(StringPool.COLON);
			if (values.length == 2) {
				try {
					id = Long.parseLong(values[1]);
				} catch (NumberFormatException nfe) {
					id = 0L;
				}
			}
		}

		return id;
	}
}