/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.Map.Entry;

import com.liferay.portal.kernel.util.StringPool;

/**
 * String util class.
 * @author sheyenrath
 * @version $Revision: 1.6 $, $Date: 2009/11/24 14:41:01 $
 */
public final class StringUtils {

	private static final String ID_PREFIX = "ID";

	private StringUtils() {
		// Empty constructor
	}

	/**
	 * Capitalize a string.
	 * 
	 * @param string
	 *            Input string
	 * @return Capitalized string
	 */
	public static String capitalize(final String string) {
		if (string.length() == 0) {
			return string;
		}

		return string.substring(0, 1).toUpperCase() + string.substring(1);
	}

	/**
	 * GetQueryMap from URL.
	 * 
	 * @param url
	 *            URL
	 * 
	 * @return Map<String, String>
	 */
	public static Map<String, String> getQueryMap(final String url) {

		final Map<String, String> map = new HashMap<String, String>();

		if (url == null) {
			return map;
		}

		final String[] params = url.split(StringPool.AMPERSAND);
		for (final String param : params) {
			final String name = param.split(StringPool.EQUAL)[0];
			final String value = param.split(StringPool.EQUAL)[1];
			map.put(name, value);
		}

		return map;
	}

	/**
	 * Create a query string from a map.
	 * 
	 * @param map
	 *            Map
	 * @return String
	 */
	public static String createQuery(final Map<String, String> map) {
		final StringBuilder sb = new StringBuilder();

		for (Entry<String, String> entry : map.entrySet()) {
			sb.append(entry.getKey());
			sb.append(StringPool.EQUAL);
			sb.append(entry.getValue());
			sb.append(StringPool.AMPERSAND);
		}

		sb.deleteCharAt(sb.length() - 1);
		return sb.toString();
	}

	/**
	 * Generate a random "NCName" - compliant Identifier.
	 * 
	 * @return the string
	 */
	public static String randomId() {
		return ID_PREFIX + UUID.randomUUID();
	}
}
