/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.utils;

import java.util.Locale;

import javax.faces.context.FacesContext;

import org.apache.commons.logging.LogFactory;

import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * The Class UserLocaleResolver.
 * 
 * This class resolves the locale of a user.
 * @version $Revision: 1.2 $, $Date: 2009/11/24 14:18:19 $
 */
public class UserLocaleResolver {
	// private static final Log LOG = LogFactoryUtil.getLog(UserLocaleResolver.class);

	/**
	 * Method tries to resolve the Liferay locale. If not found the systemlocale will be returned
	 * 
	 * @return the user locale
	 */
	public static Locale getUserLocale() {
		final FacesContext facesContext = FacesContext.getCurrentInstance();
		Locale userLocale = facesContext.getExternalContext().getRequestLocale();

		try {
			if (facesContext.getExternalContext().getUserPrincipal() != null) {
				userLocale = UserLocalServiceUtil.getUserById(
						Long.parseLong(facesContext.getExternalContext().getUserPrincipal().getName())).getLocale();
			}
		} catch (Exception e) {
			// manage anomaly
			LogFactory.getLog(UserLocaleResolver.class).warn("Could not resolve liferay local", e);
			userLocale = new Locale(System.getProperty("user.language"));
		}
		return userLocale;
	}

	/**
	 * Getter to use in a managed bean.
	 * 
	 * @return the locale
	 */
	public Locale getLocale() {
		return getUserLocale();
	}

}
