/*
 * Copyright (c) 2008, Consortium Board TENCompetence
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the TENCompetence nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CONSORTIUM BOARD TENCOMPETENCE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CONSORTIUM BOARD TENCOMPETENCE BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package apis;

import java.io.FileInputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.util.Properties;
//import org.apache.log4j.Logger;

/**
 * DAO class to open and close Database Connection
 *
 * @author UPF / Pau Melines
 * 
 * @version $Id: DAO.java,v 1.2 2009/11/25 15:38:43 javier_melero Exp $
 */

public class DAO {
    
//	private Logger apisDbLog = Logger.getLogger("apisDbLog");
	protected Properties fDefaultProps = null;
    private Connection connection = null;
    protected Statement stmt;
        
    public DAO(){

    }    
    public void setConnectionFromDAO(DAO connected_dao){
    	this.stmt = connected_dao.stmt;
    	this.fDefaultProps = connected_dao.fDefaultProps;
    	this.connection = connected_dao.connection;
    }
    public void setConnectionPropertiesFromFile(String file_location){ 	 //now refactored till QtiParser, before was "= new FileInputStream("apis-adapter.properties");"
    	try {
    		if(fDefaultProps == null){
				fDefaultProps = new Properties();
				FileInputStream in = new FileInputStream(file_location);
				fDefaultProps.load(in);
				in.close();			
    		}
		}
		catch (Exception ex){
		//	apisDbLog.error("Unable to load apis properties file. ", ex);
		}
    }
    
    public void setConnectionProperties(String driverClass, String databaseUrl,String databaseUsername,String databasePassword){
    	try {
    		if(fDefaultProps == null){
				fDefaultProps = new Properties();
			
				fDefaultProps.setProperty("driver.class",driverClass);
				fDefaultProps.setProperty("database.url",databaseUrl);
	    		fDefaultProps.setProperty("database.username",databaseUsername);
	    		fDefaultProps.setProperty("database.password",databasePassword);	
    		}
		}
		catch (Exception ex){
		//	apisDbLog.error("Unable to load apis properties file. ", ex);
		}
    }
    
    public Connection openConnection() throws Exception{
    	
    	if(fDefaultProps == null){
    		Exception e = new Exception();
    		System.out.println("Properties of the DB connection are not set");
    		throw(e);
    	}
    	try{        	
    		String driverClass = fDefaultProps.getProperty("driver.class");
    		String databaseUrl = fDefaultProps.getProperty("database.url");
    		String databaseUsername = fDefaultProps.getProperty("database.username");
    		String databasePassword = fDefaultProps.getProperty("database.password");
    		
    		if(driverClass != null && databaseUrl != null && databaseUsername != null && databasePassword != null){       			
    			Class.forName(driverClass).newInstance();
    			String url = databaseUrl;
    			connection = DriverManager.getConnection(url, databaseUsername, databasePassword);
    			connection.setAutoCommit(true);
    			stmt = connection.createStatement();
    			return connection;
    		}
    		else{   
    			System.out.println("Cannot obtain DB settings");
    			return null;
    		}
    	}
    	catch(Exception ex){
    		System.out.println("Error opening connection to apis DB");
    		return null;
    	}
    }
    
    public boolean isConnected(){
    	return connection != null;
    }
    
    public boolean closeConnection(){
        try{
        	// TODO for testing
//            stmt.close(); // close connection statement
//            connection.close(); // close connection
            return true;
        }
        catch(Exception ex){
        	//apisDbLog.error("could not close connection to apis DB:", ex);
            return false;
        }
    }
    
}