/*
 * Copyright (c) 2008, Consortium Board TENCompetence
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the TENCompetence nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CONSORTIUM BOARD TENCOMPETENCE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CONSORTIUM BOARD TENCOMPETENCE BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package apis;

/**
 * Class :  QTIv21interpolationTable(variableDeclaration)
 * 
 * Declaration : Outcome variables are declared by outcome declarations. Their value is set either from a 
 * 	default given in the declaration itself or by a responseRule during responseProcessing. Interpolation 
 *  table it'll be used in testfeedback 
 * 
 * */

import nb_utils.NBEasyXML;

public class QTIv21interpolationTable extends QTIv2variableDeclaration
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//-- Attributes specification
	protected String targetValue;			// targetValue, String
	protected float sourceValue;		// sourceValue, float 
	
	// -- contructor
	public QTIv21interpolationTable() throws InvalidQTIv2Exception
	{
		targetValue = "";
		sourceValue = -1;
	}  
	 
	public String getTargetValue()
	{
		return this.targetValue;
	}
	
	public float getSourceValue()
	{
		return this.sourceValue;
	}
	 
	// -- parse
	public int parse(int position, NBEasyXML itemXML) throws InvalidQTIv2Exception
	{
		sourceValue = Float.parseFloat(itemXML.getAttribute(position, "sourceValue"));
		targetValue = itemXML.getAttribute(position, "targetValue");
        return itemXML.findNextPeer(position);
	}
}