/*
 * Copyright (c) 2008, Consortium Board TENCompetence
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the TENCompetence nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CONSORTIUM BOARD TENCOMPETENCE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CONSORTIUM BOARD TENCOMPETENCE BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package apis;

import nb_utils.NBEasyXML;

/*
 * <p>Title: APIS test wrapper</p>
 * <p>Description: Class to process lookup outcome variable</p>
 * <p>Copyright: Copyright (c) 2008, Universitat Pompeu Fabra</p>
 * <p>Company: Universitat Pompeu Fabra</p>
 * @author Pau Melines Lopez
 * @version 1.0
 */

public class QTIv21lookupOutcomeValue implements QTIv2variableAction
{
    QTIv2item theItem;
    QTIv21test theTest;
    QTIv2expression expr;
    String identifier;

    public QTIv21lookupOutcomeValue(QTIv2item owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        
    	theItem = owner;
        identifier = itemXML.getAttribute(idx,"identifier");
        int exidx =  itemXML.findChildElement(idx);
        if(exidx==-1)
            throw new InvalidQTIv2Exception("setOutcomeValue must contain an expression.");
        expr = QTIv2expression.parseNew(theItem, itemXML, exidx);
        if(expr==null)
              throw new InvalidQTIv2Exception("setOutcomeValue must contain an expression.");

    }
    
    public QTIv21lookupOutcomeValue(QTIv21test owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        theTest = owner;
        identifier = itemXML.getAttribute(idx,"identifier");
        int exidx =  itemXML.findChildElement(idx);
        if(exidx==-1)
            throw new InvalidQTIv2Exception("lookupOutcomeValue must contain an expression.");
        expr = QTIv2expression.parseNew(theTest, itemXML, exidx);
        if(expr==null)
              throw new InvalidQTIv2Exception("lookupOutcomeValue must contain an expression.");

    }

    public boolean run() throws InvalidQTIv2Exception
    {
    	if(theItem==null)
    		theTest.setOutcomeVariable(identifier, expr.resolve());
    	else if (theTest == null)
    		theItem.setOutcomeVariable(identifier, expr.resolve());
        return true;
    }

}

