package apis;

import java.util.Vector;

import nb_utils.NBEasyXML;

/**
 * Class: sectionPart (abstract)
 * 
 * Description : Sections group together individual items and/or sub-sections. A number of common parameters are shared by both
 *			types of child element.
 * 
 * Attributes :	* required 
 * 				If a child element is required it must appear (at least once) in the selection.
 * 				It is in error if a section contains a selection rule that selects fewer child 
 * 				elements than the number of required elements it contains.
 *				* fixed 
 *				If a child element is fixed it must never be shuffled. When used in combination 
 *				with a selection rule fixed elements do not have their position fixed until after 
 *				selection has taken place.
 * Contains :  * preCondition [*] 
 * 				An optional set of conditions evaluated during the test, that determine if the 
 * 				item or section is to be skipped (in nonlinear mode, pre-conditions are ignored).
 * 			   * branchRule [*] 
 * 				An optional set of rules, evaluated during the test, for setting an alternative 
 * 	  			target as the next item or section (in nonlinear mode, branch rules are ignored).
 * 			   * itemSessionControl [0..1]
 * 				Parameters used to control the allowable states of each item session (may be 
 * 				overridden at sub-section or item level).
 * 			   * timeLimits [0..1] 
 * 				Optionally controls the amount of time a candidate is allowed for this item or
 * 				section.
 */

public abstract class QTIv21sectionPart
{
    QTIv21test theTest;
    Vector parts;
    String rawXML; // not really needed, just there for development purposes
    NBEasyXML itemXML;    
    
    //Attributes
    protected boolean required;			// required [0..1]: boolean = false 		
    protected boolean fixed;			// fixed [0..1]: boolean = false 
    //Contains
    public Vector preCondition;
    public Vector branchRule;
    protected boolean endSection;		// assessment Section finalized
    protected String finishingReason ;	// the reason for this assessmentSection ending.
    
    public int page_num;
    public int page_set_size;
    int init_part;
    int fin_part;
    
    
    // -- constructor 
    public QTIv21sectionPart(QTIv21test owner)
    {
    	theTest = owner ;
    	required = false;
    	fixed = false;
    	endSection = false;
    	
        page_num =0;
        page_set_size= 10;
    }
    
    public QTIv21sectionPart(String nItemXML)
    {
        rawXML = nItemXML;
        itemXML = new NBEasyXML(nItemXML);
    }    
    
    
    public abstract void setProcessedItems(UserTestCase processedItems);
    public abstract void setItemsToProcess(String postStr);
    public abstract void addFormDataValue(String name, String value);
    public abstract boolean findItem(String itemIdentifier, Vector item);
    public abstract String render() throws InvalidQTIv2Exception;
    public abstract int  answeredItems();
    public abstract int getNumberPresented();
    public abstract int getNumberCorrect();
    public abstract int getNumberIncorrect();
    public abstract String getResultInfo();
    public void findRelatedVar(String varIdentifier,String includeCategory, String excludeCategory, Vector relatedVarDec){}
    public void findRelatedVar(String varIdentifier,String sectionIdentifier,String includeCategory, String excludeCategory, Vector relatedVarDec){}
  
    public void setPageNum(int a){
    	page_num = a;
    }
    
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {     
    //	QTIv2expression expr;
    	
    	
    	//ATTRIBUTES
        if (itemXML.getAttribute(idx,"required")!=null)
        {
        	if(itemXML.getAttribute(idx,"required").equals("true"))
        		required = true;
        }
    	
        if (itemXML.getAttribute(idx,"fixed")!=null)
        {
        	if(itemXML.getAttribute(idx,"fixed").equals("true"))
        		fixed = true;
        }
        
   /*     int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        { 
            int id = QTIv2define.identify(itemXML.getName(cidx));    
            switch(id)
            {
             case QTIv2define.QTIv21_preCondition:   
            	  System.out.println("<preCondition>");
                  cidx =  itemXML.findChildElement(cidx);
                  if(cidx==-1)
                      throw new InvalidQTIv2Exception("problems must contain an expression.");
                  expr = QTIv2expression.parseNew(theTest, itemXML, cidx);
                  if(expr==null)
                        throw new InvalidQTIv2Exception("probs must contain an expression.");
            	 break;                 
           /*  case QTIv2define.QTIv21_branchRule:
            	 QTIv21variableMapping aVarMap= new QTIv21variableMapping();
            	 cidx = aVarMap.parse(itemXML, cidx);
            	 anItem.itemVarMapping.add(aVarMap);
            	 break;
              default:   
            	  cidx = itemXML.findChildElement(cidx);
            }	
        }               */  
    	return idx;
    	//Al final crido per cadascun d'ells al parse de la subclasse.
    }

}