package apis;
import nb_utils.NBEasyXML;


/**
 * 	Class : selection
 *		
 *	Description :  specifies the rules used to select the child elements of a section for
 *			each test session. If no selection rules are given  we assume that all elements
 *			are to be selected.
 *
 *  Attributes : 	* select  
 *  				The number of child elements to be selected. Sub-sections always count as 
 *  				1, regardless of how many child elements they have and whether or not they
 *  				are visible. The number of children to select may exceed the number of 
 *  				child elements defined only if withReplacement is true.
 *					* withReplacement	
 *					When selecting child elements each element is normally eligible for selection 
 *					once only. In other words, when selecting 3 elements from {A,B,C,D} the possible
 * 					outcomes are {A,B,C}, {A,B,D}, {A,C,D}, and {B,C,D}. By setting withReplacement 
 * 					to true each element becomes eligible for selection multiple times. Selecting 3
 * 					nodes from {A,B,C,D} can then result in combinations such as {A,A,A}, {A,A,B} 
 * 					and so on.
 */

public class QTIv21selection 
{   
    protected QTIv2structuredBodyPart prompt;
    protected QTIv21test theTest;
    
    //--  Attributes
    protected int select; 					// 	select [1] : integer 
    protected boolean withReplacement;		//  withReplacement[0..1] : boolean = false
     	
    
    public QTIv21selection(QTIv21test owner)
    {
    	theTest = owner;
    	select = -1;
        withReplacement = false;
    }
    //-- parse:  select tag parsing 
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {     
    	//-- ATTRIBUTES
        if(itemXML.getAttribute(idx,"select")!= null)
        {
        	select= Integer.parseInt(itemXML.getAttribute(idx,"select"));
        }else 
        	throw new InvalidQTIv2Exception("select attribute needed");

        //NOT APPLIED
        if(itemXML.getAttribute(idx,"withReplacement")!= null && itemXML.getAttribute(idx,"withReplacement").equals("true"))
        {
        	withReplacement = true;
        }
        return itemXML.findNextPeer(idx);
    }

    //-- render : renders selection contents. 
    public String render()
    {
        String out = "";//[LAU]<h4>NOTE : In this assessment part you can only answer "+select+" questions </h4>";
        return out;
    }

}