package apis;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.List;
import java.util.ArrayList;

import javax.portlet.PortletException;

import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.generated.service.AssessmentTestLocalServiceUtil;

//import org.tencompetence.generated.service.AssessmentTestLocalServiceUtil;

import com.liferay.portal.UserActiveException;
import com.liferay.portal.kernel.dao.orm.DynamicQuery;
import com.liferay.portal.kernel.dao.orm.DynamicQueryFactoryUtil;
import com.liferay.portal.kernel.dao.orm.PropertyFactoryUtil;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.model.Company;
import com.liferay.portal.model.Group;
import com.liferay.portal.model.Role;
import com.liferay.portal.model.User;
import com.liferay.portal.model.UserGroupRole;
import com.liferay.portal.security.auth.PrincipalThreadLocal;
import com.liferay.portal.service.CompanyLocalServiceUtil;
import com.liferay.portal.service.GroupLocalServiceUtil;
import com.liferay.portal.service.RoleLocalServiceUtil;
import com.liferay.portal.service.UserGroupRoleLocalServiceUtil;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.qti.slayer.model.Assignment;
import com.qti.slayer.model.TestResult;
import com.qti.slayer.model.Uol;
import com.qti.slayer.model.UolModel;
import com.qti.slayer.service.AssignmentLocalServiceUtil;
import com.qti.slayer.service.TestResultLocalServiceUtil;
import com.qti.slayer.service.TestResultServiceUtil;
import com.qti.slayer.service.UolLocalServiceUtil;
import com.qti.slayer.service.persistence.TestResultPK;
import com.qti.slayer.service.persistence.TestResultUtil;

//import apistesthost.*;

/**
 * Class: assessmentTest 
 * 
 * Description: A test is a group of assessmentItems with an associated set of rules that
 * 				determine which of the items the candidate sees, in what order, and in what 
 * 				way the candidate interacts with them. The rules describe the valid paths 
 * 				through the test, when responses are submitted for response processing  and
 * 				when (if at all) feedback is to be given.
 * 
 * Attributes:	- identifier : The principile identifier of the test. This identifier must 
 * 								have a corresponfing entry in the test's meta-data.
 * 				- title :	The title of the assesmentTest is intented to enable the test to 
 * 							be selected outside of any test session. 
 * 
 * Contains:	- outcomeDeclaration [*]
 * 				 	Each test has an associated set of outcomes. The 
 * 				 	value of these outcomes are set by test's 
 * 				 	outcomeProcessing rules.
 * 				- timeLimits[0..1] : 
 * 			 	 	Optionally controls  the amount of time a candidate is allowed 
 * 			     	for the entire test.
 * 				- testPart [1..*] 
 * 				 	Each test is divided  into one or more parts which may in turn be
 * 			  	 	divided into sections, sub-sections and so on. A testPart represents
 * 				 	a major division of the test and is used to control the basic mode
 * 				 	parameters that apply to all sections  and sub-sections within that 
 * 				 	part.
 * 				- outcomeProcesing [0..1]
 * 					The set of rules used for calculating the values  of the 
 * 					test outcomes.
 * 				- scoreReport [*]
 * 					Contains the test-level feedback controlled by the test outcomes.    
 */

public class QTIv21test extends QTIv2base
{
	public String embedPrefix; // to ensure unique names of item form vars. (Use of identifier could lead to too long names)  
    protected String parseErrors;
    protected QTIv21testPart testPart;
    protected Vector testParts;
    protected Vector feedbacks;
    protected Vector itemsOutDec;
    protected String mediaBase = "", itemBase=" ",appletBase = "",testBase="";
	protected Hashtable formData;
    protected String file;
    protected int numberResponded;
    protected int numberCorrect;
    protected int numberIncorrect;
    protected int numberSelected;
    protected int numberPresented;
    protected int testfeedback_id;
    protected String weightIdentifier;
    protected QTIv21outcomeProcessing outproc;
    protected boolean continueTest;
    protected String finishingReason;
    private QTIv2variableDeclaration tmp; 
    private QTIv21testFeedback feedback;
    private QTIv21interpolationTable intertable;
    private QTIv21interpolationTable tmptable;
    private QTIv2outcomeDeclaration outVars;
    private Vector intertables;
    private String outcomeId;
    public String testTitle = "";
    private float outcomeValue;
    private String feedmessageOutcome = "";
    private String testMark;
    private float finalScore = 0;
    private float percentCorrect = 0;
    private float numPRESENTED = 0;
    private float numRESPONDED = 0;
    private float summary = 0;
    private float correct = 0;
    private float incorrect = 0;
    private Hashtable xml_container;
    public boolean showOutcome = false;
    UserTestCase theUser;    
    
    //Attributes
    protected String identifier;		//identifier[1] : string
    protected String title;				//titel[1] : string 
    	
    public QTIv21test(String nTestXML, String nEmbedPrefix, String aMediaBase, String anItemBase, String aTestBase,String anAppletBase,String file, String identifier, String title, int run, String user_name,String uol,String language, Hashtable xml_container_)
    {
        super(nTestXML);
       
        embedPrefix = nEmbedPrefix;
        testBase=aTestBase;
        itemBase = anItemBase;
        mediaBase = aMediaBase;
        appletBase = anAppletBase;
        this.file = file;
        this.identifier = identifier;
        this.testTitle = title;
        numberResponded = 0;
        numberCorrect = 0;
        numberIncorrect = 0;
        numberPresented = 0;
        numberSelected = 0;
    	outcomeId = "";
    	outcomeValue = 0;
    	testMark = "";
        continueTest= true;
        finishingReason = "undefined";
        xml_container = xml_container_;
        //# at this point run through the XML looking at the immediate children of
        //# assessmentTest, and parsing or indexing these blocks as they are found
        
        try{
        	theUser = new UserTestCase(identifier,run,user_name,uol,language);
        }catch(Exception e){
        	System.out.println("User cannot be instantiated");
        }
    }
    
    public void linkTheTestWithDb(String properties_file_location)throws Exception{
    	// refactoring needed since user != test , for the moment userTestCase class contains all the persistance information for the test ... there shoud be a testCaseDAO
    	theUser.setDB(properties_file_location);
    	theUser.connectWithDB();
    	theUser.getStoredItems();
    }

    public void linkTheTestWithDb(String driverClass,String  databaseUrl,String  databaseUsername,String  databasePassword)throws Exception{
    	theUser.setDB(driverClass, databaseUrl, databaseUsername, databasePassword);
    	theUser.connectWithDB();
    	theUser.getStoredItems();
    }
    
    public void startParse(){    
        try
        {
            parseTest();
        }
        catch (InvalidQTIv2Exception ex)
        {
            System.err.println(ex);
            ex.printStackTrace();
            if(parseErrors == null)
                parseErrors = "";
            parseErrors += ex + "<br/>";
        }
        catch (Exception ex)
        {
            System.err.println(ex);
            ex.printStackTrace();
        }
    }
    
    protected void parseTest() throws InvalidQTIv2Exception, Exception
    {
    	if(!theUser.isConnected())
    		throw (new Exception());
    		
    		
        boolean inTest = false;
        int testDepth=0;
        int position = 0;
        
       DAOAssessmentTest daotest = new DAOAssessmentTest();
       try{
    	   daotest.setTest(this.identifier,this.getUser());
       }catch(SQLException e){
       		e.printStackTrace();
       }
       System.out.println("Test XML: " + testXML.toString());
        while ((position < testXML.size())&&(position != -1))       	
        {
            int op = position;
            //# need to run through them and call the indexing functions.
            //# Unfortunatly java virtual methods don't get called by what the instance is, but by
            //# what the reference is, so can't be used as a callback mechanisim.
            if(testXML.getType(position) == 1)
            {    	
                if((inTest)&&(testXML.getDepth(position)==testDepth+1))
                {
                    int id = QTIv2define.identify(testXML.getName(position));
                    System.out.println("ID: " + id);
                    System.out.println("TestXML piece: "+testXML.getAsString(position));
                    switch (id)
                    {
                        case QTIv2define.QTIv2_outcomeDeclaration:
                        	QTIv2outcomeDeclaration outDec = new QTIv2outcomeDeclaration();
                        	position = outDec.parse(position, id, testXML );
                        	if (outcomeVars == null){
              	               outcomeVars = new Vector();
                        	}
              	            outcomeVars.add(outDec);  
              	            break;                     
                        case QTIv2define.QTIv21_testPart:
                        	try
                        	{
                                testPart = new QTIv21testPart(this);
                                System.out.println("Peta Aqu... 1");
                                /*
                                System.out.println("Test Parts...");
                                Iterator it = testPart.parts.iterator();
                                while(it.hasNext())
                                {
                                	Vector v = (Vector) it.next();
                                	System.out.println("Part Content: "+v.toString());
                                }
                                */
                                
                                position = testPart.parse(testXML, position);
                                System.out.println("Peta Aqu... 2");
                                if(testParts==null)
                                	testParts= new Vector();
                                testParts.add(testPart);
                        	}
                        	catch(Exception ex)
                        	{
                        		ex.printStackTrace();
                        	}

                            break;
                        case QTIv2define.QTIv21_outcomeProcessing:
                        	int tempPos = testXML.findChildElement(op);
                        	outproc = new QTIv21outcomeProcessing(this);
                            position = outproc.parse(testXML, position); 
                        	break;
                        case QTIv2define.QTIv21_timeLimits:
                        	QTIv21timeLimits tl = new QTIv21timeLimits(this);
                        	position=tl.parse(testXML,position );
                            break;
                        case QTIv2define.QTIv21_scoreReport:
                        	break;
                        case QTIv2define.QTIv21_testFeedback:
                        	testfeedback_id = position;
                       	 	feedback = new QTIv21testFeedback(this);        	  
                       	 	position = feedback.parse(testXML,position);
                       	 	if(feedbacks == null)
                       	 		feedbacks = new Vector();
                            feedbacks.add(feedback);            	 
                       	 	break;
                   	 	default:
                   	 		throw new InvalidQTIv2Exception("Parsing Test, identifier:["+testXML.getName(position)+"]");
                    	}
                }
                else if (testXML.getName(position).equals("assessmentTest")) // an extra check of id could go here to allow xml with multiple tests.
                {
                    inTest = true;
                    testDepth = testXML.getDepth(position);
                }
            }
            else if((inTest)&&(testXML.getType(position) == 1)&&(testXML.getName(position).equals("assessmentTest")))
            {
                inTest = false;
                position = testXML.size();
            }
            if (position == op) position++;
        }
        return;
    }
    
    public UserTestCase getUser(){
    	return theUser;
    }
    
    public void setProcessedItems(UserTestCase processedItems)
    {
    //	numberResponded=processedItems.size();
        if(testParts==null)
        {
        	System.out.println("Test Parts est vaco");
        	testParts= new Vector();
            //testParts.add(testPart);
        }
        else
        	System.out.println("Test Parts:" + testParts.size());
        
    	for(int i=0;i<testParts.size();i++)
    		((QTIv21testPart)testParts.get(i)).setProcessedItems(processedItems);
    }
  
    public void setItemsToProcess(String postStr)
    {
    	for(int i=0;i<testParts.size();i++){
    		((QTIv21testPart)testParts.get(i)).setItemsToProcess(postStr);
    	}
    }
  
    public void readFormData(String postStr)
    {
    	String[] dt = postStr.split("&");
        for(int n=0; n<dt.length; n++)
        {
            int eqp = dt[n].indexOf("=");
            if(eqp>2)
            {
            	
            	String name = dt[n].substring(0,eqp);
                String value = dt[n].substring(eqp+1);

                if(name.contains("PAGE")){
                	try
                	{

                		value = URLDecoder.decode(value, "UTF-8");
                		theUser.setPage(Integer.parseInt(value));
                		((QTIv21sectionPart)((QTIv21testPart)testParts.get(1)).parts.get(1)).setPageNum( 1);
                		
                	}catch (Exception ex)
                	{
                		System.out.println("no page");
                	}
                }
                
                int i = name.indexOf("_"); // si es apisa_RESPONSE
                if(i != -1)
                {
                	try
                	{
                		value = URLDecoder.decode(value, "UTF-8");
                		addFormDataValue(name, value);
                	}
                	catch (UnsupportedEncodingException ex)
                	{
                	}
            
                }
            }
        }
    }
    
    public void updatePage(String postStr)
    {
    	String[] dt = postStr.split("&");
    	String test="";
    	String [] lol = test.split("&");
    	
        for(int n=0; n<dt.length; n++)
        {
            int eqp = dt[n].indexOf("=");
            if(eqp>2)
            {
            	
            	String name = dt[n].substring(0,eqp);
                String value = dt[n].substring(eqp+1);

                if(name.contains("PAGE")){
                	try
                	{

                		value = URLDecoder.decode(value, "UTF-8");
                		theUser.setPage(Integer.parseInt(value));
                		((QTIv21sectionPart)((QTIv21testPart)testParts.get(1)).parts.get(1)).setPageNum( 1);
                		
                	}catch (Exception ex)
                	{
                		System.out.println("no page");
                	}
                }
                
            }
        }
    	
    }
    
   public void addFormDataValue(String name, String value)
    {  for(int i=0;i<testParts.size();i++)
		   ((QTIv21testPart)testParts.get(i)).addFormDataValue(name,value);
    }
      
   // -- getOutcomeVariable :  returns an specific outcomeDeclaration given its identifier.
   public QTIv2variableDeclaration getOutcomeVariable(String identifier)
   {
       //# ideally this should return outcomeVars.get(identifier), i.e. I should
       //# make a sensible vector derived class for variable lists.
       if(outcomeVars != null)
       {
           int n = 0;
           while(n < outcomeVars.size())
           {
               QTIv2variableDeclaration var = (QTIv2variableDeclaration)outcomeVars.get(n);
               if(var.identifier.equals(identifier))
                   return var;
               n++;
           }
       }
       return null;
   }

   // vull que per cada outcomeDeclaration existent, es crei un vector amb tots els outcome Declaration dels items  que tinguin igual identifier
   public QTIv2variableDeclaration getTestVariable(String identifier, String sectionIdentifier,
		   String includeCategory,String excludeCategory) throws InvalidQTIv2Exception
   {
	   Vector relatedVarDec = new Vector();
	   QTIv2outcomeDeclaration var = null;
       //# ideally this should return outcomeVars.get(identifier), i.e. I should
       //# make a sensible vector derived class for variable lists.
	   if (outcomeVars != null)
       {
           int n = 0;
           while (n < outcomeVars.size())
           {
               var = (QTIv2outcomeDeclaration) outcomeVars.get(n);
               if (var.identifier.equals(identifier))
               {
            	   findRelatedVar(identifier, sectionIdentifier,includeCategory,excludeCategory,relatedVarDec);  
            	   if(relatedVarDec!=null)
            		   var.setRelatedItemsVarDec(relatedVarDec);   
            	   return var;
               }
               n++;
           }
       }
       return var;
   }
   
   public boolean findItem(String itemIdentifier, Vector item)
   { 
	   for(int i=0;i < testParts.size();i++)
	   {
		   if (((QTIv21testPart)testParts.get(i)).findItem(itemIdentifier,item))
			   return true;
	   }
	   return false;
   }

   public void findRelatedVar(String varIdentifier,String sectionIdentifier,String includeCategory,
		   String excludeCategory,Vector relatedVarDec)
   { 
	   for(int i=0;i < testParts.size();i++)
		   ((QTIv21testPart)testParts.get(i)).findRelatedVar(varIdentifier,sectionIdentifier,includeCategory,excludeCategory,relatedVarDec);	
   }
  
   // -- setOutcomeVariable : adds a new outcome variable to the outcomeVars vector.
   protected boolean setOutcomeVariable(String identifier, QTIv2variableDeclaration newval) throws InvalidQTIv2Exception
   {
       //# ideally this should return outcomeVars.get(identifier), i.e. I should
       //# make a sensible vector derived class for variable lists.
   	   if (outcomeVars != null)
       {
           int n = 0;
           while (n < outcomeVars.size())
           {
               QTIv2variableDeclaration var = (QTIv2variableDeclaration) outcomeVars.get(n);             
               if (var.identifier.equals(identifier))
               {
                   var.setValue(newval);
                   return true;
               }
               n++;
           }
       }
       return false;
   }
   
   // GET NUMBERS 
   public QTIv2variableDeclaration getNumberResponded(String sectionIdentifier) throws InvalidQTIv2Exception
   {
    	numberResponded= 0;
    	
    	for(int i=0;i < testParts.size();i++)
 		   numberResponded+=((QTIv21testPart)testParts.get(i)).getNumberResponded(sectionIdentifier);	
   
    	QTIv2variableDeclaration responded = 
    		new QTIv2variableDeclaration ("numberResponded", QTIv2variableDeclaration.CARD_single ,QTIv2variableDeclaration.VTYPE_integer);
    	responded.setValue(numberResponded);
    	
    	return responded;
    }
    
    public QTIv2variableDeclaration getNumberPresented(String sectionIdentifier) throws InvalidQTIv2Exception
    {
    	numberPresented = 0;
    	
    	for(int i=0;i < testParts.size();i++)
 		   numberPresented+=((QTIv21testPart)testParts.get(i)).getNumberPresented(sectionIdentifier);	
    	
    	QTIv2variableDeclaration presented = 
    		new QTIv2variableDeclaration ("numberPresented", QTIv2variableDeclaration.CARD_single ,QTIv2variableDeclaration.VTYPE_integer);
    	presented.setValue(numberPresented);
    	return presented;
    }

    public QTIv2variableDeclaration getNumberSelected(String sectionIdentifier) throws InvalidQTIv2Exception
    {
    	numberSelected= 0;
    	
    	for(int i=0;i < testParts.size();i++)
 		   numberSelected+=((QTIv21testPart)testParts.get(i)).getNumberSelected(sectionIdentifier);	
    	
    	QTIv2variableDeclaration selected= 
    		new QTIv2variableDeclaration ("numberSelected", QTIv2variableDeclaration.CARD_single ,QTIv2variableDeclaration.VTYPE_integer);
    	selected.setValue(numberSelected);
    	return selected;
    }
    
    
    public QTIv2variableDeclaration getNumberCorrect(String sectionIdentifier) throws InvalidQTIv2Exception
    { 
    	numberCorrect = 0;
	    for(int i=0;i < testParts.size();i++)
	    {
	    	if(!(testParts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	     
	    		numberCorrect +=((QTIv21testPart)testParts.get(i)).getNumberCorrect(sectionIdentifier);	 		
	    				
	    }

    	QTIv2variableDeclaration correct = 
    		new QTIv2variableDeclaration ("numberCorrect", QTIv2variableDeclaration.CARD_single ,QTIv2variableDeclaration.VTYPE_integer);
    	correct.setValue(numberCorrect);   	
    	
    	return correct;
    }
    
    public QTIv2variableDeclaration getNumberIncorrect(String sectionIdentifier) throws InvalidQTIv2Exception
    {
		numberIncorrect = 0;
    
	    for(int i=0;i < testParts.size();i++)
	    {
	    	if(!(testParts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	     
	    		numberIncorrect+=((QTIv21testPart)testParts.get(i)).getNumberIncorrect(sectionIdentifier);	
	    }
	    
    	QTIv2variableDeclaration incorrect = 
    		new QTIv2variableDeclaration ("numberIncorrect", QTIv2variableDeclaration.CARD_single ,QTIv2variableDeclaration.VTYPE_integer);
    	incorrect.setValue(numberIncorrect);
    	
    	return incorrect;
    }

    
    public float getCorrect()
    {
    	//Score score = new Score();
    	try{
    		correct = testPart.getNumberCorrect(null);
    	}catch(Exception e){
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}
    	return correct;
    }
    
    public float getIncorrect()
    {
    	try{
    		incorrect = testPart.getNumberIncorrect(null);
    	}catch(Exception e){
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}
    	return incorrect;
    }
    
	/**
	* Renders QTI Test.
	* 
	* @return QTI Test to be displayed in HTML
	* 
	* @throws InvalidQTIv2Exception if there is an exception parsing QTI element
	*/
    public String render() throws InvalidQTIv2Exception
    {
    	String out="";
    	
    	//Control if user has permission to see this content
    	
    	System.out.println("Estamos entrando en el Render del Test...." + " Titulo:" +identifier + ".Result: "+this.isUolAssignedToUser(identifier));
    	
    	if(!(containsUserRole("Teacher_")||containsUserRole("Student_")) || !isUolAssignedToUser(identifier))
    	{
    		out="<h4>Sorry, you do not have permissions to view that content</h4>";
    		System.out.println("El usuario no tiene permisos de profesor/estudiante y no tiene asignado el contenido");
    		return(out);
    	}
    	
    	//TODO: Implement this functionality...
    	//Control if the test is active... if not show msg
    	// "Sorry, test is not active"
    	if(isTestAlreadyPerformed(identifier))
    	{
    		out="<h4>Sorry, test is not enabled.</h4>";
    		System.out.println("El test no est activo... no puede hacerse.");
    		return(out);
    	}
    	
    	//TODO: Implement this functionality...
    	//Control if the test has already been performed... if performed show mssg
    	// "Sorry, test can't be performed twice."
    	if(isTestDisabled(identifier))
    	{
    		out="<h4>Sorry, test can't be performed twice.</h4>";
    		System.out.println("El usuario ya ha contestado este test... no puede hacerlo de nuevo.");
    		return(out);
    	}
    	   	

    	
    	if(continueTest)
    	{ 
			 for(int i=0; i < testParts.size() ;i++) 
				out+=((QTIv21testPart)testParts.get(i)).render(); // rendering parts
	    	 if (outproc != null) 		// processing outcomes
	         	 try
	             {	
	             	outproc.run();     
	             }
	             catch (InvalidQTIv2Exception ex)
	             {
	                 System.err.println(ex);
	                 ex.printStackTrace();
	                 if (parseErrors == null)
	                     parseErrors = "";
	                 parseErrors += ex + "<br/>";
	             }
    	}
    	else
    	{
    		out="<table>" +
    				"<tr>" +
    				"	<td><h2>TEST ACABADO</h4> </td>" +
    				"</tr>" +
    				"<tr >" +
    				"	<td ><h3> "	+ finishingReason +"<h3></td>" +
					"</tr></table>";
    	}  	
    	
    	if (this.getOutcomeVars() != null) // setting the final score attribute ( shoud it be an attribute of a test when it really is a outcome var ? )
        {
            for (int n = 0; n < this.getOutcomeVars().size(); n++)
            {
				tmp = (QTIv2variableDeclaration) this.getOutcomeVars().get(n);
                for (int n2 = 0; n2 < tmp.getSize(); n2++)
                {
                	try
                    {
                		if(tmp.getIdentifier().equals("SCORE"))
                		{
                			//finalScore = theUser.getFinalScore();
                			finalScore = Float.parseFloat(tmp.getStringValue(0));
                		}
                    }
                    catch (Exception ex)
                    {
                    	ex.printStackTrace();
                    }
                }
            }
            
            //Posem el resultat a la taula de resultats...
            
            /*
            int result = (int) this.getFinalScore();
            
            List<AssessmentTest> assessmenttests = AssessmentTestLocalServiceUtil.getAssessmentTestsByIdentifier(identifier);
            if (assessmenttests.size()==1)
            {
            	AssessmentTest assessmentAux = assessmenttests.get(0);
            	long assessmentTestId = assessmentAux.getAssessmentTestId();
            	
            	System.out.println("Asssessment Test Id: " + assessmentTestId);
            	long uolId = 0L;
            	
            	List<Uol> uols = null;
				try {
					uols = UolLocalServiceUtil.getUols(0, UolLocalServiceUtil.getUolsCount());
				} catch (SystemException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
            	
				//Encontramos el UOL que toca
            	Uol	uolAux = this.getUolByAssessmentId(uols, assessmentTestId);
            	
            	
            	if(uolAux!= null)
        		{
            		
            		//Generar entrada en la tabla de tests

            		TestResult testAux = TestResultLocalServiceUtil.getTestResult(Long.toString(uolAux.getI_Id()), Long.toString(getUserId()));
                    
            		testAux.setTitle(uolAux.getTitle());
                    testAux.setResult(result);
                    
                    try{
                		User userAux = UserLocalServiceUtil.getUserById(getUserId());                		
                        testAux.setUsername(userAux.getScreenName());
                    }catch(Exception ex)
                    {
                    ex.printStackTrace();	
                    }
                    
                    try {
                    	List<TestResult> testresults = TestResultLocalServiceUtil.getTestResults(0, TestResultLocalServiceUtil.getTestResultsCount());
                    	if(!testresults.contains(testAux))
                    		TestResultLocalServiceUtil.addTestResult(testAux);
                    	else
                    	{
                    		TestResultLocalServiceUtil.updateTestResult(testAux);
                    		System.out.println("Entrada de Resultado ya inserida anteriormente, actualizando...");

                    	} 
    				} catch (SystemException e) {
    					// TODO Auto-generated catch block
    					e.printStackTrace();
    				}
        		}
        		
        		else System.out.println("La Dynamic Query no devuelve ningn Uol para la consulta.");
            	


                
            	
            }
            
            */
            //Finalitzem Actualitzacio Taula de Resultats
            

        }
    	
    	if(this.feedbacks != null)
    	{
    		out += renderFeedbacks();
    	}
    	return out;
    }
    
    
    
    public String renderFeedbacks(){
    	
    	
    	String out = "";
    	
 		if (this.getOutcomeVars() != null)
        {
	        // Get interpolation Table
	        for(int i = 0; i < this.getOutcomeVars().size(); i++)
			{
				outVars = (QTIv2outcomeDeclaration)this.getOutcomeVars().get(i);
				if(outVars.interTables!=null)
				{				
					intertables = outVars.interTables;
   				}
			}
        }
		
		// here shoud be ta call to the function intertable. solve interpolation tables
		
		//testMark = solveIntertables();
		
		if(intertables!=null){ // this is actually not following the xml calculation of a summary value, its hardcoding suposing that intertables!=null => score agnist intertables = testmark
			for(int j = 0; j < intertables.size()-1; j++)
    		{
    			intertable = (QTIv21interpolationTable)intertables.get(j);
    			tmptable = (QTIv21interpolationTable)intertables.get(j+1);
    			if( finalScore<=1000 && finalScore >=intertable.sourceValue){
    				testMark = intertable.targetValue;
    				break;
    			}
    			else if((finalScore<intertable.sourceValue)&&(finalScore>=tmptable.sourceValue)){
    			
    				testMark = tmptable.targetValue;
    			}
    			else testMark = tmptable.targetValue;
				
    		}  
    	}  		
        
		if (this.getOutcomeVars() != null) // NPRESENTED & NRESPONDED must be (and well defined)in the xml in order to know when a test is over
        {    	
            for (int n1 = 0; n1 < this.getOutcomeVars().size(); n1++)
            {
				tmp = (QTIv2variableDeclaration) this.getOutcomeVars().get(n1);
                for (int n2 = 0; n2 < tmp.getSize(); n2++)
                {
                	try
                    {
                        if(tmp.getIdentifier().equals("NPRESENTED"))
                        		numPRESENTED = Float.parseFloat(tmp.getStringValue(n2));
                        if(tmp.getIdentifier().equals("NRESPONDED"))
                        		numRESPONDED = Float.parseFloat(tmp.getStringValue(n2));
                    }
                    catch (InvalidQTIv2Exception ex)
                    {
                    }
                }
                if(numPRESENTED == numRESPONDED)
                	showOutcome = true;
                else 
                	showOutcome = false;
            }
        }
		
   
		    
    	// TESTFEEDBACK
		DAOUserTestCase daotest = new DAOUserTestCase();
		
		if(this.feedbacks.size()!=0)
		{
			
        	out+="<h4><b>"+daotest.getTFtitle(theUser)+"</b></h4>";
        	
        	for (int n = 0; n < this.feedbacks.size(); n++)
        	{
        		feedback = (QTIv21testFeedback)this.feedbacks.get(n);
       	    	
        		if(testMark.equals(feedback.getIdentifier()) && showOutcome)
        		{
	        		out += feedback.getMessage();
        		}    
        		else feedmessageOutcome = feedback.getMessage2();
            }
        	
        	if(showOutcome){
    			out+=feedmessageOutcome;
            	out+="<table>";
            	out+="<tbody>";
            	out+="<tr></tr>";
            	if (this.getOutcomeVars() != null)
            	{    	
            		for (int n1 = 0; n1 < this.getOutcomeVars().size(); n1++)
            		{
    					tmp = (QTIv2variableDeclaration) this.getOutcomeVars().get(n1);
    					for (int n2 = 0; n2 < tmp.getSize(); n2++)
    					{
    						try
    						{
    							if(tmp.getIdentifier().equals("NPRESENTED")){
    								out+="<tr><td>"+daotest.getTFpresented(theUser)+"</td>";
    								out+="<td>"+tmp.getStringValue(n2)+"</td>";
    								out+="</tr>";                            		
    							}
    							
    							else if(tmp.getIdentifier().equals("NRESPONDED")){
    								out+="<tr><td>"+daotest.getTFresponded(theUser)+"</td>";
    								out+="<td>"+tmp.getStringValue(n2)+"</td>";
    								out+="</tr>";                            		
    							}   
    							else if(tmp.getIdentifier().equals("SCORE")){
    								
    								//temp_user.setItems();
    								theUser.setItems(); //store changes in DB when it's the end of test
    													//this is done because the setItems is usually called after rendering the test
    													// but now we need all the items in the db in order to calculate&render the results!
    				        	
    								finalScore = theUser.getFinalScore();
    								
    								//TODO: Aqui debemos modificar la tabla de resultados
    								this.updateTestResultTable();
    								
    								out+="<tr><td>Final score:</td>";
    								if(finalScore!=-1)
    								{
    									out+="<td>"+finalScore+"</td>";
    								}else{
    									out+="<td> not found </td>";
    								}
    								
    								out+="</tr>";                            		
    							}
    							else if(tmp.getIdentifier().equals("PERCENT_CORRECT")){
    								out+="<tr><td><b>"+daotest.getTFpercentage(theUser)+"</b></td>";
    								out+="<td>"+tmp.getStringValue(n2)+"%</td>";
    								out+="</tr>";                             		
    							}            
    							else if(tmp.getIdentifier().equals("NCORRECT")){
    								out+="<tr><td>"+daotest.getTFcorrect(theUser)+"</td>";
    								out+="<td>"+tmp.getStringValue(n2)+"</td>";
    								out+="</tr>";                            		
    							} 
    							else if(tmp.getIdentifier().equals("NINCORRECT")){
    								out+="<tr><td>"+daotest.getTFincorrect(theUser)+"</td>";
    								out+="<td>"+tmp.getStringValue(n2)+"</td>";
    								out+="</tr>";                            		
    							} 
    							else if(tmp.getIdentifier().equals("NSELECTED")){
    								out+="<tr><td>"+daotest.getTFselected(theUser)+"</td>";
    								out+="<td>"+tmp.getStringValue(n2)+"</td>";
    								out+="</tr>";                            		
    							} 

    						}catch (InvalidQTIv2Exception ex)
    						{
    						}
    					}
            		}
            	}else	out+="<tr ><td><h5> NON TEST OUTCOME SPECIFIED:</h5></td></tr>";
            	out+="</tbody>";
            	out+="</table>";
    		}
		}
		return out;
    }
    
    // this function will solve outcome
    public String solveIntertables(QTIv2outcomeDeclaration interpolation_tables){
    	
    	// LAU:[to fix] the use of finalScore don't follow the xml specification of the summary variable definition, that maybe is not the score calculation, here and in 621

    	
    	String testMark ="";
		if(intertables!=null){
			for(int j = 0; j < intertables.size()-1; j++)
    		{
    			intertable = (QTIv21interpolationTable)intertables.get(j);
    			tmptable = (QTIv21interpolationTable)intertables.get(j+1);
    			if( finalScore >=intertable.sourceValue){
    				testMark = intertable.targetValue;
    				break;
    			}
    			else if((finalScore<intertable.sourceValue)&&(finalScore>=tmptable.sourceValue)){
    			
    				testMark = tmptable.targetValue;
    			}
    			else testMark = tmptable.targetValue;
				
    		}  
    	}
		return testMark;
    }
    	
    //-- Getters i Setters 
    public void setMediaBase(String nMediaBase)
    {
        mediaBase = nMediaBase;
    }

    public void setItemBase(String nItemBase)
    {
        itemBase = nItemBase;
    }
    
    //UPF - Pau
    public String getFile()
    {
    	return this.file;
    }
    
    public Hashtable getXmlContainer()
    {
    	return this.xml_container;
    	
    }
    public String getId()
    {
    	return this.identifier;
    }
    /*
    public void setRequest(HttpServletRequest request)
    {
        this.request = request;
    }
     */
    
    public void setTestBase(String nTestBase)
    {
        testBase = nTestBase;
    }
    public void setAppletBase(String nAppletBase)
    {
        appletBase = nAppletBase;
    }

    public String getMediaBase()
    {
        return mediaBase ;
    }

    public String getAppletBase()
    {
        return appletBase;
    }
    public String getItemBase()
    {
        return itemBase;
    }
    public float getFinalScore() {
		return finalScore;
	}

	public boolean getContinueTest()
    {
    	return continueTest;
    }
    
    public Vector getOutcomeVars()
    {
    	return outcomeVars;
    }
    
	public List<Result> getPlainOutcomeVars() {
		List<Result> plainOutComeVars = new ArrayList<Result>();
		QTIv2outcomeDeclaration tmp;
		Object tmp_value;
		for (int i = 0; i < outcomeVars.size(); i++) {
			tmp = ((QTIv2outcomeDeclaration) outcomeVars.get(i));
			Result result = null;
			
			if (tmp.interTables == null) { // not a interpolation table outcome
				if(tmp.value.size()>0)
					tmp_value = tmp.value.firstElement();
				else
					tmp_value = null;
				result = new Result(tmp.identifier, tmp_value );
			} else {
				result = new Result(tmp.identifier, solveIntertables(tmp));
			}
			plainOutComeVars.add(result);
		}
		return plainOutComeVars;
	}

	public static class Result {
		private String key;
		private Object value;
		public Result(String inKey, Object inValue) {
			this.key = inKey;
			this.value = inValue;
		}
		public String getKey() {
			return this.key;
		}
		public Object getValue() {
			return this.value;
		}
	}
   /* public void setOutcomeVars() throws FileNotFoundException, IOException
    {
    	Score score = new Score();
    	//score.setScores(outcomeVars);   	  	
    }
    
    public void setNumberCorrect() throws FileNotFoundException, IOException
    {
    	Score score = new Score();
    	score.setNumberCorrect(numberCorrect);
    }*/
    
    public void setWeightIdentifier(String aWeightIdentifier)
    {
    	weightIdentifier= aWeightIdentifier;
    }
    
	/**
	* Shows QTI XML information (testing purposes)
	*
	* @return QTI parameters string
	*/
    
    public String toString()
    {
		StringBuffer sb =new StringBuffer();
		sb.append("QTIv21test: MediaBase: " + this.mediaBase + ", ");
		sb.append("TestTitle: "+ this.testTitle + ", ");
		sb.append("TestBase: "+ this.testBase + ", ");
		sb.append("TestXML: "+ this.sTestXML);		
		return sb.toString();
    }
    
	/**
	* Gets a Unit of Learning by AssessmentId
	* 
	* @param uols List of Units of Learning
	* @param assessmentTestId Assessment object test Id
	* 
	* @return Uol parameters string
	*/
    public Uol getUolByAssessmentId(List<Uol> uols, long assessmentTestId)
    {
    	Uol uolAux = null;
    	
    	Iterator<Uol> it = uols.iterator();
    	
    	while(it.hasNext())
    	{
    		uolAux = it.next();
    		if (uolAux.getAssessmentTestId() == assessmentTestId)
    			return uolAux;
    	}
    	
    	return uolAux;
    }
    
	/**
	* Checks if QTI Test has already been performed by the current
	* 
	* @param assessmentId Assessment object test Id
	* 
	* @return true if the test has already been performed, false instead
	*/
    public boolean isTestAlreadyPerformed(String assessmentId) 
    {
    	boolean result = false;
    	String userName = this.getUser().getName();
    	long assessmentTestId = 0L;
    	long uolId = 0L;
    	TestResult testResAux = null;
    	
        List<AssessmentTest> assessmenttests = AssessmentTestLocalServiceUtil.getAssessmentTestsByIdentifier(identifier);
        if (assessmenttests.size()==1)
        {
        	AssessmentTest assessmentAux = assessmenttests.get(0);
        	assessmentTestId = assessmentAux.getAssessmentTestId();
        }
    	
    	List<Uol> uols = null;
		try {
			uols = UolLocalServiceUtil.getUols(0, UolLocalServiceUtil.getUolsCount());
		} catch (SystemException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
    	
		//Encontramos el UOL que toca
    	Uol	uolAux = this.getUolByAssessmentId(uols, assessmentTestId);
    	if(uolAux != null)
    		uolId = uolAux.getI_Id();
    	
    	//Generamos el TestResult con la PK que toca
    	testResAux = TestResultLocalServiceUtil.getTestResult(Long.toString(uolId), userName);
    	
    	List<TestResult> testresults = null;
    	
		try {
			testresults = TestResultLocalServiceUtil.getTestResults(0, TestResultLocalServiceUtil.getTestResultsCount());
		} catch (SystemException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
    	
		
    	if(testresults.contains(testResAux))
    	{
    		System.out.println("Encontrado TestResult: " + testResAux.toString());
    		return true;
    	}
    		
    	
    	return result;
    }
    
	/**
	* Checks if QTI Test is disabled
	* 
	* @param assessmentId Assessment object test Id
	* 
	* @return true if the test has been disabled, false instead
	*/
    public boolean isTestDisabled(String assessmentId) 
    {
    	boolean result = false;
    	String userName = this.getUser().getName();
    	long assessmentTestId = 0L;
    	long uolId = 0L;
    	TestResult testResAux = null;
    	
        List<AssessmentTest> assessmenttests = AssessmentTestLocalServiceUtil.getAssessmentTestsByIdentifier(identifier);
        if (assessmenttests.size()==1)
        {
        	AssessmentTest assessmentAux = assessmenttests.get(0);
        	assessmentTestId = assessmentAux.getAssessmentTestId();
        }
    	
    	List<Uol> uols = null;
		try {
			uols = UolLocalServiceUtil.getUols(0, UolLocalServiceUtil.getUolsCount());
		} catch (SystemException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
    	
		//Encontramos el UOL que toca
    	Uol	uolAux = this.getUolByAssessmentId(uols, assessmentTestId);
    	if(uolAux != null)
    		uolId = uolAux.getI_Id();
    	
    	if(uolAux.getActive())
    		return true;
    	
    	return result;
    }
    
	/**
	* Gets current userId
	* 
	* @return current user Id
	*/
    public long getUserId()
    {

    	String userName = this.getUser().getName();
    	//long userId = PrincipalThreadLocal.getUserId();
    	System.out.println("User ID:" + userName);
    	return Long.parseLong(userName);
    	
    }
    
	/**
	* Checks if current user has a concrete role or not
	* 
	* @param userRoleName Role name to be checked
	* 
	* @return true if the current user has the concrete role, false instead
	*/
    public boolean containsUserRole(String userRoleName) 
    {
    	boolean result = false;
    	
    	System.out.println("UserRole: " + userRoleName);

    	String userIdStr = this.getUser().getName();
    	long userId = Long.parseLong(userIdStr);
    	long compId = 0;
    	Group group = null;
    	
    	
    	List<Role> roles = new ArrayList<Role>();
		List<Role> allRoles=new ArrayList<Role>();

    	List<UserGroupRole> ugroles = new ArrayList<UserGroupRole>();
    	try {

    		try {
				compId = CompanyLocalServiceUtil.getCompanyIdByUserId(Long.parseLong(userIdStr));
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    		try {
				group = GroupLocalServiceUtil.getCompanyGroup(compId);
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    		try {
    			allRoles = RoleLocalServiceUtil.getUserRelatedRoles(userId, group.getGroupId());
    			//RoleLocalServiceUtil.hasUserRole(userId, compId, userRoleName, true);
				//result = UserGroupRoleLocalServiceUtil.hasUserGroupRole(userId, group.getGroupId(), userRoleName);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			//roles = RoleLocalServiceUtil.getUserRoles(userId);
			//ugroles = UserGroupRoleLocalServiceUtil.getUserGroupRoles(userId);
			//allRoles = RoleLocalServiceUtil.getRoles(0,RoleLocalServiceUtil.getRolesCount());
			//System.out.println("Listado de Roles del Usuario " + userIdStr + ": " + roles.toString());
			//System.out.println("Listado de UGroup Roles del Usuario " + userIdStr + ": " + ugroles.toString());
			System.out.println("Listado de AllRoles del Usuario " + userIdStr + ": " + allRoles.toString());

		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Iterator<Role> it = allRoles.iterator();
		while(it.hasNext())
		{
			Role aux =it.next();
			if(aux.getName().compareTo(userRoleName)==0)
				return true;
		}
		
		System.out.println("El Usuario " + userIdStr + " no tiene asignado el rol " + userRoleName);
		 
    	return result;
    	
    }
    
	/**
	* Checks if the Unit of Learning to be performed is assigned to the current user
	* 
	* @param assessmentId Assessment object test Id
	* 
	* @return true if the unit of learning tobe performed has current user assigned, false instead
	*/
    public boolean isUolAssignedToUser(String assessmentId)
    {	
    	boolean result = false;
    	String uolTitle ="";
    	
    	System.out.println("Assessment Id: " + assessmentId);
        List<AssessmentTest> assessmenttests = AssessmentTestLocalServiceUtil.getAssessmentTestsByIdentifier(identifier);
        if (assessmenttests.size()==1)
        {
        	AssessmentTest assessmentAux = assessmenttests.get(0);
        	uolTitle = assessmentAux.getTitle();
        }

    	String userIdStr = this.getUser().getName();
    	long userId = Long.parseLong(userIdStr);
    	
    	System.out.println("User Id: " + userId);
    	
    	List<Assignment> assignments = new ArrayList<Assignment>();
    	
    	try {
    		assignments = AssignmentLocalServiceUtil.getAssignments(0, AssignmentLocalServiceUtil.getAssignmentsCount());
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	
		Iterator<Assignment> it = assignments.iterator();
		while(it.hasNext())
		{
			Assignment aux =it.next();
			if(aux.getUserId()==userId && aux.getTitle().compareTo(uolTitle)==0)
				return true;
		}
		System.out.println("El Uol " + uolTitle + "no ha sido asignado al usuario " + userId);
    	return result;
    }
    
	/**
	* Updates Test Result Table after test is performed
	* 
	*/
    public void updateTestResultTable()
    {
        //Posem el resultat a la taula de resultats...
    	System.out.println("Actualitzant la taula de resultats");
        
        int result = (int) this.getFinalScore();
        
        List<AssessmentTest> assessmenttests = AssessmentTestLocalServiceUtil.getAssessmentTestsByIdentifier(identifier);
        if (assessmenttests.size()==1)
        {
        	AssessmentTest assessmentAux = assessmenttests.get(0);
        	long assessmentTestId = assessmentAux.getAssessmentTestId();
        	
        	System.out.println("Asssessment Test Id: " + assessmentTestId);
        	long uolId = 0L;
        	
        	List<Uol> uols = null;
			try {
				uols = UolLocalServiceUtil.getUols(0, UolLocalServiceUtil.getUolsCount());
			} catch (SystemException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
        	
			//Encontramos el UOL que toca
        	Uol	uolAux = this.getUolByAssessmentId(uols, assessmentTestId);
        	
        	
        	if(uolAux!= null)
    		{
        		
        		//Generar entrada en la tabla de tests

        		TestResult testAux = TestResultLocalServiceUtil.getTestResult(Long.toString(uolAux.getI_Id()), Long.toString(getUserId()));
                
        		testAux.setTitle(uolAux.getTitle());
                testAux.setResult(result);
                
                try{
            		User userAux = UserLocalServiceUtil.getUserById(getUserId());                		
                    testAux.setUsername(userAux.getScreenName());
                }catch(Exception ex)
                {
                ex.printStackTrace();	
                }
                
                try {
                	List<TestResult> testresults = TestResultLocalServiceUtil.getTestResults(0, TestResultLocalServiceUtil.getTestResultsCount());
                	if(!testresults.contains(testAux))
                		TestResultLocalServiceUtil.addTestResult(testAux);
                	else
                	{
                		TestResultLocalServiceUtil.updateTestResult(testAux);
                		System.out.println("Entrada de Resultado ya inserida anteriormente, actualizando...");

                	} 
				} catch (SystemException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
    		}
    		
    		else System.out.println("La Dynamic Query no devuelve ningn Uol para la consulta.");
        
        	//Desprs del resultat...
    }
        
    }
    
}


