/*
 * Copyright (c) 2008, Consortium Board TENCompetence
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the TENCompetence nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CONSORTIUM BOARD TENCOMPETENCE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CONSORTIUM BOARD TENCOMPETENCE BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package apis;

import java.util.StringTokenizer;
import java.util.Vector;

import nb_utils.*;

/**
 * <p>Title: APIS test wrapper</p>
 * <p>Description: Test Feedback class</p>
 * <p>Copyright: Copyright (c) 2008, Universitat Pompeu Fabra</p>
 * <p>Company: Universitat Pompeu Fabra</p>
 * @author Pau Melines / Lau Llobet
 * @version 1.0
 */

public class QTIv21testFeedback
{
    private String outcomeIdentifier;
    private QTIv2outcomeDeclaration outcome;
    private String access;
    private boolean show;
    private String identifier;
    private String identifier2;
    private String title;
    private String message;
    private String message2;
    private QTIv21test theTest;
    private QTIv2outcomeDeclaration outVars;
    private QTIv2variableDeclaration tmp;


    // Constructor
    public QTIv21testFeedback(QTIv21test owner)
    {
    	theTest = owner;
    	access = "";
    	show = false;
    	identifier = "";
    	identifier2 = "";
    	title = "";
    	message = "";
    	message2 = "";
    	outcomeIdentifier = "";
    }     
    
    public String getMessage()
    {
    	return message;
    }
    
    public String getMessage2()
    {
    	return message2;
    }
    
    public String getIdentifier()
    {
    	return identifier;
    }

    // Parse method
    public int parse(NBEasyXML testXML, int idx) throws InvalidQTIv2Exception
    {
    	//# read in attributes, and store
        identifier = testXML.getAttribute(idx, "identifier");

        outcomeIdentifier = testXML.getAttribute(idx,"outcomeIdentifier");
        
        title = testXML.getAttribute(idx, "title");

        if(outcomeIdentifier.length()==0)
            throw new InvalidQTIv2Exception("feedback section does not have an outcomeIdentifier");
        
        if(testXML.getAttribute(idx,"showHide").equals("show"))
            show = true;
        else
            show = false;
        
        if(show == true)
        {
	        int position = testXML.findChildElement(idx);
		    int id = QTIv2define.identify(testXML.getName(position));
	        while(id == -1)
	        {
	        	message += testXML.getAsString(position);
	        	position++;
	        	id = QTIv2define.identify(testXML.getName(position));
	        }
        }
        else if(show == false)
        {
	        int position = testXML.findChildElement(idx);
		    int id = QTIv2define.identify(testXML.getName(position));
		    position++;
        	message2 += testXML.getAsString(position);
        }                 
               
        return testXML.findNextPeer(idx);
    }
}
