package apis;

import nb_utils.*;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;
import java.util.Vector;

/**
 * Class : testPart [1..*]
 * 
 * Description : Major division of the test used to control basic mode parameters for sections 
 * 				 included within this part
 * 
 * Attributes:  * navigationMode (linear / non linear) 
 * 					Determines the general paths that the candidate may take. 
 * 					In linear, attempt each item in turn. The candidate is not permited to return back
 * 					In non linear, the candidate is free to navigate to any item in the test at
 * 					any time. 
 *				* submissionMode (individual / simultaneous)
 *					Determines when the candidate's responses are submitted for response processing.
 *					Individual mode requires the candidate to submit their responses on an item-by-item	
 *					basis. 
 *					In Simultaneous mode, the candidate's responses are all submitted together at the end. 
 * Contains :   * timeLimits [0..1]
 * 					Optionally controls  the amount  of time  a candidateis allowed for the test. 
 * 				* assessmentSection [1..*]
 * 					The items contained in each testPart are arranged into sections and sub-sections.
 * 				* itemSessionControl [0..1]
 * 					Parameters used to control the allowable for this part of the test
 * 				* scoreReport [*]
 * 					Test-level feedback specific to this part of the test.
**/


public class QTIv21testPart //extends QTIv21sectionPart
{
    protected QTIv2structuredBodyPart prompt;
    
    //Attributes
    protected QTIv21test theTest;
    protected Vector parts ; 
    protected Vector assParts ; 
    protected String navigationMode;
    protected String submissionMode;
    protected boolean endPart;
    protected String finishingReason;
    
    //-- Constructor 
    public QTIv21testPart(QTIv21test owner)
    {
        theTest=owner;
        navigationMode="";	    	// navigationMode [1] : navigationMode
        submissionMode="";			// submissionMode [1] : submissionMode
        endPart = false;
        finishingReason = "";
    }
    
    //-- parsing of testPart 
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {	
    	System.out.println("Peta aqu i hem de veure pq...");
    	parts = new Vector();
    	assParts = new Vector();
    	//-- ATTRIBUTES
        if(itemXML.getAttribute(idx,"navigationMode")!= null)
        {
        	navigationMode= itemXML.getAttribute(idx,"navigationMode");
        	System.out.println("Navigation mode: " + navigationMode);
        }
            
        else
        	throw new InvalidQTIv2Exception("there must be a navegationMode specified for the testPart");

        if(itemXML.getAttribute(idx,"submissionMode")!= null)
        {
        	submissionMode= itemXML.getAttribute(idx,"submissionMode");
        	System.out.println("Submission mode: " + submissionMode);
        }
            
        else
        	throw new InvalidQTIv2Exception("there must be a submissionMode specified for the testPart");

        int cidx = itemXML.findChildElement(idx);
        
        //-- CONTAINS
        while(cidx != -1)
        {

             	int id = QTIv2define.identify(itemXML.getName(cidx));  
            try
            {
                switch(id)
                {
                    case QTIv2define.QTIv21_itemSessionControl:
                        QTIv21itemSessionControl isc = new QTIv21itemSessionControl(theTest);
                        cidx =isc.parse(itemXML, cidx );
                        System.out.println("Item Session Control Max Atts:" + isc.maxAttempts);
                        break;
                    case QTIv2define.QTIv21_timeLimits:
                    	QTIv21timeLimits tl = new QTIv21timeLimits(this);
                        System.out.println("Time limits:");
                    	cidx =tl.parse(itemXML,cidx );
                        break;
                    case QTIv2define.QTIv21_assessmentSection:
                    	
                    	System.out.println("1.- Cidx:" +cidx);
                    	
                    	QTIv21assessmentSection as = new QTIv21assessmentSection(theTest);
                    	
                    	System.out.println("2.- Cidx:" +cidx);
                    	
                    	cidx =as.parse(itemXML,cidx );
                   	 	
                    	System.out.println("Cidx:" +cidx);
                    	
                    	if(as.visible) 
                    		parts.add(as);
                    	else
                        {
                     		 for(int k=0;k<as.parts.size();k++)
                     		 {   
                     			 parts.add(as.parts.get(k));
                     		 }	 
                        }
                    	System.out.println("Assessment sect:" + as.rawXML);
                    	break;
                    case QTIv2define.QTIv21_scoreReport:
                        throw new InvalidQTIv2Exception("externalResponseProcessing is not supported by APIS.");
                    default:
                        throw new InvalidQTIv2Exception("Unexpected "+itemXML.getName(idx)+" in response processing.");
                }
        	}
        	catch(Exception ex)
        	{
        		ex.printStackTrace();
        		cidx = -1;
        	}
        }
    //Mostrar lo recuperado
        /*Iterator it = parts.iterator();
        while(it.hasNext())
        {
        	Vector v = (Vector) it.next();
        	System.out.println("Part Content: "+v.toString());
        }
        */
        //Fin mostrar
    return itemXML.findNextPeer(idx); // -1 if no more peers, needs to be handled.
    }
    
    public void setProcessedItems(UserTestCase processedItems)
    {
    	for(int i=0;i<parts.size();i++)
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
    		((QTIv21sectionPart)parts.get(i)).setProcessedItems(processedItems);
    }
    
    // setItemsToProcess : marks if the item variables should be checked or not
    public void setItemsToProcess(String postStr)
    {
    	for(int i=0;i<parts.size();i++)
    	if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock")){	
    		((QTIv21sectionPart)parts.get(i)).setItemsToProcess(postStr);
    	}
    }
    
 
    // addValueFormDataValue : updates data from all items in this testPart marked to be processed.
    public void addFormDataValue(String name, String value)
    {
    	for(int i=0;i < parts.size();i++)
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
    		((QTIv21sectionPart)parts.get(i)).addFormDataValue(name,value);
    }

    public boolean findItem(String itemIdentifier, Vector item)
    { 
 	   for(int i=0;i < parts.size();i++)
 	   {
 		  if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
 		   if (((QTIv21sectionPart)parts.get(i)).findItem(itemIdentifier,item))
 			  return true;
 	  }
 	   return false;
    }
    
    public void findRelatedVar(String varIdentifier,String sectionIdentifier,String includeCategory,
    		String excludeCategory, Vector relatedVarDec)
    {
    	for(int i=0;i < parts.size();i++)
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
   		((QTIv21sectionPart)parts.get(i)).findRelatedVar(varIdentifier,sectionIdentifier,includeCategory,excludeCategory,relatedVarDec );	
    }
    

    //GETS NUMBER
    public int getNumberPresented(String sectionIdentifier) throws InvalidQTIv2Exception
    {
    	int numberPresented = 0;
    	for(int i=0;i < parts.size();i++)
    	{
    	if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
 		   numberPresented+=((QTIv21sectionPart)parts.get(i)).getNumberPresented();	
    	}
 		 return numberPresented;
    }
    
    public int getNumberCorrect(String sectionIdentifier) throws InvalidQTIv2Exception //if string == null , all items 
    {
    	int numberCorrect = 0;
    	
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
    		{
    			if(sectionIdentifier == null)
	    			numberCorrect+=((QTIv21sectionPart)parts.get(i)).getNumberCorrect();	
	    		else
	    			if(parts.get(i).getClass().getCanonicalName().equals("apis.QTIv21assessmentSection"))
	    				if(((QTIv21assessmentSection)parts.get(i)).identifier.equals(sectionIdentifier))
	    					numberCorrect+=((QTIv21assessmentSection)parts.get(i)).getNumberCorrect();     				
	    	}
    	}
    	return numberCorrect;
    }
    
    public int getNumberIncorrect(String sectionIdentifier) throws InvalidQTIv2Exception
    {
    	int numberIncorrect= 0;
    	
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
    		{		
	    		if(sectionIdentifier == null)
	    			numberIncorrect+=((QTIv21sectionPart)parts.get(i)).getNumberIncorrect();	
	    		else 
	    			if(parts.get(i).getClass().getCanonicalName().equals("apis.QTIv21assessmentSection"))
	    				if(((QTIv21assessmentSection)parts.get(i)).identifier.equals(sectionIdentifier))
	    					numberIncorrect+=((QTIv21assessmentSection)parts.get(i)).getNumberIncorrect();	    	
	    	}
    	}
    	return numberIncorrect;
    }
    
    public int getNumberResponded(String sectionIdentifier) 
    {
    	int numberResponded = 0;
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
    		{
	    		if(sectionIdentifier == null)
	    			numberResponded+=((QTIv21assessmentSection)parts.get(i)).getNumberResponded();	
	    		else 
	    		{	if(parts.get(i).getClass().getCanonicalName().equals("apis.QTIv21assessmentSection"))
	    				if(((QTIv21assessmentSection)parts.get(i)).identifier.equals(sectionIdentifier))
	    					numberResponded +=((QTIv21assessmentSection)parts.get(i)).getNumberResponded();	   
	    	
	    		}
	    	}
    	}
    	return numberResponded;
    
    }
    
    public int getNumberSelected(String sectionIdentifier)
    {
    	int numberSelected= 0;
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
    		{
	    		if(((QTIv21assessmentSection)parts.get(i)).visible)
	    		{
	    			if(sectionIdentifier == null)
	    				numberSelected+=((QTIv21assessmentSection)parts.get(i)).getNumberSelected();	
	    			else 
	        		{	if(parts.get(i).getClass().getCanonicalName().equals("apis.QTIv21assessmentSection"))
	        				if(((QTIv21assessmentSection)parts.get(i)).identifier.equals(sectionIdentifier))
	        					numberSelected +=((QTIv21assessmentSection)parts.get(i)).getNumberSelected();	   
	        	
	        		}
	    		}
    		}
    	}
    	return numberSelected;
    	
    }
    
	// render : renders all items in this testPart
    public String render() throws InvalidQTIv2Exception
    {
    	String section ="";
        String out = "";
        if(!endPart)
        {   //out+="<table  width='90%' align='center' border ='0'><tr align = 'center'><td> <h3>  TEST PART:</h3></td></tr>";	      
        	out+="<table><tr><td> <h3></h3></td></tr>";
        	out+="<tr><td>";	
	        for (int n = 0; n < parts.size(); n++)
	        {
	        	if((parts.get(n).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))	
	        		 out+=((QTIv2rubricBlock)parts.get(n)).render();
	        	else if(parts.get(n).getClass().getCanonicalName().equals("apis.QTIv21assessmentSection"))
	        	{
	        		QTIv21assessmentSection assSection = ((QTIv21assessmentSection)parts.get(n));
	     
	        		section = assSection.render();
	        	
	        		/*if (assSection.endSection)
	        		{			
	        	out+="<table align='center' width='30%' border ='0' bgcolor='#D4DDE4'><tr><td><h4 align='center'> FINISHED SECTION : "+assSection.identifier+"" +
	        			"</h4>" + assSection.finishingReason +"</td></tr>";	    		    			
	        			
	        			if(!assSection.printedResults)
	        			{	out+="<tr><td>"+assSection.getResultInfo()+"</td></tr></table><br></br>";
	        				assSection.printedResults = true;
	        			}else
	        				out+="</table><br></br>";
	        		
	        		}	        		else*/
	        			out+=section;
	        	}
	        	else
	        		out+= ((QTIv21assessmentItemRef)parts.get(n)).render();
	        }
        }else
        {	
        	out="<table>" +
			"<tr >" +
			"	<td ><h3>TEST PART FINISHED</h3> </td>" +
			"</tr>" +
			"<tr>" +
			"	<td ><h4> "	+ finishingReason +"<h4></td>" +
			"</tr>"; 	
        }   
        out+="</td></tr>";
        out +="</table>";
        return out;
    }
}
