package apis;

import nb_utils.*;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Class : timeLimits
 * 
 * Description : In the context of a specific assessmentTest an item, or group of items, may be subject to a time 
 * 				constraint. This specification supports both minimum and maximum time constraints. The controlled 
 * 				time for a single item is simply the duration of item session as defined by duration.
 * 	
 * Attributes :  * minTime 
 * 				 * maxTime
 */



public class QTIv21timeLimits 
{   
    protected QTIv2structuredBodyPart prompt;
    protected QTIv2variableDeclaration var;
    QTIv21test theTest = null;
    QTIv21testPart theTestPart = null;
    QTIv21sectionPart theSectionPart = null;
    
    //--  Attributes
    protected String minTime;		// minTime [0..1]: duration
    protected String maxTime;		// maxTime [0..1]: duration 	 	
    
    public QTIv21timeLimits(QTIv21test owner)
    {
    	theTest = owner;
    	maxTime = "";
    	minTime = "";
    }
    
    public QTIv21timeLimits(QTIv21testPart owner)
    {
    	theTestPart = owner;
    	maxTime = "";
    	minTime = "";
    }
    
    public QTIv21timeLimits(QTIv21sectionPart owner)
    {
    	theSectionPart = owner;
    	maxTime = "";
    	minTime = "";
    }
    
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {     
    	
    	//ATTRIBUTES
    	if (itemXML.getAttribute(idx,"minTime")!=null)
        {
        	minTime=itemXML.getAttribute(idx,"minTime");
        	
            //TODO
        }
        if (itemXML.getAttribute(idx,"maxTime")!=null)
        {
        	maxTime=itemXML.getAttribute(idx,"maxTime");
	       //reading format PT4M
	        int iMaxTime = Integer.parseInt(maxTime.substring(2,maxTime.length()-1));
	        
	        String tempsType = maxTime.substring((maxTime.length()-1),maxTime.length());
	        
	        if(tempsType.equals("M"))
	        	iMaxTime *=60;
	        if(tempsType.equals("H"))
	        	iMaxTime *=(60*60);
	       
	        if(theTest !=null)
	        	new Reminder(iMaxTime,theTest);
	        else if (theTestPart != null)
	        	new Reminder(iMaxTime,theTestPart);
	        else if (theSectionPart != null)
	        	new Reminder(iMaxTime,theSectionPart);
        }
        return itemXML.findNextPeer(idx);
    }

    public String render()
    {
        String out = "";
        return out;
    }
}

/*
http://java.sun.com/docs/books/tutorial/essential/threads/timer.html
* Simple demo that uses java.util.Timer to schedule a task 
* to execute once 5 seconds have passed.
*/

class Reminder{
    Timer timer;
    QTIv21test theTest = null;
    QTIv21testPart theTestPart = null;
    QTIv21sectionPart theSectionPart = null;
    
    public Reminder(int seconds, QTIv21test owner) {
        theTest = owner ;
    	timer = new Timer();
        timer.schedule(new RemindTask(),seconds*1000);
	}
    
    public Reminder(int seconds, QTIv21testPart owner) {
        theTestPart = owner;
    	timer = new Timer();
        timer.schedule(new RemindTask(),seconds*1000);
	}
   
    public Reminder(int seconds, QTIv21sectionPart owner) {
        theSectionPart = owner;
    	timer = new Timer();
        timer.schedule(new RemindTask(),seconds*1000);
	}
    
    class RemindTask extends TimerTask {
        public void run() {
        	
        if(theTest !=null)
        {	//System.out.println("Test time's up!%n");
            theTest.continueTest=false;
            theTest.finishingReason = "<h4>Maximum time's up!</h4>";
            timer.cancel(); //Terminate the timer thread
        }else if(theTestPart != null)
        {
    		//System.out.println("TestPart time's up!%n");
            theTestPart.endPart = true;
            theTestPart.finishingReason = "<h4>Maximum time's up!</h4>";
            timer.cancel(); //Terminate the timer thread
        }else if(theSectionPart != null)
        {   
        	//System.out.println("sectionPart time's up!%n");
        	theSectionPart.endSection = true;
        	theSectionPart.finishingReason = "<h4>Maximum time's up!</h4>";
        	timer.cancel(); //Terminate the timer thread
        }   
      }
    }
}
