/**
 * Class : weight
 * 
 * Description : The contribution of an individual item score to an overall test score typically varies from test to test. 
 * 				The score of the item is said to be weighted. Weights are defined as part of each reference to an item 
 * 				(assessmentItemRef) within a test.
 * 
 * Attributes :	* identifier 
 * 				An item can have any number of weights, each one is given an identifier that is used to refer to the weight 
 * 				in outcomes processing. (See the variable and testVariables definitions.) Weights are always floating point 
 * 				values. 
 * */


package apis;

import nb_utils.NBEasyXML;

public class QTIv21weight {

	//-- Attributes specification 
	protected String identifier;	//identifier [1]: identifier 
	
	//-- Other attributes
	protected float value;			// weight value 
	
	public QTIv21weight ()
	{			
			identifier = " ";
			value = -1 ;
	}
	
	 public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
	 {
		   //ATTRIBUTES
	        if(itemXML.getAttribute(idx,"identifier")!= null)
	        {
	        	identifier=itemXML.getAttribute(idx,"identifier");
	        }else 
	        	throw new InvalidQTIv2Exception("identifer needed");
	        
	        if(itemXML.getContent(idx)!=null)
	        	value = Float.parseFloat(itemXML.getContent(idx));
	        else
	        	throw new InvalidQTIv2Exception("weight value needed");
	         
		 return itemXML.findNextPeer(idx);
	 }
	
	 public float getValue()
	 {
		 return value;
	 }
	
}
