package apis;

import nb_utils.*;

import java.util.Random;
import java.util.Vector;

/**
 * Title: Associate Interaction
 * Class : associateInteraction (blockInteraction)
 * Description: blockInteraction that presents candidates a number of choices and allows them to create associations 
 * 				between them.
 * Attributes:  * shuffle 
 *			    * maxAssociations 
 *
 * Contains :   * simpleAssociableChoice 
 *
**/

public class QTIv2associateInteraction extends QTIv2structuredBodyPart implements QTIv2responseVariableUpdate
{
    protected QTIv2structuredBodyPart prompt;
    protected QTIv2variableDeclaration var;
    protected Vector formNames;			 // contains the form field names from the simpleAssociableChoice class;
    protected String shuffleName; 		 // variable name for persistance data.
    //-- Interaction Attributes
    protected boolean shuffle; 
    protected int maxAssociations; 
    
    public QTIv2associateInteraction(QTIv2item owner)
    {
        super(owner);
        owner.registerVarUpdate(this);
        shuffle = false;
        maxAssociations=1;
    }

    // <associateInteraction responseIdentifier="RESPONSE" shuffle="true" maxAssociations="3"> 
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        formNames = new Vector();
        
        // shuffle[1]: boolean = false
        if(itemXML.getAttribute(idx,"shuffle").equals("true"))
            shuffle = true;
        
        // maxAssociations [1]: integer = 1    --> for the moment, we won't work with more than one answer 
        if(itemXML.getAttribute(idx,"maxAssociations")!=null ) 
        	maxAssociations = Integer.parseInt(itemXML.getAttribute(idx,"maxAssociations"));
        else 
        	throw new InvalidQTIv2Exception("associateInteraction does not have maxAssociation attribute specified.");
        
        
        var = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"responseIdentifier")));
        if(var==null)
            throw new InvalidQTIv2Exception("responseIdentifier "+itemXML.getAttribute(idx,"responseIdentifier")+" does not match a variable.");
       
        //-- parsing of associateInteraction content
        int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        {
            int id = QTIv2define.identify(itemXML.getName(cidx));
            //-- prompt
            if(id == QTIv2define.QTIv2_prompt)
            {
                if(prompt != null)
                    throw new InvalidQTIv2Exception("Multiple prompts in associateInteraction");
                if(parts != null)
                    throw new InvalidQTIv2Exception("prompt after simpleAssociableChoice in associateInteraction");
                
                prompt = new QTIv2structuredBodyPart(theItem);
                cidx = prompt.parse(itemXML, cidx);
                
                if(cidx==-1)
                    throw new InvalidQTIv2Exception("No simpleChoice following prompt choiceInteraction");
            }
            //-- simpleAssociableChoice [1..*]
            else if(id == QTIv2define.QTIv2_simpleAssociableChoice)
            {
                if(parts == null)
                    parts = new Vector();                
                QTIv2simpleAssociableChoice ch = new QTIv2simpleAssociableChoice(theItem, itemXML.getAttribute(cidx,"identifier"), var);
                cidx = ch.parse(itemXML, cidx);
                
                if(parts.size()==0)
                { 
                	for(int i=0;i<maxAssociations;i++)     
                		formNames.add(ch.nm+"_"+i);
                }
                parts.add(ch);
            }
           else
              throw new InvalidQTIv2Exception("Invalid child element in choiceInteraction [" + String.valueOf(id) + "]");
        }
        return itemXML.findNextPeer(idx);
    }

    public String render() throws InvalidQTIv2Exception
    {
    	if(shuffle==true)
        	doShuffle(parts, new Random());
        
        String out = "";
        
        if (prompt != null)
            out += prompt.render();
        
         
        //-- Table containing the selectable elements and lists to choose them in pairs (baseType). 
        out+="<br>" +
        		"<table width =\"90%\" align =\"center\" cellpadding=\"50\">" +
        			"<tr><td> <table align=\"center\">" +
        						"<tr><td><br><var><b> Options: </b></var></br></td></tr>" +
        						"<tr><td><ul>";
        
        //-- Selectable Options List 
        String optionList=""; 
        
        //-- Default null option to cope with less responses than the maximum allowed
        optionList+="<option name=\"apisa_RESPONSE\" value=\"NULL\" selected=\"selected\"> - </option>";
	  
	    for(int n=0; n < parts.size(); n++)
	    {
	        //-- Print option List
	    	out +="<li>"+ ((QTIv2simpleAssociableChoice)parts.get(n)).render()+"</li>";
	
	        	//-- Filling the options of select -> name: apisa_RESPONSE_D value:D
	        optionList += "<option name='" + ((QTIv2simpleAssociableChoice)parts.get(n)).nm+"' value='" + ((QTIv2simpleAssociableChoice)parts.get(n)).identifier +"'>";
	        optionList += ((QTIv2simpleAssociableChoice)parts.get(n)).render()+"</option>";
	    }
	    
	    optionList+="</select>";
	       
	        //-- Second table
	    out+="</ul></td></tr></table>" +
	     	"</td><td><table\">" +
    		"<tr><th colspan=\"3\">Pair selections</th></tr>" +
    		"<tr><td> num </td><td> <b>Source </b></td><td><b> Destination </b></td></tr> ";

        //-- As much possible responses as the indicated in maxAssociations Attribute.
        for(int i=0;i<maxAssociations;i++)
        	out+="<tr><td>"+(i+1)+"</td><td><select name=\"apisa_RESPONSE_"+i+"-A\">"+optionList+"</td><td><select name=\"apisa_RESPONSE_"+i+"-B\">"+optionList + "</td></tr>";
        
        out+="</table></td></tr></table></br>";
        return out;        
    }

    public boolean updateFromFormData()
    {
    	for (int n = 0; n < formNames.size(); n++)
        {
            String key = (String) formNames.get(n);
            if (theItem.formData.containsKey(key))
            	try
                {
            		if(!((String) theItem.formData.get(key)).equals("NULL NULL"))
                    var.setValue((String) theItem.formData.get(key));
                }
                catch (InvalidQTIv2Exception ex)
                {
                }
        }
        return false;
     } 
  }
