package apis;

import nb_utils.*;
import java.util.Vector;

/**
 * <p>Title: APIS test wrapper</p>
 * <p>Description: A very very simple HTTP server</p>
 * <p>Copyright: Copyright (c) 2004, University of Strathclyde</p>
 * <p>Company: University of Strathclyde/JISC</p>
 * @author not attributable
 * @version 1.0
 */

public abstract class QTIv2bodyPart
{
    QTIv2item theItem; // used to find response variables etc.

    abstract public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception;
    abstract public int parseLikert(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception;
    abstract public String render() throws InvalidQTIv2Exception;

    QTIv2bodyPart(QTIv2item owner)
    {
        theItem = owner;
    }

    protected String getSafeFormID(String id)
    {
        String out = "apis" + theItem.embedPrefix + "_" + getSafeIdent(id);
        return out;
    }

    protected String getSafeIdent(String id)
    {
        String out = id;
        //# some simple encryption needed, ideally one way, but resulting in unique output...
        return out;
    }

}

class XHTMLPart extends QTIv2bodyPart
{
    String content;

    XHTMLPart(String nContent, QTIv2item owner)
    {
        super(owner);
        content = nContent;
    }

    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        //assert(false);
        throw new InvalidQTIv2Exception("XHTMLPart::parse should never be called.");
        //return idx+1;
    }

    public int parseLikert(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        //assert(false);
        throw new InvalidQTIv2Exception("XHTMLPart::parse should never be called.");
        //return idx+1;
    }
    
    public String render() throws InvalidQTIv2Exception
    {
        return content;
    }
}

class InfoPart extends QTIv2bodyPart
{
    String content;

    InfoPart(QTIv2item owner)
    {
        super(owner);
        content = "<p>ERROR - InfoPart not parsed yet, and it's only a debugging placeholder class anyway!</p>";
    }

    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        content = "<p>QTI section <b>" + itemXML.getName(idx) + "</b></p>";
        int next = itemXML.findEnd(idx)+1;
        return next;
    }

    public int parseLikert(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        content = "<p>QTI section <b>" + itemXML.getName(idx) + "</b></p>";
        int next = itemXML.findEnd(idx)+1;
        return next;
    }
    
    public String render()
    {
        return content;
    }
}

class TextPart extends QTIv2bodyPart
{
    String content;

    TextPart(QTIv2item owner)
    {
        super(owner);
        content = "<p>ERROR - InfoPart not parsed yet, and it's only a debugging placeholder class anyway!</p>";
    }

    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        content = "";
        while(itemXML.getType(idx)==4)
        {
            content += itemXML.ExportBlock(idx);
            idx++;
        }
        return idx;
    }
    
    public int parseLikert(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        content = "";
        while(itemXML.getType(idx)==4)
        {
            content += itemXML.ExportBlock(idx);
            idx++;
        }
        return idx;
    }

    public String render()
    {
        return content;
    }
}

