package apis;

import nb_utils.*;
import java.util.*;

/**
 * <p>Title: APIS test wrapper</p>
 * <p>Description: A very very simple HTTP server</p>
 * <p>Copyright: Copyright (c) 2004, University of Strathclyde</p>
 * <p>Company: University of Strathclyde/JISC</p>
 * @author Niall Barr
 * @version 1.0
 */

/**
 * Title: choice Interaction
 * Class : choiceInteraction (blockInteraction)
 * Description : The choice interaction presents a set of choices to the candidate. The candidate's task is 
 * 				 to select one or more of the choices, up to a maximum of maxChoices. There is no corresponding 
 * 			 	 minimum number of choices. The interaction is always initialized with no choices selected.  
 * 				 The choiceInteraction must be bound to a responseVariable with a baseType of identifier and single 
 * 				 or multiple cardinality. 			
 * Attributes:  * shuffle:  If the shuffle attribute is true then the delivery engine must randomize the order in which 
 * 							the choices are presented subject to the fixed  attribute.
 *				* maxChoices: The maximum number of choices that the candidate is allowed to select. 
 *							 If maxChoices is 0 then there is no restriction. If maxChoices is greater than 1 (or 0)
 *							 then the interaction must be bound to a response with multiple cardinality.
 * Contains :   * simpleChoice: An ordered list of the choices that are displayed to the user. The order is the order 
 * 								of the choices presented to the user unless shuffle is true.
**/

public class QTIv2choiceInteraction extends QTIv2structuredBodyPart implements QTIv2responseVariableUpdate
{
    protected QTIv2structuredBodyPart prompt;
    protected Vector formNames; // contains the form field names from the simpleChoice class;
    protected QTIv2variableDeclaration var;
    protected String shuffleName; // variable name for persistance data.
    protected String nm;
    //-- Attributes
    protected boolean shuffle;  					//-- shuffle [1]: boolean = false
    protected int maxChoices;   					//-- maxChoices [1]: integer = 1

    public QTIv2choiceInteraction(QTIv2item owner)
    {
        super(owner);
       
        owner.registerVarUpdate(this);
        shuffle = false;
    }

    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        formNames = new Vector();
 
    //-- ATTRIBUTES
        //-- shuffle
        if(itemXML.getAttribute(idx,"shuffle").equals("true"))
        {
            shuffle = true;
        }

        //-- maxChoices 
        if(itemXML.getAttribute(idx,"maxChoices")!= null)
        {
            maxChoices= Integer.parseInt(itemXML.getAttribute(idx,"maxChoices"));
            //maxChoices= 1; // de moment nomes tractem amb singulars
        }else
        	throw new InvalidQTIv2Exception("maxChoices in ChoiceInteraction not specified");

        //-- Response Variable 
        var = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"responseIdentifier")));
       
        if(var==null)
            throw new InvalidQTIv2Exception("responseIdentifier "+itemXML.getAttribute(idx,"responseIdentifier")+" does not match a variable.");

        //-- Content Parsing 
        int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        {
            int id = QTIv2define.identify(itemXML.getName(cidx));
            //-- check for a prompt
            if(id == QTIv2define.QTIv2_prompt)
            {
                if(prompt != null)
                    throw new InvalidQTIv2Exception("Multiple prompts in choiceInteraction");
                if(parts != null)
                    throw new InvalidQTIv2Exception("prompt after simpleChoice in choiceInteraction");
                prompt = new QTIv2structuredBodyPart(theItem);
                cidx = prompt.parse(itemXML, cidx);
                if(cidx==-1)
                    throw new InvalidQTIv2Exception("No simpleChoice following prompt choiceInteraction");
            }
            //-- check simpleChoices
            else if(id == QTIv2define.QTIv2_simpleChoice)
            {
                if(parts == null)
                    parts = new Vector();
                QTIv2simpleChoice ch = new QTIv2simpleChoice(theItem, itemXML.getAttribute(cidx,"identifier"), var);
                cidx = ch.parse(itemXML, cidx);                
                parts.add(ch);
                nm=ch.nm;
            }
           else
              throw new InvalidQTIv2Exception("Invalid child element in choiceInteraction [" + String.valueOf(id) + "]");

            
        }
        
        if(maxChoices==0)
        	maxChoices = parts.size(); 
        for(int i=0;i<maxChoices;i++)     
        	 formNames.add(nm+"_"+i);


        return itemXML.findNextPeer(idx);
    }
    
    public int parseLikert(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        formNames = new Vector();
 
    //-- ATTRIBUTES
        //-- shuffle
        if(itemXML.getAttribute(idx,"shuffle").equals("true"))
        {
            shuffle = true;
        }

        //-- maxChoices 
        if(itemXML.getAttribute(idx,"maxChoices")!= null)
        {
            maxChoices= Integer.parseInt(itemXML.getAttribute(idx,"maxChoices"));
            //maxChoices= 1; // de moment nomes tractem amb singulars
        }else
        	throw new InvalidQTIv2Exception("maxChoices in ChoiceInteraction not specified");

        //-- Response Variable 
        var = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"responseIdentifier")));
       
        if(var==null)
            throw new InvalidQTIv2Exception("responseIdentifier "+itemXML.getAttribute(idx,"responseIdentifier")+" does not match a variable.");

        //-- Content Parsing 
        int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        {
            int id = QTIv2define.identify(itemXML.getName(cidx));
            //-- check for a prompt
            if(id == QTIv2define.QTIv2_prompt)
            {
                if(prompt != null)
                    throw new InvalidQTIv2Exception("Multiple prompts in choiceInteraction");
                if(parts != null)
                    throw new InvalidQTIv2Exception("prompt after simpleChoice in choiceInteraction");
                prompt = new QTIv2structuredBodyPart(theItem);
                cidx = prompt.parse(itemXML, cidx);
                if(cidx==-1)
                    throw new InvalidQTIv2Exception("No simpleChoice following prompt choiceInteraction");
            }
            //-- check simpleChoices
            else if(id == QTIv2define.QTIv2_simpleChoice)
            {
                if(parts == null)
                    parts = new Vector();
                QTIv2simpleChoice ch = new QTIv2simpleChoice(theItem, itemXML.getAttribute(cidx,"identifier"), var);
                cidx = ch.parseLikert(itemXML, cidx);                
                parts.add(ch);
                nm=ch.nm;
            }
           else
              throw new InvalidQTIv2Exception("Invalid child element in choiceInteraction [" + String.valueOf(id) + "]");

            
        }
        
        if(maxChoices==0)
        	maxChoices = parts.size(); 
        for(int i=0;i<maxChoices;i++)     
        	 formNames.add(nm+"_"+i);


        return itemXML.findNextPeer(idx);
    }

    public String render() throws InvalidQTIv2Exception
    {
        if(shuffle==true)
        {
        	doShuffle(parts, new Random());
            //# if this is not the first rendering, need to read order from post data,
            //# otherwise generate order.
            //# has to be done in render so the form data is available.
        }
        String out = "";
        
        //-- print prompt
        if (prompt != null)
            out += prompt.render();
        out += "<br />";
        	
        //-- render simpleChoices
        for(int n=0; n < parts.size(); n++)
            out += ((QTIv2simpleChoice)parts.get(n)).render(n);
        
        out += "<br /><br />";
        return out;
    }

    public boolean updateFromFormData()
    {
    	for (int n = 0; n < formNames.size(); n++)
        {
            String key = (String) formNames.get(n);
            if (theItem.formData.containsKey(key))
            {    try
                	{	 var.setValue((String) theItem.formData.get(key));	}
                catch (InvalidQTIv2Exception ex){}
            }
        }
        return false;
    }
}
