package apis;

import nb_utils.*;
import apis.expression.*;

/**
 * <p>Title: APIS test wrapper</p>
 * <p>Description: A very very simple HTTP server</p>
 * <p>Copyright: Copyright (c) 2004, University of Strathclyde</p>
 * <p>Company: University of Strathclyde/JISC</p>
 * @author not attributable
 * @version 1.0
 */

abstract public class QTIv2expression
{
    protected QTIv21test theTest;    
    protected QTIv2item theItem;


    public QTIv2expression(QTIv2item owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        theItem = owner;
        if(!parse(itemXML, idx))
            throw new InvalidQTIv2Exception("Failed to parse "+itemXML.getName(idx)+", possibly not yet implemented.");
    }    

    
    public QTIv2expression(QTIv21test owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
    	theItem = null;
        theTest = owner;
        if(!parse(itemXML, idx))
            throw new InvalidQTIv2Exception("Failed to parse "+itemXML.getName(idx)+", possibly not yet implemented.");
    }

    abstract public QTIv2variableDeclaration resolve() throws InvalidQTIv2Exception;
    abstract protected boolean parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception;

    public static QTIv2expression parseNew(QTIv2item owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        int id = QTIv2define.identify(itemXML.getName(idx));
        switch (id)
        {
            case QTIv2define.QTIv2_and:
                return new QTIv2Expr_and(owner, itemXML, idx);
            case QTIv2define.QTIv2_anyN:
                return new QTIv2Expr_anyN(owner, itemXML, idx);
            case QTIv2define.QTIv2_baseValue:
                return new QTIv2Expr_baseValue(owner, itemXML, idx);
            case QTIv2define.QTIv2_contains:
                return new QTIv2Expr_contains(owner, itemXML, idx);
            case QTIv2define.QTIv2_correct:
                return new QTIv2Expr_correct(owner, itemXML, idx);
            case QTIv2define.QTIv2_default:
                return new QTIv2Expr_default(owner, itemXML, idx);
            case QTIv2define.QTIv2_delete:
                return new QTIv2Expr_delete(owner, itemXML, idx);
            case QTIv2define.QTIv2_divide:
                return new QTIv2Expr_divide(owner, itemXML, idx);
            case QTIv2define.QTIv2_durationGTE:
                return new QTIv2Expr_durationGTE(owner, itemXML, idx);
            case QTIv2define.QTIv2_durationLT:
                return new QTIv2Expr_durationLT(owner, itemXML, idx);
            case QTIv2define.QTIv2_empty:
                return new QTIv2Expr_empty(owner, itemXML, idx);
            case QTIv2define.QTIv2_equal:
                return new QTIv2Expr_equal(owner, itemXML, idx);
            case QTIv2define.QTIv2_gt:
                return new QTIv2Expr_gt(owner, itemXML, idx);
            case QTIv2define.QTIv2_gte:
                return new QTIv2Expr_gte(owner, itemXML, idx);
            case QTIv2define.QTIv2_index:
                return new QTIv2Expr_index(owner, itemXML, idx);
            case QTIv2define.QTIv2_integerDivide:
                return new QTIv2Expr_integerDivide(owner, itemXML, idx);
            case QTIv2define.QTIv2_integerModulus:
                return new QTIv2Expr_integerModulus(owner, itemXML, idx);
            case QTIv2define.QTIv2_isNull:
                return new QTIv2Expr_isNull(owner, itemXML, idx);
            case QTIv2define.QTIv2_lt:
                return new QTIv2Expr_lt(owner, itemXML, idx);
            case QTIv2define.QTIv2_lte:
                return new QTIv2Expr_lte(owner, itemXML, idx);
            case QTIv2define.QTIv2_mapResponse:
                return new QTIv2Expr_mapResponse(owner, itemXML, idx);
            case QTIv2define.QTIv2_mapResponsePoint:
                return new QTIv2Expr_mapResponsePoint(owner, itemXML, idx);
            case QTIv2define.QTIv2_match:
                return new QTIv2Expr_match(owner, itemXML, idx);
            case QTIv2define.QTIv2_member:
                return new QTIv2Expr_member(owner, itemXML, idx);
            case QTIv2define.QTIv2_multiple:
                return new QTIv2Expr_multiple(owner, itemXML, idx);
            case QTIv2define.QTIv2_not:
                return new QTIv2Expr_not(owner, itemXML, idx);
            case QTIv2define.QTIv2_null:
                return new QTIv2Expr_null(owner, itemXML, idx);
            case QTIv2define.QTIv2_or:
                return new QTIv2Expr_or(owner, itemXML, idx);
            case QTIv2define.QTIv2_ordered:
                return new QTIv2Expr_ordered(owner, itemXML, idx);
            case QTIv2define.QTIv2_patternMatch:
                return new QTIv2Expr_patternMatch(owner, itemXML, idx);
            case QTIv2define.QTIv2_power:
                return new QTIv2Expr_power(owner, itemXML, idx);
            case QTIv2define.QTIv2_product:
                return new QTIv2Expr_product(owner, itemXML, idx);
            case QTIv2define.QTIv2_random:
                return new QTIv2Expr_random(owner, itemXML, idx);
            case QTIv2define.QTIv2_randomFloat:
                return new QTIv2Expr_randomFloat(owner, itemXML, idx);
            case QTIv2define.QTIv2_randomInteger:
                return new QTIv2Expr_randomInteger(owner, itemXML, idx);
            case QTIv2define.QTIv2_stringMatch:
                return new QTIv2Expr_stringMatch(owner, itemXML, idx);
            case QTIv2define.QTIv2_substring:
                return new QTIv2Expr_substring(owner, itemXML, idx);
            case QTIv2define.QTIv2_subtract:
                return new QTIv2Expr_subtract(owner, itemXML, idx);
            case QTIv2define.QTIv2_sum:
                return new QTIv2Expr_sum(owner, itemXML, idx);
            case QTIv2define.QTIv2_variable:
                return new QTIv2Expr_variable(owner, itemXML, idx);
            default:
                throw new InvalidQTIv2Exception("invalid expression, or not yet implemented expression");
        }
    }
    
    public static QTIv2expression parseNew(QTIv21test owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        int id = QTIv2define.identify(itemXML.getName(idx));
        switch (id)
        {
            case QTIv2define.QTIv2_and:
                return new QTIv2Expr_and(owner, itemXML, idx);
            case QTIv2define.QTIv2_anyN:
                return new QTIv2Expr_anyN(owner, itemXML, idx);
            case QTIv2define.QTIv2_baseValue:
                return new QTIv2Expr_baseValue(owner, itemXML, idx);
            case QTIv2define.QTIv2_contains:
                return new QTIv2Expr_contains(owner, itemXML, idx);
            case QTIv2define.QTIv2_correct:
                return new QTIv2Expr_correct(owner, itemXML, idx);
            case QTIv2define.QTIv2_default:
                return new QTIv2Expr_default(owner, itemXML, idx);
            case QTIv2define.QTIv2_delete:
                return new QTIv2Expr_delete(owner, itemXML, idx);
            case QTIv2define.QTIv2_divide:
                return new QTIv2Expr_divide(owner, itemXML, idx);
            case QTIv2define.QTIv2_durationGTE:
                return new QTIv2Expr_durationGTE(owner, itemXML, idx);
            case QTIv2define.QTIv2_durationLT:
                return new QTIv2Expr_durationLT(owner, itemXML, idx);
            case QTIv2define.QTIv2_empty:
                return new QTIv2Expr_empty(owner, itemXML, idx);
            case QTIv2define.QTIv2_equal:
                return new QTIv2Expr_equal(owner, itemXML, idx);
            case QTIv2define.QTIv2_gt:
                return new QTIv2Expr_gt(owner, itemXML, idx);
            case QTIv2define.QTIv2_gte:
                return new QTIv2Expr_gte(owner, itemXML, idx);
            case QTIv2define.QTIv2_index:
                return new QTIv2Expr_index(owner, itemXML, idx);
            case QTIv2define.QTIv2_integerDivide:
                return new QTIv2Expr_integerDivide(owner, itemXML, idx);
            case QTIv2define.QTIv2_integerModulus:
                return new QTIv2Expr_integerModulus(owner, itemXML, idx);
            case QTIv2define.QTIv2_isNull:
                return new QTIv2Expr_isNull(owner, itemXML, idx);
            case QTIv2define.QTIv2_lt:
                return new QTIv2Expr_lt(owner, itemXML, idx);
            case QTIv2define.QTIv2_lte:
                return new QTIv2Expr_lte(owner, itemXML, idx);
            case QTIv2define.QTIv2_mapResponse:
                return new QTIv2Expr_mapResponse(owner, itemXML, idx);
            case QTIv2define.QTIv2_mapResponsePoint:
                return new QTIv2Expr_mapResponsePoint(owner, itemXML, idx);
            case QTIv2define.QTIv2_match:
                return new QTIv2Expr_match(owner, itemXML, idx);
            case QTIv2define.QTIv2_member:
                return new QTIv2Expr_member(owner, itemXML, idx);
            case QTIv2define.QTIv2_multiple:
                return new QTIv2Expr_multiple(owner, itemXML, idx);
            case QTIv2define.QTIv2_not:
                return new QTIv2Expr_not(owner, itemXML, idx);
            case QTIv2define.QTIv2_null:
                return new QTIv2Expr_null(owner, itemXML, idx);
            case QTIv2define.QTIv21_numberCorrect:
                return new QTIv21Expr_numberCorrect(owner, itemXML, idx);
            case QTIv2define.QTIv21_numberIncorrect:
                return new QTIv21Expr_numberIncorrect(owner, itemXML, idx);
            case QTIv2define.QTIv21_numberPresented:
                return new QTIv21Expr_numberPresented(owner, itemXML, idx);
            case QTIv2define.QTIv21_numberResponded:
                return new QTIv21Expr_numberResponded(owner, itemXML, idx);
            case QTIv2define.QTIv21_numberSelected:
                return new QTIv21Expr_numberSelected(owner, itemXML, idx);
            case QTIv2define.QTIv2_or:
                return new QTIv2Expr_or(owner, itemXML, idx);
            case QTIv2define.QTIv2_ordered:
                return new QTIv2Expr_ordered(owner, itemXML, idx);
            case QTIv2define.QTIv2_patternMatch:
                return new QTIv2Expr_patternMatch(owner, itemXML, idx);
            case QTIv2define.QTIv2_power:
                return new QTIv2Expr_power(owner, itemXML, idx);
            case QTIv2define.QTIv2_product:
                return new QTIv2Expr_product(owner, itemXML, idx);
            case QTIv2define.QTIv2_random:
                return new QTIv2Expr_random(owner, itemXML, idx);
            case QTIv2define.QTIv2_randomFloat:
                return new QTIv2Expr_randomFloat(owner, itemXML, idx);
            case QTIv2define.QTIv2_randomInteger:
                return new QTIv2Expr_randomInteger(owner, itemXML, idx);
            case QTIv2define.QTIv2_stringMatch:
                return new QTIv2Expr_stringMatch(owner, itemXML, idx);
            case QTIv2define.QTIv2_substring:
                return new QTIv2Expr_substring(owner, itemXML, idx);
            case QTIv2define.QTIv2_subtract:
                return new QTIv2Expr_subtract(owner, itemXML, idx);
            case QTIv2define.QTIv2_sum:
                return new QTIv2Expr_sum(owner, itemXML, idx);
            case QTIv2define.QTIv21_testVariables:
                return new QTIv21Expr_testVariables(owner, itemXML, idx);
            case QTIv2define.QTIv2_variable:
                return new QTIv2Expr_variable(owner, itemXML, idx);
            case QTIv2define.QTIv2_integerToFloat:
                return new QTIv2Expr_integerToFloat(owner, itemXML, idx);
            default:
                throw new InvalidQTIv2Exception("invalid expression, or not yet implemented expression: "+ itemXML.getName(idx));
        }
    }    
}
