 package apis;

import java.sql.SQLException;
import java.util.*;

import nb_utils.*;
import java.io.*;
import java.net.URLDecoder;

import apis.UserTestCase.ItemStruct;



/**
 * Class : assessmentItem
 * 
 * Description : 
 * 
 * Attributes :	* identifier 
 * 				The principle identifier of the item. This identifier must have a corresponding entry in 
 * 				the item's meta-data.
 * 				* title 
 * 				The title of an assessmentItem is intended to enable the item to be selected in situations 
 * 				where the full text of the itemBody is not available, for example when a candidate is 
 * 				browsing a set of items to determine the order in which to attempt them.
 * 				* label [0..1]: string256
 * 				* lang [0..1]: language
 *  			* adaptive 
 *  			Items are classified into Adaptive Items and Non-adaptive Items.
 *  			* timeDependent 
 *  			* toolName [0..1]: string256
 *  			The tool name attribute allows the tool creating the item to identify itself. 
 *  			* toolVersion [0..1]: string256 
 *  			The tool version attribute allows the tool creating the item to identify its version. 
 *
 * Contains : 	* responseDeclaration [*]
 * 				* outcomeDeclaration [*]
 * 				* templateDeclaration [*]
 * 				* templateProcessing [0..1]
 * 				* stylesheet [0..*]
 * 				* itemBody [0..1]
 * 				* responseProcessing [0..1]
 * 				* modalFeedback [*]
 * */
public class QTIv2item extends QTIv21assessmentItemRef
{
	// Processing attributes
	protected Hashtable formData;				// responses managing
	protected Vector modalFeedback;				// feedbak managing
    protected Vector outcomeVars;				// outcoming variables  related to candidate's reponses
    protected Vector responseVars;				// candidate's reponses
    protected Vector varUpdates;				// variables to update from formData
    protected QTIv2responseActionCollection resproc;  	// mapping reponses to correct values 
    protected boolean  uncheckedData;			// true if there's new data to check (candidate reponses)
    protected boolean  process;					// true if this is the item to be checked, if there has been any change or answer for their interactions
    protected String weightIdentifier ;
    protected String resultInfo;
    
    protected NBEasyXML aItemXML;				// XML of the item that's being parsed
    protected QTIv2itemBody body;				// it will manage with the content of this item (interactions,...)
    protected String mediaBase = "", appletBase = "";  	// base directory to access media and applet objects
    protected String embedPrefix,parseErrors;   // to ensure unique names of item form vars. (Use of identifier could lead to too long names)
    protected Vector itemWeights;
    protected Vector itemVarMapping;			
    protected boolean answered; 
    protected boolean isCorrect;			//numberCorrect
    protected boolean isIncorrect ;			//numberIncorrect 
    protected String itemScore = "";
    
    //-- Attributes Item
    protected String title;						// title [1]: string  
    protected String label;						// label [0..1]: string256
    protected boolean adaptive;					// adaptive [1]: boolean = false 
    protected boolean timeDependent;			// timeDependent [1]: boolean
    
    protected UserTestCase user; //user context and responseprocessing container
    
    // -- constructor 
    public QTIv2item(QTIv21test owner)
    {
    	super(owner);
    	embedPrefix=owner.embedPrefix;
    	mediaBase = owner.getMediaBase();
    	appletBase = owner.getAppletBase();
    	title="untitled";
    	adaptive = false;
    	timeDependent = false;
    	process= false;
    	answered = false;
    	formData = new Hashtable (53);
    	itemWeights = new Vector();
    	itemVarMapping = new Vector();
    	answered = false;
    	isCorrect = false;
    	isIncorrect = false;
    	resultInfo="";
    }
    
    public QTIv2item(String nItemXML, String nEmbedPrefix) {
        super(nItemXML);
        embedPrefix = nEmbedPrefix;
        NBEasyXML aItemXML = new NBEasyXML(nItemXML);
        uncheckedData = false;
        formData = new Hashtable(53);
        mediaBase = "";
        appletBase = "";
        adaptive = false;
        process = false;
    	answered = false;
    	resultInfo="";
        // # at this point run through the XML looking at the immediate children of
        // # assessmentItem, and parsing or indexing these blocks as they are found
        try {
        	
          parse(aItemXML, 0);
        }
        catch (InvalidQTIv2Exception ex) {
          System.err.println(ex);
          ex.printStackTrace();
          if (parseErrors == null)
            parseErrors = "";
          parseErrors += ex + "<br/>";
        }
        catch (Exception ex) {
          System.err.println(ex);
          ex.printStackTrace();
        }
      }    
    
    public void setProcessedItems(UserTestCase processedItems) {
    //this picks information from userTestCase items and sets it to the item if exists

    	Vector vectprocessedItems = processedItems.getItems();
    	formData.clear();
    	user = processedItems;
        for (int i = 0 ; i<vectprocessedItems.size() ; i++)
        {
            if(((ItemStruct)vectprocessedItems.get(i)).item_id.equals(itemIdentifier))
        	{
        			itemScore = ""+((ItemStruct)vectprocessedItems.get(i)).score;
        			
        			int k=0;
        			for(;k<this.outcomeVars.size();k++){
        		    	if( ((QTIv2outcomeDeclaration)this.outcomeVars.get(k)).identifier.equals("SCORE") ){
        		    			try{
        		    				((QTIv2outcomeDeclaration)this.outcomeVars.get(k)).setValue(itemScore);
        		    			}catch(Exception e){
        		    				System.out.println("Error updating processed items from d");
        		    			}
        		    	}
        			}
        			
        			
        			break;
        	}
        } 
        vectprocessedItems = processedItems.getStoredItems();
        for (int i = 0 ; i<vectprocessedItems.size() ; i++)
        {
            if(((ItemStruct)vectprocessedItems.get(i)).item_id.equals(itemIdentifier))
        	{
        			answered= true;
        			
        			if(this.outcomeVars.size()==0)
        				return;
        			for(int j=0; j<  ((ItemStruct)vectprocessedItems.get(i)).answer_id.size();j++){
	        			String ans = (String) ((ItemStruct)vectprocessedItems.get(i)).answer_id.get(j);
	        			int k=0;
	        			for(;k<this.outcomeVars.size();k++){
	        		    	if( ((QTIv2outcomeDeclaration)this.outcomeVars.get(k)).identifier.equals("RESPONSE.FEEDBACK") ){
	        		    			try{
	        		    				((QTIv2outcomeDeclaration)this.outcomeVars.get(k)).setValue(ans);
	        		    			}catch(Exception e){
	        		    				System.out.println("Error updating processed items from d");
	        		    			}
	        		    	}
	        			}
        			}    			
        	}
        } 
    }
    
    // -- setItemsToProcess : marks this item to be processed if the itemIdentifier is equal to the
    // identifier from the answered item. 
    public void setItemsToProcess(String postStr)
    {
        if(!answered)
        {
	        String[] dt = postStr.split("&");
	        
	        for(int n=0; n<dt.length; n++)
	        {
	        	int eqp = dt[n].indexOf("=");
	            if(eqp>2)
	            {
	            	String name = dt[n].substring(0,eqp);
	                String value = dt[n].substring(eqp+1);
	        
	                if((name.equals("itemIdentifier"))&& value.equals(itemIdentifier)){
	                	process = true;
	                }
	            }
	        }
        }    
    }
    
    // -- addFormDataValue : updates data from this item 
    public void addFormDataValue(String name, String value)
    {
        if(formData == null)
            formData = new Hashtable(53); // 53 is about the right size, no particular reason for picking it.
       
        int i = name.indexOf("_");

        if((i!=-1)&&(process))
        {
     
            if (name.substring(0, i).equals("apis" + embedPrefix))
            {
            	//for some multiple cardinality variables              

  
            	if(formData.containsKey(name))
            	{
            		String valorAux = (String) formData.get(name);            			
            		valorAux = value+" "+valorAux;
            		formData.put(name, valorAux);
            		
            	//	this.setItemCloseAnswer(fs.getResponseId());
            		
           		 	uncheckedData = true;
            	}
            	else
            	{
            		formData.put(name, value);      
            		uncheckedData = true;
            	}
            }
        }
    }    
    
    // -- registerVarUpdate : response variables to update from formData
    protected void registerVarUpdate(QTIv2responseVariableUpdate u)
    {
        if(varUpdates==null)
            varUpdates = new Vector();
        varUpdates.add(u);
    }
    
    // -- parse : item parsing
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {   
    	outcomeVars = new Vector ();  
        varUpdates = new Vector();  

    	int cidx = itemXML.findChildElement(idx);
        
    	itemIdentifier= itemXML.getAttribute(idx,"identifier");
    	    	
    	while(cidx != -1)
        {
        	int id = QTIv2define.identify(itemXML.getName(cidx));
                switch (id)
                {
                    case QTIv2define.QTIv2_responseDeclaration: 
                    	QTIv2responseDeclaration resDec = new QTIv2responseDeclaration();
                    	cidx = resDec.parse(cidx, id, itemXML );
                    	if (responseVars == null)
          	               responseVars = new Vector();
          	            responseVars.add(resDec);
                    	break;                    
                    case QTIv2define.QTIv2_outcomeDeclaration:	
                    	QTIv2outcomeDeclaration outDec = new QTIv2outcomeDeclaration();
                    	cidx = outDec.parse(cidx, id, itemXML );
                    	if (outcomeVars == null)
          	               outcomeVars = new Vector();
          	            outcomeVars.add(outDec);
                    	break;                     
                    case QTIv2define.QTIv2_templateDeclaration: // hereda de variableDeclaration
                    case QTIv2define.QTIv2_templateProcessing:
                        throw new InvalidQTIv2Exception("Unsupported (not yet implemented) templateDeclaration or templateProcessing found.");
                    case QTIv2define.QTIv2_stylesheet:
                        throw new InvalidQTIv2Exception("Unsupported (not yet implemented) QTIv2_stylesheet");
                    case QTIv2define.QTIv2_itemBody:
                        body = new QTIv2itemBody(this);
                        cidx = body.parse(itemXML, cidx);
                        break;
                    case QTIv2define.QTIv2_responseProcessing:                    	
                    	int tempPos = itemXML.findChildElement(cidx);
                    	if(tempPos != -1)
                        {
                            resproc = new QTIv2responseActionCollection(this);
                            tempPos = resproc.parse(itemXML, tempPos);
                            cidx = itemXML.findNextPeer(cidx);
                        }
                        else
                        {
                            String templateName;
                            String templateURL = itemXML.getAttribute(cidx, "template");
                            
                            //# ideally here it should check this URL against a list of full localy held template ID urls
                            //# but, for now I'm going to just grab any template with the right name.
                            // Templates are a nasty, nasty fudge and the (valid) arguments put forward for them
                            // in the QTI working group only make sense if only the ones distributed with the
                            // specification ever exist.
                            int p = templateURL.lastIndexOf("/");
                            if (p == -1)
                                templateName = templateURL;
                            else
                                templateName = templateURL.substring(p + 1);
                            if(!loadResProcTemplate(templateName))
                                throw new InvalidQTIv2Exception(
                                      "External response processing template " + templateURL +
                                               " not found.");
                            cidx = itemXML.findNextPeer(cidx);
                        }
                        break;
                    case QTIv2define.QTIv2_modalFeedback:
                        if(modalFeedback==null)
                            modalFeedback = new Vector();
                        QTIv2feedbackSection tmpFbk = new QTIv2feedbackSection(this);
                        modalFeedback.add(tmpFbk);
                        cidx = tmpFbk.parse(itemXML, cidx);
                        break;
                    default:
                        throw new InvalidQTIv2Exception("Unexpected "+itemXML.getName(cidx) + " found.");
                }
        }
        return itemXML.findNextPeer(idx);
    }
    
    // -- loadResProcTemplate : given a template reference, it loads the XML content and processes it.
    protected boolean loadResProcTemplate(String templateName) throws InvalidQTIv2Exception
    {
        String rpString="";
        // load file into string (there must be a neater way of doing this!)
        try
        {
            File f = new File("rptemplates/" + templateName);
            if (f.exists() == false)
            {
                throw new InvalidQTIv2Exception("Template file "+f.getAbsolutePath()+" not found.");
            }
            FileReader fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            try
            {
                String in = br.readLine();
                while (in != null)
                {
                    rpString += in;
                    in = br.readLine();
                }
                br.close();
                fr.close();
            }
            catch (IOException ex1)
            {
                return false;
            }
        }
        catch (FileNotFoundException ex)
        {
            return false;
        }
        // set up for XML parsing
        NBEasyXML rpXML = new NBEasyXML(rpString);
        // find start of response processing
        int position = rpXML.root();
        if(position == -1)
            return false;
        // parse in in exactly the same way as in parseItem
        int tempPos = rpXML.findChildElement(position);
        if(tempPos != -1)
        {
            resproc = new QTIv2responseActionCollection(this);
            tempPos = resproc.parse(rpXML, tempPos);
        }
        else
            return false;
        // return true if it's all gone well
        return true;
    }
    
    // -- render 2.0 : item rendering
    public String renderItem( ) throws InvalidQTIv2Exception
    {
    	String out="";
    	
    	boolean noModalFdBk = true;
        
        if (body != null )
        {
        	if (uncheckedData)
            {
                if (varUpdates != null )
                {
                	if(process)
                	{
                		//resultInfo="<table>";
                		resultInfo="";
                		for (int n = 0; n < varUpdates.size(); n++)
                		{
	                        QTIv2responseVariableUpdate r = (QTIv2responseVariableUpdate) varUpdates.get(n); 
	                        r.updateFromFormData();                        
	
	                        QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) responseVars.get(n);
	                        /*[LAU] resultInfo += "<h4>Your response was :</h4> "+ tmp.identifier + " = ";
	                        for(int n2=0; n2<tmp.getSize(); n2++)
	                            try
	                            {  resultInfo += tmp.getStringValue(n2) + ", ";	}
	                            catch (InvalidQTIv2Exception ex)
	                            {  resultInfo += "[getStringValue failed], ";		}
	                        
	                            resultInfo += "<br/>\n";*/
                		}	                		
                		process = false;
                	}
                }        		
                uncheckedData = false;
                
                if (resproc != null )
                {	
                	noModalFdBk = false;
                    try
                    {	
                    	// reponse processing
                    	resproc.run();       
                    }
                    catch (InvalidQTIv2Exception ex)
                    {
                        System.err.println(ex);
                        ex.printStackTrace();
                        if (parseErrors == null)
                            parseErrors = "";
                        parseErrors += ex + "<br/>";
                    }
                }
                
                if (outcomeVars != null)
                { 
                	resultInfo+= "<h4> puntuacion conseguida :</h4>";//[LAU]resultInfo+= "<h4> and your Score is :</h4>";
        	        for (int n = 0; n < outcomeVars.size(); n++)
        	        {                	
        	        	QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) outcomeVars.get(n);             
                        try
                        {
                        	resultInfo += tmp.getStringValue(0);
                            if (Float.parseFloat(tmp.getStringValue(0))> 0)
                            	isCorrect = true;
                            else
                            	isIncorrect = true;
                        }
                        catch (InvalidQTIv2Exception ex)
                        {
                        }
                        resultInfo+= "</table><br/>\n";  
        	        }                	
                }
                
            }
        	
        	out = "<div>" ;
           	try
            {
                out += body.render();
            }
           	catch (Exception ex1) {
                out = "<div>" + ex1.getMessage() + "</div>";
                return out;
              }

        	//UPF - Pau
            // results / scoring rendering   
            // feedback managing
           	
            if((modalFeedback != null)&&(!noModalFdBk))
            {
                for(int n=0; n<modalFeedback.size(); n++)
                {
                	out += ((QTIv2feedbackSection)modalFeedback.get(n)).render();
                }
            }
                       	
           	out+=resultInfo;
        	out+="</div>";
            return out;
        }
        else  
        	return "Item failed to be parsed. (Body not found, invalid item or unsupported QTI";
    }     
    
    // -- render : item rendering 2.1
    public String render() throws InvalidQTIv2Exception
    {
    	String out="";
    	
    	boolean noModalFdBk = true;
    	boolean teest=varUpdates!=null;
    
    	if (body == null )
    		return "Item failed to be parsed. (Body not found, invalid item or unsupported QTI";

		
		
		if (uncheckedData)
        {
            //# update response vars from formData
            if (varUpdates != null )
            {
            	if(process){

            	for (int n = 0; n < varUpdates.size(); n++)
                {
                    QTIv2responseVariableUpdate r = (QTIv2responseVariableUpdate) varUpdates.get(n); 
                    r.updateFromFormData();

               //     QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) responseVars.get(n);

                }
            	this.storeItem();

            	
            	process = false; // if process==false answer == true
            	answered = true;
            	}
           }
            uncheckedData = false;
            
            if (resproc != null )
            {	
            	noModalFdBk = false;
                try
                {	
                	// reponse processing
                	resproc.run();   

                }
                catch (InvalidQTIv2Exception ex)
                {
                    System.err.println(ex);
                    ex.printStackTrace();
                    if (parseErrors == null)
                        parseErrors = "";
                    parseErrors += ex + "<br/>";
                }

                // results / scoring rendering
                if (outcomeVars != null)
                {
                	try {
                		if (weightIdentifier!=null)
                			this.applyWeight(weightIdentifier);
                		else if(theTest.weightIdentifier!=null) 
                			this.applyWeight(theTest.weightIdentifier);
                		} catch (InvalidQTIv2Exception e) {
						e.printStackTrace();
					}
                   /*[LAU] resultInfo += "<h4> and your Score is :</h4>";*/
                    itemScore = "0";
                    for (int n = 0; n < outcomeVars.size(); n++)
                    {
                        QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) outcomeVars.get(n);
                        
                        if (tmp.getIdentifier().matches("SCORE")){
                        	try{
                         		itemScore = tmp.getStringValue(0);
                              //[LAU]  resultInfo += tmp.getStringValue(0) + ", ";
                                if (Float.parseFloat(tmp.getStringValue(0))> 0)
                                 	isCorrect = true;
                                else
                                  	isIncorrect = true;
                            }
                            catch (InvalidQTIv2Exception ex)
                            {
                            }
                            //[LAU] resultInfo+= "<br/>\n";
                        }
                    }
   
                    this.setItemScore(itemScore);
	           // 	this.setItemCloseAnswer(fs.getResponseId());
	            	
                    out+=resultInfo;
                }
            }
        }
    	
    	
    	//[refactor] DAOResponseContext rc = new DAOResponseContext();
        DAOCloseAnswer daoCloseAnswer = new DAOCloseAnswer();
     
            // feedback managing
        String modalfeedback="";
        if(modalFeedback != null)//&&(!noModalFdBk))
        {
            for(int n=0; n<modalFeedback.size(); n++)
            {
            	if( ((QTIv2feedbackSection)modalFeedback.get(n)).render() != ""){
            		//out += ((QTIv2feedbackSection)modalFeedback.get(n)).getResponseId();
                	//out += " => ";
				    try{
			            
		            	QTIv2feedbackSection fs = (QTIv2feedbackSection)modalFeedback.get(n);
		            	//[refactor] out += rc.getContextFromId(fs.getResponseId());
		            	//[refactor] out += "tmp duplication";
		            
		            	//modalfeedback += "<b>"+daoCloseAnswer.getContentFromAnswerId(fs.getResponseId())+"<\b>";
		            	modalfeedback += "you said: "+daoCloseAnswer.getContentFromAnswerId(fs.getResponseId(),this.theTest.getUser());
			            
				    }catch( SQLException e ){
		            	System.out.println("unable to reach the context");
	            	}
		            modalfeedback += ":";
            	}
            	modalfeedback += "<b>"+((QTIv2feedbackSection)modalFeedback.get(n)).render()+"</b>";
            }
       }
       if(answered || process){
            out += renderBodyInGray(body.render(),modalfeedback); // Lau: render the item even if it's answered
    		return out;
    	}
    	
    	process = false; // if answered = false 

    	out += renderBodyInForm(body.render());
        return out; 
    }
 public String renderBodyInGray(String thebody,String modalfeedback){
    	
    	String out = "";
       	out+="<table class=\"answered\"><tr><td>\n";
       	out += thebody;
        out+= modalfeedback;
        out+="\n</td></tr>";
        out+="<tr><td>";
        out+="</td></tr></table>";   
        
        return out;
    
    }
    
    public String renderBodyInForm(String thebody){
    	
    	String out = "";
    	//-- item form 
    	//out+="<div class=\"whiteFiller\">";
    	out+="<form/><form method=\"post\" action=\"QTIPost.do?url="+theTest.getFile()+"&amp;identifier="+theTest.getId()+"\">"; 
        
    	  
    	out+="<form method='GET'>";
        out+="<input type='hidden' name='file' value='" + theTest.getFile()+"'>";
        out+="</input>";
        out+="<input type='hidden' name='identifier' value='" +
                      theTest.getId() + "'>";
        out+="</input>";
        out+="<input type='hidden' name='itemIdentifier' value='"+this.itemIdentifier+"'>"; 
        out+="</input>";
       	out+="<table class=\"unanswered\"><tr><td>\n";

        out += thebody;
       
        out+="\n</td></tr>";
        out+="<tr><td>";
        out+="<input type='submit' value='submit'>";
        out+="</input>";
        out+="</td></tr></table></form></form>";
        
        return out;
    
    }
    
    public boolean getIsCorrect ()
    {
    	try{
        if (Float.parseFloat(itemScore)> 0)
         	isCorrect = true;
        else
        	isCorrect = false;
    	}catch(Exception e){
    		return false;
    	}
    	return isCorrect;
    }
    public void setWeightIdentifier(String aWeightIdentifier)
	{
		weightIdentifier = aWeightIdentifier; 
	}

	public boolean getIsIncorrect ()
    {
    	try{
            if (Float.parseFloat(itemScore)> 0)
            	isIncorrect = false;
            else
            	isIncorrect = true;
        	}catch(Exception e){
        		return false;
        	}
        	return isIncorrect;
    }
    public void findRelatedVar(String varIdentifier, Vector relatedVarDec )
    {
    	if(outcomeVars!=null)
    	for(int i=0; i< outcomeVars.size();i++)
    	{	
    		if ((((QTIv2outcomeDeclaration)outcomeVars.get(i)).identifier).equals(varIdentifier))
    				relatedVarDec.add((QTIv2outcomeDeclaration)outcomeVars.get(i));
    	}
    }
    
    public boolean findItem(String itemIdentifier, Vector item)
    { 
    	if(itemIdentifier.equals(this.itemIdentifier))		
 		  {	  item.add(this);
 		  	  return true;
 		  }
 		  else
 		  return false;
    }
	public Collection getResponseVariables()
	{
		return responseVars;
	}

	protected QTIv2variableDeclaration getResponseVariable(String identifier)
	{
    //# ideally this should return responseVars.get(identifier), i.e. I should
    //# make a sensible vector derived class for variable lists.

    if(responseVars != null)
    {
        int n = 0;
        while(n < responseVars.size())
        {
            	QTIv2variableDeclaration var = (QTIv2variableDeclaration)responseVars.get(n);
    		    QTIv2responseDeclaration var2 = (QTIv2responseDeclaration)var;
    	      if(var.identifier.equals(identifier))
                    return var;
                n++;
            }
        }
        return null;
    }
    
    // -- getOutcomeVariable :  returns an specific outcomeDeclaration given its identifier.
    protected QTIv2variableDeclaration getOutcomeVariable(String identifier)
    {
        //# ideally this should return outcomeVars.get(identifier), i.e. I should
        //# make a sensible vector derived class for variable lists.
        if(outcomeVars != null)
        {
            int n = 0;
            while(n < outcomeVars.size())
            {
                QTIv2variableDeclaration var = (QTIv2variableDeclaration)outcomeVars.get(n);
                if(var.identifier.equals(identifier))
                    return var;
                n++;
            }
        }
        return null;
    }

    // -- setOutcomeVariable : adds a new outcome variable to the outcomeVars vector.
    protected boolean setOutcomeVariable(String identifier, QTIv2variableDeclaration newval) throws InvalidQTIv2Exception
    {
        //# ideally this should return outcomeVars.get(identifier), i.e. I should
        //# make a sensible vector derived class for variable lists.
    	if (outcomeVars != null)
        {
            int n = 0;
            while (n < outcomeVars.size())
            {
                QTIv2variableDeclaration var = (QTIv2variableDeclaration) outcomeVars.get(n);
                if (var.identifier.equals(identifier))
                {
                    var.setValue(newval);
                    return true;
                }
                n++;
            }
        }
        return false;
    }

    // -- getVariable : returns the variableDeclaration that matches with the given identifier.
    public QTIv2variableDeclaration getVariable(String identifier)
    {
    	identifier = identifier.trim();
        QTIv2variableDeclaration tmp = getResponseVariable(identifier);
        if (tmp == null)
            tmp = getOutcomeVariable(identifier);
        return tmp;
    }
    
    public String getRQPFormatOutcomes()
    {
        String out;
        if (outcomeVars != null)
        {
            out = "<outcomes>";
            for (int n = 0; n < outcomeVars.size(); n++)
            {
                QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) outcomeVars.get(n);
                out += "<RPQvar><identifier>" + tmp.identifier + "</identifier>";
                for (int n2 = 0; n2 < tmp.getSize(); n2++)
                    try
                    {
                        out += "<value>" + tmp.getStringValue(n2) + "</value>";
                    }
                    catch (InvalidQTIv2Exception ex)
                    {
                        out += "<value>[ERROR - "+ex.getMessage()+"]</value>";
                    }
                out += "</RPQvar>";
            }
            out += "</outcomes>";
       }
        else
            out = "<outcomes/>";
        return out;
    }
    
    // -- outcomeCount : returns the size of the Vector outcomeVars 
    public int outcomeCount()
    {
        if (outcomeVars == null)
            return 0;
        else
            return outcomeVars.size();
    }

    // -- getOutcomeIdentifier : returns the identifier of the variableDeclaration at position n
    public String getOutcomeIdentifier(int n)
    {
        if(n < outcomeVars.size())
        {
            QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) outcomeVars.get(n);
            return tmp.getIdentifier();
        }
        else
            return null;
    }
    
    
    public void applyWeight(String weightIdentifier) throws InvalidQTIv2Exception
	{
		   float weightedScore =0;
		   
		   if (itemWeights != null)
		   {
			   for(int i=0; i< itemWeights.size(); i++)
			   {
				   if(((QTIv21weight)itemWeights.get(i)).identifier.equals(weightIdentifier))
				   { 
					for(int j=0; j < outcomeVars.size(); j++)
	        		{        	
	        			weightedScore =((((QTIv2outcomeDeclaration)outcomeVars.get(j)).toFloat()).floatValue()) 
	        			* ((QTIv21weight)itemWeights.get(i)).getValue();
	        			
	        			((QTIv2outcomeDeclaration)outcomeVars.get(j)).setValue(weightedScore);
	        		}
				   }   
			   }
		   }
	}
	  	   
 
    public void doVarMappings()
    {	
    	if(itemVarMapping !=null )
    	{
    		for(int i=0; i< itemVarMapping.size(); i++)
        	{
    			String target =((QTIv21variableMapping)itemVarMapping.get(i)).targetIdentifier;
        		for(int j=0; j < outcomeVars.size(); j++)
        		{        		
        			if (((QTIv2outcomeDeclaration)outcomeVars.get(j)).identifier.equals(target))
        				((QTIv2outcomeDeclaration)outcomeVars.get(j)).identifier = ((QTIv21variableMapping)itemVarMapping.get(i)).sourceIdentifier;            		
        		}
        	}
        }    	
    }
    
    public QTIv21weight getWeights(String weightIdentifier)
    {
    	if(itemWeights != null)
    	for(int i=0; i< itemWeights.size(); i++)
    	{
    		if (((QTIv21weight)itemWeights.get(i)).identifier.equals(weightIdentifier))
    			return ((QTIv21weight)itemWeights.get(i));
    	}
    	return null;
    }
    public boolean doProcess()
    {
    	return process;
    }
    
    public boolean isAnswered()
    {
    	return answered;
    }
    
    
    public QTIv2variableDeclaration getOutcomeValue (String identifier) throws InvalidQTIv2Exception
    {
    	float score = 0;
    	QTIv2variableDeclaration outScore=null;
        boolean isInt= false;
        
    	for (int n = 0; n < outcomeVars.size(); n++)
    	{
    		QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) outcomeVars.get(n);
    		
    		for (int n2 = 0; n2 < tmp.getSize(); n2++)
	        {
    			 if (tmp.identifier.equals(identifier))
    			 {
    				 if(tmp==null)
    		               throw new InvalidQTIv2Exception("Null QTIv2Variable while resolving outcomeValue");
    				 if(tmp.getType()!=QTIv2variableDeclaration.VTYPE_integer)
    					 isInt= false;
    				 Float tmpfloat=tmp.toFloat();
    				 if(tmpfloat==null)
    		           return new QTIv2responseDeclaration(null, QTIv2variableDeclaration.CARD_single, QTIv2variableDeclaration.VTYPE_float);
    		         else
    		           score += tmpfloat.floatValue();
    			 }
	        }

    		if(isInt)
    			outScore = new QTIv2variableDeclaration(null,QTIv2variableDeclaration.CARD_single, QTIv2variableDeclaration.VTYPE_integer);
    		else
    			outScore = new QTIv2variableDeclaration(null,QTIv2variableDeclaration.CARD_single, QTIv2variableDeclaration.VTYPE_float);
    	}
    	outScore.setValue(score);
    	return outScore;
    }
    
    // -- getOutcomeValue : returns the outcomeValue, score.
    public String getOutcomeValue(int o, int v)
    {
        if (o < outcomeVars.size()) {
            QTIv2variableDeclaration tmp = (QTIv2variableDeclaration) outcomeVars.get(o);
            if (v < tmp.getSize()) {
                try {
                    return tmp.getStringValue(v);
                }
                catch (InvalidQTIv2Exception ex) {
                    return null;
                }
            }
            else
                return null;
        }
        else
            return null;
    }
    
    public void readFormData(String postStr) 
    {
    	String[] dt = postStr.split("&");
        for(int n=0; n<dt.length; n++)
        {
            int eqp = dt[n].indexOf("=");
            if(eqp>2)
            {
            	
            	String name = dt[n].substring(0,eqp);
                String value = dt[n].substring(eqp+1);

                int i = name.indexOf("_"); // if apisa_RESPONSE
                if(i != -1)
                {
                	try
                	{
                		value = URLDecoder.decode(value, "UTF-8");
                		addFormDataValue(name, value);
                	}
                	catch (UnsupportedEncodingException ex)
                	{
                	}
            
                }
            }
        }
    }
    
    public String getItemScore(){
    	return this.itemScore;
    }
    
    public void setItemScore(String itemScore) 
    {
    	float score_ = Float.parseFloat(itemScore);
    	user.setItemScore(itemIdentifier,score_);
    }
    
    public void storeItem()
    {
    	user.storeItem(itemIdentifier);
    }
    
    public void setMediaBase(String nMediaBase) {
      mediaBase = nMediaBase;
    }
    public void setAppletBase(String nAppletBase) {
      appletBase = nAppletBase;
    }    
}


