package apis;

import nb_utils.*;

import java.util.Random;
import java.util.Vector;

/**
 * <p>Title: APIS test wrapper</p>
 * <p>Description: A very very simple HTTP server</p>
 * <p>Copyright: Copyright (c) 2004, University of Strathclyde</p>
 * <p>Company: University of Strathclyde/JISC</p>
 * @author Niall Barr
 * @version 1.0
 */
/*HeLeNa*/
public class QTIv2matchInteraction extends QTIv2structuredBodyPart implements QTIv2responseVariableUpdate
{
    protected QTIv2structuredBodyPart prompt;
    
    protected QTIv2variableDeclaration var;

    protected boolean shuffle;
    
    protected int maxAssociations; 
    
    protected Vector formNames; 
    
    public QTIv2matchInteraction(QTIv2item owner)
    {
        super(owner);
        owner.registerVarUpdate(this);
        shuffle = false;
        maxAssociations=1;
    }

    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
    	formNames = new Vector();
        
    	//ATTRIBUTES
        // is it to be shuffled?
        if(itemXML.getAttribute(idx,"shuffle").equals("true"))
        {
            shuffle = true;
        }
        //maxAssociation
        maxAssociations =Integer.parseInt(itemXML.getAttribute(idx,"maxAssociations"));
       
        var = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"responseIdentifier")));
        
        if(var==null)
            throw new InvalidQTIv2Exception("responseIdentifier "+itemXML.getAttribute(idx,"responseIdentifier")+" does not match a variable.");

        // check for a prompt, then run through the 2 simpleMatchSets
        int cidx = itemXML.findChildElement(idx);
        
        while(cidx != -1)
        {
            int id = QTIv2define.identify(itemXML.getName(cidx));
            if(id == QTIv2define.QTIv2_prompt)
            {
                if(prompt != null)
                    throw new InvalidQTIv2Exception("Multiple prompts in choiceInteraction");
                if(parts != null)
                    throw new InvalidQTIv2Exception("prompt after simpleChoice in choiceInteraction");
                prompt = new QTIv2structuredBodyPart(theItem);
                cidx = prompt.parse(itemXML, cidx);
                if(cidx==-1)
                    throw new InvalidQTIv2Exception("No simpleChoice following prompt choiceInteraction");
            }
            else if(id == QTIv2define.QTIv2_simpleMatchSet)
            {       	
                if(parts == null)
                    parts = new Vector();
             
                QTIv2simpleMatchSet sms = new QTIv2simpleMatchSet(theItem, itemXML.getAttribute(cidx,"identifier"), var);
                cidx = sms.parse(itemXML, cidx);
                parts.add(sms);

                if(parts.size()==1)
                for(int j=0; j<maxAssociations;j++) 	//  a form names s'hi guarda el numero de respostes, nomes 1 cop
                	formNames.add(sms.nm+"_"+j);
            }
        }
        return itemXML.findNextPeer(idx);
    }

    public String render() throws InvalidQTIv2Exception
    {
        if(shuffle==true)
        {	        	
        	
        	doShuffle(((QTIv2simpleMatchSet)parts.get(0)).parts, new Random());
        	doShuffle(((QTIv2simpleMatchSet)parts.get(1)).parts, new Random());
        	
        	//# if this is not the first rendering, need to read order from post data,
            //# otherwise generate order.
            //# has to be done in render so the form data is available.       
        }
        String out = "";
        out+="<table>";
        if (prompt != null)
            out += "<tr><td>"+prompt.render()+"</td>";
        	
        //there are always only 2 simpleMatchSets
        //source
        out +=((QTIv2simpleMatchSet) parts.get(0)).renderSource()+"</tr>";
      	//destination
        out +="<tr>"+((QTIv2simpleMatchSet) parts.get(1)).renderDest(((QTIv2simpleMatchSet)parts.get(0)))+"</tr>";
        
        out+="</table>";
        return out;
    }

    public boolean updateFromFormData()
    {
    	for (int n = 0; n < formNames.size(); n++)
        {
            String key = (String) formNames.get(n);

            if (theItem.formData.containsKey(key))
                try
                {
                    var.setValue((String) theItem.formData.get(key));
                }
                catch (InvalidQTIv2Exception ex)
                {
                }
        }
        return false;
     }
}