package apis;

import nb_utils.*;
import java.util.*;


/**
 * <p>Title: APIS test wrapper</p>
 * <p>Description: A very very simple HTTP server</p>
 * @version 1.0
 */

public class QTIv2orderInteraction extends QTIv2structuredBodyPart implements QTIv2responseVariableUpdate
{
    protected QTIv2structuredBodyPart prompt;
    protected Vector formNames; // contains the form field names from the simpleChoice class;

    protected QTIv2variableDeclaration var;

    protected String shuffleName; // variable name for persistance data.

    protected boolean shuffle;	//if true: randomize the order of the choices presented to the user
    protected String orientation; //provides a hint to rendergin systems that the ordering has an inherent vertical/
    										   //horizontal interpretation
    protected int count;
    public QTIv2orderInteraction(QTIv2item owner)
    {
        super(owner);
        owner.registerVarUpdate(this);
        shuffle = false;
        orientation="horizontal";
        count=0;
    }

    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        formNames = new Vector();
        
        // is it to be shuffled?
        if(itemXML.getAttribute(idx,"shuffle").equals("true"))
        {
            shuffle = true;
 //#           shuffleName = theItem.persist.getNextName();
        }
 
        var = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"responseIdentifier")));
        if(var==null)
            throw new InvalidQTIv2Exception("responseIdentifier "+itemXML.getAttribute(idx,"responseIdentifier")+" does not match a variable.");

        // check for a prompt, then run through the simpleChoices
        int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        {
            int id = QTIv2define.identify(itemXML.getName(cidx));
            if(id == QTIv2define.QTIv2_prompt)
            {
                if(prompt != null)
                    throw new InvalidQTIv2Exception("Multiple prompts in choiceInteraction");
                if(parts != null)
                    throw new InvalidQTIv2Exception("prompt after simpleChoice in choiceInteraction");
                prompt = new QTIv2structuredBodyPart(theItem);
                cidx = prompt.parse(itemXML, cidx);
                if(cidx==-1)
                    throw new InvalidQTIv2Exception("No simpleChoice following prompt choiceInteraction");
            }
            else if(id == QTIv2define.QTIv2_simpleChoice)
            {
                if(parts == null)
                    parts = new Vector();
                QTIv2simpleChoice ch = new QTIv2simpleChoice(theItem, itemXML.getAttribute(cidx,"identifier"), var);
                cidx = ch.parse(itemXML, cidx);
                
                formNames.add(ch.nm+"_"+count);
                count++;
                
                parts.add(ch);
            }
           //else
             //  throw new InvalidQTIv2Exception("Invalid child element in choiceInteraction [" + String.valueOf(id) + "]");
        }
        return itemXML.findNextPeer(idx);
    }

 
    public String render() throws InvalidQTIv2Exception
    {    	
    	 if(shuffle==true)
         {
         	Vector toShuffle=new Vector();
         
         	//adding the QTIv2simpleChoice not fixed to a vector that will be randomized
         	for(int i=0;i<parts.size();i++)
         	{ 
         		if(!(((QTIv2simpleChoice)parts.get(i)).fixed))
         			toShuffle.add(((QTIv2simpleChoice)parts.get(i)));
         	}
         	// do Random
         	doShuffle(toShuffle, new Random());
         	
         	//# if this is not the first rendering, need to read order from post data,
             //# otherwise generate order.
             //# has to be done in render so the form data is available.
         	int pos=0;
         	
         	for(int i=0;i<parts.size();i++)
         	{
         		if(!(((QTIv2simpleChoice)parts.get(i)).fixed))
         		{
         			parts.setElementAt((QTIv2simpleChoice)toShuffle.get(pos),i);
         			pos++;
         		}
         	}
         }
         
        
       String out = "";
       if (prompt != null)
            out += prompt.render();
        
       //This rendering is a substitute for what I think that should be a drag'n drop interface
        if(var.cardinality == QTIv2variableDeclaration.CARD_ordered)
        {
        	out += "<br><br><table>";
        	for(int m=0;m<parts.size();m++)
        	{ 
        		out+="<tr><td>"+(m+1)+ "</td><td><select name=\"apisa_RESPONSE_"+m+"\">"; 
        		for(int n=0; n < parts.size(); n++)
        		{	
        			out+="<option value=\""+ ((QTIv2simpleChoice)parts.get(n)).identifier +"\"";		
        			
        			if ((n==m)) 
        				out+= " selected=\"selected\"";
        				        			
        			out+=">"+ ((QTIv2simpleChoice)parts.get(n)).render()+"</option>";        		
        		}
        		out+="</select></td></tr>";
        	}
        	out+="</table></br></br>";
        }
        return out;
    }	
    
    public boolean updateFromFormData()
    {
    	for (int n = 0; n < formNames.size(); n++)
        {
            String key = (String) formNames.get(n);
            
            if (theItem.formData.containsKey(key))
                try
                {
                    var.setValue((String) theItem.formData.get(key));
                }
                catch (InvalidQTIv2Exception ex)
                {
                }
        }
        return false;
    }

}
