package apis;

import java.util.Vector;
import nb_utils.NBEasyXML;

/**
 * QTIv2ResponseActionCollection is the set of response actions that are either contained directly within
 * responseProcessing or responseElse, or follow the condition in responseIf or responseElseIf. For this
 * reason the parseing is slightly different - since this is a collection of elements but not always every
 * element within the parent, parse continues through a peer group of elements.
 *
 * @author Niall Barr
 * @version 1.0
 */
public class QTIv2responseActionCollection
{
    QTIv2item theItem;
    Vector actions;

    public QTIv2responseActionCollection(QTIv2item owner)
    {
        theItem = owner;
    }

    /**
     * parse
     *
     * @param itemXML NBEasyXML
     * @param cidx int
     * @return int
     */
    public int parse(NBEasyXML itemXML, int cidx) throws InvalidQTIv2Exception
    {
        actions = new Vector();
        while(cidx != -1)
        {
            int id = QTIv2define.identify(itemXML.getName(cidx));
            switch(id)
            {
                case QTIv2define.QTIv2_responseCondition:
                	QTIv2responseCondition rc = new QTIv2responseCondition(theItem);
                    rc.parse(itemXML, cidx);
                    actions.add(rc);
                    break;
                case QTIv2define.QTIv2_setOutcomeValue:
                    actions.add(new QTIv2setOutcomeValue(theItem, itemXML, cidx));
                    break;
                case QTIv2define.QTIv2_exitResponse:
                    actions.add(new ExitResponse());
                    break;
                case QTIv2define.QTIv2_externalResponseProcessing:
                    throw new InvalidQTIv2Exception("externalResponseProcessing is not supported by APIS.");
                default:
                    throw new InvalidQTIv2Exception("Unexpected "+itemXML.getName(cidx)+" in response processing.");
            }

            cidx = itemXML.findNextPeer(cidx);
        }
        return -1;
    }

    public boolean run() throws InvalidQTIv2Exception
    {
        boolean result = true;
        int pos = 0;
        while((pos < actions.size())&&(result))
        {
            result = ((QTIv2variableAction)actions.get(pos)).run();
            pos++;
        }
        return result;
    }
}
