package apis;

import nb_utils.*;

/**
 * <p>Title: stringInteraction</p>
 * <p>Description: An abstract superclass for text interactions</p>
 * <p>Copyright: Copyright (c) 2004, University of Strathclyde</p>
 * <p>Company: University of Strathclyde/JISC</p>
 * @author Peter Crowther, refactored from unattributable work
 * @version 1.0
 */

public abstract class QTIv2stringInteraction extends QTIv2bodyPart implements QTIv2responseVariableUpdate
{
    // The attributes that APIS will definitely use.
    protected String responseIdentifier;
    protected String stringIdentifier;
    protected int base;
    protected int expectedLength;
    protected String patternMask;
    protected QTIv2variableDeclaration responseVar;
    protected QTIv2variableDeclaration stringVar;

    protected QTIv2stringInteraction(QTIv2item owner)
    {
        super(owner);
        owner.registerVarUpdate(this);
    }

    /** Note that subclasses must call this in their own parse methods. **/
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
        //# read in attributes from the element
        responseIdentifier = itemXML.getAttribute(idx, "responseIdentifier");
        base = itemXML.getIntegerAttribute(idx, "base", 10);
        stringIdentifier = itemXML.getAttribute(idx, "stringIdentifier");
        expectedLength = itemXML.getIntegerAttribute(idx, "expectedLength", 10);
        patternMask = itemXML.getAttribute(idx, "patternMask");

        responseVar = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"responseIdentifier")));
        if(null == responseVar)
            throw new InvalidQTIv2Exception("responseIdentifier "+itemXML.getAttribute(idx,"responseIdentifier")+" does not match a variable.");

        if (null != stringIdentifier)
        {
            stringVar = (QTIv2variableDeclaration)(theItem.getResponseVariable(itemXML.getAttribute(idx,"stringIdentifier")));
            if(null == stringVar)
                throw new InvalidQTIv2Exception("stringIdentifier "+itemXML.getAttribute(idx,"stringIdentifier")+" does not match a variable.");
        }

        return itemXML.findEnd(idx)+1;
    }

    public boolean updateFromFormData()
    {
        String key = getSafeFormID(responseIdentifier);
        if (theItem.formData.containsKey(key))
            try
            {
            	
                String value = (String) theItem.formData.get(key);
               
                responseVar.setValue(value);
                // TODO: Convert to number, correct base, etc.
                if (null != stringVar)
                    stringVar.setValue(value);
            }
            catch (InvalidQTIv2Exception ex)
            {
            }
        return false;
    }

}
