/*
 * Copyright (c) 2008, Consortium Board TENCompetence
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the TENCompetence nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CONSORTIUM BOARD TENCOMPETENCE ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CONSORTIUM BOARD TENCOMPETENCE BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package apis;

import java.io.Serializable;
import java.util.Vector;
import java.sql.Connection;

public class UserTestCase implements Serializable
{
	public class ItemStruct{
		public float score;
		public Vector answer_id;
		public boolean open;
		public String item_id;
		public boolean toStore;
		
		
		public ItemStruct(String item_,float score_,Vector answer_,boolean open_){
			item_id =item_;
			score = score_;
			toStore = false;
			answer_id = answer_;
			open = open_;
		}
		
		public ItemStruct(String item_){
			item_id =item_;
			score = 0;
			toStore = false;
			answer_id = new Vector();
			open = false;
		}
	}
	
	public class ItemStructVector extends Vector{
		public boolean add(Object o){
			if( o instanceof ItemStruct){
				return super.add(o);
			}
			System.out.println("This vector only handles ");
			return false;
			
		}
	}

	
	/**
	 * User serializable class
	 * UPF - Pau Melines
	 */
	
	private static final long serialVersionUID = 1L;
	private String test;
    private String userName;
    private int runID;
    private String uolId;
    String language;
    private float itemScore = 0;
    private float finalScore = 0;
    private ItemStructVector vectorItems = new ItemStructVector();
    private ItemStructVector toprocessItems = new ItemStructVector();
    private ItemStructVector storedItems = new ItemStructVector();
    private DAO connectedDao;
    
    public UserTestCase(String _assessmentTest, int _run, String _user, String _uolId,String language_) throws Exception 
    {
    	this.test = _assessmentTest;
     	this.userName = _user;
        this.runID = _run;
        this.uolId = _uolId;
        this.language = language_;
        connectedDao = null;
        
    }
    
    public void setDB(String driverClass, String databaseUrl,String databaseUsername,String databasePassword)throws Exception{
    	System.out.println("SetDB: driverClass: " + driverClass + ". databaseUrl: " + databaseUrl +". User/Pwd: " + databaseUsername +"/"+ databasePassword);
    	connectedDao = new DAO();
    	connectedDao.setConnectionProperties(driverClass, databaseUrl, databaseUsername, databasePassword);
        System.out.println("[DB] Database properties are setted");
    }
    
    public void setDB(String properties_file_location)throws Exception{
    	System.out.println("SetDB: propertiesFileLocation: " + properties_file_location);
    	connectedDao = new DAO();
    	connectedDao.setConnectionPropertiesFromFile(properties_file_location);
        System.out.println("[DB] Database properties are setted");
    }
    
    public boolean isConnected(){
    	return connectedDao.isConnected();
    }
    
    public void connectWithDB()throws Exception{
    	connectedDao.openConnection();
        System.out.println("[DB] Database connection On");
    }
    protected void finalize () throws Throwable{
    	  DAO tmp = new DAO();
    	  tmp.setConnectionFromDAO(connectedDao);
          tmp.closeConnection();
          System.out.println("[DB] Database connection off");
    }
    
  /*  public void setName(String aName)
    {
    	this.userName = aName;
    }*/
    
    public String getName()
    {
        return this.userName;
    }
    
    public DAO getConnectedDao(){
    	return connectedDao;
    }

  /*  public void setRun(int aRun)
    {
    	this.runID = aRun;
    }*/
    
    public int getRun()
    {
    	return this.runID;
    }
    
    public String getUolID()
    {
    	return this.uolId;
    }
    
   /* public void setTest(String aTest)
    {
    	this.test = aTest;
    }*/
    
    public String getTest()
    {
    	return this.test;
    }
    public boolean hasItem(String itemref){
        for (int i = 0 ; i<toprocessItems.size() ; i++)
        {
            if(((ItemStruct)toprocessItems.get(i)).item_id.equals(itemref))
        	{
            	return true;
        	}
        }
        return false;
    }
    
    public boolean storeItem(String itemref){
    	
        for (int i = 0 ; i<toprocessItems.size() ; i++)
        {
            if(((ItemStruct)toprocessItems.get(i)).item_id.equals(itemref))
        	{
            	((ItemStruct)toprocessItems.get(i)).toStore = true;
            	storedItems.add(toprocessItems.remove(i));
            	i--;
            	return true;
        	}
        }
        return false;
    }
    
    public boolean setItemScore(String itemref,float score_){ //adds score to an item which is goig to be stored
    	
        for (int i = 0 ; i<storedItems.size() ; i++)
        {
            if(((ItemStruct)storedItems.get(i)).item_id.equals(itemref))
        	{
            	if(((ItemStruct)storedItems.get(i)).toStore == false){
            		return false;
            	}
            	((ItemStruct)storedItems.get(i)).score = score_;
            	return true;
        	}
        }
    	return false;
    }
    public boolean setItemCloseAnswer(String itemref,Vector answer_id){ //adds score to an item which is goig to be stored
    	
        for (int i = 0 ; i<toprocessItems.size() ; i++)
        {
            if(((ItemStruct)toprocessItems.get(i)).item_id.equals(itemref))
        	{
            	((ItemStruct)toprocessItems.get(i)).answer_id = answer_id;
            	((ItemStruct)toprocessItems.get(i)).open = false; 
            	return true;
        	}
        }
    	return false;
    }
    
    public void setCloseAnswer(String id,String ans){
   
     	   try{
     		    DAOCloseAnswer daocloseanswer = new DAOCloseAnswer();
              	//[refactor]tmpdao.setResponseContext( getSafeIdent(identifier),context);
              	daocloseanswer.setCloseAnswer(id, ans, this);
              }catch(Exception e){
              	e.printStackTrace();
              }
     }
            
  
    
    public ItemStructVector getStoredItems()
    {
        try{
            DAOUserTestCase daouser = new DAOUserTestCase();
            storedItems = daouser.getItems(this);
            return storedItems;
        }
        catch(Exception e)
        {
        	System.out.println("Exception: "+e);
        	return null;
        }
    }
    
    public void addToProcessItem(String item){
    	ItemStruct toadd = new ItemStruct(item);
    	toprocessItems.add(toadd);
    }
    
    public Vector getToProcessItems()
    {
         return toprocessItems;
    }
    
    public Vector getItems()
    {
    	vectorItems.removeAllElements();
    	vectorItems.addAll(toprocessItems);
    	vectorItems.addAll(storedItems);
        return vectorItems;
    }

    
    public float getFinalScore()
    {
    	try{
    		DAOUserTestCase daouser = new DAOUserTestCase();
    		finalScore = daouser.getFinalScore(this);
    		return finalScore;
    	}catch(Exception e)
    	{
    		System.out.println("Exception: "+e);
    		return -1;
    	}
    }
    
    public float getMaxRepetition(){
    	float repetition;
    	try{
    		DAOUserTestCase daouser = new DAOUserTestCase();
    		repetition = daouser.getMaxRepetition(this);
    		return repetition;
    	}catch(Exception e)
    	{
    		System.out.println("Exception: "+e);
    		return -1;
    	}
    }

    public void updateLoginUserTestCase(){
    	try{
    		DAOUserTestCase daouser = new DAOUserTestCase();
    		daouser.setLoginTestCase(this);
    	}catch(Exception e)
    	{
    		System.out.println("Exception: "+e);
    	}
    }
    
    public void setItems()
    {
    	try{
    		DAOUserTestCase daouser = new DAOUserTestCase();
    		//daouser.setUser(this); USER
    		daouser.setItemsToDb(this);
    	}catch(Exception e)
    	{
    		System.out.println("Exception: "+e);
    	}
    }
    
    public void setUserTestCaseDone(){
        try{
            DAOUserTestCase daouser = new DAOUserTestCase();
            daouser.setUserTestCaseDone(this);
         }catch(Exception e)
         {
            System.out.println("Exception: "+e);
         }
    }
    
    public void checkUserTestCaseNotDone(){
    	
    	try{
    		DAOUserTestCase daouser = new DAOUserTestCase();
        	daouser.checkUserTestCaseNotDone(this);
	     }catch(Exception e)
	     {
	        System.out.println("Exception: "+e);
	     }
    }
    
    public void setPage(int page){
        try{
            DAOUserTestCase daouser = new DAOUserTestCase();
            daouser.setUserTestCasePage(this,page);
         }catch(Exception e)
         {
            System.out.println("Exception: "+e);
         }
    }
     
    public int getPage(){
        DAOUserTestCase daouser = new DAOUserTestCase();
    	return daouser.getPage(this);
    }

}   

