package apis.expression;

import nb_utils.NBEasyXML;
import apis.InvalidQTIv2Exception;
import apis.QTIv21test;
import apis.QTIv2expression;
import apis.QTIv2variableDeclaration;

/**
 * This expression, which can only be used in outcomes processing, simultaneously looks up the value of an itemVariable
 * in a sub-set of the items referred to in a test. Only variables with single cardinality are considered, all NULL
 * values are ignored. The result has cardinality multiple and base-type as specified below. Attribute :
 * variableIdentifier [1]: identifier The identifier of the variable to look up in each item. If a test brings together
 * items with different variable naming conventions variableMappings may be used to reduce the complexity of outcomes
 * processing and allow a single testVariables expression to be used. Items with no matching variable are ignored.
 * Attribute : baseType [0..1]: baseType If specified, matches only variables declared with this baseType. This also
 * becomes the base-type of the result (subject to type promotion through weighting, as described below). If omitted,
 * variables declared with base-type integer or float are matched. The base-type of the result is integer if all
 * matching values have base-type integer, otherwise it is float and integer values are subject to type promotion.
 * Attribute : weightIdentifier [0..1]: identifier If specified, the defined weight is applied to each variable as
 * described in the definition of weightIdentifier for a single variable. The behaviour of this attribute is only
 * defined if baseType is float or omitted. When a weighting is specified the result of the expression always has
 * base-type float.
 */

public class QTIv21Expr_testVariables extends QTIv2expression {
	String variableIdentifier;
	String sectionIdentifier; // no surt a l'especificacio ... pero si al'exemple!
	String baseType; // suposem que sera integer o float
	String weightIdentifier;
	String includeCategory;
	String excludeCategory;

	public QTIv21Expr_testVariables(QTIv21test owner, NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception {
		super(owner, itemXML, idx);
	}

	protected boolean parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception {
		variableIdentifier = itemXML.getAttribute(idx, "variableIdentifier");

		if (itemXML.getAttribute(idx, "sectionIdentifier") != null)
			sectionIdentifier = itemXML.getAttribute(idx, "sectionIdentifier");
		else
			sectionIdentifier = null;

		if (itemXML.getAttribute(idx, "weightIdentifier") != null)
			weightIdentifier = itemXML.getAttribute(idx, "weightIdentifier");
		else
			weightIdentifier = null;

		if (itemXML.getAttribute(idx, "includeCategory") != null)
			includeCategory = itemXML.getAttribute(idx, "includeCategory");
		else
			includeCategory = null;

		if (itemXML.getAttribute(idx, "excludeCategory") != null)
			excludeCategory = itemXML.getAttribute(idx, "excludeCategory");
		else
			excludeCategory = null;

		return true;
	}

	public QTIv2variableDeclaration resolve() throws InvalidQTIv2Exception {
		QTIv2variableDeclaration c2 = theTest.getTestVariable(variableIdentifier, sectionIdentifier, includeCategory,
				excludeCategory);
		if (weightIdentifier != null)
			theTest.setWeightIdentifier(weightIdentifier);

		if (c2 == null)
			throw new InvalidQTIv2Exception("something horrible has gone wrong resolving variable "
					+ variableIdentifier);

		if (c2.getRelatedItemsVarDec().size() == 0)
			throw new InvalidQTIv2Exception("there are no matches resolving testVariable " + variableIdentifier);

		return ((QTIv2variableDeclaration) c2).getDuplicate();
	}
}
