package apistesthost;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Hashtable;
import java.util.StringTokenizer;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import adapters.ParameterInputAdapter;
import adapters.QtiParser;

public class Connection implements Runnable {

	// Variables for each thread
	Socket linkto; // the socket
	PrintWriter out; // the output and input streams
	BufferedReader in;

	int id; // i.d. of the connection
	String from_name; // name of host connecting

	// Class Variables

	static int nextid = 1; // Increasing i.d.

	String testDir = "tests/";
	String mediaDir = "";
	String appletDir = "";

	Connection(Socket from) {
		id = nextid++;
		linkto = from;
		InetAddress source = linkto.getInetAddress();
		from_name = source.getHostName();
		try {
			out = new PrintWriter(new OutputStreamWriter(linkto.getOutputStream()));
			in = new BufferedReader(new InputStreamReader(linkto.getInputStream()));
		} catch (Exception e) {
		}
	}

	public void run() {

		try {
			HTTPRequestInfo request = new HTTPRequestInfo();

			request.readInfo(in);

			if (request.fileName().compareTo("/") == 0) // home leads to a list of published tests
				SendFileListPage(out);
			else if (request.fileName().compareToIgnoreCase("/render_test") == 0
					|| request.fileName().compareToIgnoreCase("/QTIPost.do") == 0) { // render a test
				SendRenderedTestPage(out, request, linkto.getInetAddress().toString());
			} else {
				SendFile(out, request, linkto.getOutputStream()); // get a test xml
			}

			out.close();
			in.close();
			linkto.close();
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	private void SendRenderedTestPage(PrintWriter prOut, HTTPRequestInfo request, String user) {

		SendOKHeader(prOut, "text/html");

		StringWriter result = new StringWriter();

		String xml_url;

		// xml_url = "http://localhost:1234/tests/test1.xml"; // This test is done with portlet editor before 15Oct09

		// xml_url = "http://localhost:1234/tests/Testmultiple.xml"; // This test is done with Recourse editor
		// note that it has test feedback and modal feedback

		// xml_url = "http://localhost:1234/tests/myTest.xml"; // This test is done with portlet editor and it's used to
		// test if the score calculation is OK

		// xml_url = "http://localhost:1234/tests/test2.xml";

		// xml_url = "http://localhost:1234/tests/test_exception.xml";

		// xml_url = "http://localhost:1234/tests/test2.xml";
		String file = (String) request.requestParams.get("file");
		if (file.startsWith("http"))
			xml_url = "http://localhost:1234/tests/" + file.substring(file.lastIndexOf("%2F") + 3);
		else
			xml_url = "http://localhost:1234/tests/" + file;
		// String file = (String)request.requestParams.get("file");
		// xml_url = "http://localhost:1234/tests/"+file;
		QtiParser qti_parser = new QtiParser(user, 0, result, xml_url, request.post_info, "english");

		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(false);
		factory.setNamespaceAware(true);

		String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
		String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";

		try {
			SAXParser parser = factory.newSAXParser();
			// parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
			parser.parse(xml_url, qti_parser);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("parser cannot be runned ..");
		}
		prOut.println(result.toString());

	}

	private void SendOKHeader(PrintWriter prOut, String mimeType, long contentLength) {
		// start sending... (always OK!)
		prOut.println("HTTP/1.0 200 ");
		prOut.println("Content-Type: " + mimeType);
		if (contentLength > 0)
			prOut.println("Content-length: " + String.valueOf(contentLength));
		prOut.println();
		prOut.flush();

	}

	private void SendOKHeader(PrintWriter prOut, String mimeType) {
		SendOKHeader(prOut, mimeType, -1);
	}

	private void SendErrorPage(PrintWriter prOut, int errorCode) {
		// start sending... (always OK!)
		prOut.println("HTTP/1.0 " + String.valueOf(errorCode) + " ");
		prOut.println("Content-Type: text/html");
		prOut.println();
		prOut.flush();
		prOut
				.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"><html><body>");

		switch (errorCode) {
		case 404:
			prOut.println("<p>Error 404, File not found.</p>");
			break;
		case 500:
			prOut.println("<p>Error 500, Something nasty and unexpected went wrong.</p>");
			break;
		case 501:
			prOut.println("<p>Error 501, Unsupported request.</p>");
			break;
		default:
			prOut.println("<p>Error " + String.valueOf(errorCode) + "</p>");
			break;

		}
		prOut.println("</body></html>");
		prOut.flush();
	}

	private void SendSetupRequiredPage(PrintWriter prOut) {
		prOut.println("HTTP/1.0 200 ");
		prOut.println("Content-Type: text/html");
		prOut.println();
		prOut.flush();
		prOut
				.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"><html><body>");
		prOut
				.println("<p>The item directory needs to be set to a directory containing QTIv2 XML files before APISTestHost is used.");
		prOut.println("</body></html>");
		prOut.flush();
	}

	private void SendFileListPage(PrintWriter prOut) {
		SendOKHeader(prOut, "text/html");
		FileList fl = new FileList(testDir); // disabled for the portlet version
		prOut.println(fl.getHTMLList());
		prOut
				.println(" SEND FILE LIST PAGE : Publication funciontalities have been disabled for this PortletAdapter version<p/>\n\n");
		prOut
				.println("To see a test working visit: <a href=\"http://127.0.0.1:1234/render_test\">http://127.0.0.1:1234/render_test </a> \n<p/>");
		prOut.println("The rendered test and it's parts are hardcoded as a Strings\n<p/>");
		prOut.println("To modify the rendered test see the 272 line and 162 line in Connection.java \n\n<p/>");
		prOut
				.println("If you can't see the test the most probable error is that you havent launched the database, please execute DB_run_me_first.bat\n<p/>");
		prOut.println("</body></html>");
	}

	private void SendItemsInFilePage(PrintWriter prOut, HTTPRequestInfo request) {// TODO
		SendOKHeader(prOut, "text/html");
		prOut.println("<p>Checking for items in <b>" + request.getParam("file") + "</b></p>\n");

		QTI2FileReader fr = new QTI2FileReader();

		fr.readFile(testDir + request.getParam("file"));
		prOut.println(fr.getInfo(true));
		prOut.println("</body></html>");
	}

	private void SendFile(PrintWriter prOut, HTTPRequestInfo request, OutputStream out) {
		String fname = request.fileName();
		String ftype = "";
		String folder = "";
		String baseDir = null;
		System.out.println("[***] ATENTION File requested to server: " + fname);

		int tp = fname.lastIndexOf(".");
		if (tp != -1)
			ftype = fname.substring(tp + 1);
		tp = fname.indexOf("/", 1);
		if ((tp != -1) && (fname.substring(0, 1).equals("/"))) {
			folder = fname.substring(1, tp);
			fname = fname.substring(tp + 1);
		}
		String mimeType = (String) theMimeTypes.get(ftype);
		if (folder.equals("images")) {
			if ((mediaDir != null) && (mediaDir.length() > 0))
				baseDir = mediaDir;
		} else if (folder.equals("applets")) {
			if ((appletDir != null) && (appletDir.length() > 0))
				baseDir = appletDir;
		} else if (folder.equals("tests")) {
			baseDir = "tests";
		}

		if ((baseDir == null) || (fname == null) || (fname.length() == 0) || (mimeType == null))
			SendErrorPage(prOut, 501);
		else {
			File f = new File(baseDir + "\\" + fname);
			if (!f.exists()) {
				SendErrorPage(prOut, 404);
				System.out.println("Cannot acces to files, check the path");
				System.out.println("actual path is:" + f.getAbsolutePath());
			} else {
				try {
					FileInputStream fis = new FileInputStream(f);
					SendOKHeader(prOut, mimeType, f.length());
					byte[] buff = new byte[2048];
					int read = 2048;
					while (read == 2048) {
						read = fis.read(buff, 0, 2048);
						out.write(buff, 0, read);
					}
					out.flush();
					out.close();
					if (fis != null)
						fis.close();

				} catch (Exception ex) {
					SendErrorPage(prOut, 500);
				}

			}
		}
	}

	// copied from http://elonen.iki.fi/code/nanohttpd/NanoHTTPD.java.html, see
	// that file for licence and copyright
	/**
	 * Hashtable mapping (String)FILENAME_EXTENSION -> (String)MIME_TYPE
	 */
	private static Hashtable theMimeTypes = new Hashtable();
	static {
		StringTokenizer st = new StringTokenizer("htm        text/html " + "html       text/html "
				+ "txt        text/plain " + "asc        text/plain " + "gif        image/gif "
				+ "jpg        image/jpeg " + "jpeg       image/jpeg " + "png        image/png "
				+ "mp3        audio/mpeg " + "m3u        audio/mpeg-url " + "pdf        application/pdf "
				+ "ogg        application/x-ogg " + "xml        tests " + "js         tests "
				+ "class      application/octet-stream ");
		while (st.hasMoreTokens())
			theMimeTypes.put(st.nextToken(), st.nextToken());
	}

	// HOW ParameterInputAdapter works like

	// private void SendRenderedTestPage(PrintWriter prOut, HTTPRequestInfo request, String user) {

	private void SendRenderedTestPage_____ParameterInputAdapter(PrintWriter prOut, HTTPRequestInfo request, String user) {

		SendOKHeader(prOut, "text/html");

		StringWriter result = new StringWriter();

		String xml_url;
		Hashtable xml_container = new Hashtable();// [ demostration only ] this work will be done by consulting the
													// xml_editor_portlet database

		// fillHashtableWithXmlStrings(xml_container);
		// xml_url = "http://fake_url/tests/Testmultiple.xml";// note that the uris must have a "http://fake_url/" +
		// test_name structure

		fillHashtableWithXmlStrings____PortletTest(xml_container);
		xml_url = "http://fake_url/tests/test1.xml";

		ParameterInputAdapter apis_portlet_adapter = new ParameterInputAdapter();

		String xml_ans = apis_portlet_adapter.runQtiTestuser(result, xml_url, xml_container, user, request.post_info,
				"spanish");

		if (xml_ans != null) { // [ demostration only ] Erase this for a standard rendering
			StringWriter result_xml = new StringWriter();
			prOut.println("THIS IS THE RETURNED XML :");
			prOut.println(xml_ans);
		}

		prOut.println("THIS IS THE RENDERED HTML :");
		prOut.println(result.toString());

	}

	private static String readFileAsString(String filePath) throws java.io.IOException {
		StringBuffer fileData = new StringBuffer(1000);
		BufferedReader reader = new BufferedReader(new FileReader(filePath));
		char[] buf = new char[1024];
		int numRead = 0;
		while ((numRead = reader.read(buf)) != -1) {
			String readData = String.valueOf(buf, 0, numRead);
			fileData.append(readData);
			buf = new char[1024];
		}
		reader.close();
		return fileData.toString();
	}

	public String getTestXml(String location) {
		try {
			return readFileAsString(location);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}

	public void fillHashtableWithXmlStrings(Hashtable xml_container) {

		String testmultipleXmlContent = getTestXml("./tests/Testmultiple.xml");
		String testmultipleUriKey = "http://fake_url/tests/Testmultiple.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

		testmultipleXmlContent = getTestXml("./tests/Testmultiple/AI-7ed5fb2d-a130-4470-b598-9aff2dcc0c6e-109.xml");
		testmultipleUriKey = "http://fake_url/tests/Testmultiple/AI-7ed5fb2d-a130-4470-b598-9aff2dcc0c6e-109.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

		testmultipleXmlContent = getTestXml("./tests/Testmultiple/AI-9c1770b1-6b58-45d7-86bf-1202eb7126eb-116.xml");
		testmultipleUriKey = "http://fake_url/tests/Testmultiple/AI-9c1770b1-6b58-45d7-86bf-1202eb7126eb-116.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

		testmultipleXmlContent = getTestXml("./tests/Testmultiple/AI-bb4d8169-1e30-4cfc-8110-69a68ba4e5de-123.xml");
		testmultipleUriKey = "http://fake_url/tests/Testmultiple/AI-bb4d8169-1e30-4cfc-8110-69a68ba4e5de-123.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

		testmultipleXmlContent = getTestXml("./tests/Testmultiple/AI-a16fc7b2-1077-4430-ba02-96cb9cbb2998-144.xml");
		testmultipleUriKey = "http://fake_url/tests/Testmultiple/AI-a16fc7b2-1077-4430-ba02-96cb9cbb2998-144.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

		testmultipleXmlContent = getTestXml("./tests/Testmultiple/AI-5674fb2b-5af1-47e2-95a6-83605b346520-151.xml");
		testmultipleUriKey = "http://fake_url/tests/Testmultiple/AI-5674fb2b-5af1-47e2-95a6-83605b346520-151.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

		testmultipleXmlContent = getTestXml("./tests/Testmultiple/AI-75476dab-395c-4cbc-b88d-e56a3a529eb4-158.xml");
		testmultipleUriKey = "http://fake_url/tests/Testmultiple/AI-75476dab-395c-4cbc-b88d-e56a3a529eb4-158.xml";
		xml_container.put(testmultipleUriKey, testmultipleXmlContent);

	}

	public void fillHashtableWithXmlStrings____PortletTest(Hashtable xml_container) {

		String test1XmlContent = getTestXml("./tests/test1.xml");
		String testmultipleUriKey = "http://fake_url/tests/test1.xml";
		xml_container.put(testmultipleUriKey, test1XmlContent);

		test1XmlContent = getTestXml("./tests/test1/AI-4014ad3b-4cda-4060-b124-4bbd548b7d79-396.xml");
		testmultipleUriKey = "http://fake_url/tests/test1/AI-4014ad3b-4cda-4060-b124-4bbd548b7d79-396.xml";
		xml_container.put(testmultipleUriKey, test1XmlContent);

		test1XmlContent = getTestXml("./tests/test1/AI-5442b905-052f-4329-9d78-6bb72edbcb7f-321.xml");
		testmultipleUriKey = "http://fake_url/tests/test1/AI-5442b905-052f-4329-9d78-6bb72edbcb7f-321.xml";
		xml_container.put(testmultipleUriKey, test1XmlContent);

		test1XmlContent = getTestXml("./tests/test1/AI-a51b44a3-7697-4305-8d23-ef29b7fd34c8-549.xml");
		testmultipleUriKey = "http://fake_url/tests/test1/AI-a51b44a3-7697-4305-8d23-ef29b7fd34c8-549.xml";
		xml_container.put(testmultipleUriKey, test1XmlContent);

		test1XmlContent = getTestXml("./tests/test1/AI-f0ffc462-5fce-48be-b612-1fc001aae3a2-241.xml");
		testmultipleUriKey = "http://fake_url/tests/test1/AI-f0ffc462-5fce-48be-b612-1fc001aae3a2-241.xml";
		xml_container.put(testmultipleUriKey, test1XmlContent);

	}

}
