package apistesthost;

import java.io.File;
import java.util.Vector;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * <p>
 * Title: QTI2FileReader
 * </p>
 * <p>
 * Description: Reads in a QTIv2 file, identifies tests
 * </p>
 */

public class QTI2FileReader extends DefaultHandler implements ContentHandler {
	protected String fileName;
	private String log; // !< log is a temporary message list for
	// development/learning purposes
	protected String xml; // !< the xml for the current item
	protected boolean inTest; // !< flag, indicates in atest
	protected boolean endNeeded; // !< flag, indicates an element needs its > or
	// />
	protected Vector tests;

	protected String loadItemID;
	private testInfo curTestInfo;

	public QTI2FileReader() {
		fileName = null;
		inTest = false;
	}

	public boolean readFile(String filePathAndName) {
		return readFileItem(filePathAndName, null);
	}

	private boolean readFileItem(String filePathAndName, String itemId) {
		fileName = filePathAndName;
		loadItemID = itemId;
		SAXParserFactory factory = SAXParserFactory.newInstance();
		tests = new Vector();
		try {
			// Parse the input
			endNeeded = false;
			factory.setValidating(false);
			SAXParser parser = factory.newSAXParser();
			parser.parse(new File(fileName), this);
		} catch (Throwable t) {
			log = "ERROR Parsing...";
			t.printStackTrace();
		}

		return true;
	}

	public String getTestXML(String itemID) {
		for (int n = 0; n < tests.size(); n++) {
			testInfo tmp = (testInfo) tests.get(n);

			if (tmp.identifier.compareTo(itemID) == 0) {
				return tmp.xml;
			}
		}
		return null; // only gets here if not found.
	}

	public String getInfo(boolean asHTML) {
		String brk;
		String info = "";

		if (asHTML) {
			brk = "</br>\n";
			info = "<p>";
		} else {
			brk = "\n";
		}
		if (fileName == null) {
			info += "No file is loaded.";
		} else {
			info += "File - " + fileName + brk;
			for (int n = 0; n < tests.size(); n++) {
				testInfo tmp = (testInfo) tests.get(n);
				String nm;
				if ((tmp.label != null) && (tmp.label.length() > 0))
					nm = tmp.label;
				else
					nm = tmp.identifier;

				info += "<a href='renderitem?file=" + fileName + "&identifier=" + tmp.identifier + "'>";
				info += nm + "</a> - " + tmp.title + "<br/>\n";
			}
		}
		if (asHTML)
			info += "</p>";
		return info;
	}

	/**
	 * Receive an object for locating the origin of SAX document events.
	 * 
	 * @param locator
	 *            An object that can return the location of any SAX document event.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void setDocumentLocator(Locator locator) {
	}

	/**
	 * Receive notification of the beginning of a document.
	 * 
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void startDocument() throws SAXException {
		log = "";
	}

	/**
	 * Receive notification of the end of a document.
	 * 
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void endDocument() throws SAXException {
	}

	/**
	 * Begin the scope of a prefix-URI Namespace mapping.
	 * 
	 * @param prefix
	 *            The Namespace prefix being declared.
	 * @param uri
	 *            The Namespace URI the prefix is mapped to.
	 * @throws SAXException
	 *             The client may throw an exception during processing.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void startPrefixMapping(String prefix, String uri) throws SAXException {
	}

	/**
	 * End the scope of a prefix-URI mapping.
	 * 
	 * @param prefix
	 *            The prefix that was being mapping.
	 * @throws SAXException
	 *             The client may throw an exception during processing.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void endPrefixMapping(String prefix) throws SAXException {
	}

	/**
	 * Receive notification of the beginning of an element.
	 * 
	 * @param namespaceURI
	 *            The Namespace URI, or the empty string if the element has no Namespace URI or if Namespace processing
	 *            is not being performed.
	 * @param localName
	 *            The local name (without prefix), or the empty string if Namespace processing is not being performed.
	 * @param qName
	 *            The qualified name (with prefix), or the empty string if qualified names are not available.
	 * @param atts
	 *            The attributes attached to the element. If there are no attributes, it shall be an empty Attributes
	 *            object.
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
		if (inTest) {
			if (endNeeded) {
				curTestInfo.xml += ">";
				endNeeded = false;
			}
		}
		/* HeLeNa -- > QTI v2.1 */
		if (qName.equals("assessmentTest")) {
			curTestInfo = new testInfo();
			String id = atts.getValue("identifier");
			curTestInfo.identifier = id;
			curTestInfo.label = atts.getValue("label");
			curTestInfo.title = atts.getValue("title");
			if (curTestInfo.title == null)
				curTestInfo.title = "Untitled";
			curTestInfo.xml = "";
			log += "Test " + id + " found\n";
			inTest = true;
		}
		if (inTest) {
			curTestInfo.xml += "<" + qName;
			for (int n = 0; n < atts.getLength(); n++) {
				if (atts.getValue(n).indexOf("\"") == -1)
					curTestInfo.xml += " " + atts.getQName(n) + "=\"" + atts.getValue(n) + "\"";
				else
					curTestInfo.xml += " " + atts.getQName(n) + "='" + atts.getValue(n) + "'";
			}
			endNeeded = true; // xml += ">";
		}
	}

	/**
	 * Receive notification of the end of an element.
	 * 
	 * @param namespaceURI
	 *            The Namespace URI, or the empty string if the element has no Namespace URI or if Namespace processing
	 *            is not being performed.
	 * @param localName
	 *            The local name (without prefix), or the empty string if Namespace processing is not being performed.
	 * @param qName
	 *            The qualified XML 1.0 name (with prefix), or the empty string if qualified names are not available.
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
		if (inTest) {
			if (endNeeded) {
				curTestInfo.xml += "/>";
				endNeeded = false;
			} else
				curTestInfo.xml += "</" + qName + ">";
		}
		if (qName.equals("assessmentTest")) {
			inTest = false;
			tests.add(curTestInfo);
		}
	}

	/**
	 * Receive notification of character data.
	 * 
	 * @param ch
	 *            The characters from the XML document.
	 * @param start
	 *            The start position in the array.
	 * @param length
	 *            The number of characters to read from the array.
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (inTest) {
			if (endNeeded) {
				curTestInfo.xml += ">";
				endNeeded = false;
			}
			String tmp = "";
			for (int n = start; n < length + start; n++)
				tmp += ch[n];
			if ((tmp.indexOf("<") != -1) || (tmp.indexOf("&") != -1))
				curTestInfo.xml += "<![CDATA[" + tmp + "]]>";
			else
				curTestInfo.xml += tmp;
		}
	}

	/**
	 * Receive notification of ignorable whitespace in element content.
	 * 
	 * @param ch
	 *            The characters from the XML document.
	 * @param start
	 *            The start position in the array.
	 * @param length
	 *            The number of characters to read from the array.
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
	}

	/**
	 * Receive notification of a processing instruction.
	 * 
	 * @param target
	 *            The processing instruction target.
	 * @param data
	 *            The processing instruction data, or null if none was supplied. The data does not include any
	 *            whitespace separating it from the target.
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void processingInstruction(String target, String data) throws SAXException {
	}

	/**
	 * Receive notification of a skipped entity.
	 * 
	 * @param name
	 *            The name of the skipped entity. If it is a parameter entity, the name will begin with '%', and if it
	 *            is the external DTD subset, it will be the string "[dtd]".
	 * @throws SAXException
	 *             Any SAX exception, possibly wrapping another exception.
	 * @todo Implement this org.xml.sax.ContentHandler method
	 */
	public void skippedEntity(String name) throws SAXException {
	}

	public void error(SAXParseException e) {
		log += "\nParse Exception\n";
	}
}

class testInfo {
	protected String identifier;
	protected String title;
	protected String label;
	protected String xml;

	public void testinfo() {

	}
}
